@@ -42,7 +42,7 @@ THE SOFTWARE.
 #include "xroute.h"
 #include "babel_errors.h"
 
-DEFINE_MTYPE_STATIC(BABELD, BABEL_IF, "Babel Interface")
+DEFINE_MTYPE_STATIC(BABELD, BABEL_IF, "Babel Interface");
 
 #define IS_ENABLE(ifp) (babel_enable_if_lookup(ifp->name) >= 0)
 
@@ -59,31 +59,11 @@ static void babel_interface_free (babel_interface_nfo *bi);
 
 
 static vector babel_enable_if;                 /* enable interfaces (by cmd). */
-static int interface_config_write(struct vty *vty);
-static struct cmd_node babel_interface_node = {
-    .name = "interface",
-    .node = INTERFACE_NODE,
-    .parent_node = CONFIG_NODE,
-    .prompt = "%s(config-if)# ",
-    .config_write = interface_config_write,
-};
 
-
-int
-babel_interface_up (ZAPI_CALLBACK_ARGS)
+int babel_ifp_up(struct interface *ifp)
 {
-    struct stream *s = NULL;
-    struct interface *ifp = NULL;
-
     debugf(BABEL_DEBUG_IF, "receive a 'interface up'");
 
-    s = zclient->ibuf;
-    ifp = zebra_interface_state_read(s, vrf_id); /* it updates iflist */
-
-    if (ifp == NULL) {
-        return 0;
-    }
-
     interface_recalculate(ifp);
     return 0;
 }
@@ -177,10 +157,11 @@ babel_interface_address_delete (ZAPI_CALLBACK_ARGS)
     if (prefix->family == AF_INET) {
         flush_interface_routes(ifc->ifp, 0);
         babel_ifp = babel_get_if_nfo(ifc->ifp);
-        if (babel_ifp->ipv4 != NULL
-            && memcmp(babel_ifp->ipv4, &prefix->u.prefix4, 4) == 0) {
-            free(babel_ifp->ipv4);
-            babel_ifp->ipv4 = NULL;
+	if (babel_ifp->ipv4 != NULL
+	    && memcmp(babel_ifp->ipv4, &prefix->u.prefix4, IPV4_MAX_BYTELEN)
+		       == 0) {
+		free(babel_ifp->ipv4);
+		babel_ifp->ipv4 = NULL;
         }
     }
 
@@ -492,7 +473,8 @@ DEFUN (babel_set_rtt_min,
     babel_ifp = babel_get_if_nfo(ifp);
     assert (babel_ifp != NULL);
 
-    babel_ifp->rtt_min = rtt;
+    /* The value is entered in milliseconds but stored as microseconds. */
+    babel_ifp->rtt_min = rtt * 1000;
     return CMD_SUCCESS;
 }
 
@@ -512,7 +494,8 @@ DEFUN (babel_set_rtt_max,
     babel_ifp = babel_get_if_nfo(ifp);
     assert (babel_ifp != NULL);
 
-    babel_ifp->rtt_max = rtt;
+    /* The value is entered in milliseconds but stored as microseconds. */
+    babel_ifp->rtt_max = rtt * 1000;
     return CMD_SUCCESS;
 }
 
@@ -693,7 +676,7 @@ interface_recalculate(struct interface *ifp)
 
     rc = resize_receive_buffer(mtu);
     if(rc < 0)
-        zlog_warn("couldn't resize receive buffer for interface %s (%d) (%d bytes).\n",
+        zlog_warn("couldn't resize receive buffer for interface %s (%d) (%d bytes).",
                   ifp->name, ifp->ifindex, mtu);
 
     memset(&mreq, 0, sizeof(mreq));
@@ -825,9 +808,11 @@ is_interface_ll_address(struct interface *ifp, const unsigned char *address)
         return 0;
 
     FOR_ALL_INTERFACES_ADDRESSES(ifp, connected, node) {
-        if(connected->address->family == AF_INET6 &&
-           memcmp(&connected->address->u.prefix6, address, 16) == 0)
-            return 1;
+	    if (connected->address->family == AF_INET6
+		&& memcmp(&connected->address->u.prefix6, address,
+			  IPV6_MAX_BYTELEN)
+			   == 0)
+		    return 1;
     }
 
     return 0;
@@ -941,13 +926,13 @@ static int
 babel_prefix_eq(struct prefix *prefix, unsigned char *p, int plen)
 {
     if(prefix->family == AF_INET6) {
-        if(prefix->prefixlen != plen ||
-           memcmp(&prefix->u.prefix6, p, 16) != 0)
-            return 0;
+	    if (prefix->prefixlen != plen
+		|| memcmp(&prefix->u.prefix6, p, IPV6_MAX_BYTELEN) != 0)
+		    return 0;
     } else if(prefix->family == AF_INET) {
-        if(plen < 96 || !v4mapped(p) || prefix->prefixlen != plen - 96 ||
-           memcmp(&prefix->u.prefix4, p + 12, 4) != 0)
-            return 0;
+	    if (plen < 96 || !v4mapped(p) || prefix->prefixlen != plen - 96
+		|| memcmp(&prefix->u.prefix4, p + 12, IPV4_MAX_BYTELEN) != 0)
+		    return 0;
     } else {
         return 0;
     }
@@ -959,31 +944,35 @@ static void
 show_babel_routes_sub(struct babel_route *route, struct vty *vty,
                       struct prefix *prefix)
 {
-    const unsigned char *nexthop =
-        memcmp(route->nexthop, route->neigh->address, 16) == 0 ?
-        NULL : route->nexthop;
-    char channels[100];
+	const unsigned char *nexthop =
+		memcmp(route->nexthop, route->neigh->address, IPV6_MAX_BYTELEN)
+				== 0
+			? NULL
+			: route->nexthop;
+	char channels[100];
 
-    if(prefix && !babel_prefix_eq(prefix, route->src->prefix, route->src->plen))
-        return;
+	if (prefix
+	    && !babel_prefix_eq(prefix, route->src->prefix, route->src->plen))
+		return;
 
-    if(route->channels[0] == 0)
-        channels[0] = '\0';
-    else {
-        int k, j = 0;
-        snprintf(channels, sizeof(channels), " chan (");
-        j = strlen(channels);
-        for(k = 0; k < DIVERSITY_HOPS; k++) {
-            if(route->channels[k] == 0)
-                break;
-            if(k > 0)
-                channels[j++] = ',';
-            snprintf(channels + j, 100 - j, "%u", route->channels[k]);
-            j = strlen(channels);
-        }
-        snprintf(channels + j, 100 - j, ")");
-        if(k == 0)
-            channels[0] = '\0';
+	if (route->channels[0] == 0)
+		channels[0] = '\0';
+	else {
+		int k, j = 0;
+		snprintf(channels, sizeof(channels), " chan (");
+		j = strlen(channels);
+		for (k = 0; k < DIVERSITY_HOPS; k++) {
+			if (route->channels[k] == 0)
+				break;
+			if (k > 0)
+				channels[j++] = ',';
+			snprintf(channels + j, 100 - j, "%u",
+				 route->channels[k]);
+			j = strlen(channels);
+		}
+		snprintf(channels + j, 100 - j, ")");
+		if (k == 0)
+			channels[0] = '\0';
     }
 
     vty_out (vty,
@@ -1107,6 +1096,7 @@ DEFUN (show_babel_route_addr,
 {
     struct in_addr addr;
     char buf[INET_ADDRSTRLEN + 8];
+    char buf1[INET_ADDRSTRLEN + 8];
     struct route_stream *routes = NULL;
     struct xroute_stream *xroutes = NULL;
     struct prefix prefix;
@@ -1119,7 +1109,8 @@ DEFUN (show_babel_route_addr,
     }
 
     /* Quagga has no convenient prefix constructors. */
-    snprintf(buf, sizeof(buf), "%s/%d", inet_ntoa(addr), 32);
+    snprintf(buf, sizeof(buf), "%s/%d",
+	     inet_ntop(AF_INET, &addr, buf1, sizeof(buf1)), 32);
 
     ret = str2prefix(buf, &prefix);
     if (ret == 0) {
@@ -1233,11 +1224,6 @@ DEFUN (show_babel_parameters,
     return CMD_SUCCESS;
 }
 
-int babel_ifp_up(struct interface *ifp)
-{
-	return 0;
-}
-
 void
 babel_if_init(void)
 {
@@ -1248,8 +1234,7 @@ babel_if_init(void)
     babel_enable_if = vector_init (1);
 
     /* install interface node and commands */
-    install_node(&babel_interface_node);
-    if_cmd_init();
+    if_cmd_init(interface_config_write);
 
     install_element(BABEL_NODE, &babel_network_cmd);
     install_element(BABEL_NODE, &no_babel_network_cmd);
@@ -1307,7 +1292,7 @@ interface_config_write (struct vty *vty)
     int write = 0;
 
     FOR_ALL_INTERFACES (vrf, ifp) {
-        vty_frame (vty, "interface %s\n",ifp->name);
+        if_vty_config_start(vty, ifp);
         if (ifp->desc)
             vty_out (vty, " description %s\n",ifp->desc);
         babel_interface_nfo *babel_ifp = babel_get_if_nfo (ifp);
@@ -1329,8 +1314,29 @@ interface_config_write (struct vty *vty)
                        babel_ifp->update_interval);
             write++;
         }
-        /* Some parameters have different defaults for wired/wireless. */
-        if (CHECK_FLAG (babel_ifp->flags, BABEL_IF_WIRED)) {
+	if (CHECK_FLAG(babel_ifp->flags, BABEL_IF_TIMESTAMPS)) {
+		vty_out(vty, " babel enable-timestamps\n");
+		write++;
+	}
+	if (babel_ifp->max_rtt_penalty != BABEL_DEFAULT_MAX_RTT_PENALTY) {
+		vty_out(vty, " babel max-rtt-penalty %u\n",
+			babel_ifp->max_rtt_penalty);
+		write++;
+	}
+	if (babel_ifp->rtt_decay != BABEL_DEFAULT_RTT_DECAY) {
+		vty_out(vty, " babel rtt-decay %u\n", babel_ifp->rtt_decay);
+		write++;
+	}
+	if (babel_ifp->rtt_min != BABEL_DEFAULT_RTT_MIN) {
+		vty_out(vty, " babel rtt-min %u\n", babel_ifp->rtt_min / 1000);
+		write++;
+	}
+	if (babel_ifp->rtt_max != BABEL_DEFAULT_RTT_MAX) {
+		vty_out(vty, " babel rtt-max %u\n", babel_ifp->rtt_max / 1000);
+		write++;
+	}
+	/* Some parameters have different defaults for wired/wireless. */
+	if (CHECK_FLAG (babel_ifp->flags, BABEL_IF_WIRED)) {
             if (!CHECK_FLAG (babel_ifp->flags, BABEL_IF_SPLIT_HORIZON)) {
                 vty_out (vty, " no babel split-horizon\n");
                 write++;
@@ -1363,7 +1369,7 @@ interface_config_write (struct vty *vty)
                 write++;
             }
         }
-        vty_endframe (vty, "!\n");
+        if_vty_config_end(vty);
         write++;
     }
     return write;
@@ -1396,9 +1402,10 @@ babel_interface_allocate (void)
     babel_ifp->bucket_time = babel_now.tv_sec;
     babel_ifp->bucket = BUCKET_TOKENS_MAX;
     babel_ifp->hello_seqno = (frr_weak_random() & 0xFFFF);
-    babel_ifp->rtt_min = 10000;
-    babel_ifp->rtt_max = 120000;
-    babel_ifp->max_rtt_penalty = 150;
+    babel_ifp->rtt_decay = BABEL_DEFAULT_RTT_DECAY;
+    babel_ifp->rtt_min = BABEL_DEFAULT_RTT_MIN;
+    babel_ifp->rtt_max = BABEL_DEFAULT_RTT_MAX;
+    babel_ifp->max_rtt_penalty = BABEL_DEFAULT_MAX_RTT_PENALTY;
     babel_ifp->hello_interval = BABEL_DEFAULT_HELLO_INTERVAL;
     babel_ifp->update_interval = BABEL_DEFAULT_UPDATE_INTERVAL;
     babel_ifp->channel = BABEL_IF_CHANNEL_INTERFERING;
