@@ -166,7 +166,7 @@ resize_route_table(int new_slots)
 static struct babel_route *
 insert_route(struct babel_route *route)
 {
-    int i, n;
+    int i, n = 0;
 
     assert(!route->installed);
 
@@ -399,15 +399,17 @@ install_route(struct babel_route *route)
         return;
 
     if(!route_feasible(route))
-        flog_err(EC_BABEL_ROUTE, "WARNING: installing unfeasible route (this shouldn't happen).");
+	    flog_err(EC_BABEL_ROUTE,
+		     "Installing unfeasible route (this shouldn't happen).");
 
     i = find_route_slot(route->src->prefix, route->src->plen, NULL);
     assert(i >= 0 && i < route_slots);
 
     if(routes[i] != route && routes[i]->installed) {
-        flog_err(EC_BABEL_ROUTE,
-		  "WARNING: attempting to install duplicate route (this shouldn't happen).");
-        return;
+	    flog_err(
+		    EC_BABEL_ROUTE,
+		    "Attempting to install duplicate route (this shouldn't happen).");
+	    return;
     }
 
     rc = kernel_route(ROUTE_ADD, route->src->prefix, route->src->plen,
@@ -463,7 +465,8 @@ switch_routes(struct babel_route *old, struct babel_route *new)
         return;
 
     if(!route_feasible(new))
-        flog_err(EC_BABEL_ROUTE, "WARNING: switching to unfeasible route (this shouldn't happen).");
+	    flog_err(EC_BABEL_ROUTE,
+		     "Switching to unfeasible route (this shouldn't happen).");
 
     rc = kernel_route(ROUTE_MODIFY, old->src->prefix, old->src->plen,
                       old->nexthop, old->neigh->ifp->ifindex,
