@@ -32,10 +32,10 @@
 
 #include "bfd.h"
 
-DEFINE_MTYPE_STATIC(BFDD, BFDD_CONFIG, "long-lived configuration memory")
-DEFINE_MTYPE_STATIC(BFDD, BFDD_PROFILE, "long-lived profile memory")
-DEFINE_MTYPE_STATIC(BFDD, BFDD_SESSION_OBSERVER, "Session observer")
-DEFINE_MTYPE_STATIC(BFDD, BFDD_VRF, "BFD VRF")
+DEFINE_MTYPE_STATIC(BFDD, BFDD_CONFIG, "long-lived configuration memory");
+DEFINE_MTYPE_STATIC(BFDD, BFDD_PROFILE, "long-lived profile memory");
+DEFINE_MTYPE_STATIC(BFDD, BFDD_SESSION_OBSERVER, "Session observer");
+DEFINE_MTYPE_STATIC(BFDD, BFDD_VRF, "BFD VRF");
 
 /*
  * Prototypes
@@ -51,8 +51,6 @@ static void bs_admin_down_handler(struct bfd_session *bs, int nstate);
 static void bs_down_handler(struct bfd_session *bs, int nstate);
 static void bs_init_handler(struct bfd_session *bs, int nstate);
 static void bs_up_handler(struct bfd_session *bs, int nstate);
-static void bs_neighbour_admin_down_handler(struct bfd_session *bfd,
-					    uint8_t diag);
 
 /**
  * Remove BFD profile from all BFD sessions so we don't leave dangling
@@ -85,12 +83,13 @@ struct bfd_profile *bfd_profile_lookup(const char *name)
 
 static void bfd_profile_set_default(struct bfd_profile *bp)
 {
-	bp->admin_shutdown = true;
+	bp->admin_shutdown = false;
 	bp->detection_multiplier = BFD_DEFDETECTMULT;
 	bp->echo_mode = false;
 	bp->passive = false;
 	bp->minimum_ttl = BFD_DEF_MHOP_TTL;
-	bp->min_echo_rx = BFD_DEF_REQ_MIN_ECHO;
+	bp->min_echo_rx = BFD_DEF_REQ_MIN_ECHO_RX;
+	bp->min_echo_tx = BFD_DEF_DES_MIN_ECHO_TX;
 	bp->min_rx = BFD_DEFREQUIREDMINRX;
 	bp->min_tx = BFD_DEFDESIREDMINTX;
 }
@@ -179,13 +178,19 @@ void bfd_session_apply(struct bfd_session *bs)
 
 	/* We can only apply echo options on single hop sessions. */
 	if (!CHECK_FLAG(bs->flags, BFD_SESS_FLAG_MH)) {
-		/* Configure remote echo if it was default. */
-		if (bs->peer_profile.min_echo_rx == BFD_DEF_REQ_MIN_ECHO)
-			bs->timers.required_min_echo = bp->min_echo_rx;
+		/* Configure echo timers if they were default. */
+		if (bs->peer_profile.min_echo_rx == BFD_DEF_REQ_MIN_ECHO_RX)
+			bs->timers.required_min_echo_rx = bp->min_echo_rx;
 		else
-			bs->timers.required_min_echo =
+			bs->timers.required_min_echo_rx =
 				bs->peer_profile.min_echo_rx;
 
+		if (bs->peer_profile.min_echo_tx == BFD_DEF_DES_MIN_ECHO_TX)
+			bs->timers.desired_min_echo_tx = bp->min_echo_tx;
+		else
+			bs->timers.desired_min_echo_tx =
+				bs->peer_profile.min_echo_tx;
+
 		/* Toggle echo if default value. */
 		if (bs->peer_profile.echo_mode == false)
 			bfd_set_echo(bs, bp->echo_mode);
@@ -206,7 +211,7 @@ void bfd_session_apply(struct bfd_session *bs)
 		bfd_set_passive_mode(bs, bs->peer_profile.passive);
 
 	/* Toggle 'no shutdown' if default value. */
-	if (bs->peer_profile.admin_shutdown)
+	if (bs->peer_profile.admin_shutdown == false)
 		bfd_set_shutdown(bs, bp->admin_shutdown);
 	else
 		bfd_set_shutdown(bs, bs->peer_profile.admin_shutdown);
@@ -216,6 +221,9 @@ void bfd_session_apply(struct bfd_session *bs)
 	    && (bs->timers.desired_min_tx != min_tx
 		|| bs->timers.required_min_rx != min_rx))
 		bfd_set_polling(bs);
+
+	/* Send updated information to data plane. */
+	bfd_dplane_update_session(bs);
 }
 
 void bfd_profile_remove(struct bfd_session *bs)
@@ -293,6 +301,10 @@ int bfd_session_enable(struct bfd_session *bs)
 	struct vrf *vrf = NULL;
 	int psock;
 
+	/* We are using data plane, we don't need software. */
+	if (bs->bdc)
+		return 0;
+
 	/*
 	 * If the interface or VRF doesn't exist, then we must register
 	 * the session but delay its start.
@@ -301,52 +313,42 @@ int bfd_session_enable(struct bfd_session *bs)
 		vrf = vrf_lookup_by_name(bs->key.vrfname);
 		if (vrf == NULL) {
 			zlog_err(
-				"session-enable: specified VRF doesn't exists.");
+				"session-enable: specified VRF %s doesn't exists.",
+				bs->key.vrfname);
 			return 0;
 		}
+	} else {
+		vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	}
 
-	if (!vrf_is_backend_netns() && vrf && vrf->vrf_id != VRF_DEFAULT
-	    && !if_lookup_by_name(vrf->name, vrf->vrf_id)) {
-		zlog_err("session-enable: vrf interface %s not available yet",
-			 vrf->name);
-		return 0;
-	}
+	assert(vrf);
 
 	if (bs->key.ifname[0]) {
-		if (vrf)
-			ifp = if_lookup_by_name(bs->key.ifname, vrf->vrf_id);
-		else
-			ifp = if_lookup_by_name_all_vrf(bs->key.ifname);
+		ifp = if_lookup_by_name(bs->key.ifname, vrf->vrf_id);
 		if (ifp == NULL) {
 			zlog_err(
-				"session-enable: specified interface doesn't exists.");
+				"session-enable: specified interface %s (VRF %s) doesn't exist.",
+				bs->key.ifname, vrf->name);
 			return 0;
 		}
-		if (bs->key.ifname[0] && !vrf) {
-			vrf = vrf_lookup_by_id(ifp->vrf_id);
-			if (vrf == NULL) {
-				zlog_err(
-					"session-enable: specified VRF doesn't exists.");
-				return 0;
-			}
-		}
 	}
 
 	/* Assign interface/VRF pointers. */
 	bs->vrf = vrf;
-	if (bs->vrf == NULL)
-		bs->vrf = vrf_lookup_by_id(VRF_DEFAULT);
-	assert(bs->vrf);
 
-	if (bs->key.ifname[0]
-	    && CHECK_FLAG(bs->flags, BFD_SESS_FLAG_MH) == 0)
-		bs->ifp = ifp;
+	/* Assign interface pointer (if any). */
+	bs->ifp = ifp;
+
+	/* Attempt to use data plane. */
+	if (bglobal.bg_use_dplane && bfd_dplane_add_session(bs) == 0) {
+		control_notify_config(BCM_NOTIFY_CONFIG_ADD, bs);
+		return 0;
+	}
 
 	/* Sanity check: don't leak open sockets. */
 	if (bs->sock != -1) {
 		if (bglobal.debug_peer_event)
-			zlog_debug("session-enable: previous socket open");
+			zlog_debug("%s: previous socket open", __func__);
 
 		close(bs->sock);
 		bs->sock = -1;
@@ -379,6 +381,9 @@ int bfd_session_enable(struct bfd_session *bs)
 		ptm_bfd_start_xmt_timer(bs, false);
 	}
 
+	/* initialize RTT */
+	bfd_rtt_init(bs);
+
 	return 0;
 }
 
@@ -390,6 +395,10 @@ int bfd_session_enable(struct bfd_session *bs)
  */
 void bfd_session_disable(struct bfd_session *bs)
 {
+	/* We are using data plane, we don't need software. */
+	if (bs->bdc)
+		return;
+
 	/* Free up socket resources. */
 	if (bs->sock != -1) {
 		close(bs->sock);
@@ -400,8 +409,6 @@ void bfd_session_disable(struct bfd_session *bs)
 	bfd_recvtimer_delete(bs);
 	bfd_xmttimer_delete(bs);
 	ptm_bfd_echo_stop(bs);
-	bs->vrf = NULL;
-	bs->ifp = NULL;
 
 	/* Set session down so it doesn't report UP and disabled. */
 	ptm_bfd_sess_dn(bs, BD_PATH_DOWN);
@@ -450,7 +457,17 @@ void ptm_bfd_start_xmt_timer(struct bfd_session *bfd, bool is_echo)
 static void ptm_bfd_echo_xmt_TO(struct bfd_session *bfd)
 {
 	/* Send the scheduled echo  packet */
-	ptm_bfd_echo_snd(bfd);
+	/* if ipv4 use the new echo implementation that causes
+	 * the packet to be looped in forwarding plane of peer
+	 */
+	if (CHECK_FLAG(bfd->flags, BFD_SESS_FLAG_IPV6) == 0)
+#ifdef BFD_LINUX
+		ptm_bfd_echo_fp_snd(bfd);
+#else
+		ptm_bfd_echo_snd(bfd);
+#endif
+	else
+		ptm_bfd_echo_snd(bfd);
 
 	/* Restart the timer for next time */
 	ptm_bfd_start_xmt_timer(bfd, true);
@@ -478,8 +495,10 @@ void ptm_bfd_echo_stop(struct bfd_session *bfd)
 void ptm_bfd_echo_start(struct bfd_session *bfd)
 {
 	bfd->echo_detect_TO = (bfd->remote_detect_mult * bfd->echo_xmt_TO);
-	if (bfd->echo_detect_TO > 0)
+	if (bfd->echo_detect_TO > 0) {
+		bfd_echo_recvtimer_update(bfd);
 		ptm_bfd_echo_xmt_TO(bfd);
+	}
 }
 
 void ptm_bfd_sess_up(struct bfd_session *bfd)
@@ -552,6 +571,15 @@ void ptm_bfd_sess_dn(struct bfd_session *bfd, uint8_t diag)
 				   state_list[bfd->ses_state].str,
 				   get_diag_str(bfd->local_diag));
 	}
+
+	/* clear peer's mac address */
+	UNSET_FLAG(bfd->flags, BFD_SESS_FLAG_MAC_SET);
+	memset(bfd->peer_hw_addr, 0, sizeof(bfd->peer_hw_addr));
+	/* reset local address ,it might has been be changed after bfd is up*/
+	memset(&bfd->local_address, 0, sizeof(bfd->local_address));
+
+	/* reset RTT */
+	bfd_rtt_init(bfd);
 }
 
 static struct bfd_session *bfd_find_disc(struct sockaddr_any *sa,
@@ -582,10 +610,10 @@ static struct bfd_session *bfd_find_disc(struct sockaddr_any *sa,
 struct bfd_session *ptm_bfd_sess_find(struct bfd_pkt *cp,
 				      struct sockaddr_any *peer,
 				      struct sockaddr_any *local,
-				      ifindex_t ifindex, vrf_id_t vrfid,
+				      struct interface *ifp,
+				      vrf_id_t vrfid,
 				      bool is_mhop)
 {
-	struct interface *ifp;
 	struct vrf *vrf;
 	struct bfd_key key;
 
@@ -593,21 +621,8 @@ struct bfd_session *ptm_bfd_sess_find(struct bfd_pkt *cp,
 	if (cp->discrs.remote_discr)
 		return bfd_find_disc(peer, ntohl(cp->discrs.remote_discr));
 
-	/*
-	 * Search for session without using discriminator.
-	 *
-	 * XXX: we can't trust `vrfid` because the VRF handling is not
-	 * properly implemented. Meanwhile we should use the interface
-	 * VRF to find out which one it belongs.
-	 */
-	ifp = if_lookup_by_index_all_vrf(ifindex);
-	if (ifp == NULL) {
-		if (vrfid != VRF_DEFAULT)
-			vrf = vrf_lookup_by_id(vrfid);
-		else
-			vrf = NULL;
-	} else
-		vrf = vrf_lookup_by_id(ifp->vrf_id);
+	/* Search for session without using discriminator. */
+	vrf = vrf_lookup_by_id(vrfid);
 
 	gen_bfd_key(&key, peer, local, is_mhop, ifp ? ifp->name : NULL,
 		    vrf ? vrf->name : VRF_DEFAULT_NAME);
@@ -616,27 +631,23 @@ struct bfd_session *ptm_bfd_sess_find(struct bfd_pkt *cp,
 	return bfd_key_lookup(key);
 }
 
-int bfd_xmt_cb(struct thread *t)
+void bfd_xmt_cb(struct thread *t)
 {
 	struct bfd_session *bs = THREAD_ARG(t);
 
 	ptm_bfd_xmt_TO(bs, 0);
-
-	return 0;
 }
 
-int bfd_echo_xmt_cb(struct thread *t)
+void bfd_echo_xmt_cb(struct thread *t)
 {
 	struct bfd_session *bs = THREAD_ARG(t);
 
 	if (bs->echo_xmt_TO > 0)
 		ptm_bfd_echo_xmt_TO(bs);
-
-	return 0;
 }
 
 /* Was ptm_bfd_detect_TO() */
-int bfd_recvtimer_cb(struct thread *t)
+void bfd_recvtimer_cb(struct thread *t)
 {
 	struct bfd_session *bs = THREAD_ARG(t);
 
@@ -644,22 +655,12 @@ int bfd_recvtimer_cb(struct thread *t)
 	case PTM_BFD_INIT:
 	case PTM_BFD_UP:
 		ptm_bfd_sess_dn(bs, BD_CONTROL_EXPIRED);
-		bfd_recvtimer_update(bs);
-		break;
-
-	default:
-		/* Second detect time expiration, zero remote discr (section
-		 * 6.5.1)
-		 */
-		bs->discrs.remote_discr = 0;
 		break;
 	}
-
-	return 0;
 }
 
 /* Was ptm_bfd_echo_detect_TO() */
-int bfd_echo_recvtimer_cb(struct thread *t)
+void bfd_echo_recvtimer_cb(struct thread *t)
 {
 	struct bfd_session *bs = THREAD_ARG(t);
 
@@ -669,8 +670,6 @@ int bfd_echo_recvtimer_cb(struct thread *t)
 		ptm_bfd_sess_dn(bs, BD_ECHO_FAILED);
 		break;
 	}
-
-	return 0;
 }
 
 struct bfd_session *bfd_session_new(void)
@@ -684,7 +683,8 @@ struct bfd_session *bfd_session_new(void)
 
 	bs->timers.desired_min_tx = BFD_DEFDESIREDMINTX;
 	bs->timers.required_min_rx = BFD_DEFREQUIREDMINRX;
-	bs->timers.required_min_echo = BFD_DEF_REQ_MIN_ECHO;
+	bs->timers.required_min_echo_rx = BFD_DEF_REQ_MIN_ECHO_RX;
+	bs->timers.desired_min_echo_tx = BFD_DEF_DES_MIN_ECHO_TX;
 	bs->detect_mult = BFD_DEFDETECTMULT;
 	bs->mh_ttl = BFD_DEF_MHOP_TTL;
 	bs->ses_state = PTM_BFD_DOWN;
@@ -753,9 +753,14 @@ static void _bfd_session_update(struct bfd_session *bs,
 		bs->peer_profile.detection_multiplier = bs->detect_mult;
 	}
 
-	if (bpc->bpc_has_echointerval) {
-		bs->timers.required_min_echo = bpc->bpc_echointerval * 1000;
-		bs->peer_profile.min_echo_rx = bs->timers.required_min_echo;
+	if (bpc->bpc_has_echorecvinterval) {
+		bs->timers.required_min_echo_rx = bpc->bpc_echorecvinterval * 1000;
+		bs->peer_profile.min_echo_rx = bs->timers.required_min_echo_rx;
+	}
+
+	if (bpc->bpc_has_echotxinterval) {
+		bs->timers.desired_min_echo_tx = bpc->bpc_echotxinterval * 1000;
+		bs->peer_profile.min_echo_tx = bs->timers.desired_min_echo_tx;
 	}
 
 	if (bpc->bpc_has_label)
@@ -786,7 +791,7 @@ static void _bfd_session_update(struct bfd_session *bs,
 	 * Apply profile last: it also calls `bfd_set_shutdown`.
 	 *
 	 * There is no problem calling `shutdown` twice if the value doesn't
-	 * change or if it is overriden by peer specific configuration.
+	 * change or if it is overridden by peer specific configuration.
 	 */
 	if (bpc->bpc_has_profile)
 		bfd_profile_apply(bpc->bpc_profile, bs);
@@ -811,6 +816,9 @@ void bfd_session_free(struct bfd_session *bs)
 
 	bfd_session_disable(bs);
 
+	/* Remove session from data plane if any. */
+	bfd_dplane_delete_session(bs);
+
 	bfd_key_delete(bs->key);
 	bfd_id_delete(bs->discrs.my_discr);
 
@@ -944,7 +952,7 @@ int ptm_bfd_sess_del(struct bfd_peer_cfg *bpc)
 	}
 
 	if (bglobal.debug_peer_event)
-		zlog_debug("session-delete: %s", bs_to_string(bs));
+		zlog_debug("%s: %s", __func__, bs_to_string(bs));
 
 	control_notify_config(BCM_NOTIFY_CONFIG_DELETE, bs);
 
@@ -1001,9 +1009,18 @@ static void bs_down_handler(struct bfd_session *bs, int nstate)
 		 */
 		bs->ses_state = PTM_BFD_INIT;
 
-		/* Answer peer with INIT immediately in passive mode. */
+		/*
+		 * RFC 5880, Section 6.1.
+		 * A system taking the Passive role MUST NOT begin
+		 * sending BFD packets for a particular session until
+		 * it has received a BFD packet for that session, and thus
+		 * has learned the remote system's discriminator value.
+		 *
+		 * Now we can start transmission timer in passive mode.
+		 */
 		if (CHECK_FLAG(bs->flags, BFD_SESS_FLAG_PASSIVE))
-			ptm_bfd_snd(bs, 0);
+			ptm_bfd_xmt_TO(bs, 0);
+
 		break;
 
 	case PTM_BFD_INIT:
@@ -1030,7 +1047,7 @@ static void bs_init_handler(struct bfd_session *bs, int nstate)
 		 * Remote peer doesn't want to talk, so lets make the
 		 * connection down.
 		 */
-		bs->ses_state = PTM_BFD_DOWN;
+		ptm_bfd_sess_dn(bs, BD_NEIGHBOR_DOWN);
 		break;
 
 	case PTM_BFD_DOWN:
@@ -1051,46 +1068,10 @@ static void bs_init_handler(struct bfd_session *bs, int nstate)
 	}
 }
 
-static void bs_neighbour_admin_down_handler(struct bfd_session *bfd,
-					    uint8_t diag)
-{
-	int old_state = bfd->ses_state;
-
-	bfd->local_diag = diag;
-	bfd->discrs.remote_discr = 0;
-	bfd->ses_state = PTM_BFD_DOWN;
-	bfd->polling = 0;
-	bfd->demand_mode = 0;
-	monotime(&bfd->downtime);
-
-	/* Slow down the control packets, the connection is down. */
-	bs_set_slow_timers(bfd);
-
-	/* only signal clients when going from up->down state */
-	if (old_state == PTM_BFD_UP)
-		control_notify(bfd, PTM_BFD_ADM_DOWN);
-
-	/* Stop echo packet transmission if they are active */
-	if (CHECK_FLAG(bfd->flags, BFD_SESS_FLAG_ECHO_ACTIVE))
-		ptm_bfd_echo_stop(bfd);
-
-	if (old_state != bfd->ses_state) {
-		bfd->stats.session_down++;
-		if (bglobal.debug_peer_event)
-			zlog_debug("state-change: [%s] %s -> %s reason:%s",
-				   bs_to_string(bfd), state_list[old_state].str,
-				   state_list[bfd->ses_state].str,
-				   get_diag_str(bfd->local_diag));
-	}
-}
-
 static void bs_up_handler(struct bfd_session *bs, int nstate)
 {
 	switch (nstate) {
 	case PTM_BFD_ADM_DOWN:
-		bs_neighbour_admin_down_handler(bs, BD_ADMIN_DOWN);
-		break;
-
 	case PTM_BFD_DOWN:
 		/* Peer lost or asked to shutdown connection. */
 		ptm_bfd_sess_dn(bs, BD_NEIGHBOR_DOWN);
@@ -1170,10 +1151,10 @@ void bs_echo_timer_handler(struct bfd_session *bs)
 	 * RFC 5880, Section 6.8.9.
 	 */
 	old_timer = bs->echo_xmt_TO;
-	if (bs->remote_timers.required_min_echo > bs->timers.required_min_echo)
+	if (bs->remote_timers.required_min_echo > bs->timers.desired_min_echo_tx)
 		bs->echo_xmt_TO = bs->remote_timers.required_min_echo;
 	else
-		bs->echo_xmt_TO = bs->timers.required_min_echo;
+		bs->echo_xmt_TO = bs->timers.desired_min_echo_tx;
 
 	if (CHECK_FLAG(bs->flags, BFD_SESS_FLAG_ECHO_ACTIVE) == 0
 	    || old_timer != bs->echo_xmt_TO)
@@ -1222,27 +1203,6 @@ void bs_final_handler(struct bfd_session *bs)
 	/* Apply new transmission timer immediately. */
 	ptm_bfd_start_xmt_timer(bs, false);
 
-	/*
-	 * Detection timeout calculation:
-	 * The minimum detection timeout is the remote detection
-	 * multipler (number of packets to be missed) times the agreed
-	 * transmission interval.
-	 *
-	 * RFC 5880, Section 6.8.4.
-	 *
-	 * TODO: support sending/counting more packets inside detection
-	 * timeout.
-	 */
-	if (bs->remote_timers.required_min_rx > bs->timers.desired_min_tx)
-		bs->detect_TO = bs->remote_detect_mult
-				* bs->remote_timers.required_min_rx;
-	else
-		bs->detect_TO = bs->remote_detect_mult
-				* bs->timers.desired_min_tx;
-
-	/* Apply new receive timer immediately. */
-	bfd_recvtimer_update(bs);
-
 	/* Notify watchers about changed timers. */
 	control_notify_config(BCM_NOTIFY_CONFIG_UPDATE, bs);
 }
@@ -1274,14 +1234,18 @@ void bfd_set_echo(struct bfd_session *bs, bool echo)
 		SET_FLAG(bs->flags, BFD_SESS_FLAG_ECHO);
 
 		/* Activate/update echo receive timeout timer. */
-		bs_echo_timer_handler(bs);
+		if (bs->bdc == NULL)
+			bs_echo_timer_handler(bs);
 	} else {
 		/* Check if echo mode is already disabled. */
 		if (!CHECK_FLAG(bs->flags, BFD_SESS_FLAG_ECHO))
 			return;
 
 		UNSET_FLAG(bs->flags, BFD_SESS_FLAG_ECHO);
-		ptm_bfd_echo_stop(bs);
+
+		/* Deactivate timeout timer. */
+		if (bs->bdc == NULL)
+			ptm_bfd_echo_stop(bs);
 	}
 }
 
@@ -1306,6 +1270,14 @@ void bfd_set_shutdown(struct bfd_session *bs, bool shutdown)
 
 		SET_FLAG(bs->flags, BFD_SESS_FLAG_SHUTDOWN);
 
+		/* Handle data plane shutdown case. */
+		if (bs->bdc) {
+			bs->ses_state = PTM_BFD_ADM_DOWN;
+			bfd_dplane_update_session(bs);
+			control_notify(bs, bs->ses_state);
+			return;
+		}
+
 		/* Disable all events. */
 		bfd_recvtimer_delete(bs);
 		bfd_echo_recvtimer_delete(bs);
@@ -1326,6 +1298,14 @@ void bfd_set_shutdown(struct bfd_session *bs, bool shutdown)
 
 		UNSET_FLAG(bs->flags, BFD_SESS_FLAG_SHUTDOWN);
 
+		/* Handle data plane shutdown case. */
+		if (bs->bdc) {
+			bs->ses_state = PTM_BFD_DOWN;
+			bfd_dplane_update_session(bs);
+			control_notify(bs, bs->ses_state);
+			return;
+		}
+
 		/* Change and notify state change. */
 		bs->ses_state = PTM_BFD_DOWN;
 		control_notify(bs, bs->ses_state);
@@ -1458,7 +1438,7 @@ int strtosa(const char *addr, struct sockaddr_any *sa)
 
 void integer2timestr(uint64_t time, char *buf, size_t buflen)
 {
-	unsigned int year, month, day, hour, minute, second;
+	uint64_t year, month, day, hour, minute, second;
 	int rv;
 
 #define MINUTES (60)
@@ -1470,7 +1450,7 @@ void integer2timestr(uint64_t time, char *buf, size_t buflen)
 		year = time / YEARS;
 		time -= year * YEARS;
 
-		rv = snprintf(buf, buflen, "%u year(s), ", year);
+		rv = snprintfrr(buf, buflen, "%" PRIu64 " year(s), ", year);
 		buf += rv;
 		buflen -= rv;
 	}
@@ -1478,7 +1458,7 @@ void integer2timestr(uint64_t time, char *buf, size_t buflen)
 		month = time / MONTHS;
 		time -= month * MONTHS;
 
-		rv = snprintf(buf, buflen, "%u month(s), ", month);
+		rv = snprintfrr(buf, buflen, "%" PRIu64 " month(s), ", month);
 		buf += rv;
 		buflen -= rv;
 	}
@@ -1486,7 +1466,7 @@ void integer2timestr(uint64_t time, char *buf, size_t buflen)
 		day = time / DAYS;
 		time -= day * DAYS;
 
-		rv = snprintf(buf, buflen, "%u day(s), ", day);
+		rv = snprintfrr(buf, buflen, "%" PRIu64 " day(s), ", day);
 		buf += rv;
 		buflen -= rv;
 	}
@@ -1494,7 +1474,7 @@ void integer2timestr(uint64_t time, char *buf, size_t buflen)
 		hour = time / HOURS;
 		time -= hour * HOURS;
 
-		rv = snprintf(buf, buflen, "%u hour(s), ", hour);
+		rv = snprintfrr(buf, buflen, "%" PRIu64 " hour(s), ", hour);
 		buf += rv;
 		buflen -= rv;
 	}
@@ -1502,12 +1482,12 @@ void integer2timestr(uint64_t time, char *buf, size_t buflen)
 		minute = time / MINUTES;
 		time -= minute * MINUTES;
 
-		rv = snprintf(buf, buflen, "%u minute(s), ", minute);
+		rv = snprintfrr(buf, buflen, "%" PRIu64 " minute(s), ", minute);
 		buf += rv;
 		buflen -= rv;
 	}
 	second = time % MINUTES;
-	snprintf(buf, buflen, "%u second(s)", second);
+	snprintfrr(buf, buflen, "%" PRIu64 " second(s)", second);
 }
 
 const char *bs_to_string(const struct bfd_session *bs)
@@ -1887,6 +1867,14 @@ void bfd_sessions_remove_manual(void)
 	hash_iterate(bfd_key_hash, _bfd_session_remove_manual, NULL);
 }
 
+void bfd_profiles_remove(void)
+{
+	struct bfd_profile *bp;
+
+	while ((bp = TAILQ_FIRST(&bplist)) != NULL)
+		bfd_profile_free(bp);
+}
+
 /*
  * Profile related hash functions.
  */
@@ -1943,19 +1931,6 @@ static int bfd_vrf_delete(struct vrf *vrf)
 	return 0;
 }
 
-static int bfd_vrf_update(struct vrf *vrf)
-{
-	if (!vrf_is_enabled(vrf))
-		return 0;
-
-	if (bglobal.debug_zebra)
-		zlog_debug("VRF update: %s(%u)", vrf->name, vrf->vrf_id);
-
-	/* a different name is given; update bfd list */
-	bfdd_sessions_enable_vrf(vrf);
-	return 0;
-}
-
 static int bfd_vrf_enable(struct vrf *vrf)
 {
 	struct bfd_vrf_global *bvrf;
@@ -1965,47 +1940,54 @@ static int bfd_vrf_enable(struct vrf *vrf)
 		bvrf = XCALLOC(MTYPE_BFDD_VRF, sizeof(struct bfd_vrf_global));
 		bvrf->vrf = vrf;
 		vrf->info = (void *)bvrf;
+
+		/* Disable sockets if using data plane. */
+		if (bglobal.bg_use_dplane) {
+			bvrf->bg_shop = -1;
+			bvrf->bg_mhop = -1;
+			bvrf->bg_shop6 = -1;
+			bvrf->bg_mhop6 = -1;
+			bvrf->bg_echo = -1;
+			bvrf->bg_echov6 = -1;
+		}
 	} else
 		bvrf = vrf->info;
 
 	if (bglobal.debug_zebra)
 		zlog_debug("VRF enable add %s id %u", vrf->name, vrf->vrf_id);
 
-	if (vrf->vrf_id == VRF_DEFAULT ||
-	    vrf_get_backend() == VRF_BACKEND_NETNS) {
-		if (!bvrf->bg_shop)
-			bvrf->bg_shop = bp_udp_shop(vrf);
-		if (!bvrf->bg_mhop)
-			bvrf->bg_mhop = bp_udp_mhop(vrf);
-		if (!bvrf->bg_shop6)
-			bvrf->bg_shop6 = bp_udp6_shop(vrf);
-		if (!bvrf->bg_mhop6)
-			bvrf->bg_mhop6 = bp_udp6_mhop(vrf);
-		if (!bvrf->bg_echo)
-			bvrf->bg_echo = bp_echo_socket(vrf);
-		if (!bvrf->bg_echov6)
-			bvrf->bg_echov6 = bp_echov6_socket(vrf);
+	if (!bvrf->bg_shop)
+		bvrf->bg_shop = bp_udp_shop(vrf);
+	if (!bvrf->bg_mhop)
+		bvrf->bg_mhop = bp_udp_mhop(vrf);
+	if (!bvrf->bg_shop6)
+		bvrf->bg_shop6 = bp_udp6_shop(vrf);
+	if (!bvrf->bg_mhop6)
+		bvrf->bg_mhop6 = bp_udp6_mhop(vrf);
+	if (!bvrf->bg_echo)
+		bvrf->bg_echo = bp_echo_socket(vrf);
+	if (!bvrf->bg_echov6)
+		bvrf->bg_echov6 = bp_echov6_socket(vrf);
+
+	if (!bvrf->bg_ev[0] && bvrf->bg_shop != -1)
+		thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_shop,
+				&bvrf->bg_ev[0]);
+	if (!bvrf->bg_ev[1] && bvrf->bg_mhop != -1)
+		thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_mhop,
+				&bvrf->bg_ev[1]);
+	if (!bvrf->bg_ev[2] && bvrf->bg_shop6 != -1)
+		thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_shop6,
+				&bvrf->bg_ev[2]);
+	if (!bvrf->bg_ev[3] && bvrf->bg_mhop6 != -1)
+		thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_mhop6,
+				&bvrf->bg_ev[3]);
+	if (!bvrf->bg_ev[4] && bvrf->bg_echo != -1)
+		thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_echo,
+				&bvrf->bg_ev[4]);
+	if (!bvrf->bg_ev[5] && bvrf->bg_echov6 != -1)
+		thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_echov6,
+				&bvrf->bg_ev[5]);
 
-		/* Add descriptors to the event loop. */
-		if (!bvrf->bg_ev[0])
-			thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_shop,
-					&bvrf->bg_ev[0]);
-		if (!bvrf->bg_ev[1])
-			thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_mhop,
-					&bvrf->bg_ev[1]);
-		if (!bvrf->bg_ev[2] && bvrf->bg_shop6 != -1)
-			thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_shop6,
-					&bvrf->bg_ev[2]);
-		if (!bvrf->bg_ev[3] && bvrf->bg_mhop6 != -1)
-			thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_mhop6,
-					&bvrf->bg_ev[3]);
-		if (!bvrf->bg_ev[4])
-			thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_echo,
-					&bvrf->bg_ev[4]);
-		if (!bvrf->bg_ev[5] && bvrf->bg_echov6 != -1)
-			thread_add_read(master, bfd_recv_cb, bvrf, bvrf->bg_echov6,
-					&bvrf->bg_ev[5]);
-	}
 	if (vrf->vrf_id != VRF_DEFAULT) {
 		bfdd_zclient_register(vrf->vrf_id);
 		bfdd_sessions_enable_vrf(vrf);
@@ -2058,8 +2040,7 @@ static int bfd_vrf_disable(struct vrf *vrf)
 
 void bfd_vrf_init(void)
 {
-	vrf_init(bfd_vrf_new, bfd_vrf_enable, bfd_vrf_disable,
-		 bfd_vrf_delete, bfd_vrf_update);
+	vrf_init(bfd_vrf_new, bfd_vrf_enable, bfd_vrf_disable, bfd_vrf_delete);
 }
 
 void bfd_vrf_terminate(void)
@@ -2084,64 +2065,18 @@ struct bfd_vrf_global *bfd_vrf_look_by_session(struct bfd_session *bfd)
 	return bfd->vrf->info;
 }
 
-void bfd_session_update_vrf_name(struct bfd_session *bs, struct vrf *vrf)
-{
-	if (!vrf || !bs)
-		return;
-	/* update key */
-	hash_release(bfd_key_hash, bs);
-	/*
-	 * HACK: Change the BFD VRF in the running configuration directly,
-	 * bypassing the northbound layer. This is necessary to avoid deleting
-	 * the BFD and readding it in the new VRF, which would have
-	 * several implications.
-	 */
-	if (yang_module_find("frr-bfdd") && bs->key.vrfname[0]) {
-		struct lyd_node *bfd_dnode;
-		char xpath[XPATH_MAXLEN], xpath_srcaddr[XPATH_MAXLEN + 32];
-		char oldpath[XPATH_MAXLEN], newpath[XPATH_MAXLEN];
-		char addr_buf[INET6_ADDRSTRLEN];
-		int slen;
-
-		/* build xpath */
-		if (bs->key.mhop) {
-			inet_ntop(bs->key.family, &bs->key.local, addr_buf, sizeof(addr_buf));
-			snprintf(xpath_srcaddr, sizeof(xpath_srcaddr), "[source-addr='%s']",
-				 addr_buf);
-		} else
-			xpath_srcaddr[0] = 0;
-		inet_ntop(bs->key.family, &bs->key.peer, addr_buf, sizeof(addr_buf));
-		slen = snprintf(xpath, sizeof(xpath),
-				"/frr-bfdd:bfdd/bfd/sessions/%s%s[dest-addr='%s']",
-				bs->key.mhop ? "multi-hop" : "single-hop", xpath_srcaddr,
-				addr_buf);
-		if (bs->key.ifname[0])
-			slen += snprintf(xpath + slen, sizeof(xpath) - slen,
-					 "[interface='%s']", bs->key.ifname);
-		else
-			slen += snprintf(xpath + slen, sizeof(xpath) - slen,
-					 "[interface='']");
-		snprintf(xpath + slen, sizeof(xpath) - slen, "[vrf='%s']/vrf",
-			 bs->key.vrfname);
-
-		bfd_dnode = yang_dnode_get(running_config->dnode, xpath,
-					   bs->key.vrfname);
-		if (bfd_dnode) {
-			yang_dnode_get_path(bfd_dnode->parent, oldpath,
-					    sizeof(oldpath));
-			yang_dnode_change_leaf(bfd_dnode, vrf->name);
-			yang_dnode_get_path(bfd_dnode->parent, newpath,
-					    sizeof(newpath));
-			nb_running_move_tree(oldpath, newpath);
-			running_config->version++;
-		}
-	}
-	memset(bs->key.vrfname, 0, sizeof(bs->key.vrfname));
-	strlcpy(bs->key.vrfname, vrf->name, sizeof(bs->key.vrfname));
-	hash_get(bfd_key_hash, bs, hash_alloc_intern);
-}
-
 unsigned long bfd_get_session_count(void)
 {
 	return bfd_key_hash->count;
 }
+
+void bfd_rtt_init(struct bfd_session *bfd)
+{
+	uint8_t i;
+
+	/* initialize RTT */
+	bfd->rtt_valid = 0;
+	bfd->rtt_index = 0;
+	for (i = 0; i < BFD_RTT_SAMPLE; i++)
+		bfd->rtt[i] = 0;
+}
