@@ -137,7 +137,7 @@ extern struct community_list_handler *bgp_clist;
 
 /* Prototypes.  */
 extern struct community_list_handler *community_list_init(void);
-extern void community_list_terminate(struct community_list_handler *);
+extern void community_list_terminate(struct community_list_handler *ch);
 
 extern int community_list_set(struct community_list_handler *ch,
 			      const char *name, const char *str,
@@ -160,21 +160,24 @@ extern int lcommunity_list_unset(struct community_list_handler *ch,
 				 const char *seq, int direct, int style);
 
 extern struct community_list_master *
-community_list_master_lookup(struct community_list_handler *, int);
+community_list_master_lookup(struct community_list_handler *ch, int master);
 
 extern struct community_list *
 community_list_lookup(struct community_list_handler *c, const char *name,
 		      uint32_t name_hash, int master);
 
-extern bool community_list_match(struct community *, struct community_list *);
-extern bool ecommunity_list_match(struct ecommunity *, struct community_list *);
-extern bool lcommunity_list_match(struct lcommunity *, struct community_list *);
-extern bool community_list_exact_match(struct community *,
-				       struct community_list *);
+extern bool community_list_match(struct community *com,
+				 struct community_list *list);
+extern bool ecommunity_list_match(struct ecommunity *ecom,
+				  struct community_list *list);
+extern bool lcommunity_list_match(struct lcommunity *lcom,
+				  struct community_list *list);
+extern bool community_list_exact_match(struct community *com,
+				       struct community_list *list);
 extern bool lcommunity_list_exact_match(struct lcommunity *lcom,
 					struct community_list *list);
-extern struct community *community_list_match_delete(struct community *,
-						     struct community_list *);
+extern struct community *
+community_list_match_delete(struct community *com, struct community_list *list);
 extern struct lcommunity *
 lcommunity_list_match_delete(struct lcommunity *lcom,
 			     struct community_list *list);
@@ -184,4 +187,6 @@ static inline uint32_t bgp_clist_hash_key(char *name)
 	return jhash(name, strlen(name), 0xdeadbeaf);
 }
 
+extern void bgp_community_list_command_completion_setup(void);
+
 #endif /* _QUAGGA_BGP_CLIST_H */
