@@ -40,9 +40,6 @@ struct as_list_list {
 
 /* AS path filter master. */
 struct as_list_master {
-	/* List of access_list which name is number. */
-	struct as_list_list num;
-
 	/* List of access_list which name is string. */
 	struct as_list_list str;
 
@@ -62,14 +59,15 @@ struct as_filter {
 
 	regex_t *reg;
 	char *reg_str;
+
+	/* Sequence number. */
+	int64_t seq;
 };
 
 /* AS path filter list. */
 struct as_list {
 	char *name;
 
-	enum access_type type;
-
 	struct as_list *next;
 	struct as_list *prev;
 
@@ -77,10 +75,41 @@ struct as_list {
 	struct as_filter *tail;
 };
 
+
+/* Calculate new sequential number. */
+static int64_t bgp_alist_new_seq_get(struct as_list *list)
+{
+	int64_t maxseq;
+	int64_t newseq;
+	struct as_filter *entry;
+
+	maxseq = 0;
+
+	for (entry = list->head; entry; entry = entry->next) {
+		if (maxseq < entry->seq)
+			maxseq = entry->seq;
+	}
+
+	newseq = ((maxseq / 5) * 5) + 5;
+
+	return (newseq > UINT_MAX) ? UINT_MAX : newseq;
+}
+
+/* Return as-list entry which has same seq number. */
+static struct as_filter *bgp_aslist_seq_check(struct as_list *list, int64_t seq)
+{
+	struct as_filter *entry;
+
+	for (entry = list->head; entry; entry = entry->next)
+		if (entry->seq == seq)
+			return entry;
+
+	return NULL;
+}
+
 /* as-path access-list 10 permit AS1. */
 
 static struct as_list_master as_list_master = {{NULL, NULL},
-					       {NULL, NULL},
 					       NULL,
 					       NULL};
 
@@ -125,18 +154,71 @@ static struct as_filter *as_filter_lookup(struct as_list *aslist,
 	return NULL;
 }
 
+static void as_filter_entry_replace(struct as_list *list,
+				    struct as_filter *replace,
+				    struct as_filter *entry)
+{
+	if (replace->next) {
+		entry->next = replace->next;
+		replace->next->prev = entry;
+	} else {
+		entry->next = NULL;
+		list->tail = entry;
+	}
+
+	if (replace->prev) {
+		entry->prev = replace->prev;
+		replace->prev->next = entry;
+	} else {
+		entry->prev = NULL;
+		list->head = entry;
+	}
+
+	as_filter_free(replace);
+}
+
 static void as_list_filter_add(struct as_list *aslist,
 			       struct as_filter *asfilter)
 {
-	asfilter->next = NULL;
-	asfilter->prev = aslist->tail;
+	struct as_filter *point;
+	struct as_filter *replace;
 
-	if (aslist->tail)
-		aslist->tail->next = asfilter;
-	else
-		aslist->head = asfilter;
-	aslist->tail = asfilter;
+	if (aslist->tail && asfilter->seq > aslist->tail->seq)
+		point = NULL;
+	else {
+		replace = bgp_aslist_seq_check(aslist, asfilter->seq);
+		if (replace) {
+			as_filter_entry_replace(aslist, replace, asfilter);
+			goto hook;
+		}
 
+		/* Check insert point. */
+		for (point = aslist->head; point; point = point->next)
+			if (point->seq >= asfilter->seq)
+				break;
+	}
+
+	asfilter->next = point;
+
+	if (point) {
+		if (point->prev)
+			point->prev->next = asfilter;
+		else
+			aslist->head = asfilter;
+
+		asfilter->prev = point->prev;
+		point->prev = asfilter;
+	} else {
+		if (aslist->tail)
+			aslist->tail->next = asfilter;
+		else
+			aslist->head = asfilter;
+
+		asfilter->prev = aslist->tail;
+		aslist->tail = asfilter;
+	}
+
+hook:
 	/* Run hook function. */
 	if (as_list_master.add_hook)
 		(*as_list_master.add_hook)(aslist->name);
@@ -150,10 +232,6 @@ struct as_list *as_list_lookup(const char *name)
 	if (name == NULL)
 		return NULL;
 
-	for (aslist = as_list_master.num.head; aslist; aslist = aslist->next)
-		if (strcmp(aslist->name, name) == 0)
-			return aslist;
-
 	for (aslist = as_list_master.str.head; aslist; aslist = aslist->next)
 		if (strcmp(aslist->name, name) == 0)
 			return aslist;
@@ -176,8 +254,6 @@ static void as_list_free(struct as_list *aslist)
    the name. */
 static struct as_list *as_list_insert(const char *name)
 {
-	size_t i;
-	long number;
 	struct as_list *aslist;
 	struct as_list *point;
 	struct as_list_list *list;
@@ -187,36 +263,13 @@ static struct as_list *as_list_insert(const char *name)
 	aslist->name = XSTRDUP(MTYPE_AS_STR, name);
 	assert(aslist->name);
 
-	/* If name is made by all digit character.  We treat it as
-	   number. */
-	for (number = 0, i = 0; i < strlen(name); i++) {
-		if (isdigit((unsigned char)name[i]))
-			number = (number * 10) + (name[i] - '0');
-		else
+	/* Set access_list to string list. */
+	list = &as_list_master.str;
+
+	/* Set point to insertion point. */
+	for (point = list->head; point; point = point->next)
+		if (strcmp(point->name, name) >= 0)
 			break;
-	}
-
-	/* In case of name is all digit character */
-	if (i == strlen(name)) {
-		aslist->type = ACCESS_TYPE_NUMBER;
-
-		/* Set access_list to number list. */
-		list = &as_list_master.num;
-
-		for (point = list->head; point; point = point->next)
-			if (atol(point->name) >= number)
-				break;
-	} else {
-		aslist->type = ACCESS_TYPE_STRING;
-
-		/* Set access_list to string list. */
-		list = &as_list_master.str;
-
-		/* Set point to insertion point. */
-		for (point = list->head; point; point = point->next)
-			if (strcmp(point->name, name) >= 0)
-				break;
-	}
 
 	/* In case of this is the first element of master. */
 	if (list->head == NULL) {
@@ -284,10 +337,7 @@ static void as_list_delete(struct as_list *aslist)
 		as_filter_free(filter);
 	}
 
-	if (aslist->type == ACCESS_TYPE_NUMBER)
-		list = &as_list_master.num;
-	else
-		list = &as_list_master.str;
+	list = &as_list_master.str;
 
 	if (aslist->next)
 		aslist->next->prev = aslist->prev;
@@ -391,11 +441,13 @@ bool config_bgp_aspath_validate(const char *regstr)
 }
 
 DEFUN(as_path, bgp_as_path_cmd,
-      "bgp as-path access-list WORD <deny|permit> LINE...",
+      "bgp as-path access-list AS_PATH_FILTER_NAME [seq (0-4294967295)] <deny|permit> LINE...",
       BGP_STR
       "BGP autonomous system path filter\n"
       "Specify an access list name\n"
       "Regular expression access list name\n"
+      "Sequence number of an entry\n"
+      "Sequence number\n"
       "Specify packets to reject\n"
       "Specify packets to forward\n"
       "A regular-expression (1234567890_^|[,{}() ]$*+.?-\\) to match the BGP AS paths\n")
@@ -406,11 +458,15 @@ DEFUN(as_path, bgp_as_path_cmd,
 	struct as_list *aslist;
 	regex_t *regex;
 	char *regstr;
+	int64_t seqnum = ASPATH_SEQ_NUMBER_AUTO;
 
 	/* Retrieve access list name */
-	argv_find(argv, argc, "WORD", &idx);
+	argv_find(argv, argc, "AS_PATH_FILTER_NAME", &idx);
 	char *alname = argv[idx]->arg;
 
+	if (argv_find(argv, argc, "(0-4294967295)", &idx))
+		seqnum = (int64_t)atol(argv[idx]->arg);
+
 	/* Check the filter type. */
 	type = argv_find(argv, argc, "deny", &idx) ? AS_FILTER_DENY
 						   : AS_FILTER_PERMIT;
@@ -429,6 +485,7 @@ DEFUN(as_path, bgp_as_path_cmd,
 	if (!config_bgp_aspath_validate(regstr)) {
 		vty_out(vty, "Invalid character in as-path access-list %s\n",
 			regstr);
+		XFREE(MTYPE_TMP, regstr);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
@@ -439,6 +496,11 @@ DEFUN(as_path, bgp_as_path_cmd,
 	/* Install new filter to the access_list. */
 	aslist = as_list_get(alname);
 
+	if (seqnum == ASPATH_SEQ_NUMBER_AUTO)
+		seqnum = bgp_alist_new_seq_get(aslist);
+
+	asfilter->seq = seqnum;
+
 	/* Duplicate insertion check. */;
 	if (as_list_dup_check(aslist, asfilter))
 		as_filter_free(asfilter);
@@ -449,12 +511,14 @@ DEFUN(as_path, bgp_as_path_cmd,
 }
 
 DEFUN(no_as_path, no_bgp_as_path_cmd,
-      "no bgp as-path access-list WORD <deny|permit> LINE...",
+      "no bgp as-path access-list AS_PATH_FILTER_NAME [seq (0-4294967295)] <deny|permit> LINE...",
       NO_STR
       BGP_STR
       "BGP autonomous system path filter\n"
       "Specify an access list name\n"
       "Regular expression access list name\n"
+      "Sequence number of an entry\n"
+      "Sequence number\n"
       "Specify packets to reject\n"
       "Specify packets to forward\n"
       "A regular-expression (1234567890_^|[,{}() ]$*+.?-\\) to match the BGP AS paths\n")
@@ -467,7 +531,7 @@ DEFUN(no_as_path, no_bgp_as_path_cmd,
 	regex_t *regex;
 
 	char *aslistname =
-		argv_find(argv, argc, "WORD", &idx) ? argv[idx]->arg : NULL;
+		argv_find(argv, argc, "AS_PATH_FILTER_NAME", &idx) ? argv[idx]->arg : NULL;
 
 	/* Lookup AS list from AS path list. */
 	aslist = as_list_lookup(aslistname);
@@ -524,7 +588,7 @@ DEFUN(no_as_path, no_bgp_as_path_cmd,
 
 DEFUN (no_as_path_all,
        no_bgp_as_path_all_cmd,
-       "no bgp as-path access-list WORD",
+       "no bgp as-path access-list AS_PATH_FILTER_NAME",
        NO_STR
        BGP_STR
        "BGP autonomous system path filter\n"
@@ -550,89 +614,110 @@ DEFUN (no_as_path_all,
 	return CMD_SUCCESS;
 }
 
-static void as_list_show(struct vty *vty, struct as_list *aslist)
+static void as_list_show(struct vty *vty, struct as_list *aslist,
+			 json_object *json)
 {
 	struct as_filter *asfilter;
+	json_object *json_aslist = NULL;
 
-	vty_out(vty, "AS path access list %s\n", aslist->name);
+	if (json) {
+		json_aslist = json_object_new_array();
+		json_object_object_add(json, aslist->name, json_aslist);
+	} else
+		vty_out(vty, "AS path access list %s\n", aslist->name);
 
 	for (asfilter = aslist->head; asfilter; asfilter = asfilter->next) {
-		vty_out(vty, "    %s %s\n", filter_type_str(asfilter->type),
-			asfilter->reg_str);
+		if (json) {
+			json_object *json_asfilter = json_object_new_object();
+
+			json_object_int_add(json_asfilter, "sequenceNumber",
+					    asfilter->seq);
+			json_object_string_add(json_asfilter, "type",
+					       filter_type_str(asfilter->type));
+			json_object_string_add(json_asfilter, "regExp",
+					       asfilter->reg_str);
+
+			json_object_array_add(json_aslist, json_asfilter);
+		} else
+			vty_out(vty, "    %s %s\n",
+				filter_type_str(asfilter->type),
+				asfilter->reg_str);
 	}
 }
 
-static void as_list_show_all(struct vty *vty)
+static void as_list_show_all(struct vty *vty, json_object *json)
 {
 	struct as_list *aslist;
-	struct as_filter *asfilter;
 
-	for (aslist = as_list_master.num.head; aslist; aslist = aslist->next) {
-		vty_out(vty, "AS path access list %s\n", aslist->name);
-
-		for (asfilter = aslist->head; asfilter;
-		     asfilter = asfilter->next) {
-			vty_out(vty, "    %s %s\n",
-				filter_type_str(asfilter->type),
-				asfilter->reg_str);
-		}
-	}
-
-	for (aslist = as_list_master.str.head; aslist; aslist = aslist->next) {
-		vty_out(vty, "AS path access list %s\n", aslist->name);
-
-		for (asfilter = aslist->head; asfilter;
-		     asfilter = asfilter->next) {
-			vty_out(vty, "    %s %s\n",
-				filter_type_str(asfilter->type),
-				asfilter->reg_str);
-		}
-	}
+	for (aslist = as_list_master.str.head; aslist; aslist = aslist->next)
+		as_list_show(vty, aslist, json);
 }
 
 DEFUN (show_as_path_access_list,
        show_bgp_as_path_access_list_cmd,
-       "show bgp as-path-access-list WORD",
+       "show bgp as-path-access-list AS_PATH_FILTER_NAME [json]",
        SHOW_STR
        BGP_STR
        "List AS path access lists\n"
-       "AS path access list name\n")
+       "AS path access list name\n"
+       JSON_STR)
 {
 	int idx_word = 3;
 	struct as_list *aslist;
+	bool uj = use_json(argc, argv);
+	json_object *json = NULL;
+
+	if (uj)
+		json = json_object_new_object();
 
 	aslist = as_list_lookup(argv[idx_word]->arg);
 	if (aslist)
-		as_list_show(vty, aslist);
+		as_list_show(vty, aslist, json);
+
+	if (uj)
+		vty_json(vty, json);
 
 	return CMD_SUCCESS;
 }
 
 ALIAS (show_as_path_access_list,
        show_ip_as_path_access_list_cmd,
-       "show ip as-path-access-list WORD",
+       "show ip as-path-access-list AS_PATH_FILTER_NAME [json]",
        SHOW_STR
        IP_STR
        "List AS path access lists\n"
-       "AS path access list name\n")
+       "AS path access list name\n"
+       JSON_STR)
 
 DEFUN (show_as_path_access_list_all,
        show_bgp_as_path_access_list_all_cmd,
-       "show bgp as-path-access-list",
+       "show bgp as-path-access-list [json]",
        SHOW_STR
        BGP_STR
-       "List AS path access lists\n")
+       "List AS path access lists\n"
+       JSON_STR)
 {
-	as_list_show_all(vty);
+	bool uj = use_json(argc, argv);
+	json_object *json = NULL;
+
+	if (uj)
+		json = json_object_new_object();
+
+	as_list_show_all(vty, json);
+
+	if (uj)
+		vty_json(vty, json);
+
 	return CMD_SUCCESS;
 }
 
 ALIAS (show_as_path_access_list_all,
        show_ip_as_path_access_list_all_cmd,
-       "show ip as-path-access-list",
+       "show ip as-path-access-list [json]",
        SHOW_STR
        IP_STR
-       "List AS path access lists\n")
+       "List AS path access lists\n"
+       JSON_STR)
 
 static int config_write_as_list(struct vty *vty)
 {
@@ -640,20 +725,14 @@ static int config_write_as_list(struct vty *vty)
 	struct as_filter *asfilter;
 	int write = 0;
 
-	for (aslist = as_list_master.num.head; aslist; aslist = aslist->next)
-		for (asfilter = aslist->head; asfilter;
-		     asfilter = asfilter->next) {
-			vty_out(vty, "bgp as-path access-list %s %s %s\n",
-				aslist->name, filter_type_str(asfilter->type),
-				asfilter->reg_str);
-			write++;
-		}
-
 	for (aslist = as_list_master.str.head; aslist; aslist = aslist->next)
 		for (asfilter = aslist->head; asfilter;
 		     asfilter = asfilter->next) {
-			vty_out(vty, "bgp as-path access-list %s %s %s\n",
-				aslist->name, filter_type_str(asfilter->type),
+			vty_out(vty,
+				"bgp as-path access-list %s seq %" PRId64
+				" %s %s\n",
+				aslist->name, asfilter->seq,
+				filter_type_str(asfilter->type),
 				asfilter->reg_str);
 			write++;
 		}
@@ -668,6 +747,20 @@ static struct cmd_node as_list_node = {
 	.config_write = config_write_as_list,
 };
 
+static void bgp_aspath_filter_cmd_completion(vector comps,
+					     struct cmd_token *token)
+{
+	struct as_list *aslist;
+
+	for (aslist = as_list_master.str.head; aslist; aslist = aslist->next)
+		vector_set(comps, XSTRDUP(MTYPE_COMPLETION, aslist->name));
+}
+
+static const struct cmd_variable_handler aspath_filter_handlers[] = {
+	{.tokenname = "AS_PATH_FILTER_NAME",
+	 .completions = bgp_aspath_filter_cmd_completion},
+	{.completions = NULL}};
+
 /* Register functions. */
 void bgp_filter_init(void)
 {
@@ -681,6 +774,8 @@ void bgp_filter_init(void)
 	install_element(VIEW_NODE, &show_ip_as_path_access_list_cmd);
 	install_element(VIEW_NODE, &show_bgp_as_path_access_list_all_cmd);
 	install_element(VIEW_NODE, &show_ip_as_path_access_list_all_cmd);
+
+	cmd_variable_handler_register(aspath_filter_handlers);
 }
 
 void bgp_filter_reset(void)
@@ -688,19 +783,11 @@ void bgp_filter_reset(void)
 	struct as_list *aslist;
 	struct as_list *next;
 
-	for (aslist = as_list_master.num.head; aslist; aslist = next) {
-		next = aslist->next;
-		as_list_delete(aslist);
-	}
-
 	for (aslist = as_list_master.str.head; aslist; aslist = next) {
 		next = aslist->next;
 		as_list_delete(aslist);
 	}
 
-	assert(as_list_master.num.head == NULL);
-	assert(as_list_master.num.tail == NULL);
-
 	assert(as_list_master.str.head == NULL);
 	assert(as_list_master.str.tail == NULL);
 }
