@@ -268,6 +268,7 @@ void route_vty_out_flowspec(struct vty *vty, const struct prefix *p,
 	json_object *json_ecom_path = NULL;
 	json_object *json_time_path = NULL;
 	char timebuf[BGP_UPTIME_LEN];
+	struct ecommunity *ipv6_ecomm = NULL;
 
 	if (p == NULL || p->family != AF_FLOWSPEC)
 		return;
@@ -298,16 +299,20 @@ void route_vty_out_flowspec(struct vty *vty, const struct prefix *p,
 		json_object_array_add(json_paths, json_nlri_path);
 	if (!path)
 		return;
-	if (path->attr &&
-	    (path->attr->ecommunity || path->attr->ipv6_ecommunity)) {
+
+	if (path->attr)
+		ipv6_ecomm = bgp_attr_get_ipv6_ecommunity(path->attr);
+
+	if (path->attr && (bgp_attr_get_ecommunity(path->attr) || ipv6_ecomm)) {
 		/* Print attribute */
 		attr = path->attr;
-		if (attr->ecommunity)
-			s1 = ecommunity_ecom2str(attr->ecommunity,
-						ECOMMUNITY_FORMAT_ROUTE_MAP, 0);
-		if (attr->ipv6_ecommunity)
-			s2 = ecommunity_ecom2str(attr->ipv6_ecommunity,
-						ECOMMUNITY_FORMAT_ROUTE_MAP, 0);
+		if (bgp_attr_get_ecommunity(attr))
+			s1 = ecommunity_ecom2str(bgp_attr_get_ecommunity(attr),
+						 ECOMMUNITY_FORMAT_ROUTE_MAP,
+						 0);
+		if (ipv6_ecomm)
+			s2 = ecommunity_ecom2str(
+				ipv6_ecomm, ECOMMUNITY_FORMAT_ROUTE_MAP, 0);
 		if (!s1 && !s2)
 			return;
 		if (display == NLRI_STRING_FORMAT_LARGE)
@@ -331,8 +336,8 @@ void route_vty_out_flowspec(struct vty *vty, const struct prefix *p,
 			char local_buff[INET6_ADDRSTRLEN];
 
 			local_buff[0] = '\0';
-			if (p->u.prefix_flowspec.family == AF_INET &&
-			    attr->nexthop.s_addr != 0)
+			if (p->u.prefix_flowspec.family == AF_INET
+			    && attr->nexthop.s_addr != INADDR_ANY)
 				inet_ntop(AF_INET,
 					  &attr->nexthop.s_addr,
 					  local_buff,
@@ -445,11 +450,7 @@ int bgp_show_table_flowspec(struct vty *vty, struct bgp *bgp, afi_t afi,
 					       pi, display, json_paths);
 		}
 		if (use_json) {
-			vty_out(vty, "%s\n",
-				json_object_to_json_string_ext(
-						json_paths,
-						JSON_C_TO_STRING_PRETTY));
-			json_object_free(json_paths);
+			vty_json(vty, json_paths);
 			json_paths = NULL;
 		}
 	}
