@@ -29,12 +29,6 @@
 			thread_add_timer(bm->master, (F), peer, (V), &(T));    \
 	} while (0)
 
-#define BGP_TIMER_OFF(T)                                                       \
-	do {                                                                   \
-		if (T)                                                         \
-			THREAD_TIMER_OFF(T);                                   \
-	} while (0)
-
 #define BGP_EVENT_ADD(P, E)                                                    \
 	do {                                                                   \
 		if ((P)->status != Deleted)                                    \
@@ -45,9 +39,21 @@
 #define BGP_EVENT_FLUSH(P)                                                     \
 	do {                                                                   \
 		assert(peer);                                                  \
-		thread_cancel_event(bm->master, (P));                          \
+		thread_cancel_event_ready(bm->master, (P));                    \
 	} while (0)
 
+#define BGP_UPDATE_GROUP_TIMER_ON(T, F)					       \
+	do {								       \
+		if (BGP_SUPPRESS_FIB_ENABLED(peer->bgp) &&		       \
+		    PEER_ROUTE_ADV_DELAY(peer))				       \
+			thread_add_timer_msec(bm->master, (F), peer,	       \
+				(BGP_DEFAULT_UPDATE_ADVERTISEMENT_TIME * 1000),\
+				(T));					       \
+		else							       \
+			thread_add_timer_msec(bm->master, (F), peer,	       \
+					      0, (T));			       \
+	} while (0)							       \
+
 #define BGP_MSEC_JITTER 10
 
 /* Status codes for bgp_event_update() */
@@ -99,22 +105,26 @@
 	 && CHECK_FLAG(peer->cap, PEER_CAP_RESTART_RCV))
 
 #define BGP_PEER_RESTARTING_MODE(peer)                                         \
-	(CHECK_FLAG(peer->flags, PEER_FLAG_GRACEFUL_RESTART)                   \
-	 && CHECK_FLAG(peer->cap, PEER_CAP_RESTART_BIT_ADV)                    \
-	 && !CHECK_FLAG(peer->cap, PEER_CAP_RESTART_BIT_RCV))
+	(CHECK_FLAG(peer->flags, PEER_FLAG_GRACEFUL_RESTART) &&                \
+	 CHECK_FLAG(peer->cap, PEER_CAP_GRACEFUL_RESTART_R_BIT_ADV) &&         \
+	 !CHECK_FLAG(peer->cap, PEER_CAP_GRACEFUL_RESTART_R_BIT_RCV))
 
 #define BGP_PEER_HELPER_MODE(peer)                                             \
-	(CHECK_FLAG(peer->flags, PEER_FLAG_GRACEFUL_RESTART_HELPER)            \
-	 && CHECK_FLAG(peer->cap, PEER_CAP_RESTART_BIT_RCV)                    \
-	 && !CHECK_FLAG(peer->cap, PEER_CAP_RESTART_BIT_ADV))
+	(CHECK_FLAG(peer->flags, PEER_FLAG_GRACEFUL_RESTART_HELPER) &&         \
+	 CHECK_FLAG(peer->cap, PEER_CAP_GRACEFUL_RESTART_R_BIT_RCV) &&         \
+	 !CHECK_FLAG(peer->cap, PEER_CAP_GRACEFUL_RESTART_R_BIT_ADV))
 
 /* Prototypes. */
-extern void bgp_fsm_event_update(struct peer *peer, int valid);
-extern int bgp_event(struct thread *);
+
+/*
+ * Update FSM for peer based on whether we have valid nexthops or not.
+ */
+extern void bgp_fsm_nht_update(struct peer *peer, bool has_valid_nexthops);
+extern void bgp_event(struct thread *);
 extern int bgp_event_update(struct peer *, enum bgp_fsm_events event);
 extern int bgp_stop(struct peer *peer);
 extern void bgp_timer_set(struct peer *);
-extern int bgp_routeadv_timer(struct thread *);
+extern void bgp_routeadv_timer(struct thread *);
 extern void bgp_fsm_change_status(struct peer *peer, int status);
 extern const char *const peer_down_str[];
 extern void bgp_update_delay_end(struct bgp *);
@@ -140,8 +150,8 @@ extern void bgp_start_routeadv(struct bgp *);
 extern void bgp_adjust_routeadv(struct peer *);
 
 #include "hook.h"
-DECLARE_HOOK(peer_backward_transition, (struct peer *peer), (peer))
-DECLARE_HOOK(peer_established, (struct peer *peer), (peer))
+DECLARE_HOOK(peer_backward_transition, (struct peer *peer), (peer));
+DECLARE_HOOK(peer_established, (struct peer *peer), (peer));
 
 int bgp_gr_update_all(struct bgp *bgp, int global_gr_cmd);
 int bgp_neighbor_graceful_restart(struct peer *peer, int peer_gr_cmd);
@@ -164,4 +174,5 @@ const char *print_peer_gr_mode(enum peer_mode pr_mode);
 const char *print_peer_gr_cmd(enum peer_gr_command pr_gr_cmd);
 const char *print_global_gr_mode(enum global_mode gl_mode);
 const char *print_global_gr_cmd(enum global_gr_command gl_gr_cmd);
+int bgp_peer_reg_with_nht(struct peer *peer);
 #endif /* _QUAGGA_BGP_FSM_H */
