@@ -55,7 +55,7 @@ int bgp_parse_fec_update(void)
 
 	s = zclient->ibuf;
 
-	memset(&p, 0, sizeof(struct prefix));
+	memset(&p, 0, sizeof(p));
 	p.family = stream_getw(s);
 	p.prefixlen = stream_getc(s);
 	stream_get(p.u.val, s, PSIZE(p.prefixlen));
@@ -85,12 +85,12 @@ int bgp_parse_fec_update(void)
 	if (label == MPLS_INVALID_LABEL)
 		bgp_unset_valid_label(&dest->local_label);
 	else {
-		label_ntop(label, 1, &dest->local_label);
+		dest->local_label = mpls_lse_encode(label, 0, 0, 1);
 		bgp_set_valid_label(&dest->local_label);
 	}
 	SET_FLAG(dest->flags, BGP_NODE_LABEL_CHANGED);
-	bgp_dest_unlock_node(dest);
 	bgp_process(bgp, dest, afi, safi);
+	bgp_dest_unlock_node(dest);
 	return 1;
 }
 
@@ -120,149 +120,36 @@ mpls_label_t bgp_adv_label(struct bgp_dest *dest, struct bgp_path_info *pi,
 	return dest->local_label;
 }
 
-/**
- * This is passed as the callback function to bgp_labelpool.c:bgp_lp_get()
- * by bgp_reg_dereg_for_label() when a label needs to be obtained from
- * label pool.
- * Note that it will reject the allocated label if a label index is found,
- * because the label index supposes predictable labels
- */
-int bgp_reg_for_label_callback(mpls_label_t new_label, void *labelid,
-			       bool allocated)
+static void bgp_send_fec_register_label_msg(struct bgp_dest *dest, bool reg,
+					    uint32_t label_index)
 {
-	struct bgp_path_info *pi;
-	struct bgp_dest *dest;
-
-	pi = labelid;
-	/* Is this path still valid? */
-	if (!bgp_path_info_unlock(pi)) {
-		if (BGP_DEBUG(labelpool, LABELPOOL))
-			zlog_debug(
-				"%s: bgp_path_info is no longer valid, ignoring",
-				__func__);
-		return -1;
-	}
-
-	dest = pi->net;
-
-	if (BGP_DEBUG(labelpool, LABELPOOL))
-		zlog_debug("%s: FEC %pRN label=%u, allocated=%d", __func__,
-			   bgp_dest_to_rnode(dest), new_label, allocated);
-
-	if (!allocated) {
-		/*
-		 * previously-allocated label is now invalid
-		 */
-		if (pi->attr->label_index == MPLS_INVALID_LABEL_INDEX
-		    && pi->attr->label != MPLS_LABEL_NONE
-		    && CHECK_FLAG(dest->flags, BGP_NODE_REGISTERED_FOR_LABEL)) {
-			bgp_unregister_for_label(dest);
-			label_ntop(MPLS_LABEL_IMPLICIT_NULL, 1,
-				   &dest->local_label);
-			bgp_set_valid_label(&dest->local_label);
-		}
-		return 0;
-	}
-
-	/*
-	 * label index is assigned, this should be handled by SR-related code,
-	 * so retry FEC registration and then reject label allocation for
-	 * it to be released to label pool
-	 */
-	if (pi->attr->label_index != MPLS_INVALID_LABEL_INDEX) {
-		flog_err(
-			EC_BGP_LABEL,
-			"%s: FEC %pRN Rejecting allocated label %u as Label Index is %u",
-			__func__, bgp_dest_to_rnode(dest), new_label,
-			pi->attr->label_index);
-
-		bgp_register_for_label(pi->net, pi);
-
-		return -1;
-	}
-
-	if (pi->attr->label != MPLS_INVALID_LABEL) {
-		if (new_label == pi->attr->label) {
-			/* already have same label, accept but do nothing */
-			return 0;
-		}
-		/* Shouldn't happen: different label allocation */
-		flog_err(EC_BGP_LABEL,
-			 "%s: %pRN had label %u but got new assignment %u",
-			 __func__, bgp_dest_to_rnode(dest), pi->attr->label,
-			 new_label);
-		/* continue means use new one */
-	}
-
-	label_ntop(new_label, 1, &dest->local_label);
-	bgp_set_valid_label(&dest->local_label);
-
-	/*
-	 * Get back to registering the FEC
-	 */
-	bgp_register_for_label(pi->net, pi);
-
-	return 0;
-}
-
-void bgp_reg_dereg_for_label(struct bgp_dest *dest, struct bgp_path_info *pi,
-			     bool reg)
-{
-	bool with_label_index = false;
 	struct stream *s;
-	const struct prefix *p;
-	mpls_label_t *local_label;
 	int command;
+	const struct prefix *p;
 	uint16_t flags = 0;
 	size_t flags_pos = 0;
-	char addr[PREFIX_STRLEN];
+	mpls_label_t *local_label = &(dest->local_label);
+	uint32_t ttl = 0;
+	uint32_t bos = 0;
+	uint32_t exp = 0;
+	mpls_label_t label = MPLS_INVALID_LABEL;
+	bool have_label_to_reg;
+
+	mpls_lse_decode(*local_label, &label, &ttl, &exp, &bos);
+
+	have_label_to_reg = bgp_is_valid_label(local_label) &&
+			    label != MPLS_LABEL_IMPLICIT_NULL;
 
 	p = bgp_dest_get_prefix(dest);
-	local_label = &(dest->local_label);
-	/* this prevents the loop when we're called by
-	 * bgp_reg_for_label_callback()
-	 */
-	bool have_label_to_reg = bgp_is_valid_label(local_label)
-			&& label_pton(local_label) != MPLS_LABEL_IMPLICIT_NULL;
-
-	if (reg) {
-		assert(pi);
-		/*
-		 * Determine if we will let zebra should derive label from
-		 * label index instead of bgpd requesting from label pool
-		 */
-		if (CHECK_FLAG(pi->attr->flag,
-			    ATTR_FLAG_BIT(BGP_ATTR_PREFIX_SID))
-			&& pi->attr->label_index != BGP_INVALID_LABEL_INDEX) {
-			with_label_index = true;
-		} else {
-			/*
-			 * If no label index was provided -- assume any label
-			 * from label pool will do. This means that label index
-			 * always takes precedence over auto-assigned labels.
-			 */
-			if (!have_label_to_reg) {
-				if (BGP_DEBUG(labelpool, LABELPOOL)) {
-					prefix2str(p, addr, PREFIX_STRLEN);
-					zlog_debug("%s: Requesting label from LP for %s",
-						 __func__, addr);
-				}
-				/* bgp_reg_for_label_callback() will call back
-				 * __func__ when it gets a label from the pool.
-				 * This means we'll never register FECs without
-				 * valid labels.
-				 */
-				bgp_lp_get(LP_TYPE_BGP_LU, pi,
-				    bgp_reg_for_label_callback);
-				return;
-			}
-		}
-	}
 
 	/* Check socket. */
 	if (!zclient || zclient->sock < 0)
 		return;
 
+	if (BGP_DEBUG(labelpool, LABELPOOL))
+		zlog_debug("%s: FEC %sregister %pRN label_index=%u label=%u",
+			   __func__, reg ? "" : "un", bgp_dest_to_rnode(dest),
+			   label_index, label);
 	/* If the route node has a local_label assigned or the
 	 * path node has an MPLS SR label index allowing zebra to
 	 * derive the label, proceed with registration. */
@@ -275,12 +162,13 @@ void bgp_reg_dereg_for_label(struct bgp_dest *dest, struct bgp_path_info *pi,
 	stream_putw(s, PREFIX_FAMILY(p));
 	stream_put_prefix(s, p);
 	if (reg) {
-		if (have_label_to_reg) {
-			flags |= ZEBRA_FEC_REGISTER_LABEL;
-			stream_putl(s, label_pton(local_label));
-		} else if (with_label_index) {
+		/* label index takes precedence over auto-assigned label. */
+		if (label_index != 0) {
 			flags |= ZEBRA_FEC_REGISTER_LABEL_INDEX;
-			stream_putl(s, pi->attr->label_index);
+			stream_putl(s, label_index);
+		} else if (have_label_to_reg) {
+			flags |= ZEBRA_FEC_REGISTER_LABEL;
+			stream_putl(s, label);
 		}
 		SET_FLAG(dest->flags, BGP_NODE_REGISTERED_FOR_LABEL);
 	} else
@@ -298,6 +186,124 @@ void bgp_reg_dereg_for_label(struct bgp_dest *dest, struct bgp_path_info *pi,
 	zclient_send_message(zclient);
 }
 
+/**
+ * This is passed as the callback function to bgp_labelpool.c:bgp_lp_get()
+ * by bgp_reg_dereg_for_label() when a label needs to be obtained from
+ * label pool.
+ * Note that it will reject the allocated label if a label index is found,
+ * because the label index supposes predictable labels
+ */
+int bgp_reg_for_label_callback(mpls_label_t new_label, void *labelid,
+			       bool allocated)
+{
+	struct bgp_dest *dest;
+
+	dest = labelid;
+
+	/*
+	 * if the route had been removed or the request has gone then reject
+	 * the allocated label. The requesting code will have done what is
+	 * required to allocate the correct label
+	 */
+	if (!CHECK_FLAG(dest->flags, BGP_NODE_LABEL_REQUESTED)) {
+		bgp_dest_unlock_node(dest);
+		return -1;
+	}
+
+	bgp_dest_unlock_node(dest);
+
+	if (BGP_DEBUG(labelpool, LABELPOOL))
+		zlog_debug("%s: FEC %pRN label=%u, allocated=%d", __func__,
+			   bgp_dest_to_rnode(dest), new_label, allocated);
+
+	if (!allocated) {
+		/*
+		 * previously-allocated label is now invalid, set to implicit
+		 * null until new label arrives
+		 */
+		if (CHECK_FLAG(dest->flags, BGP_NODE_REGISTERED_FOR_LABEL)) {
+			UNSET_FLAG(dest->flags, BGP_NODE_LABEL_REQUESTED);
+			dest->local_label = mpls_lse_encode(
+				MPLS_LABEL_IMPLICIT_NULL, 0, 0, 1);
+			bgp_set_valid_label(&dest->local_label);
+		}
+	}
+
+	dest->local_label = mpls_lse_encode(new_label, 0, 0, 1);
+	bgp_set_valid_label(&dest->local_label);
+
+	/*
+	 * Get back to registering the FEC
+	 */
+	bgp_send_fec_register_label_msg(dest, true, 0);
+
+	return 0;
+}
+
+void bgp_reg_dereg_for_label(struct bgp_dest *dest, struct bgp_path_info *pi,
+			     bool reg)
+{
+	bool with_label_index = false;
+	const struct prefix *p;
+	bool have_label_to_reg;
+	uint32_t ttl = 0;
+	uint32_t bos = 0;
+	uint32_t exp = 0;
+	mpls_label_t label = MPLS_INVALID_LABEL;
+
+	mpls_lse_decode(dest->local_label, &label, &ttl, &exp, &bos);
+
+	have_label_to_reg = bgp_is_valid_label(&dest->local_label) &&
+			    label != MPLS_LABEL_IMPLICIT_NULL;
+
+	p = bgp_dest_get_prefix(dest);
+
+	if (BGP_DEBUG(labelpool, LABELPOOL))
+		zlog_debug("%s: %pFX: %s ", __func__, p,
+			   (reg ? "reg" : "dereg"));
+
+	if (reg) {
+		assert(pi);
+		/*
+		 * Determine if we will let zebra should derive label from
+		 * label index instead of bgpd requesting from label pool
+		 */
+		if (CHECK_FLAG(pi->attr->flag,
+			    ATTR_FLAG_BIT(BGP_ATTR_PREFIX_SID))
+			&& pi->attr->label_index != BGP_INVALID_LABEL_INDEX) {
+			with_label_index = true;
+			UNSET_FLAG(dest->flags, BGP_NODE_LABEL_REQUESTED);
+		} else {
+			/*
+			 * If no label has been registered -- assume any label
+			 * from label pool will do. This means that label index
+			 * always takes precedence over auto-assigned labels.
+			 */
+			if (!have_label_to_reg) {
+				SET_FLAG(dest->flags, BGP_NODE_LABEL_REQUESTED);
+				if (BGP_DEBUG(labelpool, LABELPOOL))
+					zlog_debug(
+						"%s: Requesting label from LP for %pFX",
+						__func__, p);
+				/* bgp_reg_for_label_callback() will deal with
+				 * fec registration when it gets a label from
+				 * the pool. This means we'll never register
+				 * FECs withoutvalid labels.
+				 */
+				bgp_lp_get(LP_TYPE_BGP_LU, dest,
+					   bgp_reg_for_label_callback);
+				return;
+			}
+		}
+	} else {
+		UNSET_FLAG(dest->flags, BGP_NODE_LABEL_REQUESTED);
+		bgp_lp_release(LP_TYPE_BGP_LU, dest, label);
+	}
+
+	bgp_send_fec_register_label_msg(
+		dest, reg, with_label_index ? pi->attr->label_index : 0);
+}
+
 static int bgp_nlri_get_labels(struct peer *peer, uint8_t *pnt, uint8_t plen,
 			       mpls_label_t *label)
 {
@@ -320,14 +326,14 @@ static int bgp_nlri_get_labels(struct peer *peer, uint8_t *pnt, uint8_t plen,
 	/* If we RX multiple labels we will end up keeping only the last
 	 * one. We do not yet support a label stack greater than 1. */
 	if (label_depth > 1)
-		zlog_info("%s rcvd UPDATE with label stack %d deep", peer->host,
+		zlog_info("%pBP rcvd UPDATE with label stack %d deep", peer,
 			  label_depth);
 
 	if (!(bgp_is_withdraw_label(label) || label_bos(label)))
 		flog_warn(
 			EC_BGP_INVALID_LABEL_STACK,
-			"%s rcvd UPDATE with invalid label stack - no bottom of stack",
-			peer->host);
+			"%pBP rcvd UPDATE with invalid label stack - no bottom of stack",
+			peer);
 
 	return llen;
 }
@@ -342,7 +348,7 @@ int bgp_nlri_parse_label(struct peer *peer, struct attr *attr,
 	int prefixlen;
 	afi_t afi;
 	safi_t safi;
-	int addpath_encoded;
+	bool addpath_capable;
 	uint32_t addpath_id;
 	mpls_label_t label = MPLS_INVALID_LABEL;
 	uint8_t llen;
@@ -353,16 +359,13 @@ int bgp_nlri_parse_label(struct peer *peer, struct attr *attr,
 	safi = packet->safi;
 	addpath_id = 0;
 
-	addpath_encoded =
-		(CHECK_FLAG(peer->af_cap[afi][safi], PEER_CAP_ADDPATH_AF_RX_ADV)
-		 && CHECK_FLAG(peer->af_cap[afi][safi],
-			       PEER_CAP_ADDPATH_AF_TX_RCV));
+	addpath_capable = bgp_addpath_encode_rx(peer, afi, safi);
 
 	for (; pnt < lim; pnt += psize) {
 		/* Clear prefix structure. */
-		memset(&p, 0, sizeof(struct prefix));
+		memset(&p, 0, sizeof(p));
 
-		if (addpath_encoded) {
+		if (addpath_capable) {
 
 			/* When packet overflow occurs return immediately. */
 			if (pnt + BGP_ADDPATH_ID_LEN > lim)
@@ -401,8 +404,8 @@ int bgp_nlri_parse_label(struct peer *peer, struct attr *attr,
 			return BGP_NLRI_PARSE_ERROR_LABEL_LENGTH;
 		}
 
-		if ((afi == AFI_IP && p.prefixlen > 32)
-		    || (afi == AFI_IP6 && p.prefixlen > 128))
+		if ((afi == AFI_IP && p.prefixlen > IPV4_MAX_BITLEN)
+		    || (afi == AFI_IP6 && p.prefixlen > IPV6_MAX_BITLEN))
 			return BGP_NLRI_PARSE_ERROR_PREFIX_LENGTH;
 
 		/* Fetch prefix from NLRI packet */
@@ -422,8 +425,8 @@ int bgp_nlri_parse_label(struct peer *peer, struct attr *attr,
 				  */
 				flog_err(
 					EC_BGP_UPDATE_RCV,
-					"%s: IPv4 labeled-unicast NLRI is multicast address %s, ignoring",
-					peer->host, inet_ntoa(p.u.prefix4));
+					"%s: IPv4 labeled-unicast NLRI is multicast address %pI4, ignoring",
+					peer->host, &p.u.prefix4);
 				continue;
 			}
 		}
@@ -431,27 +434,19 @@ int bgp_nlri_parse_label(struct peer *peer, struct attr *attr,
 		/* Check address. */
 		if (afi == AFI_IP6 && safi == SAFI_LABELED_UNICAST) {
 			if (IN6_IS_ADDR_LINKLOCAL(&p.u.prefix6)) {
-				char buf[BUFSIZ];
-
 				flog_err(
 					EC_BGP_UPDATE_RCV,
-					"%s: IPv6 labeled-unicast NLRI is link-local address %s, ignoring",
-					peer->host,
-					inet_ntop(AF_INET6, &p.u.prefix6, buf,
-						  BUFSIZ));
+					"%s: IPv6 labeled-unicast NLRI is link-local address %pI6, ignoring",
+					peer->host, &p.u.prefix6);
 
 				continue;
 			}
 
 			if (IN6_IS_ADDR_MULTICAST(&p.u.prefix6)) {
-				char buf[BUFSIZ];
-
 				flog_err(
 					EC_BGP_UPDATE_RCV,
-					"%s: IPv6 unicast NLRI is multicast address %s, ignoring",
-					peer->host,
-					inet_ntop(AF_INET6, &p.u.prefix6, buf,
-						  BUFSIZ));
+					"%s: IPv6 unicast NLRI is multicast address %pI6, ignoring",
+					peer->host, &p.u.prefix6);
 
 				continue;
 			}
@@ -459,8 +454,8 @@ int bgp_nlri_parse_label(struct peer *peer, struct attr *attr,
 
 		if (attr) {
 			bgp_update(peer, &p, addpath_id, attr, packet->afi,
-				   SAFI_UNICAST, ZEBRA_ROUTE_BGP,
-				   BGP_ROUTE_NORMAL, NULL, &label, 1, 0, NULL);
+				   safi, ZEBRA_ROUTE_BGP, BGP_ROUTE_NORMAL,
+				   NULL, &label, 1, 0, NULL);
 		} else {
 			bgp_withdraw(peer, &p, addpath_id, attr, packet->afi,
 				     SAFI_UNICAST, ZEBRA_ROUTE_BGP,
