@@ -31,7 +31,7 @@
 #define LP_TYPE_VRF	0x00000001
 #define LP_TYPE_BGP_LU	0x00000002
 
-PREDECL_LIST(lp_fifo)
+PREDECL_LIST(lp_fifo);
 
 struct labelpool {
 	struct skiplist		*ledger;	/* all requests */
@@ -40,6 +40,8 @@ struct labelpool {
 	struct lp_fifo_head	requests;	/* blocked on zebra */
 	struct work_queue	*callback_q;
 	uint32_t		pending_count;	/* requested from zebra */
+	uint32_t reconnect_count;		/* zebra reconnections */
+	uint32_t next_chunksize;		/* request this many labels */
 };
 
 extern void bgp_lp_init(struct thread_master *master, struct labelpool *pool);
@@ -50,5 +52,6 @@ extern void bgp_lp_release(int type, void *labelid, mpls_label_t label);
 extern void bgp_lp_event_chunk(uint8_t keep, uint32_t first, uint32_t last);
 extern void bgp_lp_event_zebra_down(void);
 extern void bgp_lp_event_zebra_up(void);
+extern void bgp_lp_vty_init(void);
 
 #endif /* _FRR_BGP_LABELPOOL_H */
