@@ -42,6 +42,13 @@ struct bgp_table {
 
 	int lock;
 
+	/* soft_reconfig_table in progress */
+	bool soft_reconfig_init;
+	struct thread *soft_reconfig_thread;
+
+	/* list of peers on which soft_reconfig_table has to run */
+	struct list *soft_reconfig_peers;
+
 	struct route_table *route_table;
 	uint64_t version;
 };
@@ -96,14 +103,17 @@ struct bgp_node {
 
 	mpls_label_t local_label;
 
-	uint8_t flags;
+	uint16_t flags;
 #define BGP_NODE_PROCESS_SCHEDULED	(1 << 0)
 #define BGP_NODE_USER_CLEAR             (1 << 1)
 #define BGP_NODE_LABEL_CHANGED          (1 << 2)
 #define BGP_NODE_REGISTERED_FOR_LABEL   (1 << 3)
 #define BGP_NODE_SELECT_DEFER           (1 << 4)
-	/* list node pointer */
-	struct listnode *rt_node;
+#define BGP_NODE_FIB_INSTALL_PENDING    (1 << 5)
+#define BGP_NODE_FIB_INSTALLED          (1 << 6)
+#define BGP_NODE_LABEL_REQUESTED        (1 << 7)
+#define BGP_NODE_SOFT_RECONFIG (1 << 8)
+
 	struct bgp_addpath_node_data tx_addpath;
 
 	enum bgp_path_selection_reason reason;
@@ -124,6 +134,9 @@ extern struct bgp_table *bgp_table_init(struct bgp *bgp, afi_t, safi_t);
 extern void bgp_table_lock(struct bgp_table *);
 extern void bgp_table_unlock(struct bgp_table *);
 extern void bgp_table_finish(struct bgp_table **);
+extern void bgp_dest_unlock_node(struct bgp_dest *dest);
+extern struct bgp_dest *bgp_dest_lock_node(struct bgp_dest *dest);
+extern const char *bgp_dest_get_prefix_str(struct bgp_dest *dest);
 
 
 /*
@@ -168,15 +181,6 @@ static inline struct bgp_dest *bgp_dest_parent_nolock(struct bgp_dest *dest)
 	return bgp_dest_from_rnode(rn);
 }
 
-/*
- * bgp_dest_unlock_node
- */
-static inline void bgp_dest_unlock_node(struct bgp_dest *dest)
-{
-	bgp_delete_listnode(dest);
-	route_unlock_node(bgp_dest_to_rnode(dest));
-}
-
 /*
  * bgp_table_top_nolock
  *
@@ -241,16 +245,6 @@ bgp_node_lookup(const struct bgp_table *const table, const struct prefix *p)
 	return bgp_dest_from_rnode(rn);
 }
 
-/*
- * bgp_dest_lock_node
- */
-static inline struct bgp_dest *bgp_dest_lock_node(struct bgp_dest *dest)
-{
-	struct route_node *rn = route_lock_node(bgp_dest_to_rnode(dest));
-
-	return bgp_dest_from_rnode(rn);
-}
-
 /*
  * bgp_node_match
  */
@@ -262,28 +256,6 @@ static inline struct bgp_dest *bgp_node_match(const struct bgp_table *table,
 	return bgp_dest_from_rnode(rn);
 }
 
-/*
- * bgp_node_match_ipv4
- */
-static inline struct bgp_dest *
-bgp_node_match_ipv4(const struct bgp_table *table, struct in_addr *addr)
-{
-	struct route_node *rn = route_node_match_ipv4(table->route_table, addr);
-
-	return bgp_dest_from_rnode(rn);
-}
-
-/*
- * bgp_node_match_ipv6
- */
-static inline struct bgp_dest *
-bgp_node_match_ipv6(const struct bgp_table *table, struct in6_addr *addr)
-{
-	struct route_node *rn = route_node_match_ipv6(table->route_table, addr);
-
-	return bgp_dest_from_rnode(rn);
-}
-
 static inline unsigned long bgp_table_count(const struct bgp_table *const table)
 {
 	return route_table_count(table->route_table);
@@ -298,59 +270,6 @@ static inline struct bgp_dest *bgp_table_get_next(const struct bgp_table *table,
 	return bgp_dest_from_rnode(route_table_get_next(table->route_table, p));
 }
 
-/*
- * bgp_table_iter_init
- */
-static inline void bgp_table_iter_init(bgp_table_iter_t *iter,
-				       struct bgp_table *table)
-{
-	bgp_table_lock(table);
-	iter->table = table;
-	route_table_iter_init(&iter->rt_iter, table->route_table);
-}
-
-/*
- * bgp_table_iter_next
- */
-static inline struct bgp_dest *bgp_table_iter_next(bgp_table_iter_t *iter)
-{
-	return bgp_dest_from_rnode(route_table_iter_next(&iter->rt_iter));
-}
-
-/*
- * bgp_table_iter_cleanup
- */
-static inline void bgp_table_iter_cleanup(bgp_table_iter_t *iter)
-{
-	route_table_iter_cleanup(&iter->rt_iter);
-	bgp_table_unlock(iter->table);
-	iter->table = NULL;
-}
-
-/*
- * bgp_table_iter_pause
- */
-static inline void bgp_table_iter_pause(bgp_table_iter_t *iter)
-{
-	route_table_iter_pause(&iter->rt_iter);
-}
-
-/*
- * bgp_table_iter_is_done
- */
-static inline int bgp_table_iter_is_done(bgp_table_iter_t *iter)
-{
-	return route_table_iter_is_done(&iter->rt_iter);
-}
-
-/*
- * bgp_table_iter_started
- */
-static inline int bgp_table_iter_started(bgp_table_iter_t *iter)
-{
-	return route_table_iter_started(&iter->rt_iter);
-}
-
 /* This would benefit from a real atomic operation...
  * until then. */
 static inline uint64_t bgp_table_next_version(struct bgp_table *table)
@@ -469,8 +388,14 @@ static inline const struct prefix *bgp_dest_get_prefix(const struct bgp_dest *de
 	return &dest->p;
 }
 
+static inline unsigned int bgp_dest_get_lock_count(const struct bgp_dest *dest)
+{
+	return dest->lock;
+}
+
 #ifdef _FRR_ATTRIBUTE_PRINTFRR
 #pragma FRR printfrr_ext "%pRN"  (struct bgp_node *)
+#pragma FRR printfrr_ext "%pBD"  (struct bgp_dest *)
 #endif
 
 #endif /* _QUAGGA_BGP_TABLE_H */
