@@ -22,6 +22,7 @@
 #include "command.h"
 #include "prefix.h"
 #include "lib/json.h"
+#include "lib/printfrr.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_route.h"
@@ -119,9 +120,9 @@ int show_adj_route_vpn(struct vty *vty, struct peer *peer,
 				if (use_json) {
 					json_object_int_add(
 						json, "bgpTableVersion", 0);
-					json_object_string_add(
+					json_object_string_addf(
 						json, "bgpLocalRouterId",
-						inet_ntoa(bgp->router_id));
+						"%pI4", &bgp->router_id);
 					json_object_int_add(
 						json,
 						"defaultLocPrf",
@@ -137,8 +138,8 @@ int show_adj_route_vpn(struct vty *vty, struct peer *peer,
 							       json_ocode);
 				} else {
 					vty_out(vty,
-						"BGP table version is 0, local router ID is %s\n",
-						inet_ntoa(bgp->router_id));
+						"BGP table version is 0, local router ID is %pI4\n",
+						&bgp->router_id);
 					vty_out(vty, "Default local pref %u, ",
 						bgp->default_local_pref);
 					vty_out(vty, "local AS %u\n", bgp->as);
@@ -184,10 +185,10 @@ int show_adj_route_vpn(struct vty *vty, struct peer *peer,
 							 "%u:%d", rd_as.as,
 							 rd_as.val);
 					else if (type == RD_TYPE_IP)
-						snprintf(rd_str, sizeof(rd_str),
-							 "%s:%d",
-							 inet_ntoa(rd_ip.ip),
-							 rd_ip.val);
+						snprintfrr(rd_str,
+							   sizeof(rd_str),
+							   "%pI4:%d", &rd_ip.ip,
+							   rd_ip.val);
 					json_object_string_add(
 						json_routes,
 						"rd", rd_str);
@@ -199,9 +200,8 @@ int show_adj_route_vpn(struct vty *vty, struct peer *peer,
 						vty_out(vty, "%u:%d", rd_as.as,
 							rd_as.val);
 					else if (type == RD_TYPE_IP)
-						vty_out(vty, "%s:%d",
-							inet_ntoa(rd_ip.ip),
-							rd_ip.val);
+						vty_out(vty, "%pI4:%d",
+							&rd_ip.ip, rd_ip.val);
 #ifdef ENABLE_BGP_VNC
 					else if (type == RD_TYPE_VNC_ETH)
 						vty_out(vty,
@@ -225,8 +225,9 @@ int show_adj_route_vpn(struct vty *vty, struct peer *peer,
 				}
 				rd_header = 0;
 			}
-			route_vty_out_tmp(vty, bgp_dest_get_prefix(rm), attr,
-					  safi, use_json, json_routes, false);
+			route_vty_out_tmp(vty, rm, bgp_dest_get_prefix(rm),
+					  attr, safi, use_json, json_routes,
+					  false);
 			output_count++;
 		}
 
@@ -238,9 +239,7 @@ int show_adj_route_vpn(struct vty *vty, struct peer *peer,
 		json_object_object_add(json, "advertisedRoutes", json_adv);
 		json_object_int_add(json,
 			"totalPrefixCounter", output_count);
-		vty_out(vty, "%s\n", json_object_to_json_string_ext(
-					     json, JSON_C_TO_STRING_PRETTY));
-		json_object_free(json);
+		vty_json(vty, json);
 	} else
 		vty_out(vty, "\nTotal number of prefixes %ld\n", output_count);
 
