@@ -74,19 +74,18 @@ static const struct message eigrp_general_tlv_type_str[] = {
  *
  * @param[in]   thread  current execution thread timer is associated with
  *
- * @return int  always returns 0
+ * @return void
  *
  * @par
  * Called once per "hello" time interval, default 5 seconds
  * Sends hello packet via multicast for all interfaces eigrp
  * is configured for
  */
-int eigrp_hello_timer(struct thread *thread)
+void eigrp_hello_timer(struct thread *thread)
 {
 	struct eigrp_interface *ei;
 
 	ei = THREAD_ARG(thread);
-	ei->t_hello = NULL;
 
 	if (IS_DEBUG_EIGRP(0, TIMERS))
 		zlog_debug("Start Hello Timer (%s) Expire [%u]", IF_NAME(ei),
@@ -96,11 +95,8 @@ int eigrp_hello_timer(struct thread *thread)
 	eigrp_hello_send(ei, EIGRP_HELLO_NORMAL, NULL);
 
 	/* Hello timer set. */
-	ei->t_hello = NULL;
 	thread_add_timer(master, eigrp_hello_timer, ei, ei->params.v_hello,
 			 &ei->t_hello);
-
-	return 0;
 }
 
 /**
@@ -125,6 +121,10 @@ eigrp_hello_parameter_decode(struct eigrp_neighbor *nbr,
 	struct eigrp *eigrp = nbr->ei->eigrp;
 	struct TLV_Parameter_Type *param = (struct TLV_Parameter_Type *)tlv;
 
+	/* First validate TLV length */
+	if (tlv->length < sizeof(struct TLV_Parameter_Type))
+		return NULL;
+
 	/* copy over the values passed in by the neighbor */
 	nbr->K1 = param->K1;
 	nbr->K2 = param->K2;
@@ -144,10 +144,11 @@ eigrp_hello_parameter_decode(struct eigrp_neighbor *nbr,
 	    && (eigrp->k_values[4] == nbr->K5)) {
 
 		if (eigrp_nbr_state_get(nbr) == EIGRP_NEIGHBOR_DOWN) {
-			zlog_info("Neighbor %s (%s) is pending: new adjacency",
-				  inet_ntoa(nbr->src),
-				  ifindex2ifname(nbr->ei->ifp->ifindex,
-						 VRF_DEFAULT));
+			zlog_info(
+				"Neighbor %pI4 (%s) is pending: new adjacency",
+				&nbr->src,
+				ifindex2ifname(nbr->ei->ifp->ifindex,
+					       eigrp->vrf_id));
 
 			/* Expedited hello sent */
 			eigrp_hello_send(nbr->ei, EIGRP_HELLO_NORMAL, NULL);
@@ -164,18 +165,18 @@ eigrp_hello_parameter_decode(struct eigrp_neighbor *nbr,
 			     & param->K5)
 			    == 255) {
 				zlog_info(
-					"Neighbor %s (%s) is down: Interface PEER-TERMINATION received",
-					inet_ntoa(nbr->src),
+					"Neighbor %pI4 (%s) is down: Interface PEER-TERMINATION received",
+					&nbr->src,
 					ifindex2ifname(nbr->ei->ifp->ifindex,
-						       VRF_DEFAULT));
+						       eigrp->vrf_id));
 				eigrp_nbr_delete(nbr);
 				return NULL;
 			} else {
 				zlog_info(
-					"Neighbor %s (%s) going down: Kvalue mismatch",
-					inet_ntoa(nbr->src),
+					"Neighbor %pI4 (%s) going down: Kvalue mismatch",
+					&nbr->src,
 					ifindex2ifname(nbr->ei->ifp->ifindex,
-						       VRF_DEFAULT));
+						       eigrp->vrf_id));
 				eigrp_nbr_state_set(nbr, EIGRP_NEIGHBOR_DOWN);
 			}
 		}
@@ -193,13 +194,22 @@ eigrp_hello_authentication_decode(struct stream *s,
 
 	md5 = (struct TLV_MD5_Authentication_Type *)tlv_header;
 
-	if (md5->auth_type == EIGRP_AUTH_TYPE_MD5)
+	if (md5->auth_type == EIGRP_AUTH_TYPE_MD5) {
+		/* Validate tlv length */
+		if (md5->length < sizeof(struct TLV_MD5_Authentication_Type))
+			return 0;
+
 		return eigrp_check_md5_digest(s, md5, nbr,
 					      EIGRP_AUTH_BASIC_HELLO_FLAG);
-	else if (md5->auth_type == EIGRP_AUTH_TYPE_SHA256)
+	} else if (md5->auth_type == EIGRP_AUTH_TYPE_SHA256) {
+		/* Validate tlv length */
+		if (md5->length < sizeof(struct TLV_SHA256_Authentication_Type))
+			return 0;
+
 		return eigrp_check_sha256_digest(
 			s, (struct TLV_SHA256_Authentication_Type *)tlv_header,
 			nbr, EIGRP_AUTH_BASIC_HELLO_FLAG);
+	}
 
 	return 0;
 }
@@ -222,6 +232,10 @@ static void eigrp_sw_version_decode(struct eigrp_neighbor *nbr,
 {
 	struct TLV_Software_Type *version = (struct TLV_Software_Type *)tlv;
 
+	/* Validate TLV length */
+	if (tlv->length < sizeof(struct TLV_Software_Type))
+		return;
+
 	nbr->os_rel_major = version->vender_major;
 	nbr->os_rel_minor = version->vender_minor;
 	nbr->tlv_rel_major = version->eigrp_major;
@@ -245,16 +259,22 @@ static void eigrp_sw_version_decode(struct eigrp_neighbor *nbr,
 static void eigrp_peer_termination_decode(struct eigrp_neighbor *nbr,
 					  struct eigrp_tlv_hdr_type *tlv)
 {
+	struct eigrp *eigrp = nbr->ei->eigrp;
 	struct TLV_Peer_Termination_type *param =
 		(struct TLV_Peer_Termination_type *)tlv;
 
-	uint32_t my_ip = nbr->ei->address->u.prefix4.s_addr;
+	/* Validate TLV length */
+	if (tlv->length < sizeof(struct TLV_Peer_Termination_type))
+		return;
+
+	uint32_t my_ip = nbr->ei->address.u.prefix4.s_addr;
 	uint32_t received_ip = param->neighbor_ip;
 
 	if (my_ip == received_ip) {
-		zlog_info("Neighbor %s (%s) is down: Peer Termination received",
-			  inet_ntoa(nbr->src),
-			  ifindex2ifname(nbr->ei->ifp->ifindex, VRF_DEFAULT));
+		zlog_info(
+			"Neighbor %pI4 (%s) is down: Peer Termination received",
+			&nbr->src,
+			ifindex2ifname(nbr->ei->ifp->ifindex, eigrp->vrf_id));
 		/* set neighbor to DOWN */
 		nbr->state = EIGRP_NEIGHBOR_DOWN;
 		/* delete neighbor */
@@ -329,9 +349,9 @@ void eigrp_hello_receive(struct eigrp *eigrp, struct ip *iph,
 	assert(nbr);
 
 	if (IS_DEBUG_EIGRP_PACKET(eigrph->opcode - 1, RECV))
-		zlog_debug("Processing Hello size[%u] int(%s) nbr(%s)", size,
-			   ifindex2ifname(nbr->ei->ifp->ifindex, VRF_DEFAULT),
-			   inet_ntoa(nbr->src));
+		zlog_debug("Processing Hello size[%u] int(%s) nbr(%pI4)", size,
+			   ifindex2ifname(nbr->ei->ifp->ifindex, eigrp->vrf_id),
+			   &nbr->src);
 
 	size -= EIGRP_HEADER_LEN;
 	if (size < 0)
@@ -343,6 +363,10 @@ void eigrp_hello_receive(struct eigrp *eigrp, struct ip *iph,
 		type = ntohs(tlv_header->type);
 		length = ntohs(tlv_header->length);
 
+		/* Validate length against packet size */
+		if (length > size)
+			return;
+
 		if ((length > 0) && (length <= size)) {
 			if (IS_DEBUG_EIGRP_PACKET(0, RECV))
 				zlog_debug(
@@ -402,8 +426,7 @@ void eigrp_hello_receive(struct eigrp *eigrp, struct ip *iph,
 	}
 
 	if (IS_DEBUG_EIGRP_PACKET(0, RECV))
-		zlog_debug("Hello Packet received from %s",
-			   inet_ntoa(nbr->src));
+		zlog_debug("Hello Packet received from %pI4", &nbr->src);
 }
 
 uint32_t FRR_MAJOR;
@@ -421,9 +444,9 @@ void eigrp_sw_version_initialize(void)
 	ret = sscanf(ver_string, "%" SCNu32 ".%" SCNu32, &FRR_MAJOR,
 		     &FRR_MINOR);
 	if (ret != 2)
-		flog_err(EIGRP_ERR_PACKET,
-			  "Did not Properly parse %s, please fix VERSION string",
-			  VERSION);
+		flog_err(EC_EIGRP_PACKET,
+			 "Did not Properly parse %s, please fix VERSION string",
+			 VERSION);
 }
 
 /**
@@ -484,21 +507,15 @@ static uint16_t eigrp_tidlist_encode(struct stream *s)
  * Part of conditional receive process
  *
  */
-static uint16_t eigrp_sequence_encode(struct stream *s)
+static uint16_t eigrp_sequence_encode(struct eigrp *eigrp, struct stream *s)
 {
 	uint16_t length = EIGRP_TLV_SEQ_BASE_LEN;
-	struct eigrp *eigrp;
 	struct eigrp_interface *ei;
 	struct listnode *node, *node2, *nnode2;
 	struct eigrp_neighbor *nbr;
 	size_t backup_end, size_end;
 	int found;
 
-	eigrp = eigrp_lookup();
-	if (eigrp == NULL) {
-		return 0;
-	}
-
 	// add in the parameters TLV
 	backup_end = stream_get_endp(s);
 	stream_putw(s, EIGRP_TLV_SEQ);
@@ -541,15 +558,10 @@ static uint16_t eigrp_sequence_encode(struct stream *s)
  * Part of conditional receive process
  *
  */
-static uint16_t eigrp_next_sequence_encode(struct stream *s)
+static uint16_t eigrp_next_sequence_encode(struct eigrp *eigrp,
+					   struct stream *s)
 {
 	uint16_t length = EIGRP_NEXT_SEQUENCE_TLV_SIZE;
-	struct eigrp *eigrp;
-
-	eigrp = eigrp_lookup();
-	if (eigrp == NULL) {
-		return 0;
-	}
 
 	// add in the parameters TLV
 	stream_putw(s, EIGRP_TLV_NEXT_MCAST_SEQ);
@@ -577,8 +589,6 @@ static uint16_t eigrp_next_sequence_encode(struct stream *s)
 static uint16_t eigrp_hello_parameter_encode(struct eigrp_interface *ei,
 					     struct stream *s, uint8_t flags)
 {
-	uint16_t length = EIGRP_TLV_PARAMETER_LEN;
-
 	// add in the parameters TLV
 	stream_putw(s, EIGRP_TLV_PARAMETER);
 	stream_putw(s, EIGRP_TLV_PARAMETER_LEN);
@@ -605,7 +615,7 @@ static uint16_t eigrp_hello_parameter_encode(struct eigrp_interface *ei,
 	// and set hold time value..
 	stream_putw(s, ei->params.v_wait);
 
-	return length;
+	return EIGRP_TLV_PARAMETER_LEN;
 }
 
 /**
@@ -661,8 +671,8 @@ static struct eigrp_packet *eigrp_hello_encode(struct eigrp_interface *ei,
 		length += eigrp_sw_version_encode(ep->s);
 
 		if (flags & EIGRP_HELLO_ADD_SEQUENCE) {
-			length += eigrp_sequence_encode(ep->s);
-			length += eigrp_next_sequence_encode(ep->s);
+			length += eigrp_sequence_encode(ei->eigrp, ep->s);
+			length += eigrp_next_sequence_encode(ei->eigrp, ep->s);
 		}
 
 		// add in the TID list if doing multi-topology
@@ -720,9 +730,8 @@ void eigrp_hello_send_ack(struct eigrp_neighbor *nbr)
 
 	if (ep) {
 		if (IS_DEBUG_EIGRP_PACKET(0, SEND))
-			zlog_debug("Queueing [Hello] Ack Seq [%u] nbr [%s]",
-				   nbr->recv_sequence_number,
-				   inet_ntoa(nbr->src));
+			zlog_debug("Queueing [Hello] Ack Seq [%u] nbr [%pI4]",
+				   nbr->recv_sequence_number, &nbr->src);
 
 		/* Add packet to the top of the interface output queue*/
 		eigrp_fifo_push(nbr->ei->obuf, ep);
@@ -757,12 +766,6 @@ void eigrp_hello_send(struct eigrp_interface *ei, uint8_t flags,
 {
 	struct eigrp_packet *ep = NULL;
 
-	/* If this is passive interface, do not send EIGRP Hello.
-	   if ((EIGRP_IF_PASSIVE_STATUS (ei) == EIGRP_IF_PASSIVE) ||
-	   (ei->type != EIGRP_IFTYPE_NBMA))
-	   return;
-	*/
-
 	if (IS_DEBUG_EIGRP_PACKET(0, SEND))
 		zlog_debug("Queueing [Hello] Interface(%s)", IF_NAME(ei));
 
