@@ -55,10 +55,13 @@
 #include "eigrpd/eigrp_packet.h"
 #include "eigrpd/eigrp_network.h"
 #include "eigrpd/eigrp_topology.h"
-#include "eigrpd/eigrp_memory.h"
 #include "eigrpd/eigrp_filter.h"
 
-DEFINE_QOBJ_TYPE(eigrp)
+DEFINE_MGROUP(EIGRPD, "eigrpd");
+
+DEFINE_MTYPE_STATIC(EIGRPD, EIGRP_TOP, "EIGRP structure");
+
+DEFINE_QOBJ_TYPE(eigrp);
 
 static struct eigrp_master eigrp_master;
 
@@ -110,10 +113,6 @@ void eigrp_router_id_update(struct eigrp *eigrp)
 
 	eigrp->router_id = router_id;
 	if (router_id_old.s_addr != router_id.s_addr) {
-		//      if (IS_DEBUG_EIGRP_EVENT)
-		//        zlog_debug("Router-ID[NEW:%s]: Update",
-		//        inet_ntoa(eigrp->router_id));
-
 		/* update eigrp_interface's */
 		FOR_ALL_INTERFACES (vrf, ifp)
 			eigrp_if_update(ifp);
@@ -124,7 +123,7 @@ void eigrp_master_init(void)
 {
 	struct timeval tv;
 
-	memset(&eigrp_master, 0, sizeof(struct eigrp_master));
+	memset(&eigrp_master, 0, sizeof(eigrp_master));
 
 	eigrp_om = &eigrp_master;
 	eigrp_om->eigrp = list_new();
@@ -164,7 +163,8 @@ static struct eigrp *eigrp_new(uint16_t as, vrf_id_t vrf_id)
 	if (eigrp->fd < 0) {
 		flog_err_sys(
 			EC_LIB_SOCKET,
-			"eigrp_new: fatal error: eigrp_sock_init was unable to open a socket");
+			"%s: fatal error: eigrp_sock_init was unable to open a socket",
+			__func__);
 		exit(1);
 	}
 
@@ -172,7 +172,6 @@ static struct eigrp *eigrp_new(uint16_t as, vrf_id_t vrf_id)
 
 	eigrp->ibuf = stream_new(EIGRP_PACKET_MAX_LEN + 1);
 
-	eigrp->t_read = NULL;
 	thread_add_read(master, eigrp_read, eigrp, eigrp->fd, &eigrp->t_read);
 	eigrp->oi_write_q = list_new();
 
