@@ -59,10 +59,13 @@
 #include "isisd/isis_errors.h"
 #include "isisd/isis_tx_queue.h"
 #include "isisd/isis_nb.h"
+#include "isisd/isis_ldp_sync.h"
 
-DEFINE_QOBJ_TYPE(isis_circuit)
+DEFINE_MTYPE_STATIC(ISISD, ISIS_CIRCUIT, "ISIS circuit");
 
-DEFINE_HOOK(isis_if_new_hook, (struct interface *ifp), (ifp))
+DEFINE_QOBJ_TYPE(isis_circuit);
+
+DEFINE_HOOK(isis_if_new_hook, (struct interface *ifp), (ifp));
 
 /*
  * Prototypes.
@@ -70,18 +73,50 @@ DEFINE_HOOK(isis_if_new_hook, (struct interface *ifp), (ifp))
 int isis_if_new_hook(struct interface *);
 int isis_if_delete_hook(struct interface *);
 
-struct isis_circuit *isis_circuit_new(void)
+DEFINE_HOOK(isis_circuit_new_hook, (struct isis_circuit *circuit), (circuit));
+DEFINE_HOOK(isis_circuit_del_hook, (struct isis_circuit *circuit), (circuit));
+
+static void isis_circuit_enable(struct isis_circuit *circuit)
+{
+	struct isis_area *area = circuit->area;
+	struct interface *ifp = circuit->interface;
+
+	if (!area) {
+		area = isis_area_lookup(circuit->tag, ifp->vrf->vrf_id);
+		if (area)
+			isis_area_add_circuit(area, circuit);
+	}
+
+	if (if_is_operative(ifp))
+		isis_csm_state_change(IF_UP_FROM_Z, circuit, ifp);
+}
+
+static void isis_circuit_disable(struct isis_circuit *circuit)
+{
+	struct isis_area *area = circuit->area;
+	struct interface *ifp = circuit->interface;
+
+	if (if_is_operative(ifp))
+		isis_csm_state_change(IF_DOWN_FROM_Z, circuit, ifp);
+
+	if (area)
+		isis_area_del_circuit(area, circuit);
+}
+
+struct isis_circuit *isis_circuit_new(struct interface *ifp, const char *tag)
 {
 	struct isis_circuit *circuit;
 	int i;
 
 	circuit = XCALLOC(MTYPE_ISIS_CIRCUIT, sizeof(struct isis_circuit));
 
+	circuit->tag = XSTRDUP(MTYPE_ISIS_CIRCUIT, tag);
+
 	/*
 	 * Default values
 	 */
 #ifndef FABRICD
-	circuit->is_type = yang_get_default_enum(
+	circuit->is_type_config = yang_get_default_enum(
 		"/frr-interface:lib/interface/frr-isisd:isis/circuit-type");
 	circuit->flags = 0;
 
@@ -121,7 +156,7 @@ struct isis_circuit *isis_circuit_new(void)
 		circuit->level_arg[i].circuit = circuit;
 	}
 #else
-	circuit->is_type = IS_LEVEL_1_AND_2;
+	circuit->is_type_config = IS_LEVEL_1_AND_2;
 	circuit->flags = 0;
 	circuit->pad_hellos = 1;
 	for (i = 0; i < 2; i++) {
@@ -137,10 +172,26 @@ struct isis_circuit *isis_circuit_new(void)
 	}
 #endif /* ifndef FABRICD */
 
+	circuit->is_type = circuit->is_type_config;
+
 	circuit_mt_init(circuit);
+	isis_lfa_excluded_ifaces_init(circuit, ISIS_LEVEL1);
+	isis_lfa_excluded_ifaces_init(circuit, ISIS_LEVEL2);
+
+	circuit->ldp_sync_info = ldp_sync_info_create();
+	circuit->ldp_sync_info->enabled = LDP_IGP_SYNC_ENABLED;
 
 	QOBJ_REG(circuit, isis_circuit);
 
+	isis_circuit_if_bind(circuit, ifp);
+
+	circuit->ip_addrs = list_new();
+	circuit->ipv6_link = list_new();
+	circuit->ipv6_non_link = list_new();
+
+	if (ifp->ifindex != IFINDEX_INTERNAL)
+		isis_circuit_enable(circuit);
+
 	return circuit;
 }
 
@@ -149,11 +200,30 @@ void isis_circuit_del(struct isis_circuit *circuit)
 	if (!circuit)
 		return;
 
-	QOBJ_UNREG(circuit);
+	if (circuit->interface->ifindex != IFINDEX_INTERNAL)
+		isis_circuit_disable(circuit);
 
 	isis_circuit_if_unbind(circuit, circuit->interface);
 
+	QOBJ_UNREG(circuit);
+
+	ldp_sync_info_free(&circuit->ldp_sync_info);
+
 	circuit_mt_finish(circuit);
+	isis_lfa_excluded_ifaces_clear(circuit, ISIS_LEVEL1);
+	isis_lfa_excluded_ifaces_clear(circuit, ISIS_LEVEL2);
+
+	list_delete(&circuit->ip_addrs);
+	list_delete(&circuit->ipv6_link);
+	list_delete(&circuit->ipv6_non_link);
+
+	if (circuit->ext) {
+		isis_del_ext_subtlvs(circuit->ext);
+		circuit->ext = NULL;
+	}
+
+	XFREE(MTYPE_TMP, circuit->bfd_config.profile);
+	XFREE(MTYPE_ISIS_CIRCUIT, circuit->tag);
 
 	/* and lastly the circuit itself */
 	XFREE(MTYPE_ISIS_CIRCUIT, circuit);
@@ -165,6 +235,7 @@ void isis_circuit_configure(struct isis_circuit *circuit,
 			    struct isis_area *area)
 {
 	assert(area);
+	circuit->isis = area->isis;
 	circuit->area = area;
 
 	/*
@@ -184,76 +255,45 @@ void isis_circuit_configure(struct isis_circuit *circuit,
 
 	circuit->idx = flags_get_index(&area->flags);
 
+	hook_call(isis_circuit_new_hook, circuit);
+
 	return;
 }
 
 void isis_circuit_deconfigure(struct isis_circuit *circuit,
 			      struct isis_area *area)
 {
+	hook_call(isis_circuit_del_hook, circuit);
+
 	/* Free the index of SRM and SSN flags */
 	flags_free_index(&area->flags, circuit->idx);
 	circuit->idx = 0;
+
+	/* Reset IS type to configured */
+	circuit->is_type = circuit->is_type_config;
+
 	/* Remove circuit from area */
 	assert(circuit->area == area);
 	listnode_delete(area->circuit_list, circuit);
 	circuit->area = NULL;
+	circuit->isis = NULL;
 
 	return;
 }
 
-struct isis_circuit *circuit_lookup_by_ifp(struct interface *ifp,
-					   struct list *list)
-{
-	struct isis_circuit *circuit = NULL;
-	struct listnode *node;
-
-	if (!list)
-		return NULL;
-
-	for (ALL_LIST_ELEMENTS_RO(list, node, circuit))
-		if (circuit->interface == ifp) {
-			assert(ifp->info == circuit);
-			return circuit;
-		}
-
-	return NULL;
-}
-
 struct isis_circuit *circuit_scan_by_ifp(struct interface *ifp)
 {
-	struct isis_area *area;
-	struct listnode *node;
-	struct isis_circuit *circuit;
-	struct isis *isis = NULL;
-
-	if (ifp->info)
-		return (struct isis_circuit *)ifp->info;
-
-	isis = isis_lookup_by_vrfid(ifp->vrf_id);
-	if (isis == NULL) {
-		zlog_warn(" %s : ISIS routing instance not found", __func__);
-		return NULL;
-	}
-
-	if (isis->area_list) {
-		for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
-			circuit =
-				circuit_lookup_by_ifp(ifp, area->circuit_list);
-			if (circuit)
-				return circuit;
-		}
-	}
-	return circuit_lookup_by_ifp(ifp, isis->init_circ_list);
+	return (struct isis_circuit *)ifp->info;
 }
 
+DEFINE_HOOK(isis_circuit_add_addr_hook, (struct isis_circuit *circuit),
+	    (circuit));
+
 void isis_circuit_add_addr(struct isis_circuit *circuit,
 			   struct connected *connected)
 {
 	struct listnode *node;
 	struct prefix_ipv4 *ipv4;
-#if defined(EXTREME_DEBUG)
-	char buf[PREFIX2STR_BUFFER];
-#endif
 	struct prefix_ipv6 *ipv6;
 
 	if (connected->address->family == AF_INET) {
@@ -285,9 +325,10 @@ void isis_circuit_add_addr(struct isis_circuit *circuit,
 						0);
 
 #ifdef EXTREME_DEBUG
-		prefix2str(connected->address, buf, sizeof(buf));
-		zlog_debug("Added IP address %s to circuit %s", buf,
-			   circuit->interface->name);
+		if (IS_DEBUG_EVENTS)
+			zlog_debug("Added IP address %pFX to circuit %s",
+				   connected->address,
+				   circuit->interface->name);
 #endif /* EXTREME_DEBUG */
 	}
 	if (connected->address->family == AF_INET6) {
@@ -309,18 +350,30 @@ void isis_circuit_add_addr(struct isis_circuit *circuit,
 
 		if (IN6_IS_ADDR_LINKLOCAL(&ipv6->prefix))
 			listnode_add(circuit->ipv6_link, ipv6);
-		else
+		else {
 			listnode_add(circuit->ipv6_non_link, ipv6);
+			/* Update Local IPv6 address param. if MPLS TE is on */
+			if (circuit->ext && circuit->area
+			    && IS_MPLS_TE(circuit->area->mta)) {
+				IPV6_ADDR_COPY(&circuit->ext->local_addr6,
+					       &ipv6->prefix);
+				SET_SUBTLV(circuit->ext, EXT_LOCAL_ADDR6);
+			}
+		}
 		if (circuit->area)
 			lsp_regenerate_schedule(circuit->area, circuit->is_type,
 						0);
 
 #ifdef EXTREME_DEBUG
-		prefix2str(connected->address, buf, sizeof(buf));
-		zlog_debug("Added IPv6 address %s to circuit %s", buf,
-			   circuit->interface->name);
+		if (IS_DEBUG_EVENTS)
+			zlog_debug("Added IPv6 address %pFX to circuit %s",
+				   connected->address,
+				   circuit->interface->name);
 #endif /* EXTREME_DEBUG */
 	}
+
+	hook_call(isis_circuit_add_addr_hook, circuit);
+
 	return;
 }
 
@@ -329,7 +382,6 @@ void isis_circuit_del_addr(struct isis_circuit *circuit,
 {
 	struct prefix_ipv4 *ipv4, *ip = NULL;
 	struct listnode *node;
-	char buf[PREFIX2STR_BUFFER];
 	struct prefix_ipv6 *ipv6, *ip6 = NULL;
 	int found = 0;
 
@@ -350,16 +402,14 @@ void isis_circuit_del_addr(struct isis_circuit *circuit,
 				lsp_regenerate_schedule(circuit->area,
 							circuit->is_type, 0);
 		} else {
-			prefix2str(connected->address, buf, sizeof(buf));
 			zlog_warn(
-				"Nonexistent ip address %s removal attempt from circuit %s",
-				buf, circuit->interface->name);
+				"Nonexistent ip address %pFX removal attempt from circuit %s",
+				connected->address, circuit->interface->name);
 			zlog_warn("Current ip addresses on %s:",
 				  circuit->interface->name);
 			for (ALL_LIST_ELEMENTS_RO(circuit->ip_addrs, node,
 						  ip)) {
-				prefix2str(ip, buf, sizeof(buf));
-				zlog_warn("  %s", buf);
+				zlog_warn("  %pFX", ip);
 			}
 			zlog_warn("End of addresses");
 		}
@@ -398,25 +448,18 @@ void isis_circuit_del_addr(struct isis_circuit *circuit,
 		}
 
 		if (!found) {
-			prefix2str(connected->address, buf, sizeof(buf));
 			zlog_warn(
-				"Nonexistent ip address %s removal attempt from circuit %s",
-				buf, circuit->interface->name);
+				"Nonexistent ip address %pFX removal attempt from circuit %s",
+				connected->address, circuit->interface->name);
 			zlog_warn("Current ip addresses on %s:",
 				  circuit->interface->name);
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_link, node,
-						  ip6)) {
-				prefix2str((struct prefix *)ip6, (char *)buf,
-					   sizeof(buf));
-				zlog_warn("  %s", buf);
-			}
+						  ip6))
+				zlog_warn("  %pFX", (struct prefix *)ip6);
 			zlog_warn(" -----");
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_non_link, node,
-						  ip6)) {
-				prefix2str((struct prefix *)ip6, (char *)buf,
-					   sizeof(buf));
-				zlog_warn("  %s", buf);
-			}
+						  ip6))
+				zlog_warn("  %pFX", (struct prefix *)ip6);
 			zlog_warn("End of addresses");
 		} else if (circuit->area)
 			lsp_regenerate_schedule(circuit->area, circuit->is_type,
@@ -462,8 +505,6 @@ void isis_circuit_if_add(struct isis_circuit *circuit, struct interface *ifp)
 	struct listnode *node, *nnode;
 	struct connected *conn;
 
-	isis_circuit_if_bind(circuit, ifp);
-
 	if (if_is_broadcast(ifp)) {
 		if (fabricd || circuit->circ_type_config == CIRCUIT_T_P2P)
 			circuit->circ_type = CIRCUIT_T_P2P;
@@ -477,14 +518,10 @@ void isis_circuit_if_add(struct isis_circuit *circuit, struct interface *ifp)
 	} else {
 		/* It's normal in case of loopback etc. */
 		if (IS_DEBUG_EVENTS)
-			zlog_debug("isis_circuit_if_add: unsupported media");
+			zlog_debug("%s: unsupported media", __func__);
 		circuit->circ_type = CIRCUIT_T_UNKNOWN;
 	}
 
-	circuit->ip_addrs = list_new();
-	circuit->ipv6_link = list_new();
-	circuit->ipv6_non_link = list_new();
-
 	for (ALL_LIST_ELEMENTS(ifp->connected, node, nnode, conn))
 		isis_circuit_add_addr(circuit, conn);
 
@@ -501,21 +538,6 @@ void isis_circuit_if_del(struct isis_circuit *circuit, struct interface *ifp)
 	for (ALL_LIST_ELEMENTS(ifp->connected, node, nnode, conn))
 		isis_circuit_del_addr(circuit, conn);
 
-	if (circuit->ip_addrs) {
-		assert(listcount(circuit->ip_addrs) == 0);
-		list_delete(&circuit->ip_addrs);
-	}
-
-	if (circuit->ipv6_link) {
-		assert(listcount(circuit->ipv6_link) == 0);
-		list_delete(&circuit->ipv6_link);
-	}
-
-	if (circuit->ipv6_non_link) {
-		assert(listcount(circuit->ipv6_non_link) == 0);
-		list_delete(&circuit->ipv6_non_link);
-	}
-
 	circuit->circ_type = CIRCUIT_T_UNKNOWN;
 }
 
@@ -612,6 +634,7 @@ int isis_circuit_up(struct isis_circuit *circuit)
 		return ISIS_OK;
 
 	if (circuit->is_passive) {
+		circuit->last_uptime = time(NULL);
 		/* make sure the union fields are initialized, else we
 		 * could end with garbage values from a previous circuit
 		 * type, which would then cause a segfault when building
@@ -632,13 +655,16 @@ int isis_circuit_up(struct isis_circuit *circuit)
 			"Interface MTU %zu on %s is too low to support area lsp mtu %u!",
 			isis_circuit_pdu_size(circuit),
 			circuit->interface->name, circuit->area->lsp_mtu);
-		isis_circuit_update_all_srmflags(circuit, 0);
-		return ISIS_ERROR;
+
+		/* Allow ISIS to continue configuration.   With this
+		 * configuration failure ISIS will attempt to send lsp
+		 * packets but will fail until the mtu is configured properly
+		 */
 	}
 
 	if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
-		circuit->circuit_id = isis_circuit_id_gen(circuit->area->isis,
-							  circuit->interface);
+		circuit->circuit_id =
+			isis_circuit_id_gen(circuit->isis, circuit->interface);
 		if (!circuit->circuit_id) {
 			flog_err(
 				EC_ISIS_CONFIG,
@@ -656,9 +682,11 @@ int isis_circuit_up(struct isis_circuit *circuit)
 			       ETH_ALEN);
 		}
 #ifdef EXTREME_DEGUG
-		zlog_debug("isis_circuit_if_add: if_id %d, isomtu %d snpa %s",
-			   circuit->interface->ifindex, ISO_MTU(circuit),
-			   snpa_print(circuit->u.bc.snpa));
+		if (IS_DEBUG_EVENTS)
+			zlog_debug("%s: if_id %d, isomtu %d snpa %s", __func__,
+				   circuit->interface->ifindex,
+				   ISO_MTU(circuit),
+				   snpa_print(circuit->u.bc.snpa));
 #endif /* EXTREME_DEBUG */
 
 		circuit->u.bc.adjdb[0] = list_new();
@@ -725,6 +753,13 @@ int isis_circuit_up(struct isis_circuit *circuit)
 
 	circuit->tx_queue = isis_tx_queue_new(circuit, send_lsp);
 
+	circuit->last_uptime = time(NULL);
+
+	if (circuit->area->mta && circuit->area->mta->status)
+		isis_link_params_update(circuit, circuit->interface);
+
+	isis_if_ldp_sync_enable(circuit);
+
 #ifndef FABRICD
 	/* send northbound notification */
 	isis_notif_if_state_change(circuit, false);
@@ -740,6 +775,8 @@ void isis_circuit_down(struct isis_circuit *circuit)
 	isis_notif_if_state_change(circuit, true);
 #endif /* ifndef FABRICD */
 
+	isis_if_ldp_sync_disable(circuit);
+
 	/* log adjacency changes if configured to do so */
 	if (circuit->area->log_adj_changes) {
 		struct isis_adjacency *adj = NULL;
@@ -813,29 +850,38 @@ void isis_circuit_down(struct isis_circuit *circuit)
 		memset(circuit->u.bc.l2_desig_is, 0, ISIS_SYS_ID_LEN + 1);
 		memset(circuit->u.bc.snpa, 0, ETH_ALEN);
 
-		THREAD_TIMER_OFF(circuit->u.bc.t_send_lan_hello[0]);
-		THREAD_TIMER_OFF(circuit->u.bc.t_send_lan_hello[1]);
-		THREAD_TIMER_OFF(circuit->u.bc.t_run_dr[0]);
-		THREAD_TIMER_OFF(circuit->u.bc.t_run_dr[1]);
-		THREAD_TIMER_OFF(circuit->u.bc.t_refresh_pseudo_lsp[0]);
-		THREAD_TIMER_OFF(circuit->u.bc.t_refresh_pseudo_lsp[1]);
+		THREAD_OFF(circuit->u.bc.t_send_lan_hello[0]);
+		THREAD_OFF(circuit->u.bc.t_send_lan_hello[1]);
+		THREAD_OFF(circuit->u.bc.t_run_dr[0]);
+		THREAD_OFF(circuit->u.bc.t_run_dr[1]);
+		THREAD_OFF(circuit->u.bc.t_refresh_pseudo_lsp[0]);
+		THREAD_OFF(circuit->u.bc.t_refresh_pseudo_lsp[1]);
 		circuit->lsp_regenerate_pending[0] = 0;
 		circuit->lsp_regenerate_pending[1] = 0;
 
-		_ISIS_CLEAR_FLAG(circuit->area->isis->circuit_ids_used,
+		_ISIS_CLEAR_FLAG(circuit->isis->circuit_ids_used,
 				 circuit->circuit_id);
 		circuit->circuit_id = 0;
 	} else if (circuit->circ_type == CIRCUIT_T_P2P) {
 		isis_delete_adj(circuit->u.p2p.neighbor);
 		circuit->u.p2p.neighbor = NULL;
-		THREAD_TIMER_OFF(circuit->u.p2p.t_send_p2p_hello);
+		THREAD_OFF(circuit->u.p2p.t_send_p2p_hello);
 	}
 
+	/*
+	 * All adjacencies have to be gone, delete snmp list
+	 * and reset snmpd idx generator
+	 */
+	if (circuit->snmp_adj_list != NULL)
+		list_delete(&circuit->snmp_adj_list);
+
+	circuit->snmp_adj_idx_gen = 0;
+
 	/* Cancel all active threads */
-	THREAD_TIMER_OFF(circuit->t_send_csnp[0]);
-	THREAD_TIMER_OFF(circuit->t_send_csnp[1]);
-	THREAD_TIMER_OFF(circuit->t_send_psnp[0]);
-	THREAD_TIMER_OFF(circuit->t_send_psnp[1]);
+	THREAD_OFF(circuit->t_send_csnp[0]);
+	THREAD_OFF(circuit->t_send_csnp[1]);
+	THREAD_OFF(circuit->t_send_psnp[0]);
+	THREAD_OFF(circuit->t_send_psnp[1]);
 	THREAD_OFF(circuit->t_read);
 
 	if (circuit->tx_queue) {
@@ -890,6 +936,150 @@ void circuit_update_nlpids(struct isis_circuit *circuit)
 	return;
 }
 
+void isis_circuit_print_json(struct isis_circuit *circuit,
+			     struct json_object *json, char detail)
+{
+	int level;
+	json_object *iface_json, *ipv4_addr_json, *ipv6_link_json,
+		*ipv6_non_link_json, *hold_json, *lan_prio_json, *levels_json,
+		*level_json;
+	char buf_prx[INET6_BUFSIZ];
+	char buf[255];
+
+	snprintfrr(buf, sizeof(buf), "0x%x", circuit->circuit_id);
+	if (detail == ISIS_UI_LEVEL_BRIEF) {
+		iface_json = json_object_new_object();
+		json_object_object_add(json, "interface", iface_json);
+		json_object_string_add(iface_json, "name",
+				       circuit->interface->name);
+		json_object_string_add(iface_json, "circuit-id", buf);
+		json_object_string_add(iface_json, "state",
+				       circuit_state2string(circuit->state));
+		json_object_string_add(iface_json, "type",
+				       circuit_type2string(circuit->circ_type));
+		json_object_string_add(iface_json, "level",
+				       circuit_t2string(circuit->is_type));
+	}
+
+	if (detail == ISIS_UI_LEVEL_DETAIL) {
+		struct listnode *node;
+		struct prefix *ip_addr;
+
+		iface_json = json_object_new_object();
+		json_object_object_add(json, "interface", iface_json);
+		json_object_string_add(iface_json, "name",
+				       circuit->interface->name);
+		json_object_string_add(iface_json, "state",
+				       circuit_state2string(circuit->state));
+		if (circuit->is_passive)
+			json_object_string_add(iface_json, "is-passive",
+					       "passive");
+		else
+			json_object_string_add(iface_json, "is-passive",
+					       "active");
+		json_object_string_add(iface_json, "circuit-id", buf);
+		json_object_string_add(iface_json, "type",
+				       circuit_type2string(circuit->circ_type));
+		json_object_string_add(iface_json, "level",
+				       circuit_t2string(circuit->is_type));
+		if (circuit->circ_type == CIRCUIT_T_BROADCAST)
+			json_object_string_add(iface_json, "snpa",
+					       snpa_print(circuit->u.bc.snpa));
+
+
+		levels_json = json_object_new_array();
+		json_object_object_add(iface_json, "levels", levels_json);
+		for (level = ISIS_LEVEL1; level <= ISIS_LEVELS; level++) {
+			if ((circuit->is_type & level) == 0)
+				continue;
+			level_json = json_object_new_object();
+			json_object_string_add(level_json, "level",
+					       circuit_t2string(level));
+			if (circuit->area->newmetric)
+				json_object_int_add(level_json, "metric",
+						    circuit->te_metric[0]);
+			else
+				json_object_int_add(level_json, "metric",
+						    circuit->metric[0]);
+			if (!circuit->is_passive) {
+				json_object_int_add(level_json,
+						    "active-neighbors",
+						    circuit->upadjcount[0]);
+				json_object_int_add(level_json,
+						    "hello-interval",
+						    circuit->hello_interval[0]);
+				hold_json = json_object_new_object();
+				json_object_object_add(level_json, "holddown",
+						       hold_json);
+				json_object_int_add(
+					hold_json, "count",
+					circuit->hello_multiplier[0]);
+				json_object_string_add(
+					hold_json, "pad",
+					(circuit->pad_hellos ? "yes" : "no"));
+				json_object_int_add(level_json, "cnsp-interval",
+						    circuit->csnp_interval[0]);
+				json_object_int_add(level_json, "psnp-interval",
+						    circuit->psnp_interval[0]);
+				if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
+					lan_prio_json =
+						json_object_new_object();
+					json_object_object_add(level_json,
+							       "lan",
+							       lan_prio_json);
+					json_object_int_add(
+						lan_prio_json, "priority",
+						circuit->priority[0]);
+					json_object_string_add(
+						lan_prio_json, "is-dis",
+						(circuit->u.bc.is_dr[0]
+							 ? "yes"
+							 : "no"));
+				}
+			}
+			json_object_array_add(levels_json, level_json);
+		}
+
+		if (listcount(circuit->ip_addrs) > 0) {
+			ipv4_addr_json = json_object_new_object();
+			json_object_object_add(iface_json, "ip-prefix",
+					       ipv4_addr_json);
+			for (ALL_LIST_ELEMENTS_RO(circuit->ip_addrs, node,
+						  ip_addr)) {
+				snprintfrr(buf_prx, INET6_BUFSIZ, "%pFX",
+					   ip_addr);
+				json_object_string_add(ipv4_addr_json, "ip",
+						       buf_prx);
+			}
+		}
+		if (listcount(circuit->ipv6_link) > 0) {
+			ipv6_link_json = json_object_new_object();
+			json_object_object_add(iface_json, "ipv6-link-locals",
+					       ipv6_link_json);
+			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_link, node,
+						  ip_addr)) {
+				snprintfrr(buf_prx, INET6_BUFSIZ, "%pFX",
+					   ip_addr);
+				json_object_string_add(ipv6_link_json, "ipv6",
+						       buf_prx);
+			}
+		}
+		if (listcount(circuit->ipv6_non_link) > 0) {
+			ipv6_non_link_json = json_object_new_object();
+			json_object_object_add(iface_json, "ipv6-prefixes",
+					       ipv6_non_link_json);
+			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_non_link, node,
+						  ip_addr)) {
+				snprintfrr(buf_prx, INET6_BUFSIZ, "%pFX",
+					   ip_addr);
+				json_object_string_add(ipv6_non_link_json,
+						       "ipv6", buf_prx);
+			}
+		}
+	}
+	return;
+}
+
 void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 			    char detail)
 {
@@ -905,7 +1095,6 @@ void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 	if (detail == ISIS_UI_LEVEL_DETAIL) {
 		struct listnode *node;
 		struct prefix *ip_addr;
-		char buf[BUFSIZ];
 
 		vty_out(vty, "  Interface: %s", circuit->interface->name);
 		vty_out(vty, ", State: %s",
@@ -987,30 +1176,23 @@ void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 				vty_out(vty, "\n");
 			}
 		}
-		if (circuit->ip_addrs && listcount(circuit->ip_addrs) > 0) {
+		if (listcount(circuit->ip_addrs) > 0) {
 			vty_out(vty, "    IP Prefix(es):\n");
 			for (ALL_LIST_ELEMENTS_RO(circuit->ip_addrs, node,
-						  ip_addr)) {
-				prefix2str(ip_addr, buf, sizeof(buf));
-				vty_out(vty, "      %s\n", buf);
-			}
+						  ip_addr))
+				vty_out(vty, "      %pFX\n", ip_addr);
 		}
-		if (circuit->ipv6_link && listcount(circuit->ipv6_link) > 0) {
+		if (listcount(circuit->ipv6_link) > 0) {
 			vty_out(vty, "    IPv6 Link-Locals:\n");
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_link, node,
-						  ip_addr)) {
-				prefix2str(ip_addr, (char *)buf, BUFSIZ);
-				vty_out(vty, "      %s\n", buf);
-			}
+						  ip_addr))
+				vty_out(vty, "      %pFX\n", ip_addr);
 		}
-		if (circuit->ipv6_non_link
-		    && listcount(circuit->ipv6_non_link) > 0) {
+		if (listcount(circuit->ipv6_non_link) > 0) {
 			vty_out(vty, "    IPv6 Prefixes:\n");
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_non_link, node,
-						  ip_addr)) {
-				prefix2str(ip_addr, (char *)buf, BUFSIZ);
-				vty_out(vty, "      %s\n", buf);
-			}
+						  ip_addr))
+				vty_out(vty, "      %pFX\n", ip_addr);
 		}
 
 		vty_out(vty, "\n");
@@ -1021,29 +1203,19 @@ void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 #ifdef FABRICD
 DEFINE_HOOK(isis_circuit_config_write,
 	    (struct isis_circuit *circuit, struct vty *vty),
-	    (circuit, vty))
+	    (circuit, vty));
 
 static int isis_interface_config_write(struct vty *vty)
 {
 	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	int write = 0;
-	struct listnode *node;
 	struct interface *ifp;
-	struct isis_area *area;
 	struct isis_circuit *circuit;
 	int i;
-	struct isis *isis = NULL;
-
-	isis = isis_lookup_by_vrfid(vrf->vrf_id);
-
-	if (isis == NULL) {
-		vty_out(vty, "ISIS routing instance not found");
-		return 0;
-	}
 
 	FOR_ALL_INTERFACES (vrf, ifp) {
 		/* IF name */
-		vty_frame(vty, "interface %s\n", ifp->name);
+		if_vty_config_start(vty, ifp);
 		write++;
 		/* IF desc */
 		if (ifp->desc) {
@@ -1051,14 +1223,13 @@ static int isis_interface_config_write(struct vty *vty)
 			write++;
 		}
 		/* ISIS Circuit */
-		for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
-			circuit =
-				circuit_lookup_by_ifp(ifp, area->circuit_list);
+		do {
+			circuit = circuit_scan_by_ifp(ifp);
 			if (circuit == NULL)
-				continue;
+				break;
 			if (circuit->ip_router) {
 				vty_out(vty, " ip router " PROTO_NAME " %s\n",
-					area->area_tag);
+					circuit->tag);
 				write++;
 			}
 			if (circuit->is_passive) {
@@ -1071,7 +1242,7 @@ static int isis_interface_config_write(struct vty *vty)
 			}
 			if (circuit->ipv6_router) {
 				vty_out(vty, " ipv6 router " PROTO_NAME " %s\n",
-					area->area_tag);
+					circuit->tag);
 				write++;
 			}
 
@@ -1245,56 +1416,20 @@ static int isis_interface_config_write(struct vty *vty)
 					circuit->passwd.passwd);
 				write++;
 			}
+			if (circuit->bfd_config.enabled) {
+				vty_out(vty, " " PROTO_NAME " bfd\n");
+				write++;
+			}
 			write += hook_call(isis_circuit_config_write,
 					   circuit, vty);
-		}
-		vty_endframe(vty, "!\n");
+		} while (0);
+		if_vty_config_end(vty);
 	}
 
 	return write;
 }
-#else
-static int isis_interface_config_write(struct vty *vty)
-{
-	struct vrf *vrf = NULL;
-	int write = 0;
-
-	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
-		struct interface *ifp;
-
-		FOR_ALL_INTERFACES (vrf, ifp) {
-			struct lyd_node *dnode;
-			dnode = yang_dnode_get(
-				running_config->dnode,
-				"/frr-interface:lib/interface[name='%s'][vrf='%s']",
-				ifp->name, vrf->name);
-			if (dnode == NULL)
-				continue;
-
-			write++;
-			nb_cli_show_dnode_cmds(vty, dnode, false);
-		}
-	}
-	return write;
-}
 #endif /* ifdef FABRICD */
 
-struct isis_circuit *isis_circuit_create(struct isis_area *area,
-					 struct interface *ifp)
-{
-	struct isis_circuit *circuit = circuit_scan_by_ifp(ifp);
-
-	if (circuit && circuit->area)
-		return NULL;
-	circuit = isis_csm_state_change(ISIS_ENABLE, circuit, area);
-	if (circuit->state != C_STATE_CONF && circuit->state != C_STATE_UP)
-		return circuit;
-	isis_circuit_if_bind(circuit, ifp);
-	if (circuit->area->mta && circuit->area->mta->status)
-		isis_link_params_update(circuit, ifp);
-	return circuit;
-}
-
 void isis_circuit_af_set(struct isis_circuit *circuit, bool ip_router,
 			 bool ipv6_router)
 {
@@ -1310,24 +1445,13 @@ void isis_circuit_af_set(struct isis_circuit *circuit, bool ip_router,
 	circuit->ipv6_router = ipv6_router;
 	circuit_update_nlpids(circuit);
 
-	/* the area should always be there if we get here, but in the past
-	 * there were corner cases where the area was NULL (e.g. because the
-	 * circuit was deconfigured following a validation error). Do not
-	 * segfault if this happens again.
-	 */
-	if (!area) {
-		zlog_err("%s: NULL area for circuit %u", __func__,
-			 circuit->circuit_id);
-		return;
+	if (area) {
+		area->ip_circuits += ip_router - old_ipr;
+		area->ipv6_circuits += ipv6_router - old_ipv6r;
+
+		if (ip_router || ipv6_router)
+			lsp_regenerate_schedule(area, circuit->is_type, 0);
 	}
-
-	area->ip_circuits += ip_router - old_ipr;
-	area->ipv6_circuits += ipv6_router - old_ipv6r;
-
-	if (!ip_router && !ipv6_router)
-		isis_csm_state_change(ISIS_DISABLE, circuit, area);
-	else
-		lsp_regenerate_schedule(area, circuit->is_type, 0);
 }
 
 ferr_r isis_circuit_passive_set(struct isis_circuit *circuit, bool passive)
@@ -1362,11 +1486,16 @@ ferr_r isis_circuit_metric_set(struct isis_circuit *circuit, int level,
 		return ferr_cfg_invalid("metric %d too large for narrow metric",
 					metric);
 
-	circuit->te_metric[level - 1] = metric;
-	circuit->metric[level - 1] = metric;
-
-	if (circuit->area)
-		lsp_regenerate_schedule(circuit->area, level, 0);
+	/* inform ldp-sync of metric change
+         *   if ldp-sync is running need to save metric
+         *   and restore new values after ldp-sync completion.
+	 */
+	if (isis_ldp_sync_if_metric_config(circuit, level, metric)) {
+		circuit->te_metric[level - 1] = metric;
+		circuit->metric[level - 1] = metric;
+		if (circuit->area)
+			lsp_regenerate_schedule(circuit->area, level, 0);
+	}
 	return ferr_ok();
 }
 
@@ -1410,14 +1539,6 @@ ferr_r isis_circuit_passwd_hmac_md5_set(struct isis_circuit *circuit,
 				       passwd);
 }
 
-struct cmd_node interface_node = {
-	.name = "interface",
-	.node = INTERFACE_NODE,
-	.parent_node = CONFIG_NODE,
-	.prompt = "%s(config-if)# ",
-	.config_write = isis_interface_config_write,
-};
-
 void isis_circuit_circ_type_set(struct isis_circuit *circuit, int circ_type)
 {
 	if (circuit->circ_type == circ_type)
@@ -1444,8 +1565,9 @@ int isis_circuit_mt_enabled_set(struct isis_circuit *circuit, uint16_t mtid,
 	setting = circuit_get_mt_setting(circuit, mtid);
 	if (setting->enabled != enabled) {
 		setting->enabled = enabled;
-		lsp_regenerate_schedule(circuit->area, IS_LEVEL_1 | IS_LEVEL_2,
-					0);
+		if (circuit->area)
+			lsp_regenerate_schedule(circuit->area,
+						IS_LEVEL_1 | IS_LEVEL_2, 0);
 	}
 
 	return CMD_SUCCESS;
@@ -1458,27 +1580,19 @@ int isis_if_new_hook(struct interface *ifp)
 
 int isis_if_delete_hook(struct interface *ifp)
 {
-	struct isis_circuit *circuit;
-	/* Clean up the circuit data */
-	if (ifp && ifp->info) {
-		circuit = ifp->info;
-		isis_csm_state_change(IF_DOWN_FROM_Z, circuit, ifp);
-	}
+	if (ifp->info)
+		isis_circuit_del(ifp->info);
 
 	return 0;
 }
 
 static int isis_ifp_create(struct interface *ifp)
 {
-	struct vrf *vrf = NULL;
+	struct isis_circuit *circuit = ifp->info;
+
+	if (circuit)
+		isis_circuit_enable(circuit);
 
-	if (if_is_operative(ifp)) {
-		vrf = vrf_lookup_by_id(ifp->vrf_id);
-		if (vrf)
-			isis_global_instance_create(vrf->name);
-		isis_csm_state_change(IF_UP_FROM_Z, circuit_scan_by_ifp(ifp),
-				      ifp);
-	}
 	hook_call(isis_if_new_hook, ifp);
 
 	return 0;
@@ -1486,34 +1600,33 @@ static int isis_ifp_create(struct interface *ifp)
 
 static int isis_ifp_up(struct interface *ifp)
 {
-	isis_csm_state_change(IF_UP_FROM_Z, circuit_scan_by_ifp(ifp), ifp);
+	struct isis_circuit *circuit = ifp->info;
+
+	if (circuit)
+		isis_csm_state_change(IF_UP_FROM_Z, circuit, ifp);
 
 	return 0;
 }
 
 static int isis_ifp_down(struct interface *ifp)
 {
-	struct isis_circuit *circuit;
+	struct isis_circuit *circuit = ifp->info;
+
+	if (circuit) {
+		isis_csm_state_change(IF_DOWN_FROM_Z, circuit, ifp);
 
-	circuit = isis_csm_state_change(IF_DOWN_FROM_Z,
-					circuit_scan_by_ifp(ifp), ifp);
-	if (circuit)
 		SET_FLAG(circuit->flags, ISIS_CIRCUIT_FLAPPED_AFTER_SPF);
+	}
 
 	return 0;
 }
 
 static int isis_ifp_destroy(struct interface *ifp)
 {
-	if (if_is_operative(ifp))
-		zlog_warn("Zebra: got delete of %s, but interface is still up",
-			  ifp->name);
+	struct isis_circuit *circuit = ifp->info;
 
-	isis_csm_state_change(IF_DOWN_FROM_Z, circuit_scan_by_ifp(ifp), ifp);
-
-	/* Cannot call if_delete because we should retain the pseudo interface
-	   in case there is configuration info attached to it. */
-	if_delete_retain(ifp);
+	if (circuit)
+		isis_circuit_disable(circuit);
 
 	return 0;
 }
@@ -1525,8 +1638,11 @@ void isis_circuit_init(void)
 	hook_register_prio(if_del, 0, isis_if_delete_hook);
 
 	/* Install interface node */
-	install_node(&interface_node);
-	if_cmd_init();
+#ifdef FABRICD
+	if_cmd_init(isis_interface_config_write);
+#else
+	if_cmd_init_default();
+#endif
 	if_zapi_callbacks(isis_ifp_create, isis_ifp_up,
 			  isis_ifp_down, isis_ifp_destroy);
 }
