@@ -57,11 +57,6 @@ static t_uscalar_t dlpi_ctl[1024]; /* DLPI control messages */
 static const uint8_t ALL_L1_ISS[6] = {0x01, 0x80, 0xC2, 0x00, 0x00, 0x14};
 static const uint8_t ALL_L2_ISS[6] = {0x01, 0x80, 0xC2, 0x00, 0x00, 0x15};
 static const uint8_t ALL_ISS[6] = {0x09, 0x00, 0x2B, 0x00, 0x00, 0x05};
-#if 0
-/* missing support for ES-IS on Solaris */
-static const uint8_t ALL_ESS[6] = {0x09, 0x00, 0x2B, 0x00, 0x00, 0x04};
-#endif
-
 static uint8_t sock_buff[16384];
 
 static unsigned short pf_filter[] = {
@@ -341,9 +336,8 @@ static int open_dlpi_dev(struct isis_circuit *circuit)
 
 		/* Double check the DLPI style */
 		if (dia->dl_provider_style != DL_STYLE2) {
-			zlog_warn(
-				"open_dlpi_dev(): interface %s: %s is not style 2",
-				circuit->interface->name, devpath);
+			zlog_warn("%s: interface %s: %s is not style 2",
+				  __func__, circuit->interface->name, devpath);
 			close(fd);
 			return ISIS_WARNING;
 		}
@@ -360,9 +354,8 @@ static int open_dlpi_dev(struct isis_circuit *circuit)
 	} else {
 		/* Double check the DLPI style */
 		if (dia->dl_provider_style != DL_STYLE1) {
-			zlog_warn(
-				"open_dlpi_dev(): interface %s: %s is not style 1",
-				circuit->interface->name, devpath);
+			zlog_warn("%s: interface %s: %s is not style 1",
+				  __func__, circuit->interface->name, devpath);
 			close(fd);
 			return ISIS_WARNING;
 		}
@@ -409,9 +402,8 @@ static int open_dlpi_dev(struct isis_circuit *circuit)
 	 * so we need to be careful and use DL_PHYS_ADDR_REQ instead.
 	 */
 	if (dlpiaddr(fd, circuit->u.bc.snpa) == -1) {
-		zlog_warn(
-			"open_dlpi_dev(): interface %s: unable to get MAC address",
-			circuit->interface->name);
+		zlog_warn("%s: interface %s: unable to get MAC address",
+			  __func__, circuit->interface->name);
 		close(fd);
 		return ISIS_WARNING;
 	}
@@ -484,7 +476,7 @@ int isis_sock_init(struct isis_circuit *circuit)
 			circuit->tx = isis_send_pdu_bcast;
 			circuit->rx = isis_recv_pdu_bcast;
 		} else {
-			zlog_warn("isis_sock_init(): unknown circuit type");
+			zlog_warn("%s: unknown circuit type", __func__);
 			retval = ISIS_WARNING;
 			break;
 		}
@@ -516,7 +508,7 @@ int isis_recv_pdu_bcast(struct isis_circuit *circuit, uint8_t *ssnpa)
 	retv = getmsg(circuit->fd, &ctlbuf, &databuf, &flags);
 
 	if (retv < 0) {
-		zlog_warn("isis_recv_pdu_bcast: getmsg failed: %s",
+		zlog_warn("%s: getmsg failed: %s", __func__,
 			  safe_strerror(errno));
 		return ISIS_WARNING;
 	}
@@ -566,8 +558,9 @@ int isis_send_pdu_bcast(struct isis_circuit *circuit, int level)
 	buflen = stream_get_endp(circuit->snd_stream) + LLC_LEN;
 	if ((size_t)buflen > sizeof(sock_buff)) {
 		zlog_warn(
-			"isis_send_pdu_bcast: sock_buff size %zu is less than output pdu size %d on circuit %s",
-			sizeof(sock_buff), buflen, circuit->interface->name);
+			"%s: sock_buff size %zu is less than output pdu size %d on circuit %s",
+			__func__, sizeof(sock_buff), buflen,
+			circuit->interface->name);
 		return ISIS_WARNING;
 	}
 
