@@ -49,14 +49,6 @@
 #include "isisd/isis_spf.h"
 #include "isisd/isis_errors.h"
 
-/* debug isis-spf spf-events
- 4w4d: ISIS-Spf (tlt): L2 SPF needed, new adjacency, from 0x609229F4
- 4w4d: ISIS-Spf (tlt): L2, 0000.0000.0042.01-00 TLV contents changed, code 0x2
- 4w4d: ISIS-Spf (tlt): L2, new LSP 0 DEAD.BEEF.0043.00-00
- 4w5d: ISIS-Spf (tlt): L1 SPF needed, periodic SPF, from 0x6091C844
- 4w5d: ISIS-Spf (tlt): L2 SPF needed, periodic SPF, from 0x6091C844
-*/
-
 void isis_event_circuit_state_change(struct isis_circuit *circuit,
 				     struct isis_area *area, int up)
 {
@@ -117,13 +109,13 @@ static void circuit_resign_level(struct isis_circuit *circuit, int level)
 			circuit->area->area_tag, circuit->circuit_id,
 			circuit->interface->name, level);
 
-	THREAD_TIMER_OFF(circuit->t_send_csnp[idx]);
-	THREAD_TIMER_OFF(circuit->t_send_psnp[idx]);
+	THREAD_OFF(circuit->t_send_csnp[idx]);
+	THREAD_OFF(circuit->t_send_psnp[idx]);
 
 	if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
-		THREAD_TIMER_OFF(circuit->u.bc.t_send_lan_hello[idx]);
-		THREAD_TIMER_OFF(circuit->u.bc.t_run_dr[idx]);
-		THREAD_TIMER_OFF(circuit->u.bc.t_refresh_pseudo_lsp[idx]);
+		THREAD_OFF(circuit->u.bc.t_send_lan_hello[idx]);
+		THREAD_OFF(circuit->u.bc.t_run_dr[idx]);
+		THREAD_OFF(circuit->u.bc.t_refresh_pseudo_lsp[idx]);
 		circuit->lsp_regenerate_pending[idx] = 0;
 		circuit->u.bc.run_dr_elect[idx] = 0;
 		circuit->u.bc.is_dr[idx] = 0;
@@ -136,7 +128,7 @@ static void circuit_resign_level(struct isis_circuit *circuit, int level)
 
 void isis_circuit_is_type_set(struct isis_circuit *circuit, int newtype)
 {
-	if (circuit->state != C_STATE_UP) {
+	if (!circuit->area) {
 		circuit->is_type = newtype;
 		return;
 	}
@@ -159,6 +151,11 @@ void isis_circuit_is_type_set(struct isis_circuit *circuit, int newtype)
 		return;
 	}
 
+	if (circuit->state != C_STATE_UP) {
+		circuit->is_type = newtype;
+		return;
+	}
+
 	if (!circuit->is_passive) {
 		switch (circuit->is_type) {
 		case IS_LEVEL_1:
@@ -212,7 +209,7 @@ void isis_circuit_is_type_set(struct isis_circuit *circuit, int newtype)
 
 /* events supporting code */
 
-int isis_event_dis_status_change(struct thread *thread)
+void isis_event_dis_status_change(struct thread *thread)
 {
 	struct isis_circuit *circuit;
 
@@ -220,15 +217,13 @@ int isis_event_dis_status_change(struct thread *thread)
 
 	/* invalid arguments */
 	if (!circuit || !circuit->area)
-		return 0;
+		return;
 	if (IS_DEBUG_EVENTS)
 		zlog_debug("ISIS-Evt (%s) DIS status change",
 			   circuit->area->area_tag);
 
 	/* LSP generation again */
 	lsp_regenerate_schedule(circuit->area, IS_LEVEL_1 | IS_LEVEL_2, 0);
-
-	return 0;
 }
 
 void isis_event_auth_failure(char *area_tag, const char *error_string,
