@@ -23,6 +23,7 @@
 #include "ldpe.h"
 #include "lde.h"
 #include "log.h"
+#include "rlfa.h"
 
 #include "mpls.h"
 
@@ -601,6 +602,10 @@ lde_check_mapping(struct map *map, struct lde_nbr *ln, int rcvd_label_mapping)
 			break;
 		}
 	}
+
+	/* Update RLFA clients. */
+	lde_rlfa_update_clients(&fec, ln, map->label);
+
 	/* LMp.13 & LMp.16: Record the mapping from this peer */
 	if (me == NULL)
 		me = lde_map_add(ln, fn, 0);
@@ -858,6 +863,9 @@ lde_check_withdraw(struct map *map, struct lde_nbr *ln)
 		fnh->remote_label = NO_LABEL;
 	}
 
+	/* Update RLFA clients. */
+	lde_rlfa_update_clients(&fec, ln, MPLS_INVALID_LABEL);
+
 	/* LWd.2: send label release */
 	lde_send_labelrelease(ln, fn, NULL, map->label);
 
@@ -940,6 +948,9 @@ lde_check_withdraw_wcard(struct map *map, struct lde_nbr *ln)
 			fnh->remote_label = NO_LABEL;
 		}
 
+		/* Update RLFA clients. */
+		lde_rlfa_update_clients(f, ln, MPLS_INVALID_LABEL);
+
 		/* LWd.3: check previously received label mapping */
 		if (me && (map->label == NO_LABEL ||
 		    map->label == me->map.label))
@@ -1026,8 +1037,7 @@ lde_wildcard_apply(struct map *wcard, struct fec *fec, struct lde_map *me)
 /* gabage collector timer: timer to remove dead entries from the LIB */
 
 /* ARGSUSED */
-int
-lde_gc_timer(struct thread *thread)
+void lde_gc_timer(struct thread *thread)
 {
 	struct fec	*fec, *safe;
 	struct fec_node	*fn;
@@ -1053,15 +1063,12 @@ lde_gc_timer(struct thread *thread)
 		log_debug("%s: %u entries removed", __func__, count);
 
 	lde_gc_start_timer();
-
-	return (0);
 }
 
 void
 lde_gc_start_timer(void)
 {
-	THREAD_TIMER_OFF(gc_timer);
-	gc_timer = NULL;
+	THREAD_OFF(gc_timer);
 	thread_add_timer(master, lde_gc_timer, NULL, LDE_GC_INTERVAL,
 			 &gc_timer);
 }
@@ -1069,5 +1076,5 @@ lde_gc_start_timer(void)
 void
 lde_gc_stop_timer(void)
 {
-	THREAD_TIMER_OFF(gc_timer);
+	THREAD_OFF(gc_timer);
 }
