@@ -28,12 +28,12 @@
 
 static struct iface		*disc_find_iface(unsigned int, int,
 				    union ldpd_addr *);
-static int			 session_read(struct thread *);
-static int			 session_write(struct thread *);
+static void session_read(struct thread *thread);
+static void session_write(struct thread *thread);
 static ssize_t			 session_get_pdu(struct ibuf_read *, char **);
 static void			 tcp_close(struct tcp_conn *);
 static struct pending_conn	*pending_conn_new(int, int, union ldpd_addr *);
-static int			 pending_conn_timeout(struct thread *);
+static void pending_conn_timeout(struct thread *thread);
 
 int
 gen_ldp_hdr(struct ibuf *buf, uint16_t size)
@@ -106,8 +106,7 @@ send_packet(int fd, int af, union ldpd_addr *dst, struct iface_af *ia,
 }
 
 /* Discovery functions */
-int
-disc_recv_packet(struct thread *thread)
+void disc_recv_packet(struct thread *thread)
 {
 	int			 fd = THREAD_FD(thread);
 	struct thread		**threadp = THREAD_ARG(thread);
@@ -141,7 +140,6 @@ disc_recv_packet(struct thread *thread)
 	struct in_addr		 lsr_id;
 
 	/* reschedule read */
-	*threadp = NULL;
 	thread_add_read(master, disc_recv_packet, threadp, fd, threadp);
 
 	/* setup buffer */
@@ -159,7 +157,7 @@ disc_recv_packet(struct thread *thread)
 		if (errno != EAGAIN && errno != EINTR)
 			log_debug("%s: read error: %s", __func__,
 			    strerror(errno));
-		return (0);
+		return;
 	}
 
 	sa2addr((struct sockaddr *)&from, &af, &src, NULL);
@@ -206,7 +204,7 @@ disc_recv_packet(struct thread *thread)
 	if (bad_addr(af, &src)) {
 		log_debug("%s: invalid source address: %s", __func__,
 		    log_addr(af, &src));
-		return (0);
+		return;
 	}
 	ifindex = getsockopt_ifindex(af, &m);
 
@@ -214,7 +212,7 @@ disc_recv_packet(struct thread *thread)
 	if (multicast) {
 		iface = disc_find_iface(ifindex, af, &src);
 		if (iface == NULL)
-			return (0);
+			return;
 	}
 
 	/* check packet size */
@@ -222,7 +220,7 @@ disc_recv_packet(struct thread *thread)
 	if (len < (LDP_HDR_SIZE + LDP_MSG_SIZE) || len > LDP_MAX_LEN) {
 		log_debug("%s: bad packet size, source %s", __func__,
 		    log_addr(af, &src));
-		return (0);
+		return;
 	}
 
 	/* LDP header sanity checks */
@@ -230,12 +228,12 @@ disc_recv_packet(struct thread *thread)
 	if (ntohs(ldp_hdr.version) != LDP_VERSION) {
 		log_debug("%s: invalid LDP version %d, source %s", __func__,
 		    ntohs(ldp_hdr.version), log_addr(af, &src));
-		return (0);
+		return;
 	}
 	if (ntohs(ldp_hdr.lspace_id) != 0) {
 		log_debug("%s: invalid label space %u, source %s", __func__,
 		    ntohs(ldp_hdr.lspace_id), log_addr(af, &src));
-		return (0);
+		return;
 	}
 	/* check "PDU Length" field */
 	pdu_len = ntohs(ldp_hdr.length);
@@ -243,7 +241,7 @@ disc_recv_packet(struct thread *thread)
 	    (pdu_len > (len - LDP_HDR_DEAD_LEN))) {
 		log_debug("%s: invalid LDP packet length %u, source %s",
 		    __func__, ntohs(ldp_hdr.length), log_addr(af, &src));
-		return (0);
+		return;
 	}
 	buf += LDP_HDR_SIZE;
 	len -= LDP_HDR_SIZE;
@@ -262,7 +260,7 @@ disc_recv_packet(struct thread *thread)
 	if (msg_len < LDP_MSG_LEN || ((msg_len + LDP_MSG_DEAD_LEN) > pdu_len)) {
 		log_debug("%s: invalid LDP message length %u, source %s",
 		    __func__, ntohs(msg.length), log_addr(af, &src));
-		return (0);
+		return;
 	}
 	buf += LDP_MSG_SIZE;
 	len -= LDP_MSG_SIZE;
@@ -276,8 +274,6 @@ disc_recv_packet(struct thread *thread)
 		log_debug("%s: unknown LDP packet type, source %s", __func__,
 		    log_addr(af, &src));
 	}
-
-	return (0);
 }
 
 static struct iface *
@@ -305,8 +301,7 @@ disc_find_iface(unsigned int ifindex, int af, union ldpd_addr *src)
 	return (iface);
 }
 
-int
-session_accept(struct thread *thread)
+void session_accept(struct thread *thread)
 {
 	int			 fd = THREAD_FD(thread);
 	struct sockaddr_storage	 src;
@@ -329,7 +324,7 @@ session_accept(struct thread *thread)
 		    errno != ECONNABORTED)
 			log_debug("%s: accept error: %s", __func__,
 			    strerror(errno));
-		return (0);
+		return;
 	}
 	sock_set_nonblock(newfd);
 
@@ -358,22 +353,20 @@ session_accept(struct thread *thread)
 			close(newfd);
 		else
 			pending_conn_new(newfd, af, &addr);
-		return (0);
+		return;
 	}
 	/* protection against buggy implementations */
 	if (nbr_session_active_role(nbr)) {
 		close(newfd);
-		return (0);
+		return;
 	}
 	if (nbr->state != NBR_STA_PRESENT) {
-		log_debug("%s: lsr-id %s: rejecting additional transport connection", __func__, inet_ntoa(nbr->id));
+		log_debug("%s: lsr-id %pI4: rejecting additional transport connection", __func__, &nbr->id);
 		close(newfd);
-		return (0);
+		return;
 	}
 
 	session_accept_nbr(nbr, newfd);
-
-	return (0);
 }
 
 void
@@ -412,8 +405,7 @@ session_accept_nbr(struct nbr *nbr, int fd)
 	nbr_fsm(nbr, NBR_EVT_MATCH_ADJ);
 }
 
-static int
-session_read(struct thread *thread)
+static void session_read(struct thread *thread)
 {
 	int		 fd = THREAD_FD(thread);
 	struct nbr	*nbr = THREAD_ARG(thread);
@@ -425,7 +417,6 @@ session_read(struct thread *thread)
 	uint16_t	 pdu_len, msg_len, msg_size, max_pdu_len;
 	int		 ret;
 
-	tcp->rev = NULL;
 	thread_add_read(master, session_read, nbr, fd, &tcp->rev);
 
 	if ((n = read(fd, tcp->rbuf->buf + tcp->rbuf->wpos,
@@ -433,16 +424,16 @@ session_read(struct thread *thread)
 		if (errno != EINTR && errno != EAGAIN) {
 			log_warn("%s: read error", __func__);
 			nbr_fsm(nbr, NBR_EVT_CLOSE_SESSION);
-			return (0);
+			return;
 		}
 		/* retry read */
-		return (0);
+		return;
 	}
 	if (n == 0) {
 		/* connection closed */
 		log_debug("%s: connection closed by remote end", __func__);
 		nbr_fsm(nbr, NBR_EVT_CLOSE_SESSION);
-		return (0);
+		return;
 	}
 	tcp->rbuf->wpos += n;
 
@@ -452,7 +443,7 @@ session_read(struct thread *thread)
 		if (ntohs(ldp_hdr->version) != LDP_VERSION) {
 			session_shutdown(nbr, S_BAD_PROTO_VER, 0, 0);
 			free(buf);
-			return (0);
+			return;
 		}
 
 		pdu_len = ntohs(ldp_hdr->length);
@@ -469,14 +460,14 @@ session_read(struct thread *thread)
 		    pdu_len > max_pdu_len) {
 			session_shutdown(nbr, S_BAD_PDU_LEN, 0, 0);
 			free(buf);
-			return (0);
+			return;
 		}
 		pdu_len -= LDP_HDR_PDU_LEN;
 		if (ldp_hdr->lsr_id != nbr->id.s_addr ||
 		    ldp_hdr->lspace_id != 0) {
 			session_shutdown(nbr, S_BAD_LDP_ID, 0, 0);
 			free(buf);
-			return (0);
+			return;
 		}
 		pdu += LDP_HDR_SIZE;
 		len -= LDP_HDR_SIZE;
@@ -494,7 +485,7 @@ session_read(struct thread *thread)
 				session_shutdown(nbr, S_BAD_MSG_LEN, msg->id,
 				    msg->type);
 				free(buf);
-				return (0);
+				return;
 			}
 			msg_size = msg_len + LDP_MSG_DEAD_LEN;
 			pdu_len -= msg_size;
@@ -507,7 +498,7 @@ session_read(struct thread *thread)
 					session_shutdown(nbr, S_SHUTDOWN,
 					    msg->id, msg->type);
 					free(buf);
-					return (0);
+					return;
 				}
 				break;
 			case MSG_TYPE_KEEPALIVE:
@@ -516,7 +507,7 @@ session_read(struct thread *thread)
 					session_shutdown(nbr, S_SHUTDOWN,
 					    msg->id, msg->type);
 					free(buf);
-					return (0);
+					return;
 				}
 				break;
 			case MSG_TYPE_NOTIFICATION:
@@ -526,7 +517,7 @@ session_read(struct thread *thread)
 					session_shutdown(nbr, S_SHUTDOWN,
 					    msg->id, msg->type);
 					free(buf);
-					return (0);
+					return;
 				}
 				break;
 			}
@@ -558,11 +549,13 @@ session_read(struct thread *thread)
 				    type);
 				break;
 			default:
-				log_debug("%s: unknown LDP message from nbr %s",
-				    __func__, inet_ntoa(nbr->id));
-				if (!(ntohs(msg->type) & UNKNOWN_FLAG))
+				log_debug("%s: unknown LDP message from nbr %pI4",
+				    __func__, &nbr->id);
+				if (!(ntohs(msg->type) & UNKNOWN_FLAG)) {
+					nbr->stats.unknown_msg++;
 					send_notification(nbr->tcp,
 					    S_UNKNOWN_MSG, msg->id, msg->type);
+				}
 				/* ignore the message */
 				ret = 0;
 				break;
@@ -571,7 +564,7 @@ session_read(struct thread *thread)
 			if (ret == -1) {
 				/* parser failed, giving up */
 				free(buf);
-				return (0);
+				return;
 			}
 
 			/* no errors - update per neighbor message counters */
@@ -618,7 +611,7 @@ session_read(struct thread *thread)
 		buf = NULL;
 		if (len != 0) {
 			session_shutdown(nbr, S_BAD_PDU_LEN, 0, 0);
-			return (0);
+			return;
 		}
 	}
 
@@ -626,11 +619,9 @@ session_read(struct thread *thread)
 	 * allocated - but let's get rid of the SA warning.
 	 */
 	free(buf);
-	return (0);
 }
 
-static int
-session_write(struct thread *thread)
+static void session_write(struct thread *thread)
 {
 	struct tcp_conn *tcp = THREAD_ARG(thread);
 	struct nbr	*nbr = tcp->nbr;
@@ -647,12 +638,10 @@ session_write(struct thread *thread)
 		 * close the socket.
 		 */
 		tcp_close(tcp);
-		return (0);
+		return;
 	}
 
 	evbuf_event_add(&tcp->wbuf);
-
-	return (0);
 }
 
 void
@@ -662,11 +651,17 @@ session_shutdown(struct nbr *nbr, uint32_t status, uint32_t msg_id,
 	switch (nbr->state) {
 	case NBR_STA_PRESENT:
 		if (nbr_pending_connect(nbr))
-			THREAD_WRITE_OFF(nbr->ev_connect);
+			THREAD_OFF(nbr->ev_connect);
 		break;
 	case NBR_STA_INITIAL:
 	case NBR_STA_OPENREC:
 	case NBR_STA_OPENSENT:
+		/* update SNMP session counters during initialization */
+		leconf->stats.session_attempts++;
+		send_notification(nbr->tcp, status, msg_id, msg_type);
+
+		nbr_fsm(nbr, NBR_EVT_CLOSE_SESSION);
+		break;
 	case NBR_STA_OPER:
 		send_notification(nbr->tcp, status, msg_id, msg_type);
 
@@ -680,8 +675,10 @@ session_shutdown(struct nbr *nbr, uint32_t status, uint32_t msg_id,
 void
 session_close(struct nbr *nbr)
 {
-	log_debug("%s: closing session with lsr-id %s", __func__,
-	    inet_ntoa(nbr->id));
+	log_debug("%s: closing session with lsr-id %pI4", __func__,
+	    &nbr->id);
+
+	ldp_sync_fsm_nbr_event(nbr, LDP_SYNC_EVT_SESSION_CLOSE);
 
 	tcp_close(nbr->tcp);
 	nbr_stop_ktimer(nbr);
@@ -735,7 +732,6 @@ tcp_new(int fd, struct nbr *nbr)
 		if ((tcp->rbuf = calloc(1, sizeof(struct ibuf_read))) == NULL)
 			fatal(__func__);
 
-		tcp->rev = NULL;
 		thread_add_read(master, session_read, nbr, tcp->fd, &tcp->rev);
 		tcp->nbr = nbr;
 	}
@@ -760,7 +756,7 @@ tcp_close(struct tcp_conn *tcp)
 	evbuf_clear(&tcp->wbuf);
 
 	if (tcp->nbr) {
-		THREAD_READ_OFF(tcp->rev);
+		THREAD_OFF(tcp->rev);
 		free(tcp->rbuf);
 		tcp->nbr->tcp = NULL;
 	}
@@ -792,7 +788,7 @@ pending_conn_new(int fd, int af, union ldpd_addr *addr)
 void
 pending_conn_del(struct pending_conn *pconn)
 {
-	THREAD_TIMER_OFF(pconn->ev_timeout);
+	THREAD_OFF(pconn->ev_timeout);
 	TAILQ_REMOVE(&global.pending_conns, pconn, entry);
 	free(pconn);
 }
@@ -810,8 +806,7 @@ pending_conn_find(int af, union ldpd_addr *addr)
 	return (NULL);
 }
 
-static int
-pending_conn_timeout(struct thread *thread)
+static void pending_conn_timeout(struct thread *thread)
 {
 	struct pending_conn	*pconn = THREAD_ARG(thread);
 	struct tcp_conn		*tcp;
@@ -830,6 +825,4 @@ pending_conn_timeout(struct thread *thread)
 	msgbuf_write(&tcp->wbuf.wbuf);
 
 	pending_conn_del(pconn);
-
-	return (0);
 }
