@@ -36,10 +36,7 @@
 #endif /* VTYSH_EXTRACT_PL */
 
 #define ACCESS_LIST_STR "Access list entry\n"
-#define ACCESS_LIST_LEG_STR "IP standard access list\n"
-#define ACCESS_LIST_ELEG_STR "IP extended access list\n"
-#define ACCESS_LIST_ELEG_EXT_STR "IP extended access list (expanded range)\n"
-#define ACCESS_LIST_ZEBRA_STR "Access list entry\n"
+#define ACCESS_LIST_ZEBRA_STR "Access list name\n"
 #define ACCESS_LIST_SEQ_STR                                                    \
 	"Sequence number of an entry\n"                                        \
 	"Sequence number\n"
@@ -51,69 +48,6 @@
 
 #define PREFIX_LIST_NAME_STR "Prefix list entry name\n"
 
-/*
- * Helper function to locate filter data structures for Cisco-style ACLs.
- */
-static int64_t acl_cisco_get_seq(struct access_list *acl, const char *action,
-				 const char *src, const char *src_mask,
-				 const char *dst, const char *dst_mask)
-{
-	struct filter_cisco *fc;
-	struct filter f, *fn;
-
-	memset(&f, 0, sizeof(f));
-	f.cisco = 1;
-	if (strcmp(action, "permit") == 0)
-		f.type = FILTER_PERMIT;
-	else
-		f.type = FILTER_DENY;
-
-	fc = &f.u.cfilter;
-	inet_pton(AF_INET, src, &fc->addr);
-	inet_pton(AF_INET, src_mask, &fc->addr_mask);
-	fc->addr.s_addr &= ~fc->addr_mask.s_addr;
-	if (dst != NULL) {
-		fc->extended = 1;
-		inet_pton(AF_INET, dst, &fc->mask);
-		inet_pton(AF_INET, dst_mask, &fc->mask_mask);
-		fc->mask.s_addr &= ~fc->mask_mask.s_addr;
-	}
-
-	fn = filter_lookup_cisco(acl, &f);
-	if (fn == NULL)
-		return -1;
-
-	return fn->seq;
-}
-
-/*
- * Helper function to locate filter data structures for zebra-style ACLs.
- */
-static int64_t acl_zebra_get_seq(struct access_list *acl, const char *action,
-				 const struct prefix *p, bool exact)
-{
-	struct filter_zebra *fz;
-	struct filter f, *fn;
-
-	memset(&f, 0, sizeof(f));
-	memset(&fz, 0, sizeof(fz));
-	if (strcmp(action, "permit") == 0)
-		f.type = FILTER_PERMIT;
-	else
-		f.type = FILTER_DENY;
-
-	fz = &f.u.zfilter;
-	if (p->family)
-		prefix_copy(&fz->prefix, p);
-	fz->exact = exact;
-
-	fn = filter_lookup_zebra(acl, &f);
-	if (fn == NULL)
-		return -1;
-
-	return fn->seq;
-}
-
 /*
  * Helper function to generate a sequence number for legacy commands.
  */
@@ -146,6 +80,53 @@ static long acl_get_seq(struct vty *vty, const char *xpath)
 	return seq + 5;
 }
 
+static int acl_remove_if_empty(struct vty *vty, const char *iptype,
+			       const char *name)
+{
+	char xpath[XPATH_MAXLEN];
+
+	snprintf(xpath, sizeof(xpath),
+		 "/frr-filter:lib/access-list[type='%s'][name='%s']/remark",
+		 iptype, name);
+	/* List is not empty if there is a remark, check that: */
+	if (yang_dnode_exists(vty->candidate_config->dnode, xpath))
+		return CMD_SUCCESS;
+
+	/* Check if we have any entries: */
+	snprintf(xpath, sizeof(xpath),
+		 "/frr-filter:lib/access-list[type='%s'][name='%s']", iptype,
+		 name);
+	/*
+	 * NOTE: if the list is empty it will return the first sequence
+	 * number: 5.
+	 */
+	if (acl_get_seq(vty, xpath) != 5)
+		return CMD_SUCCESS;
+
+	/* Nobody is using this list, lets remove it. */
+	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
+	return nb_cli_apply_changes(vty, NULL);
+}
+
+static int acl_remove(struct vty *vty, const char *iptype, const char *name,
+		      int64_t sseq)
+{
+	char xpath[XPATH_MAXLEN];
+	int rv;
+
+	snprintfrr(
+		xpath, sizeof(xpath),
+		"/frr-filter:lib/access-list[type='%s'][name='%s']/entry[sequence='%" PRId64 "']",
+		iptype, name, sseq);
+	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
+
+	rv = nb_cli_apply_changes(vty, NULL);
+	if (rv == CMD_SUCCESS)
+		return acl_remove_if_empty(vty, iptype, name);
+
+	return rv;
+}
+
 /*
  * Cisco (legacy) access lists.
  */
@@ -153,7 +134,7 @@ DEFPY_YANG(
 	access_list_std, access_list_std_cmd,
 	"access-list WORD$name [seq (1-4294967295)$seq] <deny|permit>$action <[host] A.B.C.D$host|A.B.C.D$host A.B.C.D$mask>",
 	ACCESS_LIST_STR
-	ACCESS_LIST_LEG_STR
+	ACCESS_LIST_ZEBRA_STR
 	ACCESS_LIST_SEQ_STR
 	ACCESS_LIST_ACTION_STR
 	"A single host address\n"
@@ -170,27 +151,25 @@ DEFPY_YANG(
 	 * Backward compatibility: don't complain about duplicated values,
 	 * just silently accept.
 	 */
-	if (seq_str == NULL) {
-		ada.ada_type = "ipv4";
-		ada.ada_name = name;
-		if (host_str && mask_str == NULL) {
-			ada.ada_xpath[0] = "./host";
-			ada.ada_value[0] = host_str;
-		} else if (host_str && mask_str) {
-			ada.ada_xpath[0] = "./network/address";
-			ada.ada_value[0] = host_str;
-			ada.ada_xpath[1] = "./network/mask";
-			ada.ada_value[1] = mask_str;
-		} else {
-			ada.ada_xpath[0] = "./source-any";
-			ada.ada_value[0] = "true";
-		}
-
-		/* Duplicated entry without sequence, just quit. */
-		if (acl_is_dup(vty->candidate_config->dnode, &ada))
-			return CMD_SUCCESS;
+	ada.ada_type = "ipv4";
+	ada.ada_name = name;
+	ada.ada_action = action;
+	if (host_str && mask_str == NULL) {
+		ada.ada_xpath[0] = "./host";
+		ada.ada_value[0] = host_str;
+	} else if (host_str && mask_str) {
+		ada.ada_xpath[0] = "./network/address";
+		ada.ada_value[0] = host_str;
+		ada.ada_xpath[1] = "./network/mask";
+		ada.ada_value[1] = mask_str;
+	} else {
+		ada.ada_xpath[0] = "./source-any";
+		ada.ada_value[0] = "";
 	}
 
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		return CMD_SUCCESS;
+
 	/*
 	 * Create the access-list first, so we can generate sequence if
 	 * none given (backward compatibility).
@@ -229,7 +208,7 @@ DEFPY_YANG(
 	"no access-list WORD$name [seq (1-4294967295)$seq] <deny|permit>$action <[host] A.B.C.D$host|A.B.C.D$host A.B.C.D$mask>",
 	NO_STR
 	ACCESS_LIST_STR
-	ACCESS_LIST_LEG_STR
+	ACCESS_LIST_ZEBRA_STR
 	ACCESS_LIST_SEQ_STR
 	ACCESS_LIST_ACTION_STR
 	"A single host address\n"
@@ -237,51 +216,43 @@ DEFPY_YANG(
 	"Address to match\n"
 	"Wildcard bits\n")
 {
-	struct access_list *acl;
-	struct lyd_node *dnode;
 	int64_t sseq;
-	char xpath[XPATH_MAXLEN];
-	char xpath_entry[XPATH_MAXLEN + 32];
+	struct acl_dup_args ada = {};
 
 	/* If the user provided sequence number, then just go for it. */
-	if (seq_str != NULL) {
-		snprintf(
-			xpath, sizeof(xpath),
-			"/frr-filter:lib/access-list[type='ipv4'][name='%s']/entry[sequence='%s']",
-			name, seq_str);
-		nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
-		return nb_cli_apply_changes(vty, NULL);
-	}
+	if (seq_str != NULL)
+		return acl_remove(vty, "ipv4", name, seq);
 
 	/* Otherwise, to keep compatibility, we need to figure it out. */
-	snprintf(xpath, sizeof(xpath),
-		 "/frr-filter:lib/access-list[type='ipv4'][name='%s']", name);
+	ada.ada_type = "ipv4";
+	ada.ada_name = name;
+	ada.ada_action = action;
+	if (host_str && mask_str == NULL) {
+		ada.ada_xpath[0] = "./host";
+		ada.ada_value[0] = host_str;
+	} else if (host_str && mask_str) {
+		ada.ada_xpath[0] = "./network/address";
+		ada.ada_value[0] = host_str;
+		ada.ada_xpath[1] = "./network/mask";
+		ada.ada_value[1] = mask_str;
+	} else {
+		ada.ada_xpath[0] = "./source-any";
+		ada.ada_value[0] = "";
+	}
 
-	/* Access-list must exist before entries. */
-	if (yang_dnode_exists(running_config->dnode, xpath) == false)
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		sseq = ada.ada_seq;
+	else
 		return CMD_WARNING_CONFIG_FAILED;
 
-	/* Use access-list data structure to fetch sequence. */
-	dnode = yang_dnode_get(running_config->dnode, xpath);
-	acl = nb_running_get_entry(dnode, NULL, true);
-	sseq = acl_cisco_get_seq(acl, action, host_str,
-				 mask_str ? mask_str : CISCO_HOST_WILDCARD_MASK,
-				 NULL, NULL);
-	if (sseq == -1)
-		return CMD_WARNING_CONFIG_FAILED;
-
-	snprintfrr(xpath_entry, sizeof(xpath_entry),
-		   "%s/entry[sequence='%" PRId64 "']", xpath, sseq);
-	nb_cli_enqueue_change(vty, xpath_entry, NB_OP_DESTROY, NULL);
-
-	return nb_cli_apply_changes(vty, NULL);
+	return acl_remove(vty, "ipv4", name, sseq);
 }
 
 DEFPY_YANG(
 	access_list_ext, access_list_ext_cmd,
 	"access-list WORD$name [seq (1-4294967295)$seq] <deny|permit>$action ip <A.B.C.D$src A.B.C.D$src_mask|host A.B.C.D$src|any> <A.B.C.D$dst A.B.C.D$dst_mask|host A.B.C.D$dst|any>",
 	ACCESS_LIST_STR
-	ACCESS_LIST_ELEG_STR
+	ACCESS_LIST_ZEBRA_STR
 	ACCESS_LIST_SEQ_STR
 	ACCESS_LIST_ACTION_STR
 	"IPv4 address\n"
@@ -306,48 +277,46 @@ DEFPY_YANG(
 	 * Backward compatibility: don't complain about duplicated values,
 	 * just silently accept.
 	 */
-	if (seq_str == NULL) {
-		ada.ada_type = "ipv4";
-		ada.ada_name = name;
-		if (src_str && src_mask_str == NULL) {
-			ada.ada_xpath[idx] = "./host";
-			ada.ada_value[idx] = src_str;
-			idx++;
-		} else if (src_str && src_mask_str) {
-			ada.ada_xpath[idx] = "./network/address";
-			ada.ada_value[idx] = src_str;
-			idx++;
-			ada.ada_xpath[idx] = "./network/mask";
-			ada.ada_value[idx] = src_mask_str;
-			idx++;
-		} else {
-			ada.ada_xpath[idx] = "./source-any";
-			ada.ada_value[idx] = "true";
-			idx++;
-		}
-
-		if (dst_str && dst_mask_str == NULL) {
-			ada.ada_xpath[idx] = "./destination-host";
-			ada.ada_value[idx] = dst_str;
-			idx++;
-		} else if (dst_str && dst_mask_str) {
-			ada.ada_xpath[idx] = "./destination-network/address";
-			ada.ada_value[idx] = dst_str;
-			idx++;
-			ada.ada_xpath[idx] = "./destination-network/mask";
-			ada.ada_value[idx] = dst_mask_str;
-			idx++;
-		} else {
-			ada.ada_xpath[idx] = "./destination-any";
-			ada.ada_value[idx] = "true";
-			idx++;
-		}
-
-		/* Duplicated entry without sequence, just quit. */
-		if (acl_is_dup(vty->candidate_config->dnode, &ada))
-			return CMD_SUCCESS;
+	ada.ada_type = "ipv4";
+	ada.ada_name = name;
+	ada.ada_action = action;
+	if (src_str && src_mask_str == NULL) {
+		ada.ada_xpath[idx] = "./host";
+		ada.ada_value[idx] = src_str;
+		idx++;
+	} else if (src_str && src_mask_str) {
+		ada.ada_xpath[idx] = "./network/address";
+		ada.ada_value[idx] = src_str;
+		idx++;
+		ada.ada_xpath[idx] = "./network/mask";
+		ada.ada_value[idx] = src_mask_str;
+		idx++;
+	} else {
+		ada.ada_xpath[idx] = "./source-any";
+		ada.ada_value[idx] = "";
+		idx++;
 	}
 
+	if (dst_str && dst_mask_str == NULL) {
+		ada.ada_xpath[idx] = "./destination-host";
+		ada.ada_value[idx] = dst_str;
+		idx++;
+	} else if (dst_str && dst_mask_str) {
+		ada.ada_xpath[idx] = "./destination-network/address";
+		ada.ada_value[idx] = dst_str;
+		idx++;
+		ada.ada_xpath[idx] = "./destination-network/mask";
+		ada.ada_value[idx] = dst_mask_str;
+		idx++;
+	} else {
+		ada.ada_xpath[idx] = "./destination-any";
+		ada.ada_value[idx] = "";
+		idx++;
+	}
+
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		return CMD_SUCCESS;
+
 	/*
 	 * Create the access-list first, so we can generate sequence if
 	 * none given (backward compatibility).
@@ -399,7 +368,7 @@ DEFPY_YANG(
 	"no access-list WORD$name [seq (1-4294967295)$seq] <deny|permit>$action ip <A.B.C.D$src A.B.C.D$src_mask|host A.B.C.D$src|any> <A.B.C.D$dst A.B.C.D$dst_mask|host A.B.C.D$dst|any>",
 	NO_STR
 	ACCESS_LIST_STR
-	ACCESS_LIST_ELEG_STR
+	ACCESS_LIST_ZEBRA_STR
 	ACCESS_LIST_SEQ_STR
 	ACCESS_LIST_ACTION_STR
 	"Any Internet Protocol\n"
@@ -414,68 +383,58 @@ DEFPY_YANG(
 	"Destination address to match\n"
 	"Any destination host\n")
 {
-	struct access_list *acl;
-	struct lyd_node *dnode;
+	int idx = 0;
 	int64_t sseq;
-	char xpath[XPATH_MAXLEN];
-	char xpath_entry[XPATH_MAXLEN + 32];
+	struct acl_dup_args ada = {};
 
 	/* If the user provided sequence number, then just go for it. */
-	if (seq_str != NULL) {
-		snprintfrr(
-			xpath, sizeof(xpath),
-			"/frr-filter:lib/access-list[type='ipv4'][name='%s']/entry[sequence='%s']",
-			name, seq_str);
-		nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
-		return nb_cli_apply_changes(vty, NULL);
-	}
+	if (seq_str != NULL)
+		return acl_remove(vty, "ipv4", name, seq);
 
 	/* Otherwise, to keep compatibility, we need to figure it out. */
-	snprintf(xpath, sizeof(xpath),
-		 "/frr-filter:lib/access-list[type='ipv4'][name='%s']", name);
-
-	/* Access-list must exist before entries. */
-	if (yang_dnode_exists(running_config->dnode, xpath) == false)
-		return CMD_WARNING_CONFIG_FAILED;
-
-	/* Use access-list data structure to fetch sequence. */
-	dnode = yang_dnode_get(running_config->dnode, xpath);
-	acl = nb_running_get_entry(dnode, NULL, true);
-	if (src_str != NULL) {
-		if (dst_str != NULL)
-			sseq = acl_cisco_get_seq(
-				acl, action, src_str,
-				src_mask_str ? src_mask_str
-					     : CISCO_HOST_WILDCARD_MASK,
-				dst_str,
-				dst_mask_str ? dst_mask_str
-					     : CISCO_HOST_WILDCARD_MASK);
-		else
-			sseq = acl_cisco_get_seq(
-				acl, action, src_str,
-				src_mask_str ? src_mask_str
-					     : CISCO_HOST_WILDCARD_MASK,
-				"0.0.0.0", CISCO_ANY_WILDCARD_MASK);
+	ada.ada_type = "ipv4";
+	ada.ada_name = name;
+	ada.ada_action = action;
+	if (src_str && src_mask_str == NULL) {
+		ada.ada_xpath[idx] = "./host";
+		ada.ada_value[idx] = src_str;
+		idx++;
+	} else if (src_str && src_mask_str) {
+		ada.ada_xpath[idx] = "./network/address";
+		ada.ada_value[idx] = src_str;
+		idx++;
+		ada.ada_xpath[idx] = "./network/mask";
+		ada.ada_value[idx] = src_mask_str;
+		idx++;
 	} else {
-		if (dst_str != NULL)
-			sseq = acl_cisco_get_seq(
-				acl, action, "0.0.0.0", CISCO_ANY_WILDCARD_MASK,
-				dst_str,
-				dst_mask_str ? dst_mask_str
-					     : CISCO_HOST_WILDCARD_MASK);
-		else
-			sseq = acl_cisco_get_seq(
-				acl, action, "0.0.0.0", CISCO_ANY_WILDCARD_MASK,
-				"0.0.0.0", CISCO_ANY_WILDCARD_MASK);
+		ada.ada_xpath[idx] = "./source-any";
+		ada.ada_value[idx] = "";
+		idx++;
 	}
-	if (sseq == -1)
+
+	if (dst_str && dst_mask_str == NULL) {
+		ada.ada_xpath[idx] = "./destination-host";
+		ada.ada_value[idx] = dst_str;
+		idx++;
+	} else if (dst_str && dst_mask_str) {
+		ada.ada_xpath[idx] = "./destination-network/address";
+		ada.ada_value[idx] = dst_str;
+		idx++;
+		ada.ada_xpath[idx] = "./destination-network/mask";
+		ada.ada_value[idx] = dst_mask_str;
+		idx++;
+	} else {
+		ada.ada_xpath[idx] = "./destination-any";
+		ada.ada_value[idx] = "";
+		idx++;
+	}
+
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		sseq = ada.ada_seq;
+	else
 		return CMD_WARNING_CONFIG_FAILED;
 
-	snprintfrr(xpath_entry, sizeof(xpath_entry),
-		   "%s/entry[sequence='%" PRId64 "']", xpath, sseq);
-	nb_cli_enqueue_change(vty, xpath_entry, NB_OP_DESTROY, NULL);
-
-	return nb_cli_apply_changes(vty, NULL);
+	return acl_remove(vty, "ipv4", name, sseq);
 }
 
 /*
@@ -501,27 +460,25 @@ DEFPY_YANG(
 	 * Backward compatibility: don't complain about duplicated values,
 	 * just silently accept.
 	 */
-	if (seq_str == NULL) {
-		ada.ada_type = "ipv4";
-		ada.ada_name = name;
+	ada.ada_type = "ipv4";
+	ada.ada_name = name;
+	ada.ada_action = action;
 
-		if (prefix_str) {
-			ada.ada_xpath[0] = "./ipv4-prefix";
-			ada.ada_value[0] = prefix_str;
-			if (exact) {
-				ada.ada_xpath[1] = "./ipv4-exact-match";
-				ada.ada_value[1] = "true";
-			}
-		} else {
-			ada.ada_xpath[0] = "./any";
-			ada.ada_value[0] = "true";
+	if (prefix_str) {
+		ada.ada_xpath[0] = "./ipv4-prefix";
+		ada.ada_value[0] = prefix_str;
+		if (exact) {
+			ada.ada_xpath[1] = "./ipv4-exact-match";
+			ada.ada_value[1] = "true";
 		}
-
-		/* Duplicated entry without sequence, just quit. */
-		if (acl_is_dup(vty->candidate_config->dnode, &ada))
-			return CMD_SUCCESS;
+	} else {
+		ada.ada_xpath[0] = "./any";
+		ada.ada_value[0] = "";
 	}
 
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		return CMD_SUCCESS;
+
 	/*
 	 * Create the access-list first, so we can generate sequence if
 	 * none given (backward compatibility).
@@ -565,49 +522,36 @@ DEFPY_YANG(
 	"Exact match of the prefixes\n"
 	"Match any IPv4\n")
 {
-	struct access_list *acl;
-	struct lyd_node *dnode;
 	int64_t sseq;
-	struct prefix pany;
-	char xpath[XPATH_MAXLEN];
-	char xpath_entry[XPATH_MAXLEN + 32];
+	struct acl_dup_args ada = {};
 
 	/* If the user provided sequence number, then just go for it. */
-	if (seq_str != NULL) {
-		snprintf(
-			xpath, sizeof(xpath),
-			"/frr-filter:lib/access-list[type='ipv4'][name='%s']/entry[sequence='%s']",
-			name, seq_str);
-		nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
-		return nb_cli_apply_changes(vty, NULL);
-	}
+	if (seq_str != NULL)
+		return acl_remove(vty, "ipv4", name, seq);
 
 	/* Otherwise, to keep compatibility, we need to figure it out. */
-	snprintf(xpath, sizeof(xpath),
-		 "/frr-filter:lib/access-list[type='ipv4'][name='%s']", name);
+	ada.ada_type = "ipv4";
+	ada.ada_name = name;
+	ada.ada_action = action;
 
-	/* Access-list must exist before entries. */
-	if (yang_dnode_exists(running_config->dnode, xpath) == false)
+	if (prefix_str) {
+		ada.ada_xpath[0] = "./ipv4-prefix";
+		ada.ada_value[0] = prefix_str;
+		if (exact) {
+			ada.ada_xpath[1] = "./ipv4-exact-match";
+			ada.ada_value[1] = "true";
+		}
+	} else {
+		ada.ada_xpath[0] = "./any";
+		ada.ada_value[0] = "";
+	}
+
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		sseq = ada.ada_seq;
+	else
 		return CMD_WARNING_CONFIG_FAILED;
 
-	/* Use access-list data structure to fetch sequence. */
-	dnode = yang_dnode_get(running_config->dnode, xpath);
-	acl = nb_running_get_entry(dnode, NULL, true);
-	if (prefix_str == NULL) {
-		memset(&pany, 0, sizeof(pany));
-		pany.family = AF_INET;
-		sseq = acl_zebra_get_seq(acl, action, &pany, exact);
-	} else
-		sseq = acl_zebra_get_seq(acl, action, (struct prefix *)prefix,
-					 exact);
-	if (sseq == -1)
-		return CMD_WARNING_CONFIG_FAILED;
-
-	snprintfrr(xpath_entry, sizeof(xpath_entry),
-		   "%s/entry[sequence='%" PRId64 "']", xpath, sseq);
-	nb_cli_enqueue_change(vty, xpath_entry, NB_OP_DESTROY, NULL);
-
-	return nb_cli_apply_changes(vty, NULL);
+	return acl_remove(vty, "ipv4", name, sseq);
 }
 
 DEFPY_YANG(
@@ -659,13 +603,18 @@ DEFPY_YANG(
 	ACCESS_LIST_REMARK_STR)
 {
 	char xpath[XPATH_MAXLEN];
+	int rv;
 
 	snprintf(xpath, sizeof(xpath),
 		 "/frr-filter:lib/access-list[type='ipv4'][name='%s']/remark",
 		 name);
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
-	return nb_cli_apply_changes(vty, NULL);
+	rv = nb_cli_apply_changes(vty, NULL);
+	if (rv == CMD_SUCCESS)
+		return acl_remove_if_empty(vty, "ipv4", name);
+
+	return rv;
 }
 
 ALIAS(
@@ -698,27 +647,25 @@ DEFPY_YANG(
 	 * Backward compatibility: don't complain about duplicated values,
 	 * just silently accept.
 	 */
-	if (seq_str == NULL) {
-		ada.ada_type = "ipv6";
-		ada.ada_name = name;
+	ada.ada_type = "ipv6";
+	ada.ada_name = name;
+	ada.ada_action = action;
 
-		if (prefix_str) {
-			ada.ada_xpath[0] = "./ipv6-prefix";
-			ada.ada_value[0] = prefix_str;
-			if (exact) {
-				ada.ada_xpath[1] = "./ipv6-exact-match";
-				ada.ada_value[1] = "true";
-			}
-		} else {
-			ada.ada_xpath[0] = "./any";
-			ada.ada_value[0] = "true";
+	if (prefix_str) {
+		ada.ada_xpath[0] = "./ipv6-prefix";
+		ada.ada_value[0] = prefix_str;
+		if (exact) {
+			ada.ada_xpath[1] = "./ipv6-exact-match";
+			ada.ada_value[1] = "true";
 		}
-
-		/* Duplicated entry without sequence, just quit. */
-		if (acl_is_dup(vty->candidate_config->dnode, &ada))
-			return CMD_SUCCESS;
+	} else {
+		ada.ada_xpath[0] = "./any";
+		ada.ada_value[0] = "";
 	}
 
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		return CMD_SUCCESS;
+
 	/*
 	 * Create the access-list first, so we can generate sequence if
 	 * none given (backward compatibility).
@@ -763,49 +710,36 @@ DEFPY_YANG(
 	"Exact match of the prefixes\n"
 	"Match any IPv6\n")
 {
-	struct access_list *acl;
-	struct lyd_node *dnode;
 	int64_t sseq;
-	struct prefix pany;
-	char xpath[XPATH_MAXLEN];
-	char xpath_entry[XPATH_MAXLEN + 32];
+	struct acl_dup_args ada = {};
 
 	/* If the user provided sequence number, then just go for it. */
-	if (seq_str != NULL) {
-		snprintf(
-			xpath, sizeof(xpath),
-			"/frr-filter:lib/access-list[type='ipv6'][name='%s']/entry[sequence='%s']",
-			name, seq_str);
-		nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
-		return nb_cli_apply_changes(vty, NULL);
-	}
+	if (seq_str != NULL)
+		return acl_remove(vty, "ipv6", name, seq);
 
 	/* Otherwise, to keep compatibility, we need to figure it out. */
-	snprintf(xpath, sizeof(xpath),
-		 "/frr-filter:lib/access-list[type='ipv6'][name='%s']", name);
+	ada.ada_type = "ipv6";
+	ada.ada_name = name;
+	ada.ada_action = action;
 
-	/* Access-list must exist before entries. */
-	if (yang_dnode_exists(running_config->dnode, xpath) == false)
+	if (prefix_str) {
+		ada.ada_xpath[0] = "./ipv6-prefix";
+		ada.ada_value[0] = prefix_str;
+		if (exact) {
+			ada.ada_xpath[1] = "./ipv6-exact-match";
+			ada.ada_value[1] = "true";
+		}
+	} else {
+		ada.ada_xpath[0] = "./any";
+		ada.ada_value[0] = "";
+	}
+
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		sseq = ada.ada_seq;
+	else
 		return CMD_WARNING_CONFIG_FAILED;
 
-	/* Use access-list data structure to fetch sequence. */
-	dnode = yang_dnode_get(running_config->dnode, xpath);
-	acl = nb_running_get_entry(dnode, NULL, true);
-	if (prefix == NULL) {
-		memset(&pany, 0, sizeof(pany));
-		pany.family = AF_INET6;
-		sseq = acl_zebra_get_seq(acl, action, &pany, exact);
-	} else
-		sseq = acl_zebra_get_seq(acl, action, (struct prefix *)prefix,
-					 exact);
-	if (sseq == -1)
-		return CMD_WARNING_CONFIG_FAILED;
-
-	snprintfrr(xpath_entry, sizeof(xpath_entry),
-		   "%s/entry[sequence='%" PRId64 "']", xpath, sseq);
-	nb_cli_enqueue_change(vty, xpath_entry, NB_OP_DESTROY, NULL);
-
-	return nb_cli_apply_changes(vty, NULL);
+	return acl_remove(vty, "ipv6", name, sseq);
 }
 
 DEFPY_YANG(
@@ -860,18 +794,23 @@ DEFPY_YANG(
 	ACCESS_LIST_REMARK_STR)
 {
 	char xpath[XPATH_MAXLEN];
+	int rv;
 
 	snprintf(xpath, sizeof(xpath),
 		 "/frr-filter:lib/access-list[type='ipv6'][name='%s']/remark",
 		 name);
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
-	return nb_cli_apply_changes(vty, NULL);
+	rv = nb_cli_apply_changes(vty, NULL);
+	if (rv == CMD_SUCCESS)
+		return acl_remove_if_empty(vty, "ipv6", name);
+
+	return rv;
 }
 
 ALIAS(
 	no_ipv6_access_list_remark, no_ipv6_access_list_remark_line_cmd,
-	"no ipv6 access-list WORD$name remark LINE...",
+	"no ipv6 access-list ACCESSLIST6_NAME$name remark LINE...",
 	NO_STR
 	IPV6_STR
 	ACCESS_LIST_STR
@@ -881,7 +820,7 @@ ALIAS(
 
 DEFPY_YANG(
 	mac_access_list, mac_access_list_cmd,
-	"mac access-list WORD$name [seq (1-4294967295)$seq] <deny|permit>$action <X:X:X:X:X:X$mac|any>",
+	"mac access-list ACCESSLIST_MAC_NAME$name [seq (1-4294967295)$seq] <deny|permit>$action <X:X:X:X:X:X$mac|any>",
 	MAC_STR
 	ACCESS_LIST_STR
 	ACCESS_LIST_ZEBRA_STR
@@ -899,23 +838,21 @@ DEFPY_YANG(
 	 * Backward compatibility: don't complain about duplicated values,
 	 * just silently accept.
 	 */
-	if (seq_str == NULL) {
-		ada.ada_type = "mac";
-		ada.ada_name = name;
+	ada.ada_type = "mac";
+	ada.ada_name = name;
+	ada.ada_action = action;
 
-		if (mac_str) {
-			ada.ada_xpath[0] = "./mac";
-			ada.ada_value[0] = mac_str;
-		} else {
-			ada.ada_xpath[0] = "./any";
-			ada.ada_value[0] = "true";
-		}
-
-		/* Duplicated entry without sequence, just quit. */
-		if (acl_is_dup(vty->candidate_config->dnode, &ada))
-			return CMD_SUCCESS;
+	if (mac_str) {
+		ada.ada_xpath[0] = "./mac";
+		ada.ada_value[0] = mac_str;
+	} else {
+		ada.ada_xpath[0] = "./any";
+		ada.ada_value[0] = "";
 	}
 
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		return CMD_SUCCESS;
+
 	/*
 	 * Create the access-list first, so we can generate sequence if
 	 * none given (backward compatibility).
@@ -946,7 +883,7 @@ DEFPY_YANG(
 
 DEFPY_YANG(
 	no_mac_access_list, no_mac_access_list_cmd,
-	"no mac access-list WORD$name [seq (1-4294967295)$seq] <deny|permit>$action <X:X:X:X:X:X$prefix|any>",
+	"no mac access-list ACCESSLIST_MAC_NAME$name [seq (1-4294967295)$seq] <deny|permit>$action <X:X:X:X:X:X$mac|any>",
 	NO_STR
 	MAC_STR
 	ACCESS_LIST_STR
@@ -956,54 +893,37 @@ DEFPY_YANG(
 	"MAC address\n"
 	"Match any MAC address\n")
 {
-	struct access_list *acl;
-	struct lyd_node *dnode;
 	int64_t sseq;
-	struct prefix pany;
-	char xpath[XPATH_MAXLEN];
-	char xpath_entry[XPATH_MAXLEN + 32];
+	struct acl_dup_args ada = {};
 
 	/* If the user provided sequence number, then just go for it. */
-	if (seq_str != NULL) {
-		snprintf(
-			xpath, sizeof(xpath),
-			"/frr-filter:lib/access-list[type='mac'][name='%s']/entry[sequence='%s']",
-			name, seq_str);
-		nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
-		return nb_cli_apply_changes(vty, NULL);
-	}
+	if (seq_str != NULL)
+		return acl_remove(vty, "mac", name, seq);
 
 	/* Otherwise, to keep compatibility, we need to figure it out. */
-	snprintf(xpath, sizeof(xpath),
-		 "/frr-filter:lib/access-list[type='mac'][name='%s']", name);
+	ada.ada_type = "mac";
+	ada.ada_name = name;
+	ada.ada_action = action;
 
-	/* Access-list must exist before entries. */
-	if (yang_dnode_exists(running_config->dnode, xpath) == false)
+	if (mac_str) {
+		ada.ada_xpath[0] = "./mac";
+		ada.ada_value[0] = mac_str;
+	} else {
+		ada.ada_xpath[0] = "./any";
+		ada.ada_value[0] = "";
+	}
+
+	if (acl_is_dup(vty->candidate_config->dnode, &ada))
+		sseq = ada.ada_seq;
+	else
 		return CMD_WARNING_CONFIG_FAILED;
 
-	/* Use access-list data structure to fetch sequence. */
-	dnode = yang_dnode_get(running_config->dnode, xpath);
-	acl = nb_running_get_entry(dnode, NULL, true);
-	if (prefix == NULL) {
-		memset(&pany, 0, sizeof(pany));
-		pany.family = AF_ETHERNET;
-		sseq = acl_zebra_get_seq(acl, action, &pany, false);
-	} else
-		sseq = acl_zebra_get_seq(acl, action, (struct prefix *)prefix,
-					 false);
-	if (sseq == -1)
-		return CMD_WARNING_CONFIG_FAILED;
-
-	snprintfrr(xpath_entry, sizeof(xpath_entry),
-		   "%s/entry[sequence='%" PRId64 "']", xpath, sseq);
-	nb_cli_enqueue_change(vty, xpath_entry, NB_OP_DESTROY, NULL);
-
-	return nb_cli_apply_changes(vty, NULL);
+	return acl_remove(vty, "mac", name, sseq);
 }
 
 DEFPY_YANG(
 	no_mac_access_list_all, no_mac_access_list_all_cmd,
-	"no mac access-list WORD$name",
+	"no mac access-list ACCESSLIST_MAC_NAME$name",
 	NO_STR
 	MAC_STR
 	ACCESS_LIST_STR
@@ -1020,7 +940,7 @@ DEFPY_YANG(
 
 DEFPY_YANG(
 	mac_access_list_remark, mac_access_list_remark_cmd,
-	"mac access-list WORD$name remark LINE...",
+	"mac access-list ACCESSLIST_MAC_NAME$name remark LINE...",
 	MAC_STR
 	ACCESS_LIST_STR
 	ACCESS_LIST_ZEBRA_STR
@@ -1045,7 +965,7 @@ DEFPY_YANG(
 
 DEFPY_YANG(
 	no_mac_access_list_remark, no_mac_access_list_remark_cmd,
-	"no mac access-list WORD$name remark",
+	"no mac access-list ACCESSLIST_MAC_NAME$name remark",
 	NO_STR
 	MAC_STR
 	ACCESS_LIST_STR
@@ -1053,18 +973,23 @@ DEFPY_YANG(
 	ACCESS_LIST_REMARK_STR)
 {
 	char xpath[XPATH_MAXLEN];
+	int rv;
 
 	snprintf(xpath, sizeof(xpath),
 		 "/frr-filter:lib/access-list[type='mac'][name='%s']/remark",
 		 name);
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
-	return nb_cli_apply_changes(vty, NULL);
+	rv = nb_cli_apply_changes(vty, NULL);
+	if (rv == CMD_SUCCESS)
+		return acl_remove_if_empty(vty, "mac", name);
+
+	return rv;
 }
 
 ALIAS(
 	no_mac_access_list_remark, no_mac_access_list_remark_line_cmd,
-	"no mac access-list WORD$name remark LINE...",
+	"no mac access-list ACCESSLIST_MAC_NAME$name remark LINE...",
 	NO_STR
 	MAC_STR
 	ACCESS_LIST_STR
@@ -1072,7 +997,16 @@ ALIAS(
 	ACCESS_LIST_REMARK_STR
 	ACCESS_LIST_REMARK_LINE_STR)
 
-void access_list_show(struct vty *vty, struct lyd_node *dnode,
+int access_list_cmp(const struct lyd_node *dnode1,
+		    const struct lyd_node *dnode2)
+{
+	uint32_t seq1 = yang_dnode_get_uint32(dnode1, "./sequence");
+	uint32_t seq2 = yang_dnode_get_uint32(dnode2, "./sequence");
+
+	return seq1 - seq2;
+}
+
+void access_list_show(struct vty *vty, const struct lyd_node *dnode,
 		      bool show_defaults)
 {
 	int type = yang_dnode_get_enum(dnode, "../type");
@@ -1186,7 +1120,7 @@ void access_list_show(struct vty *vty, struct lyd_node *dnode,
 	vty_out(vty, "\n");
 }
 
-void access_list_remark_show(struct vty *vty, struct lyd_node *dnode,
+void access_list_remark_show(struct vty *vty, const struct lyd_node *dnode,
 			     bool show_defaults)
 {
 	int type = yang_dnode_get_enum(dnode, "../type");
@@ -1244,13 +1178,11 @@ static int plist_remove_if_empty(struct vty *vty, const char *iptype,
 }
 
 static int plist_remove(struct vty *vty, const char *iptype, const char *name,
-			const char *seq, const char *action, struct prefix *p,
-			long ge, long le)
+			const char *seq, const char *action,
+			union prefixconstptr prefix, int ge, int le)
 {
-	struct prefix_list_entry *pentry;
-	enum prefix_list_type plt;
-	struct prefix_list *pl;
-	struct lyd_node *dnode;
+	int64_t sseq;
+	struct plist_dup_args pda = {};
 	char xpath[XPATH_MAXLEN];
 	char xpath_entry[XPATH_MAXLEN + 32];
 	int rv;
@@ -1271,29 +1203,27 @@ static int plist_remove(struct vty *vty, const char *iptype, const char *name,
 	}
 
 	/* Otherwise, to keep compatibility, we need to figure it out. */
-	snprintf(xpath, sizeof(xpath),
-		 "/frr-filter:lib/prefix-list[type='%s'][name='%s']", iptype,
-		 name);
+	pda.pda_type = iptype;
+	pda.pda_name = name;
+	pda.pda_action = action;
+	if (prefix.p) {
+		prefix_copy(&pda.prefix, prefix);
+		apply_mask(&pda.prefix);
+		pda.ge = ge;
+		pda.le = le;
+	} else {
+		pda.any = true;
+	}
 
-	/* Access-list must exist before entries. */
-	if (yang_dnode_exists(running_config->dnode, xpath) == false)
-		return CMD_WARNING_CONFIG_FAILED;
-
-	/* Use access-list data structure to fetch sequence. */
-	assert(action != NULL);
-	if (strcmp(action, "permit") == 0)
-		plt = PREFIX_PERMIT;
+	if (plist_is_dup(vty->candidate_config->dnode, &pda))
+		sseq = pda.pda_seq;
 	else
-		plt = PREFIX_DENY;
-
-	dnode = yang_dnode_get(running_config->dnode, xpath);
-	pl = nb_running_get_entry(dnode, NULL, true);
-	pentry = prefix_list_entry_lookup(pl, p, plt, -1, le, ge);
-	if (pentry == NULL)
 		return CMD_WARNING_CONFIG_FAILED;
 
-	snprintfrr(xpath_entry, sizeof(xpath_entry),
-		   "%s/entry[sequence='%" PRId64 "']", xpath, pentry->seq);
+	snprintfrr(
+		xpath_entry, sizeof(xpath_entry),
+		"/frr-filter:lib/prefix-list[type='%s'][name='%s']/entry[sequence='%" PRId64 "']",
+		iptype, name, sseq);
 	nb_cli_enqueue_change(vty, xpath_entry, NB_OP_DESTROY, NULL);
 
 	rv = nb_cli_apply_changes(vty, NULL);
@@ -1319,7 +1249,6 @@ DEFPY_YANG(
 	"Maximum prefix length\n")
 {
 	int64_t sseq;
-	int arg_idx = 0;
 	struct plist_dup_args pda = {};
 	char xpath[XPATH_MAXLEN];
 	char xpath_entry[XPATH_MAXLEN + 128];
@@ -1328,35 +1257,20 @@ DEFPY_YANG(
 	 * Backward compatibility: don't complain about duplicated values,
 	 * just silently accept.
 	 */
-	if (seq_str == NULL) {
-		pda.pda_type = "ipv4";
-		pda.pda_name = name;
-		if (prefix_str) {
-			pda.pda_xpath[arg_idx] = "./ipv4-prefix";
-			pda.pda_value[arg_idx] = prefix_str;
-			arg_idx++;
-			if (ge_str) {
-				pda.pda_xpath[arg_idx] =
-					"./ipv4-prefix-length-greater-or-equal";
-				pda.pda_value[arg_idx] = ge_str;
-				arg_idx++;
-			}
-			if (le_str) {
-				pda.pda_xpath[arg_idx] =
-					"./ipv4-prefix-length-lesser-or-equal";
-				pda.pda_value[arg_idx] = le_str;
-				arg_idx++;
-			}
-		} else {
-			pda.pda_xpath[0] = "./any";
-			pda.pda_value[0] = "";
-		}
-
-		/* Duplicated entry without sequence, just quit. */
-		if (plist_is_dup(vty->candidate_config->dnode, &pda))
-			return CMD_SUCCESS;
+	pda.pda_type = "ipv4";
+	pda.pda_name = name;
+	pda.pda_action = action;
+	if (prefix_str) {
+		prefix_copy(&pda.prefix, prefix);
+		pda.ge = ge;
+		pda.le = le;
+	} else {
+		pda.any = true;
 	}
 
+	if (plist_is_dup(vty->candidate_config->dnode, &pda))
+		return CMD_SUCCESS;
+
 	/*
 	 * Create the prefix-list first, so we can generate sequence if
 	 * none given (backward compatibility).
@@ -1380,14 +1294,32 @@ DEFPY_YANG(
 		nb_cli_enqueue_change(vty, "./ipv4-prefix", NB_OP_MODIFY,
 				      prefix_str);
 
-		if (ge_str)
+		if (ge_str) {
 			nb_cli_enqueue_change(
 				vty, "./ipv4-prefix-length-greater-or-equal",
 				NB_OP_MODIFY, ge_str);
-		if (le_str)
+		} else {
+			/*
+			 * Remove old ge if not being modified
+			 */
+			nb_cli_enqueue_change(
+				vty, "./ipv4-prefix-length-greater-or-equal",
+				NB_OP_DESTROY, NULL);
+		}
+
+		if (le_str) {
 			nb_cli_enqueue_change(
 				vty, "./ipv4-prefix-length-lesser-or-equal",
 				NB_OP_MODIFY, le_str);
+		} else {
+			/*
+			 * Remove old le if not being modified
+			 */
+			nb_cli_enqueue_change(
+				vty, "./ipv4-prefix-length-lesser-or-equal",
+				NB_OP_DESTROY, NULL);
+		}
+		nb_cli_enqueue_change(vty, "./any", NB_OP_DESTROY, NULL);
 	} else {
 		nb_cli_enqueue_change(vty, "./any", NB_OP_CREATE, NULL);
 	}
@@ -1412,7 +1344,7 @@ DEFPY_YANG(
 	"Maximum prefix length\n")
 {
 	return plist_remove(vty, "ipv4", name, seq_str, action,
-			    (struct prefix *)prefix, ge, le);
+			    prefix_str ? prefix : NULL, ge, le);
 }
 
 DEFPY_YANG(
@@ -1479,13 +1411,18 @@ DEFPY_YANG(
 	ACCESS_LIST_REMARK_STR)
 {
 	char xpath[XPATH_MAXLEN];
+	int rv;
 
 	snprintf(xpath, sizeof(xpath),
 		 "/frr-filter:lib/prefix-list[type='ipv4'][name='%s']/remark",
 		 name);
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
-	return nb_cli_apply_changes(vty, NULL);
+	rv = nb_cli_apply_changes(vty, NULL);
+	if (rv == CMD_SUCCESS)
+		return plist_remove_if_empty(vty, "ipv4", name);
+
+	return rv;
 }
 
 ALIAS(
@@ -1514,7 +1451,6 @@ DEFPY_YANG(
 	"Minimum prefix length\n")
 {
 	int64_t sseq;
-	int arg_idx = 0;
 	struct plist_dup_args pda = {};
 	char xpath[XPATH_MAXLEN];
 	char xpath_entry[XPATH_MAXLEN + 128];
@@ -1523,35 +1459,20 @@ DEFPY_YANG(
 	 * Backward compatibility: don't complain about duplicated values,
 	 * just silently accept.
 	 */
-	if (seq_str == NULL) {
-		pda.pda_type = "ipv6";
-		pda.pda_name = name;
-		if (prefix_str) {
-			pda.pda_xpath[arg_idx] = "./ipv6-prefix";
-			pda.pda_value[arg_idx] = prefix_str;
-			arg_idx++;
-			if (ge_str) {
-				pda.pda_xpath[arg_idx] =
-					"./ipv6-prefix-length-greater-or-equal";
-				pda.pda_value[arg_idx] = ge_str;
-				arg_idx++;
-			}
-			if (le_str) {
-				pda.pda_xpath[arg_idx] =
-					"./ipv6-prefix-length-lesser-or-equal";
-				pda.pda_value[arg_idx] = le_str;
-				arg_idx++;
-			}
-		} else {
-			pda.pda_xpath[0] = "./any";
-			pda.pda_value[0] = "";
-		}
-
-		/* Duplicated entry without sequence, just quit. */
-		if (plist_is_dup(vty->candidate_config->dnode, &pda))
-			return CMD_SUCCESS;
+	pda.pda_type = "ipv6";
+	pda.pda_name = name;
+	pda.pda_action = action;
+	if (prefix_str) {
+		prefix_copy(&pda.prefix, prefix);
+		pda.ge = ge;
+		pda.le = le;
+	} else {
+		pda.any = true;
 	}
 
+	if (plist_is_dup(vty->candidate_config->dnode, &pda))
+		return CMD_SUCCESS;
+
 	/*
 	 * Create the prefix-list first, so we can generate sequence if
 	 * none given (backward compatibility).
@@ -1575,14 +1496,32 @@ DEFPY_YANG(
 		nb_cli_enqueue_change(vty, "./ipv6-prefix", NB_OP_MODIFY,
 				      prefix_str);
 
-		if (ge_str)
+		if (ge_str) {
 			nb_cli_enqueue_change(
 				vty, "./ipv6-prefix-length-greater-or-equal",
 				NB_OP_MODIFY, ge_str);
-		if (le_str)
+		} else {
+			/*
+			 * Remove old ge if not being modified
+			 */
+			nb_cli_enqueue_change(
+				vty, "./ipv6-prefix-length-greater-or-equal",
+				NB_OP_DESTROY, NULL);
+		}
+
+		if (le_str) {
 			nb_cli_enqueue_change(
 				vty, "./ipv6-prefix-length-lesser-or-equal",
 				NB_OP_MODIFY, le_str);
+		} else {
+			/*
+			 * Remove old le if not being modified
+			 */
+			nb_cli_enqueue_change(
+				vty, "./ipv6-prefix-length-lesser-or-equal",
+				NB_OP_DESTROY, NULL);
+		}
+		nb_cli_enqueue_change(vty, "./any", NB_OP_DESTROY, NULL);
 	} else {
 		nb_cli_enqueue_change(vty, "./any", NB_OP_CREATE, NULL);
 	}
@@ -1607,7 +1546,7 @@ DEFPY_YANG(
 	"Minimum prefix length\n")
 {
 	return plist_remove(vty, "ipv6", name, seq_str, action,
-			    (struct prefix *)prefix, ge, le);
+			    prefix_str ? prefix : NULL, ge, le);
 }
 
 DEFPY_YANG(
@@ -1674,13 +1613,18 @@ DEFPY_YANG(
 	ACCESS_LIST_REMARK_STR)
 {
 	char xpath[XPATH_MAXLEN];
+	int rv;
 
 	snprintf(xpath, sizeof(xpath),
 		 "/frr-filter:lib/prefix-list[type='ipv6'][name='%s']/remark",
 		 name);
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
-	return nb_cli_apply_changes(vty, NULL);
+	rv = nb_cli_apply_changes(vty, NULL);
+	if (rv == CMD_SUCCESS)
+		return plist_remove_if_empty(vty, "ipv6", name);
+
+	return rv;
 }
 
 ALIAS(
@@ -1693,7 +1637,16 @@ ALIAS(
 	ACCESS_LIST_REMARK_STR
 	ACCESS_LIST_REMARK_LINE_STR)
 
-void prefix_list_show(struct vty *vty, struct lyd_node *dnode,
+int prefix_list_cmp(const struct lyd_node *dnode1,
+		    const struct lyd_node *dnode2)
+{
+	uint32_t seq1 = yang_dnode_get_uint32(dnode1, "./sequence");
+	uint32_t seq2 = yang_dnode_get_uint32(dnode2, "./sequence");
+
+	return seq1 - seq2;
+}
+
+void prefix_list_show(struct vty *vty, const struct lyd_node *dnode,
 		      bool show_defaults)
 {
 	int type = yang_dnode_get_enum(dnode, "../type");
@@ -1752,7 +1705,7 @@ void prefix_list_show(struct vty *vty, struct lyd_node *dnode,
 	vty_out(vty, "\n");
 }
 
-void prefix_list_remark_show(struct vty *vty, struct lyd_node *dnode,
+void prefix_list_remark_show(struct vty *vty, const struct lyd_node *dnode,
 			     bool show_defaults)
 {
 	int type = yang_dnode_get_enum(dnode, "../type");
