@@ -28,9 +28,11 @@
 #include "memory.h"
 #include "linklist.h"
 #include "zlog.h"
+#include "libfrr.h"
+#include "libfrr_trace.h"
 
-DEFINE_MTYPE_STATIC(LIB, FRR_PTHREAD, "FRR POSIX Thread")
-DEFINE_MTYPE_STATIC(LIB, PTHREAD_PRIM, "POSIX sync primitives")
+DEFINE_MTYPE_STATIC(LIB, FRR_PTHREAD, "FRR POSIX Thread");
+DEFINE_MTYPE_STATIC(LIB, PTHREAD_PRIM, "POSIX sync primitives");
 
 /* default frr_pthread start/stop routine prototypes */
 static void *fpt_run(void *arg);
@@ -53,7 +55,7 @@ static struct list *frr_pthread_list;
 
 void frr_pthread_init(void)
 {
-	frr_with_mutex(&frr_pthread_list_mtx) {
+	frr_with_mutex (&frr_pthread_list_mtx) {
 		frr_pthread_list = list_new();
 	}
 }
@@ -62,7 +64,7 @@ void frr_pthread_finish(void)
 {
 	frr_pthread_stop_all();
 
-	frr_with_mutex(&frr_pthread_list_mtx) {
+	frr_with_mutex (&frr_pthread_list_mtx) {
 		struct listnode *n, *nn;
 		struct frr_pthread *fpt;
 
@@ -103,7 +105,7 @@ struct frr_pthread *frr_pthread_new(const struct frr_pthread_attr *attr,
 	pthread_mutex_init(fpt->running_cond_mtx, NULL);
 	pthread_cond_init(fpt->running_cond, NULL);
 
-	frr_with_mutex(&frr_pthread_list_mtx) {
+	frr_with_mutex (&frr_pthread_list_mtx) {
 		listnode_add(frr_pthread_list, fpt);
 	}
 
@@ -124,7 +126,7 @@ static void frr_pthread_destroy_nolock(struct frr_pthread *fpt)
 
 void frr_pthread_destroy(struct frr_pthread *fpt)
 {
-	frr_with_mutex(&frr_pthread_list_mtx) {
+	frr_with_mutex (&frr_pthread_list_mtx) {
 		listnode_delete(frr_pthread_list, fpt);
 	}
 
@@ -161,12 +163,16 @@ int frr_pthread_run(struct frr_pthread *fpt, const pthread_attr_t *attr)
 	int ret;
 	sigset_t oldsigs, blocksigs;
 
+	assert(frr_is_after_fork || !"trying to start thread before fork()");
+
 	/* Ensure we never handle signals on a background thread by blocking
 	 * everything here (new thread inherits signal mask)
 	 */
 	sigfillset(&blocksigs);
 	pthread_sigmask(SIG_BLOCK, &blocksigs, &oldsigs);
 
+	frrtrace(1, frr_libfrr, frr_pthread_run, fpt->name);
+
 	fpt->rcu_thread = rcu_thread_prepare();
 	ret = pthread_create(&fpt->thread, attr, frr_pthread_inner, fpt);
 
@@ -187,7 +193,7 @@ int frr_pthread_run(struct frr_pthread *fpt, const pthread_attr_t *attr)
 
 void frr_pthread_wait_running(struct frr_pthread *fpt)
 {
-	frr_with_mutex(fpt->running_cond_mtx) {
+	frr_with_mutex (fpt->running_cond_mtx) {
 		while (!fpt->running)
 			pthread_cond_wait(fpt->running_cond,
 					  fpt->running_cond_mtx);
@@ -196,7 +202,7 @@ void frr_pthread_wait_running(struct frr_pthread *fpt)
 
 void frr_pthread_notify_running(struct frr_pthread *fpt)
 {
-	frr_with_mutex(fpt->running_cond_mtx) {
+	frr_with_mutex (fpt->running_cond_mtx) {
 		fpt->running = true;
 		pthread_cond_signal(fpt->running_cond);
 	}
@@ -204,6 +210,8 @@ void frr_pthread_notify_running(struct frr_pthread *fpt)
 
 int frr_pthread_stop(struct frr_pthread *fpt, void **result)
 {
+	frrtrace(1, frr_libfrr, frr_pthread_stop, fpt->name);
+
 	int ret = (*fpt->attr.stop)(fpt, result);
 	memset(&fpt->thread, 0x00, sizeof(fpt->thread));
 	return ret;
@@ -211,7 +219,7 @@ int frr_pthread_stop(struct frr_pthread *fpt, void **result)
 
 void frr_pthread_stop_all(void)
 {
-	frr_with_mutex(&frr_pthread_list_mtx) {
+	frr_with_mutex (&frr_pthread_list_mtx) {
 		struct listnode *n;
 		struct frr_pthread *fpt;
 		for (ALL_LIST_ELEMENTS_RO(frr_pthread_list, n, fpt)) {
@@ -229,18 +237,16 @@ void frr_pthread_stop_all(void)
  */
 
 /* dummy task for sleeper pipe */
-static int fpt_dummy(struct thread *thread)
+static void fpt_dummy(struct thread *thread)
 {
-	return 0;
 }
 
 /* poison pill task to end event loop */
-static int fpt_finish(struct thread *thread)
+static void fpt_finish(struct thread *thread)
 {
 	struct frr_pthread *fpt = THREAD_ARG(thread);
 
 	atomic_store_explicit(&fpt->running, false, memory_order_relaxed);
-	return 0;
 }
 
 /* stop function, called from other threads to halt this one */
