@@ -26,10 +26,11 @@
 #include "if.h"
 #include "if_rmap.h"
 
-DEFINE_MTYPE_STATIC(LIB, IF_RMAP_CTX, "Interface route map container")
-DEFINE_MTYPE_STATIC(LIB, IF_RMAP_CTX_NAME, "Interface route map container name")
-DEFINE_MTYPE_STATIC(LIB, IF_RMAP, "Interface route map")
-DEFINE_MTYPE_STATIC(LIB, IF_RMAP_NAME, "I.f. route map name")
+DEFINE_MTYPE_STATIC(LIB, IF_RMAP_CTX, "Interface route map container");
+DEFINE_MTYPE_STATIC(LIB, IF_RMAP_CTX_NAME,
+		    "Interface route map container name");
+DEFINE_MTYPE_STATIC(LIB, IF_RMAP, "Interface route map");
+DEFINE_MTYPE_STATIC(LIB, IF_RMAP_NAME, "I.f. route map name");
 
 static struct list *if_rmap_ctx_list;
 
@@ -292,7 +293,7 @@ void if_rmap_ctx_delete(struct if_rmap_ctx *ctx)
 	listnode_delete(if_rmap_ctx_list, ctx);
 	hash_clean(ctx->ifrmaphash, (void (*)(void *))if_rmap_free);
 	if (ctx->name)
-		XFREE(MTYPE_IF_RMAP_CTX_NAME, ctx);
+		XFREE(MTYPE_IF_RMAP_CTX_NAME, ctx->name);
 	XFREE(MTYPE_IF_RMAP_CTX, ctx);
 }
 
