@@ -24,8 +24,10 @@
 #include "command.h"
 #include "lib/log.h"
 #include "lib/zlog_targets.h"
+#include "lib/zlog_5424.h"
 #include "lib/lib_errors.h"
 #include "lib/printfrr.h"
+#include "lib/systemd.h"
 
 #ifndef VTYSH_EXTRACT_PL
 #include "lib/log_vty_clippy.c"
@@ -33,7 +35,10 @@
 
 #define ZLOG_MAXLVL(a, b) MAX(a, b)
 
-DEFINE_HOOK(zlog_rotate, (), ())
+DEFINE_HOOK(zlog_rotate, (), ());
+DEFINE_HOOK(zlog_cli_show, (struct vty * vty), (vty));
+
+static unsigned logmsgs_with_persist_bt;
 
 static const int log_default_lvl = LOG_DEBUG;
 
@@ -48,16 +53,35 @@ static struct zlog_cfg_file zt_file_cmdline = {
 static struct zlog_cfg_file zt_file = {
 	.prio_min = ZLOG_DISABLED,
 };
-static struct zlog_cfg_file zt_stdout = {
-	.prio_min = ZLOG_DISABLED,
-};
 static struct zlog_cfg_filterfile zt_filterfile = {
 	.parent = {
 		.prio_min = ZLOG_DISABLED,
 	},
 };
 
-static const char *zlog_progname;
+static struct zlog_cfg_file zt_stdout_file = {
+	.prio_min = ZLOG_DISABLED,
+};
+static struct zlog_cfg_5424 zt_stdout_journald = {
+	.prio_min = ZLOG_DISABLED,
+
+	.fmt = ZLOG_FMT_JOURNALD,
+	.dst = ZLOG_5424_DST_UNIX,
+	.filename = "/run/systemd/journal/socket",
+
+	/* this can't be changed through config since this target substitutes
+	 * in for the "plain" stdout target
+	 */
+	.facility = LOG_DAEMON,
+	.kw_version = false,
+	.kw_location = true,
+	.kw_uid = true,
+	.kw_ec = true,
+	.kw_args = true,
+};
+static bool stdout_journald_in_use;
+
+const char *zlog_progname;
 static const char *zlog_protoname;
 
 static const struct facility_map {
@@ -94,7 +118,14 @@ static const char * const zlog_priority[] = {
 	"notifications", "informational", "debugging", NULL,
 };
 
-static const char *facility_name(int facility)
+const char *zlog_priority_str(int priority)
+{
+	if (priority > LOG_DEBUG)
+		return "???";
+	return zlog_priority[priority];
+}
+
+const char *facility_name(int facility)
 {
 	const struct facility_map *fm;
 
@@ -104,7 +135,7 @@ static const char *facility_name(int facility)
 	return "";
 }
 
-static int facility_match(const char *str)
+int facility_match(const char *str)
 {
 	const struct facility_map *fm;
 
@@ -128,6 +159,7 @@ void zlog_rotate(void)
 {
 	zlog_file_rotate(&zt_file);
 	zlog_file_rotate(&zt_filterfile.parent);
+	zlog_file_rotate(&zt_file_cmdline);
 	hook_call(zlog_rotate);
 }
 
@@ -146,20 +178,24 @@ void log_show_syslog(struct vty *vty)
 			zlog_progname);
 }
 
-DEFUN (show_logging,
-       show_logging_cmd,
-       "show logging",
-       SHOW_STR
-       "Show current logging configuration\n")
+DEFUN_NOSH (show_logging,
+	    show_logging_cmd,
+	    "show logging",
+	    SHOW_STR
+	    "Show current logging configuration\n")
 {
+	int stdout_prio;
+
 	log_show_syslog(vty);
 
+	stdout_prio = stdout_journald_in_use ? zt_stdout_journald.prio_min
+					     : zt_stdout_file.prio_min;
+
 	vty_out(vty, "Stdout logging: ");
-	if (zt_stdout.prio_min == ZLOG_DISABLED)
+	if (stdout_prio == ZLOG_DISABLED)
 		vty_out(vty, "disabled");
 	else
-		vty_out(vty, "level %s",
-			zlog_priority[zt_stdout.prio_min]);
+		vty_out(vty, "level %s", zlog_priority[stdout_prio]);
 	vty_out(vty, "\n");
 
 	vty_out(vty, "File logging: ");
@@ -194,9 +230,26 @@ DEFUN (show_logging,
 	vty_out(vty, "Record priority: %s\n",
 		(zt_file.record_priority ? "enabled" : "disabled"));
 	vty_out(vty, "Timestamp precision: %d\n", zt_file.ts_subsec);
+
+	hook_call(zlog_cli_show, vty);
 	return CMD_SUCCESS;
 }
 
+static void log_stdout_apply_level(void)
+{
+	int maxlvl;
+
+	maxlvl = ZLOG_MAXLVL(log_config_stdout_lvl, log_cmdline_stdout_lvl);
+
+	if (stdout_journald_in_use) {
+		zt_stdout_journald.prio_min = maxlvl;
+		zlog_5424_apply_meta(&zt_stdout_journald);
+	} else {
+		zt_stdout_file.prio_min = maxlvl;
+		zlog_file_set_other(&zt_stdout_file);
+	}
+}
+
 DEFPY (config_log_stdout,
        config_log_stdout_cmd,
        "log stdout [<emergencies|alerts|critical|errors|warnings|notifications|informational|debugging>$levelarg]",
@@ -214,9 +267,7 @@ DEFPY (config_log_stdout,
 		level = log_default_lvl;
 
 	log_config_stdout_lvl = level;
-	zt_stdout.prio_min = ZLOG_MAXLVL(log_config_stdout_lvl,
-					 log_cmdline_stdout_lvl);
-	zlog_file_set_other(&zt_stdout);
+	log_stdout_apply_level();
 	return CMD_SUCCESS;
 }
 
@@ -229,9 +280,7 @@ DEFUN (no_config_log_stdout,
        LOG_LEVEL_DESC)
 {
 	log_config_stdout_lvl = ZLOG_DISABLED;
-	zt_stdout.prio_min = ZLOG_MAXLVL(log_config_stdout_lvl,
-					 log_cmdline_stdout_lvl);
-	zlog_file_set_other(&zt_stdout);
+	log_stdout_apply_level();
 	return CMD_SUCCESS;
 }
 
@@ -257,6 +306,43 @@ DEFUN_HIDDEN (no_config_log_monitor,
 	return CMD_SUCCESS;
 }
 
+DEFPY_NOSH (debug_uid_backtrace,
+	    debug_uid_backtrace_cmd,
+	    "[no] debug unique-id UID backtrace",
+	    NO_STR
+	    DEBUG_STR
+	    "Options per individual log message, by unique ID\n"
+	    "Log message unique ID (XXXXX-XXXXX)\n"
+	    "Add backtrace to log when message is printed\n")
+{
+	struct xrefdata search, *xrd;
+	struct xrefdata_logmsg *xrdl;
+	uint8_t flag;
+
+	strlcpy(search.uid, uid, sizeof(search.uid));
+	xrd = xrefdata_uid_find(&xrefdata_uid, &search);
+
+	if (!xrd)
+		return CMD_ERR_NOTHING_TODO;
+
+	if (xrd->xref->type != XREFT_LOGMSG) {
+		vty_out(vty, "%% ID \"%s\" is not a log message\n", uid);
+		return CMD_WARNING;
+	}
+	xrdl = container_of(xrd, struct xrefdata_logmsg, xrefdata);
+
+	flag = (vty->node == CONFIG_NODE) ? LOGMSG_FLAG_PERSISTENT
+					  : LOGMSG_FLAG_EPHEMERAL;
+
+	if ((xrdl->fl_print_bt & flag) == (no ? 0 : flag))
+		return CMD_SUCCESS;
+	if (flag == LOGMSG_FLAG_PERSISTENT)
+		logmsgs_with_persist_bt += no ? -1 : 1;
+
+	xrdl->fl_print_bt ^= flag;
+	return CMD_SUCCESS;
+}
+
 static int set_log_file(struct zlog_cfg_file *target, struct vty *vty,
 			const char *fname, int loglevel)
 {
@@ -328,9 +414,7 @@ void command_setup_early_logging(const char *dest, const char *level)
 
 	if (strcmp(type, "stdout") == 0) {
 		log_cmdline_stdout_lvl = nlevel;
-		zt_stdout.prio_min = ZLOG_MAXLVL(log_config_stdout_lvl,
-						 log_cmdline_stdout_lvl);
-		zlog_file_set_other(&zt_stdout);
+		log_stdout_apply_level();
 		return;
 	}
 	if (strcmp(type, "syslog") == 0) {
@@ -344,6 +428,22 @@ void command_setup_early_logging(const char *dest, const char *level)
 		set_log_file(&zt_file_cmdline, NULL, sep, nlevel);
 		return;
 	}
+	if (strcmp(type, "monitor") == 0 && sep) {
+		struct zlog_live_cfg cfg = {};
+		unsigned long fd;
+		char *endp;
+
+		sep++;
+		fd = strtoul(sep, &endp, 10);
+		if (!*sep || *endp) {
+			fprintf(stderr, "invalid monitor fd \"%s\"\n", sep);
+			exit(1);
+		}
+
+		zlog_live_open_fd(&cfg, nlevel, fd);
+		zlog_live_disown(&cfg);
+		return;
+	}
 
 	fprintf(stderr, "invalid log target \"%s\" (\"%s\")\n", type, dest);
 	exit(1);
@@ -364,9 +464,7 @@ DEFUN (clear_log_cmdline,
 					     log_cmdline_syslog_lvl));
 
 	log_cmdline_stdout_lvl = ZLOG_DISABLED;
-	zt_stdout.prio_min = ZLOG_MAXLVL(log_config_stdout_lvl,
-					 log_cmdline_stdout_lvl);
-	zlog_file_set_other(&zt_stdout);
+	log_stdout_apply_level();
 
 	return CMD_SUCCESS;
 }
@@ -474,8 +572,10 @@ DEFUN (config_log_record_priority,
 {
 	zt_file.record_priority = true;
 	zlog_file_set_other(&zt_file);
-	zt_stdout.record_priority = true;
-	zlog_file_set_other(&zt_stdout);
+	if (!stdout_journald_in_use) {
+		zt_stdout_file.record_priority = true;
+		zlog_file_set_other(&zt_stdout_file);
+	}
 	zt_filterfile.parent.record_priority = true;
 	zlog_file_set_other(&zt_filterfile.parent);
 	return CMD_SUCCESS;
@@ -490,8 +590,10 @@ DEFUN (no_config_log_record_priority,
 {
 	zt_file.record_priority = false;
 	zlog_file_set_other(&zt_file);
-	zt_stdout.record_priority = false;
-	zlog_file_set_other(&zt_stdout);
+	if (!stdout_journald_in_use) {
+		zt_stdout_file.record_priority = false;
+		zlog_file_set_other(&zt_stdout_file);
+	}
 	zt_filterfile.parent.record_priority = false;
 	zlog_file_set_other(&zt_filterfile.parent);
 	return CMD_SUCCESS;
@@ -507,8 +609,10 @@ DEFPY (config_log_timestamp_precision,
 {
 	zt_file.ts_subsec = precision;
 	zlog_file_set_other(&zt_file);
-	zt_stdout.ts_subsec = precision;
-	zlog_file_set_other(&zt_stdout);
+	if (!stdout_journald_in_use) {
+		zt_stdout_file.ts_subsec = precision;
+		zlog_file_set_other(&zt_stdout_file);
+	}
 	zt_filterfile.parent.ts_subsec = precision;
 	zlog_file_set_other(&zt_filterfile.parent);
 	return CMD_SUCCESS;
@@ -525,13 +629,37 @@ DEFUN (no_config_log_timestamp_precision,
 {
 	zt_file.ts_subsec = 0;
 	zlog_file_set_other(&zt_file);
-	zt_stdout.ts_subsec = 0;
-	zlog_file_set_other(&zt_stdout);
+	if (!stdout_journald_in_use) {
+		zt_stdout_file.ts_subsec = 0;
+		zlog_file_set_other(&zt_stdout_file);
+	}
 	zt_filterfile.parent.ts_subsec = 0;
 	zlog_file_set_other(&zt_filterfile.parent);
 	return CMD_SUCCESS;
 }
 
+DEFPY (config_log_ec,
+       config_log_ec_cmd,
+       "[no] log error-category",
+       NO_STR
+       "Logging control\n"
+       "Prefix log message text with [EC 9999] code\n")
+{
+	zlog_set_prefix_ec(!no);
+	return CMD_SUCCESS;
+}
+
+DEFPY (config_log_xid,
+       config_log_xid_cmd,
+       "[no] log unique-id",
+       NO_STR
+       "Logging control\n"
+       "Prefix log message text with [XXXXX-XXXXX] identifier\n")
+{
+	zlog_set_prefix_xid(!no);
+	return CMD_SUCCESS;
+}
+
 DEFPY (config_log_filterfile,
        config_log_filterfile_cmd,
        "log filtered-file FILENAME [<emergencies|alerts|critical|errors|warnings|notifications|informational|debugging>$levelarg]",
@@ -566,8 +694,9 @@ DEFUN (no_config_log_filterfile,
 
 DEFPY (log_filter,
        log_filter_cmd,
-       "[no] log-filter WORD$filter",
+       "[no] log filter-text WORD$filter",
        NO_STR
+       "Logging control\n"
        FILTER_LOG_STR
        "String to filter by\n")
 {
@@ -594,8 +723,9 @@ DEFPY (log_filter,
 /* Clear all log filters */
 DEFPY (log_filter_clear,
        log_filter_clear_cmd,
-       "clear log-filter",
+       "clear log filter-text",
        CLEAR_STR
+       "Logging control\n"
        FILTER_LOG_STR)
 {
 	zlog_filter_clear();
@@ -605,8 +735,9 @@ DEFPY (log_filter_clear,
 /* Show log filter */
 DEFPY (show_log_filter,
        show_log_filter_cmd,
-       "show log-filter",
+       "show logging filter-text",
        SHOW_STR
+       "Show current logging configuration\n"
        FILTER_LOG_STR)
 {
 	char log_filters[ZLOG_FILTERS_MAX * (ZLOG_FILTER_LENGTH_MAX + 3)] = "";
@@ -625,6 +756,18 @@ DEFPY (show_log_filter,
 	return CMD_SUCCESS;
 }
 
+/* Enable/disable 'immediate' mode, with no output buffering */
+DEFPY (log_immediate_mode,
+       log_immediate_mode_cmd,
+       "[no] log immediate-mode",
+       NO_STR
+       "Logging control"
+       "Output immediately, without buffering")
+{
+	zlog_set_immediate(!no);
+	return CMD_SUCCESS;
+}
+
 void log_config_write(struct vty *vty)
 {
 	bool show_cmdline_hint = false;
@@ -699,6 +842,29 @@ void log_config_write(struct vty *vty)
 	if (zt_file.ts_subsec > 0)
 		vty_out(vty, "log timestamp precision %d\n",
 			zt_file.ts_subsec);
+
+	if (!zlog_get_prefix_ec())
+		vty_out(vty, "no log error-category\n");
+	if (!zlog_get_prefix_xid())
+		vty_out(vty, "no log unique-id\n");
+
+	if (logmsgs_with_persist_bt) {
+		struct xrefdata *xrd;
+		struct xrefdata_logmsg *xrdl;
+
+		vty_out(vty, "!\n");
+
+		frr_each (xrefdata_uid, &xrefdata_uid, xrd) {
+			if (xrd->xref->type != XREFT_LOGMSG)
+				continue;
+
+			xrdl = container_of(xrd, struct xrefdata_logmsg,
+					    xrefdata);
+			if (xrdl->fl_print_bt & LOGMSG_FLAG_PERSISTENT)
+				vty_out(vty, "debug unique-id %s backtrace\n",
+					xrd->uid);
+		}
+	}
 }
 
 static int log_vty_init(const char *progname, const char *protoname,
@@ -707,9 +873,17 @@ static int log_vty_init(const char *progname, const char *protoname,
 	zlog_progname = progname;
 	zlog_protoname = protoname;
 
+	zlog_set_prefix_ec(true);
+	zlog_set_prefix_xid(true);
+
 	zlog_filterfile_init(&zt_filterfile);
 
-	zlog_file_set_fd(&zt_stdout, STDOUT_FILENO);
+	if (sd_stdout_is_journal) {
+		stdout_journald_in_use = true;
+		zlog_5424_init(&zt_stdout_journald);
+		zlog_5424_apply_dst(&zt_stdout_journald);
+	} else
+		zlog_file_set_fd(&zt_stdout_file, STDOUT_FILENO);
 	return 0;
 }
 
@@ -737,10 +911,18 @@ void log_cmd_init(void)
 	install_element(CONFIG_NODE, &no_config_log_record_priority_cmd);
 	install_element(CONFIG_NODE, &config_log_timestamp_precision_cmd);
 	install_element(CONFIG_NODE, &no_config_log_timestamp_precision_cmd);
+	install_element(CONFIG_NODE, &config_log_ec_cmd);
+	install_element(CONFIG_NODE, &config_log_xid_cmd);
 
 	install_element(VIEW_NODE, &show_log_filter_cmd);
 	install_element(CONFIG_NODE, &log_filter_cmd);
 	install_element(CONFIG_NODE, &log_filter_clear_cmd);
 	install_element(CONFIG_NODE, &config_log_filterfile_cmd);
 	install_element(CONFIG_NODE, &no_config_log_filterfile_cmd);
+	install_element(CONFIG_NODE, &log_immediate_mode_cmd);
+
+	install_element(ENABLE_NODE, &debug_uid_backtrace_cmd);
+	install_element(CONFIG_NODE, &debug_uid_backtrace_cmd);
+
+	log_5424_cmd_init();
 }
