@@ -83,7 +83,7 @@ struct qobj_nodetype {
 	RESERVED_SPACE_STRUCT(qobj_nodetype_capnp, capnp, 256)
 };
 
-PREDECL_HASH(qobj_nodes)
+PREDECL_HASH(qobj_nodes);
 
 /* anchor to be embedded somewhere in the object's struct */
 struct qobj_node {
@@ -92,7 +92,7 @@ struct qobj_node {
 	const struct qobj_nodetype *type;
 };
 
-#define QOBJ_FIELDS struct qobj_node qobj_node;
+#define QOBJ_FIELDS struct qobj_node qobj_node
 
 /* call these at the end of any _create function (QOBJ_REG)
  * and beginning of any _destroy function (QOBJ_UNREG) */
@@ -118,16 +118,19 @@ void *qobj_get_typed(uint64_t id, const struct qobj_nodetype *type);
 
 /* type declarations */
 #define DECLARE_QOBJ_TYPE(structname)                                          \
-	extern const struct qobj_nodetype qobj_t_##structname;
+	extern const struct qobj_nodetype qobj_t_##structname                  \
+	/* end */
 #define DEFINE_QOBJ_TYPE(structname)                                           \
 	const struct qobj_nodetype qobj_t_##structname = {                     \
 		.node_member_offset =                                          \
-			(ptrdiff_t)offsetof(struct structname, qobj_node)};
+			(ptrdiff_t)offsetof(struct structname, qobj_node)}     \
+	/* end */
 #define DEFINE_QOBJ_TYPE_INIT(structname, ...)                                 \
 	const struct qobj_nodetype qobj_t_##structname = {                     \
 		.node_member_offset =                                          \
 			(ptrdiff_t)offsetof(struct structname, qobj_node),     \
-		__VA_ARGS__};
+		__VA_ARGS__}                                                   \
+	/* end */
 
 /* ID dereference with typecheck.
  * will return NULL if id not found or wrong type. */
