@@ -21,7 +21,7 @@
  */
 
 /*
- * Skip List impementation based on code from William Pugh.
+ * Skip List implementation based on code from William Pugh.
  * ftp://ftp.cs.umd.edu/pub/skipLists/
  */
 
@@ -60,7 +60,7 @@ struct skiplist {
 	int level; /* max lvl (1 + current # of levels in list) */
 	unsigned int count;
 	struct skiplistnode *header;
-	struct skiplistnode *stats;
+	int *level_stats;
 	struct skiplistnode
 		*last; /* last real list item (NULL if empty list) */
 
@@ -123,6 +123,7 @@ extern int skiplist_empty(register struct skiplist *l); /* in */
 
 extern unsigned int skiplist_count(register struct skiplist *l); /* in */
 
+struct vty;
 extern void skiplist_debug(struct vty *vty, struct skiplist *l);
 
 extern void skiplist_test(struct vty *vty);
