@@ -20,15 +20,21 @@
 
 #include <zebra.h>
 
-#ifdef SUNOS_5
-#include <ifaddrs.h>
-#endif
-
 #include "log.h"
 #include "sockopt.h"
 #include "sockunion.h"
 #include "lib_errors.h"
 
+#if (defined(__FreeBSD__)                                                      \
+     && ((__FreeBSD_version >= 500022 && __FreeBSD_version < 700000)           \
+	 || (__FreeBSD_version < 500000 && __FreeBSD_version >= 440000)))      \
+	|| (defined(__NetBSD__) && defined(__NetBSD_Version__)                 \
+	    && __NetBSD_Version__ >= 106010000)                                \
+	|| defined(__OpenBSD__) || defined(__APPLE__)                          \
+	|| defined(__DragonFly__) || defined(__sun)
+#define HAVE_BSD_STRUCT_IP_MREQ_HACK
+#endif
+
 void setsockopt_so_recvbuf(int sock, int size)
 {
 	int orig_req = size;
@@ -38,8 +44,9 @@ void setsockopt_so_recvbuf(int sock, int size)
 		size /= 2;
 
 	if (size != orig_req)
-		zlog_warn("%s: fd %d: SO_RCVBUF set to %d (requested %d)",
-			  __func__, sock, size, orig_req);
+		flog_err(EC_LIB_SOCKET,
+			 "%s: fd %d: SO_RCVBUF set to %d (requested %d)",
+			 __func__, sock, size, orig_req);
 }
 
 void setsockopt_so_sendbuf(const int sock, int size)
@@ -51,8 +58,9 @@ void setsockopt_so_sendbuf(const int sock, int size)
 		size /= 2;
 
 	if (size != orig_req)
-		zlog_warn("%s: fd %d: SO_SNDBUF set to %d (requested %d)",
-			  __func__, sock, size, orig_req);
+		flog_err(EC_LIB_SOCKET,
+			 "%s: fd %d: SO_SNDBUF set to %d (requested %d)",
+			 __func__, sock, size, orig_req);
 }
 
 int getsockopt_so_sendbuf(const int sock)
@@ -62,7 +70,7 @@ int getsockopt_so_sendbuf(const int sock)
 	int ret = getsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&optval,
 			     &optlen);
 	if (ret < 0) {
-		flog_err_sys(LIB_ERR_SYSTEM_CALL,
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
 			     "fd %d: can't getsockopt SO_SNDBUF: %d (%s)", sock,
 			     errno, safe_strerror(errno));
 		return ret;
@@ -70,15 +78,29 @@ int getsockopt_so_sendbuf(const int sock)
 	return optval;
 }
 
+int getsockopt_so_recvbuf(const int sock)
+{
+	uint32_t optval;
+	socklen_t optlen = sizeof(optval);
+	int ret = getsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&optval,
+			     &optlen);
+	if (ret < 0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "fd %d: can't getsockopt SO_RCVBUF: %d (%s)", sock,
+			     errno, safe_strerror(errno));
+		return ret;
+	}
+	return optval;
+}
+
 static void *getsockopt_cmsg_data(struct msghdr *msgh, int level, int type)
 {
 	struct cmsghdr *cmsg;
-	void *ptr = NULL;
 
 	for (cmsg = CMSG_FIRSTHDR(msgh); cmsg != NULL;
 	     cmsg = CMSG_NXTHDR(msgh, cmsg))
 		if (cmsg->cmsg_level == level && cmsg->cmsg_type == type)
-			return (ptr = CMSG_DATA(cmsg));
+			return CMSG_DATA(cmsg);
 
 	return NULL;
 }
@@ -92,29 +114,15 @@ int setsockopt_ipv6_pktinfo(int sock, int val)
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_RECVPKTINFO, &val,
 			 sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_RECVPKTINFO : %s",
-			  safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET,
+			 "can't setsockopt IPV6_RECVPKTINFO : %s",
+			 safe_strerror(errno));
 #else  /*RFC2292*/
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_PKTINFO, &val, sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_PKTINFO : %s",
-			  safe_strerror(errno));
-#endif /* INIA_IPV6 */
-	return ret;
-}
-
-/* Set multicast hops val to the socket. */
-int setsockopt_ipv6_checksum(int sock, int val)
-{
-	int ret;
-
-#ifdef GNU_LINUX
-	ret = setsockopt(sock, IPPROTO_RAW, IPV6_CHECKSUM, &val, sizeof(val));
-#else
-	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_CHECKSUM, &val, sizeof(val));
-#endif /* GNU_LINUX */
-	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_CHECKSUM");
+		flog_err(EC_LIB_SOCKET, "can't setsockopt IPV6_PKTINFO : %s",
+			 safe_strerror(errno));
+#endif /* IANA_IPV6 */
 	return ret;
 }
 
@@ -126,7 +134,7 @@ int setsockopt_ipv6_multicast_hops(int sock, int val)
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &val,
 			 sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_MULTICAST_HOPS");
+		flog_err(EC_LIB_SOCKET, "can't setsockopt IPV6_MULTICAST_HOPS");
 	return ret;
 }
 
@@ -138,7 +146,7 @@ int setsockopt_ipv6_unicast_hops(int sock, int val)
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_UNICAST_HOPS, &val,
 			 sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_UNICAST_HOPS");
+		flog_err(EC_LIB_SOCKET, "can't setsockopt IPV6_UNICAST_HOPS");
 	return ret;
 }
 
@@ -150,11 +158,11 @@ int setsockopt_ipv6_hoplimit(int sock, int val)
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_RECVHOPLIMIT, &val,
 			 sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_RECVHOPLIMIT");
+		flog_err(EC_LIB_SOCKET, "can't setsockopt IPV6_RECVHOPLIMIT");
 #else /*RFC2292*/
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_HOPLIMIT, &val, sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_HOPLIMIT");
+		flog_err(EC_LIB_SOCKET, "can't setsockopt IPV6_HOPLIMIT");
 #endif
 	return ret;
 }
@@ -167,7 +175,7 @@ int setsockopt_ipv6_multicast_loop(int sock, int val)
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, &val,
 			 sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IPV6_MULTICAST_LOOP");
+		flog_err(EC_LIB_SOCKET, "can't setsockopt IPV6_MULTICAST_LOOP");
 	return ret;
 }
 
@@ -188,8 +196,9 @@ int setsockopt_ipv6_tclass(int sock, int tclass)
 	ret = setsockopt(sock, IPPROTO_IPV6, IPV6_TCLASS, &tclass,
 			 sizeof(tclass));
 	if (ret < 0)
-		zlog_warn("Can't set IPV6_TCLASS option for fd %d to %#x: %s",
-			  sock, tclass, safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET,
+			 "Can't set IPV6_TCLASS option for fd %d to %#x: %s",
+			 sock, tclass, safe_strerror(errno));
 #endif
 	return ret;
 }
@@ -259,13 +268,9 @@ int setsockopt_ipv4_multicast(int sock, int optname, struct in_addr if_addr,
 	    && (errno == EADDRINUSE)) {
 		/* see above: handle possible problem when interface comes back
 		 * up */
-		char buf[1][INET_ADDRSTRLEN];
 		zlog_info(
-			"setsockopt_ipv4_multicast attempting to drop and "
-			"re-add (fd %d, mcast %s, ifindex %u)",
-			sock, inet_ntop(AF_INET, &mreqn.imr_multiaddr, buf[0],
-					sizeof(buf[0])),
-			ifindex);
+			"setsockopt_ipv4_multicast attempting to drop and re-add (fd %d, mcast %pI4, ifindex %u)",
+			sock, &mreqn.imr_multiaddr, ifindex);
 		setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP, (void *)&mreqn,
 			   sizeof(mreqn));
 		ret = setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
@@ -301,13 +306,9 @@ int setsockopt_ipv4_multicast(int sock, int optname, struct in_addr if_addr,
 	    && (errno == EADDRINUSE)) {
 		/* see above: handle possible problem when interface comes back
 		 * up */
-		char buf[1][INET_ADDRSTRLEN];
 		zlog_info(
-			"setsockopt_ipv4_multicast attempting to drop and "
-			"re-add (fd %d, mcast %s, ifindex %u)",
-			sock, inet_ntop(AF_INET, &mreq.imr_multiaddr, buf[0],
-					sizeof(buf[0])),
-			ifindex);
+			"setsockopt_ipv4_multicast attempting to drop and re-add (fd %d, mcast %pI4, ifindex %u)",
+			sock, &mreq.imr_multiaddr, ifindex);
 		setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP, (void *)&mreq,
 			   sizeof(mreq));
 		ret = setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
@@ -350,35 +351,6 @@ int setsockopt_ipv4_multicast_if(int sock, struct in_addr if_addr,
 
 	return setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, (void *)&m,
 			  sizeof(m));
-#elif defined(SUNOS_5)
-	char ifname[IF_NAMESIZE];
-	struct ifaddrs *ifa, *ifap;
-	struct in_addr ifaddr;
-
-	if (if_indextoname(ifindex, ifname) == NULL)
-		return -1;
-
-	if (getifaddrs(&ifa) != 0)
-		return -1;
-
-	for (ifap = ifa; ifap != NULL; ifap = ifap->ifa_next) {
-		struct sockaddr_in *sa;
-
-		if (strcmp(ifap->ifa_name, ifname) != 0)
-			continue;
-		if (ifap->ifa_addr->sa_family != AF_INET)
-			continue;
-		sa = (struct sockaddr_in *)ifap->ifa_addr;
-		memcpy(&ifaddr, &sa->sin_addr, sizeof(ifaddr));
-		break;
-	}
-
-	freeifaddrs(ifa);
-	if (!ifap) /* This means we did not find an IP */
-		return -1;
-
-	return setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, (void *)&ifaddr,
-			  sizeof(ifaddr));
 #else
 #error "Unsupported multicast API"
 #endif
@@ -391,7 +363,7 @@ int setsockopt_ipv4_multicast_loop(int sock, uint8_t val)
 	ret = setsockopt(sock, IPPROTO_IP, IP_MULTICAST_LOOP, (void *)&val,
 			 sizeof(val));
 	if (ret < 0)
-		zlog_warn("can't setsockopt IP_MULTICAST_LOOP");
+		flog_err(EC_LIB_SOCKET, "can't setsockopt IP_MULTICAST_LOOP");
 
 	return ret;
 }
@@ -401,15 +373,17 @@ static int setsockopt_ipv4_ifindex(int sock, ifindex_t val)
 	int ret;
 
 #if defined(IP_PKTINFO)
-	if ((ret = setsockopt(sock, IPPROTO_IP, IP_PKTINFO, &val, sizeof(val)))
-	    < 0)
-		zlog_warn("Can't set IP_PKTINFO option for fd %d to %d: %s",
-			  sock, val, safe_strerror(errno));
+	ret = setsockopt(sock, IPPROTO_IP, IP_PKTINFO, &val, sizeof(val));
+	if (ret < 0)
+		flog_err(EC_LIB_SOCKET,
+			 "Can't set IP_PKTINFO option for fd %d to %d: %s",
+			 sock, val, safe_strerror(errno));
 #elif defined(IP_RECVIF)
-	if ((ret = setsockopt(sock, IPPROTO_IP, IP_RECVIF, &val, sizeof(val)))
-	    < 0)
-		zlog_warn("Can't set IP_RECVIF option for fd %d to %d: %s",
-			  sock, val, safe_strerror(errno));
+	ret = setsockopt(sock, IPPROTO_IP, IP_RECVIF, &val, sizeof(val));
+	if (ret < 0)
+		flog_err(EC_LIB_SOCKET,
+			 "Can't set IP_RECVIF option for fd %d to %d: %s", sock,
+			 val, safe_strerror(errno));
 #else
 #warning "Neither IP_PKTINFO nor IP_RECVIF is available."
 #warning "Will not be able to receive link info."
@@ -427,8 +401,9 @@ int setsockopt_ipv4_tos(int sock, int tos)
 
 	ret = setsockopt(sock, IPPROTO_IP, IP_TOS, &tos, sizeof(tos));
 	if (ret < 0)
-		zlog_warn("Can't set IP_TOS option for fd %d to %#x: %s", sock,
-			  tos, safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET,
+			 "Can't set IP_TOS option for fd %d to %#x: %s", sock,
+			 tos, safe_strerror(errno));
 	return ret;
 }
 
@@ -445,7 +420,8 @@ int setsockopt_ifindex(int af, int sock, ifindex_t val)
 		ret = setsockopt_ipv6_pktinfo(sock, val);
 		break;
 	default:
-		zlog_warn("setsockopt_ifindex: unknown address family %d", af);
+		flog_err(EC_LIB_DEVELOPMENT,
+			 "setsockopt_ifindex: unknown address family %d", af);
 	}
 	return ret;
 }
@@ -478,15 +454,9 @@ static ifindex_t getsockopt_ipv4_ifindex(struct msghdr *msgh)
 
 /* retrieval based on IP_RECVIF */
 
-#ifndef SUNOS_5
 	/* BSD systems use a sockaddr_dl as the control message payload. */
 	struct sockaddr_dl *sdl;
-#else
-	/* SUNOS_5 uses an integer with the index. */
-	ifindex_t *ifindex_p;
-#endif /* SUNOS_5 */
 
-#ifndef SUNOS_5
 	/* BSD */
 	sdl = (struct sockaddr_dl *)getsockopt_cmsg_data(msgh, IPPROTO_IP,
 							 IP_RECVIF);
@@ -494,18 +464,6 @@ static ifindex_t getsockopt_ipv4_ifindex(struct msghdr *msgh)
 		ifindex = sdl->sdl_index;
 	else
 		ifindex = 0;
-#else
-	/*
-	 * Solaris.  On Solaris 8, IP_RECVIF is defined, but the call to
-	 * enable it fails with errno=99, and the struct msghdr has
-	 * controllen 0.
-	 */
-	ifindex_p = (uint_t *)getsockopt_cmsg_data(msgh, IPPROTO_IP, IP_RECVIF);
-	if (ifindex_p != NULL)
-		ifindex = *ifindex_p;
-	else
-		ifindex = 0;
-#endif /* SUNOS_5 */
 
 #else
 /*
@@ -529,12 +487,11 @@ ifindex_t getsockopt_ifindex(int af, struct msghdr *msgh)
 	switch (af) {
 	case AF_INET:
 		return (getsockopt_ipv4_ifindex(msgh));
-		break;
 	case AF_INET6:
 		return (getsockopt_ipv6_ifindex(msgh));
-		break;
 	default:
-		zlog_warn("getsockopt_ifindex: unknown address family %d", af);
+		flog_err(EC_LIB_DEVELOPMENT,
+			 "getsockopt_ifindex: unknown address family %d", af);
 		return 0;
 	}
 }
@@ -578,10 +535,30 @@ int sockopt_tcp_rtt(int sock)
 #endif
 }
 
-int sockopt_tcp_signature(int sock, union sockunion *su, const char *password)
+int sockopt_tcp_signature_ext(int sock, union sockunion *su, uint16_t prefixlen,
+			      const char *password)
 {
+#ifndef HAVE_DECL_TCP_MD5SIG
+	/*
+	 * We have been asked to enable MD5 auth for an address, but our
+	 * platform doesn't support that
+	 */
+	return -2;
+#endif
+
+#ifndef TCP_MD5SIG_EXT
+	/*
+	 * We have been asked to enable MD5 auth for a prefix, but our platform
+	 * doesn't support that
+	 */
+	if (prefixlen > 0)
+		return -2;
+#endif
+
 #if HAVE_DECL_TCP_MD5SIG
 	int ret;
+
+	int optname = TCP_MD5SIG;
 #ifndef GNU_LINUX
 	/*
 	 * XXX Need to do PF_KEY operation here to add/remove an SA entry,
@@ -612,7 +589,7 @@ int sockopt_tcp_signature(int sock, union sockunion *su, const char *password)
 
 		/* If this does not work, then all users of this sockopt will
 		 * need to
-		 * differentiate between IPv4 and IPv6, and keep seperate
+		 * differentiate between IPv4 and IPv6, and keep separate
 		 * sockets for
 		 * each.
 		 *
@@ -634,27 +611,85 @@ int sockopt_tcp_signature(int sock, union sockunion *su, const char *password)
 
 	memset(&md5sig, 0, sizeof(md5sig));
 	memcpy(&md5sig.tcpm_addr, su2, sizeof(*su2));
+
 	md5sig.tcpm_keylen = keylen;
 	if (keylen)
 		memcpy(md5sig.tcpm_key, password, keylen);
 	sockunion_free(susock);
+
+	/*
+	 * Handle support for MD5 signatures on prefixes, if available and
+	 * requested. Technically the #ifdef check below is not needed because
+	 * if prefixlen > 0 and we don't have support for this feature we would
+	 * have already returned by now, but leaving it there to be explicit.
+	 */
+#ifdef TCP_MD5SIG_EXT
+	if (prefixlen > 0) {
+		md5sig.tcpm_prefixlen = prefixlen;
+		md5sig.tcpm_flags = TCP_MD5SIG_FLAG_PREFIX;
+		optname = TCP_MD5SIG_EXT;
+	}
+#endif /* TCP_MD5SIG_EXT */
+
 #endif /* GNU_LINUX */
-	if ((ret = setsockopt(sock, IPPROTO_TCP, TCP_MD5SIG, &md5sig,
-			      sizeof md5sig))
-	    < 0) {
-		/* ENOENT is harmless.  It is returned when we clear a password
-		   for which
-		   one was not previously set. */
+
+	ret = setsockopt(sock, IPPROTO_TCP, optname, &md5sig, sizeof(md5sig));
+	if (ret < 0) {
 		if (ENOENT == errno)
 			ret = 0;
 		else
 			flog_err_sys(
-				LIB_ERR_SYSTEM_CALL,
+				EC_LIB_SYSTEM_CALL,
 				"sockopt_tcp_signature: setsockopt(%d): %s",
 				sock, safe_strerror(errno));
 	}
 	return ret;
-#else  /* HAVE_TCP_MD5SIG */
+#endif /* HAVE_TCP_MD5SIG */
+
+	/*
+	 * Making compiler happy.  If we get to this point we probably
+	 * have done something really really wrong.
+	 */
 	return -2;
-#endif /* !HAVE_TCP_MD5SIG */
+}
+
+int sockopt_tcp_signature(int sock, union sockunion *su, const char *password)
+{
+	return sockopt_tcp_signature_ext(sock, su, 0, password);
+}
+
+/* set TCP mss value to socket */
+int sockopt_tcp_mss_set(int sock, int tcp_maxseg)
+{
+	int ret = 0;
+	socklen_t tcp_maxseg_len = sizeof(tcp_maxseg);
+
+	ret = setsockopt(sock, IPPROTO_TCP, TCP_MAXSEG, &tcp_maxseg,
+			 tcp_maxseg_len);
+	if (ret != 0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: setsockopt(%d): %s", __func__, sock,
+			     safe_strerror(errno));
+	}
+
+	return ret;
+}
+
+/* get TCP mss value synced by socket */
+int sockopt_tcp_mss_get(int sock)
+{
+	int ret = 0;
+	int tcp_maxseg = 0;
+	socklen_t tcp_maxseg_len = sizeof(tcp_maxseg);
+
+	ret = getsockopt(sock, IPPROTO_TCP, TCP_MAXSEG, &tcp_maxseg,
+			 &tcp_maxseg_len);
+	if (ret != 0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: getsockopt(%d): %s", __func__, sock,
+			     safe_strerror(errno));
+		return 0;
+	}
+
+	return tcp_maxseg;
 }
