@@ -31,8 +31,8 @@
 #include "frr_pthread.h"
 #include "lib_errors.h"
 
-DEFINE_MTYPE_STATIC(LIB, STREAM, "Stream")
-DEFINE_MTYPE_STATIC(LIB, STREAM_FIFO, "Stream FIFO")
+DEFINE_MTYPE_STATIC(LIB, STREAM, "Stream");
+DEFINE_MTYPE_STATIC(LIB, STREAM_FIFO, "Stream FIFO");
 
 /* Tests whether a position is valid */
 #define GETP_VALID(S, G) ((G) <= (S)->endp)
@@ -57,7 +57,7 @@ DEFINE_MTYPE_STATIC(LIB, STREAM_FIFO, "Stream FIFO")
 #define STREAM_WARN_OFFSETS(S)                                                 \
 	do {                                                                   \
 		flog_warn(EC_LIB_STREAM,				       \
-			  "&(struct stream): %p, size: %lu, getp: %lu, endp: %lu\n", \
+			  "&(struct stream): %p, size: %lu, getp: %lu, endp: %lu", \
 			  (void *)(S), (unsigned long)(S)->size,	       \
 			  (unsigned long)(S)->getp, (unsigned long)(S)->endp); \
 		zlog_backtrace(LOG_WARNING);				       \
@@ -93,7 +93,7 @@ DEFINE_MTYPE_STATIC(LIB, STREAM_FIFO, "Stream FIFO")
 		if (((S)->endp + (Z)) > (S)->size) {                           \
 			flog_warn(                                             \
 				EC_LIB_STREAM,                                 \
-				"CHECK_SIZE: truncating requested size %lu\n", \
+				"CHECK_SIZE: truncating requested size %lu",   \
 				(unsigned long)(Z));                           \
 			STREAM_WARN_OFFSETS(S);                                \
 			(Z) = (S)->size - (S)->endp;                           \
@@ -145,8 +145,7 @@ struct stream *stream_dup(const struct stream *s)
 
 	STREAM_VERIFY_SANE(s);
 
-	if ((snew = stream_new(s->endp)) == NULL)
-		return NULL;
+	snew = stream_new(s->endp);
 
 	return (stream_copy(snew, s));
 }
@@ -691,7 +690,7 @@ double stream_getd(struct stream *s)
 	return u.r;
 }
 
-/* Copy to source to stream.
+/* Copy from source to stream.
  *
  * XXX: This uses CHECK_SIZE and hence has funny semantics -> Size will wrap
  * around. This should be fixed once the stream updates are working.
@@ -990,7 +989,7 @@ int stream_put_in6_addr_at(struct stream *s, size_t putp,
 
 /* Put prefix by nlri type format. */
 int stream_put_prefix_addpath(struct stream *s, const struct prefix *p,
-			      int addpath_encode, uint32_t addpath_tx_id)
+			      bool addpath_capable, uint32_t addpath_tx_id)
 {
 	size_t psize;
 	size_t psize_with_addpath;
@@ -999,7 +998,7 @@ int stream_put_prefix_addpath(struct stream *s, const struct prefix *p,
 
 	psize = PSIZE(p->prefixlen);
 
-	if (addpath_encode)
+	if (addpath_capable)
 		psize_with_addpath = psize + 4;
 	else
 		psize_with_addpath = psize;
@@ -1009,7 +1008,7 @@ int stream_put_prefix_addpath(struct stream *s, const struct prefix *p,
 		return 0;
 	}
 
-	if (addpath_encode) {
+	if (addpath_capable) {
 		s->data[s->endp++] = (uint8_t)(addpath_tx_id >> 24);
 		s->data[s->endp++] = (uint8_t)(addpath_tx_id >> 16);
 		s->data[s->endp++] = (uint8_t)(addpath_tx_id >> 8);
@@ -1030,7 +1029,7 @@ int stream_put_prefix(struct stream *s, const struct prefix *p)
 
 /* Put NLRI with label */
 int stream_put_labeled_prefix(struct stream *s, const struct prefix *p,
-			      mpls_label_t *label, int addpath_encode,
+			      mpls_label_t *label, bool addpath_capable,
 			      uint32_t addpath_tx_id)
 {
 	size_t psize;
@@ -1040,14 +1039,14 @@ int stream_put_labeled_prefix(struct stream *s, const struct prefix *p,
 	STREAM_VERIFY_SANE(s);
 
 	psize = PSIZE(p->prefixlen);
-	psize_with_addpath = psize + (addpath_encode ? 4 : 0);
+	psize_with_addpath = psize + (addpath_capable ? 4 : 0);
 
 	if (STREAM_WRITEABLE(s) < (psize_with_addpath + 3)) {
 		STREAM_BOUND_WARN(s, "put");
 		return 0;
 	}
 
-	if (addpath_encode) {
+	if (addpath_capable) {
 		s->data[s->endp++] = (uint8_t)(addpath_tx_id >> 24);
 		s->data[s->endp++] = (uint8_t)(addpath_tx_id >> 16);
 		s->data[s->endp++] = (uint8_t)(addpath_tx_id >> 8);
@@ -1097,7 +1096,8 @@ ssize_t stream_read_try(struct stream *s, int fd, size_t size)
 		return -1;
 	}
 
-	if ((nbytes = read(fd, s->data + s->endp, size)) >= 0) {
+	nbytes = read(fd, s->data + s->endp, size);
+	if (nbytes >= 0) {
 		s->endp += nbytes;
 		return nbytes;
 	}
@@ -1126,9 +1126,8 @@ ssize_t stream_recvfrom(struct stream *s, int fd, size_t size, int flags,
 		return -1;
 	}
 
-	if ((nbytes = recvfrom(fd, s->data + s->endp, size, flags, from,
-			       fromlen))
-	    >= 0) {
+	nbytes = recvfrom(fd, s->data + s->endp, size, flags, from, fromlen);
+	if (nbytes >= 0) {
 		s->endp += nbytes;
 		return nbytes;
 	}
@@ -1281,7 +1280,7 @@ void stream_fifo_push(struct stream_fifo *fifo, struct stream *s)
 
 void stream_fifo_push_safe(struct stream_fifo *fifo, struct stream *s)
 {
-	frr_with_mutex(&fifo->mtx) {
+	frr_with_mutex (&fifo->mtx) {
 		stream_fifo_push(fifo, s);
 	}
 }
@@ -1313,7 +1312,7 @@ struct stream *stream_fifo_pop_safe(struct stream_fifo *fifo)
 {
 	struct stream *ret;
 
-	frr_with_mutex(&fifo->mtx) {
+	frr_with_mutex (&fifo->mtx) {
 		ret = stream_fifo_pop(fifo);
 	}
 
@@ -1329,7 +1328,7 @@ struct stream *stream_fifo_head_safe(struct stream_fifo *fifo)
 {
 	struct stream *ret;
 
-	frr_with_mutex(&fifo->mtx) {
+	frr_with_mutex (&fifo->mtx) {
 		ret = stream_fifo_head(fifo);
 	}
 
@@ -1351,7 +1350,7 @@ void stream_fifo_clean(struct stream_fifo *fifo)
 
 void stream_fifo_clean_safe(struct stream_fifo *fifo)
 {
-	frr_with_mutex(&fifo->mtx) {
+	frr_with_mutex (&fifo->mtx) {
 		stream_fifo_clean(fifo);
 	}
 }
@@ -1372,3 +1371,19 @@ void stream_fifo_free(struct stream_fifo *fifo)
 	stream_fifo_deinit(fifo);
 	XFREE(MTYPE_STREAM_FIFO, fifo);
 }
+
+void stream_pulldown(struct stream *s)
+{
+	size_t rlen = STREAM_READABLE(s);
+
+	/* No more data, so just move the pointers. */
+	if (rlen == 0) {
+		stream_reset(s);
+		return;
+	}
+
+	/* Move the available data to the beginning. */
+	memmove(s->data, &s->data[s->getp], rlen);
+	s->getp = 0;
+	s->endp = rlen;
+}
