@@ -21,9 +21,6 @@
 
 #include <zebra.h>
 
-/* for basename */
-#include <libgen.h>
-
 #include "if.h"
 #include "vrf.h"
 #include "vrf_int.h"
@@ -42,10 +39,10 @@
 /* default VRF name value used when VRF backend is not NETNS */
 #define VRF_DEFAULT_NAME_INTERNAL "default"
 
-DEFINE_MTYPE_STATIC(LIB, VRF, "VRF")
-DEFINE_MTYPE_STATIC(LIB, VRF_BITMAP, "VRF bit-map")
+DEFINE_MTYPE_STATIC(LIB, VRF, "VRF");
+DEFINE_MTYPE_STATIC(LIB, VRF_BITMAP, "VRF bit-map");
 
-DEFINE_QOBJ_TYPE(vrf)
+DEFINE_QOBJ_TYPE(vrf);
 
 static __inline int vrf_id_compare(const struct vrf *, const struct vrf *);
 static __inline int vrf_name_compare(const struct vrf *, const struct vrf *);
@@ -58,7 +55,6 @@ struct vrf_name_head vrfs_by_name = RB_INITIALIZER(&vrfs_by_name);
 
 static int vrf_backend;
 static int vrf_backend_configured;
-static struct zebra_privs_t *vrf_daemon_privs;
 static char vrf_default_name[VRF_NAMSIZ] = VRF_DEFAULT_NAME_INTERNAL;
 
 /*
@@ -73,7 +69,6 @@ static struct vrf_master {
 	int (*vrf_delete_hook)(struct vrf *);
 	int (*vrf_enable_hook)(struct vrf *);
 	int (*vrf_disable_hook)(struct vrf *);
-	int (*vrf_update_name_hook)(struct vrf *vrf);
 } vrf_master = {
 	0,
 };
@@ -98,28 +93,6 @@ static int vrf_name_compare(const struct vrf *a, const struct vrf *b)
 	return strcmp(a->name, b->name);
 }
 
-/* if ns_id is different and not VRF_UNKNOWN,
- * then update vrf identifier, and enable VRF
- */
-static void vrf_update_vrf_id(ns_id_t ns_id, void *opaqueptr)
-{
-	ns_id_t vrf_id = (vrf_id_t)ns_id;
-	vrf_id_t old_vrf_id;
-	struct vrf *vrf = (struct vrf *)opaqueptr;
-
-	if (!vrf)
-		return;
-	old_vrf_id = vrf->vrf_id;
-	if (vrf_id == vrf->vrf_id)
-		return;
-	if (vrf->vrf_id != VRF_UNKNOWN)
-		RB_REMOVE(vrf_id_head, &vrfs_by_id, vrf);
-	vrf->vrf_id = vrf_id;
-	RB_INSERT(vrf_id_head, &vrfs_by_id, vrf);
-	if (old_vrf_id == VRF_UNKNOWN)
-		vrf_enable(vrf);
-}
-
 int vrf_switch_to_netns(vrf_id_t vrf_id)
 {
 	char *name;
@@ -202,8 +175,6 @@ struct vrf *vrf_get(vrf_id_t vrf_id, const char *name)
 			name, NS_NAMSIZ);
 		strlcpy(vrf->name, name, sizeof(vrf->name));
 		RB_INSERT(vrf_name_head, &vrfs_by_name, vrf);
-		if (vrf->vrf_id == VRF_DEFAULT)
-			vrf_set_default_name(vrf->name, false);
 	} else if (name && vrf->name[0] == '\0') {
 		strlcpy(vrf->name, name, sizeof(vrf->name));
 		RB_INSERT(vrf_name_head, &vrfs_by_name, vrf);
@@ -214,6 +185,53 @@ struct vrf *vrf_get(vrf_id_t vrf_id, const char *name)
 	return vrf;
 }
 
+/* Update a VRF. If not found, create one.
+ * Arg:
+ *   name   - The name of the vrf.
+ *   vrf_id - The vrf_id of the vrf.
+ * Description: This function first finds the vrf using its name. If the vrf is
+ * found and the vrf-id of the existing vrf does not match the new vrf id, it
+ * will disable the existing vrf and update it with new vrf-id. If the vrf is
+ * not found, it will create the vrf with given name and the new vrf id.
+ */
+struct vrf *vrf_update(vrf_id_t new_vrf_id, const char *name)
+{
+	struct vrf *vrf = NULL;
+
+	/*Treat VRF add for existing vrf as update
+	 * Update VRF ID and also update in VRF ID table
+	 */
+	if (name)
+		vrf = vrf_lookup_by_name(name);
+	if (vrf && new_vrf_id != VRF_UNKNOWN && vrf->vrf_id != VRF_UNKNOWN
+	    && vrf->vrf_id != new_vrf_id) {
+		if (debug_vrf) {
+			zlog_debug(
+				"Vrf Update event: %s old id: %u, new id: %u",
+				name, vrf->vrf_id, new_vrf_id);
+		}
+
+		/*Disable the vrf to simulate implicit delete
+		 * so that all stale routes are deleted
+		 * This vrf will be enabled down the line
+		 */
+		vrf_disable(vrf);
+
+
+		RB_REMOVE(vrf_id_head, &vrfs_by_id, vrf);
+		vrf->vrf_id = new_vrf_id;
+		RB_INSERT(vrf_id_head, &vrfs_by_id, vrf);
+
+	} else {
+
+		/*
+		 * vrf_get is implied creation if it does not exist
+		 */
+		vrf = vrf_get(new_vrf_id, name);
+	}
+	return vrf;
+}
+
 /* Delete a VRF. This is called when the underlying VRF goes away, a
  * pre-configured VRF is deleted or when shutting down (vrf_terminate()).
  */
@@ -226,31 +244,27 @@ void vrf_delete(struct vrf *vrf)
 	if (vrf_is_enabled(vrf))
 		vrf_disable(vrf);
 
+	if (vrf->vrf_id != VRF_UNKNOWN) {
+		RB_REMOVE(vrf_id_head, &vrfs_by_id, vrf);
+		vrf->vrf_id = VRF_UNKNOWN;
+	}
+
 	/* If the VRF is user configured, it'll stick around, just remove
 	 * the ID mapping. Interfaces assigned to this VRF should've been
 	 * removed already as part of the VRF going down.
 	 */
-	if (vrf_is_user_cfged(vrf)) {
-		if (vrf->vrf_id != VRF_UNKNOWN) {
-			/* Delete any VRF interfaces - should be only
-			 * the VRF itself, other interfaces should've
-			 * been moved out of the VRF.
-			 */
-			if_terminate(vrf);
-			RB_REMOVE(vrf_id_head, &vrfs_by_id, vrf);
-			vrf->vrf_id = VRF_UNKNOWN;
-		}
+	if (vrf_is_user_cfged(vrf))
+		return;
+
+	/* Do not delete the VRF if it has interfaces configured in it. */
+	if (!RB_EMPTY(if_name_head, &vrf->ifaces_by_name))
 		return;
-	}
 
 	if (vrf_master.vrf_delete_hook)
 		(*vrf_master.vrf_delete_hook)(vrf);
 
 	QOBJ_UNREG(vrf);
-	if_terminate(vrf);
 
-	if (vrf->vrf_id != VRF_UNKNOWN)
-		RB_REMOVE(vrf_id_head, &vrfs_by_id, vrf);
 	if (vrf->name[0] != '\0')
 		RB_REMOVE(vrf_name_head, &vrfs_by_name, vrf);
 
@@ -337,28 +351,6 @@ const char *vrf_id_to_name(vrf_id_t vrf_id)
 	return VRF_LOGNAME(vrf);
 }
 
-vrf_id_t vrf_name_to_id(const char *name)
-{
-	struct vrf *vrf;
-	vrf_id_t vrf_id = VRF_DEFAULT; // Pending: need a way to return invalid
-				       // id/ routine not used.
-
-	if (!name)
-		return vrf_id;
-	vrf = vrf_lookup_by_name(name);
-	if (vrf)
-		vrf_id = vrf->vrf_id;
-
-	return vrf_id;
-}
-
-/* Get the data pointer of the specified VRF. If not found, create one. */
-void *vrf_info_get(vrf_id_t vrf_id)
-{
-	struct vrf *vrf = vrf_get(vrf_id, NULL);
-	return vrf->info;
-}
-
 /* Look up the data pointer of the specified VRF. */
 void *vrf_info_lookup(vrf_id_t vrf_id)
 {
@@ -492,8 +484,7 @@ static const struct cmd_variable_handler vrf_var_handlers[] = {
 
 /* Initialize VRF module. */
 void vrf_init(int (*create)(struct vrf *), int (*enable)(struct vrf *),
-	      int (*disable)(struct vrf *), int (*destroy)(struct vrf *),
-	      int ((*update)(struct vrf *)))
+	      int (*disable)(struct vrf *), int (*destroy)(struct vrf *))
 {
 	struct vrf *default_vrf;
 
@@ -506,7 +497,6 @@ void vrf_init(int (*create)(struct vrf *), int (*enable)(struct vrf *),
 	vrf_master.vrf_enable_hook = enable;
 	vrf_master.vrf_disable_hook = disable;
 	vrf_master.vrf_delete_hook = destroy;
-	vrf_master.vrf_update_name_hook = update;
 
 	/* The default VRF always exists. */
 	default_vrf = vrf_get(VRF_DEFAULT, VRF_DEFAULT_NAME);
@@ -535,29 +525,41 @@ void vrf_init(int (*create)(struct vrf *), int (*enable)(struct vrf *),
 	cmd_variable_handler_register(vrf_var_handlers);
 }
 
+static void vrf_terminate_single(struct vrf *vrf)
+{
+	/* Clear configured flag and invoke delete. */
+	vrf_disable(vrf);
+	UNSET_FLAG(vrf->status, VRF_CONFIGURED);
+	if_terminate(vrf);
+	vrf_delete(vrf);
+}
+
 /* Terminate VRF module. */
 void vrf_terminate(void)
 {
-	struct vrf *vrf;
+	struct vrf *vrf, *tmp;
 
 	if (debug_vrf)
 		zlog_debug("%s: Shutting down vrf subsystem", __func__);
 
-	while (!RB_EMPTY(vrf_id_head, &vrfs_by_id)) {
-		vrf = RB_ROOT(vrf_id_head, &vrfs_by_id);
+	RB_FOREACH_SAFE (vrf, vrf_id_head, &vrfs_by_id, tmp) {
+		if (vrf->vrf_id == VRF_DEFAULT)
+			continue;
 
-		/* Clear configured flag and invoke delete. */
-		UNSET_FLAG(vrf->status, VRF_CONFIGURED);
-		vrf_delete(vrf);
+		vrf_terminate_single(vrf);
 	}
 
-	while (!RB_EMPTY(vrf_name_head, &vrfs_by_name)) {
-		vrf = RB_ROOT(vrf_name_head, &vrfs_by_name);
+	RB_FOREACH_SAFE (vrf, vrf_name_head, &vrfs_by_name, tmp) {
+		if (vrf->vrf_id == VRF_DEFAULT)
+			continue;
 
-		/* Clear configured flag and invoke delete. */
-		UNSET_FLAG(vrf->status, VRF_CONFIGURED);
-		vrf_delete(vrf);
+		vrf_terminate_single(vrf);
 	}
+
+	/* Finally terminate default VRF */
+	vrf = vrf_lookup_by_id(VRF_DEFAULT);
+	if (vrf)
+		vrf_terminate_single(vrf);
 }
 
 int vrf_socket(int domain, int type, int protocol, vrf_id_t vrf_id,
@@ -618,124 +620,12 @@ int vrf_configure_backend(enum vrf_backend_type backend)
 	return 0;
 }
 
-int vrf_handler_create(struct vty *vty, const char *vrfname,
-		       struct vrf **vrf)
-{
-	struct vrf *vrfp;
-	char xpath_list[XPATH_MAXLEN];
-	int ret;
-
-	if (strlen(vrfname) > VRF_NAMSIZ) {
-		if (vty)
-			vty_out(vty,
-				"%% VRF name %s invalid: length exceeds %d bytes\n",
-				vrfname, VRF_NAMSIZ);
-		else
-			flog_warn(
-				EC_LIB_VRF_LENGTH,
-				"%% VRF name %s invalid: length exceeds %d bytes\n",
-				vrfname, VRF_NAMSIZ);
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-
-	if (vty) {
-		snprintf(xpath_list, sizeof(xpath_list), FRR_VRF_KEY_XPATH,
-			 vrfname);
-
-		nb_cli_enqueue_change(vty, xpath_list, NB_OP_CREATE, NULL);
-		ret = nb_cli_apply_changes(vty, xpath_list);
-		if (ret == CMD_SUCCESS) {
-			VTY_PUSH_XPATH(VRF_NODE, xpath_list);
-			nb_cli_pending_commit_check(vty);
-			vrfp = vrf_lookup_by_name(vrfname);
-			if (vrfp)
-				VTY_PUSH_CONTEXT(VRF_NODE, vrfp);
-		}
-	} else {
-		vrfp = vrf_get(VRF_UNKNOWN, vrfname);
-
-		if (vrf)
-			*vrf = vrfp;
-	}
-	return CMD_SUCCESS;
-}
-
-int vrf_netns_handler_create(struct vty *vty, struct vrf *vrf, char *pathname,
-			     ns_id_t ns_id, ns_id_t internal_ns_id,
-			     ns_id_t rel_def_ns_id)
-{
-	struct ns *ns = NULL;
-
-	if (!vrf)
-		return CMD_WARNING_CONFIG_FAILED;
-	if (vrf->vrf_id != VRF_UNKNOWN && vrf->ns_ctxt == NULL) {
-		if (vty)
-			vty_out(vty,
-				"VRF %u is already configured with VRF %s\n",
-				vrf->vrf_id, vrf->name);
-		else
-			zlog_info("VRF %u is already configured with VRF %s",
-				  vrf->vrf_id, vrf->name);
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-	if (vrf->ns_ctxt != NULL) {
-		ns = (struct ns *)vrf->ns_ctxt;
-		if (!strcmp(ns->name, pathname)) {
-			if (vty)
-				vty_out(vty,
-					"VRF %u already configured with NETNS %s\n",
-					vrf->vrf_id, ns->name);
-			else
-				zlog_info(
-					"VRF %u already configured with NETNS %s",
-					vrf->vrf_id, ns->name);
-			return CMD_WARNING_CONFIG_FAILED;
-		}
-	}
-	ns = ns_lookup_name(pathname);
-	if (ns && ns->vrf_ctxt) {
-		struct vrf *vrf2 = (struct vrf *)ns->vrf_ctxt;
-
-		if (vrf2 == vrf)
-			return CMD_SUCCESS;
-		if (vty)
-			vty_out(vty,
-				"NS %s is already configured with VRF %u(%s)\n",
-				ns->name, vrf2->vrf_id, vrf2->name);
-		else
-			zlog_info("NS %s is already configured with VRF %u(%s)",
-				  ns->name, vrf2->vrf_id, vrf2->name);
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-	ns = ns_get_created(ns, pathname, ns_id);
-	ns->internal_ns_id = internal_ns_id;
-	ns->relative_default_ns = rel_def_ns_id;
-	ns->vrf_ctxt = (void *)vrf;
-	vrf->ns_ctxt = (void *)ns;
-	/* update VRF netns NAME */
-	strlcpy(vrf->data.l.netns_name, basename(pathname), NS_NAMSIZ);
-
-	if (!ns_enable(ns, vrf_update_vrf_id)) {
-		if (vty)
-			vty_out(vty, "Can not associate NS %u with NETNS %s\n",
-				ns->ns_id, ns->name);
-		else
-			zlog_info("Can not associate NS %u with NETNS %s",
-				  ns->ns_id, ns->name);
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-
-	return CMD_SUCCESS;
-}
-
 /* vrf CLI commands */
 DEFUN_NOSH(vrf_exit,
            vrf_exit_cmd,
 	   "exit-vrf",
 	   "Exit current mode and down to previous mode\n")
 {
-	/* We have to set vrf context to default vrf */
-	VTY_PUSH_CONTEXT(VRF_NODE, vrf_get(VRF_DEFAULT, VRF_DEFAULT_NAME));
 	cmd_exit(vty);
 	return CMD_SUCCESS;
 }
@@ -748,8 +638,29 @@ DEFUN_YANG_NOSH (vrf,
 {
 	int idx_name = 1;
 	const char *vrfname = argv[idx_name]->arg;
+	char xpath_list[XPATH_MAXLEN];
+	struct vrf *vrf;
+	int ret;
 
-	return vrf_handler_create(vty, vrfname, NULL);
+	if (strlen(vrfname) > VRF_NAMSIZ) {
+		vty_out(vty,
+			"%% VRF name %s invalid: length exceeds %d bytes\n",
+			vrfname, VRF_NAMSIZ);
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
+	snprintf(xpath_list, sizeof(xpath_list), FRR_VRF_KEY_XPATH, vrfname);
+
+	nb_cli_enqueue_change(vty, xpath_list, NB_OP_CREATE, NULL);
+	ret = nb_cli_apply_changes_clear_pending(vty, xpath_list);
+	if (ret == CMD_SUCCESS) {
+		VTY_PUSH_XPATH(VRF_NODE, xpath_list);
+		vrf = vrf_lookup_by_name(vrfname);
+		if (vrf)
+			VTY_PUSH_CONTEXT(VRF_NODE, vrf);
+	}
+
+	return ret;
 }
 
 DEFUN_YANG (no_vrf,
@@ -774,10 +685,19 @@ DEFUN_YANG (no_vrf,
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
+	if (vrf_get_backend() == VRF_BACKEND_VRF_LITE) {
+		/*
+		 * Remove the VRF interface config when removing the VRF.
+		 */
+		snprintf(xpath_list, sizeof(xpath_list),
+			 "/frr-interface:lib/interface[name='%s']", vrfname);
+		nb_cli_enqueue_change(vty, xpath_list, NB_OP_DESTROY, NULL);
+	}
+
 	snprintf(xpath_list, sizeof(xpath_list), FRR_VRF_KEY_XPATH, vrfname);
 
 	nb_cli_enqueue_change(vty, xpath_list, NB_OP_DESTROY, NULL);
-	return nb_cli_apply_changes(vty, xpath_list);
+	return nb_cli_apply_changes(vty, NULL);
 }
 
 
@@ -788,62 +708,6 @@ static struct cmd_node vrf_node = {
 	.prompt = "%s(config-vrf)# ",
 };
 
-DEFUN_NOSH (vrf_netns,
-       vrf_netns_cmd,
-       "netns NAME",
-       "Attach VRF to a Namespace\n"
-       "The file name in " NS_RUN_DIR ", or a full pathname\n")
-{
-	int idx_name = 1, ret;
-	char *pathname = ns_netns_pathname(vty, argv[idx_name]->arg);
-
-	VTY_DECLVAR_CONTEXT(vrf, vrf);
-
-	if (!pathname)
-		return CMD_WARNING_CONFIG_FAILED;
-
-	frr_with_privs(vrf_daemon_privs) {
-		ret = vrf_netns_handler_create(vty, vrf, pathname,
-					       NS_UNKNOWN,
-					       NS_UNKNOWN,
-					       NS_UNKNOWN);
-	}
-	return ret;
-}
-
-DEFUN_NOSH (no_vrf_netns,
-	no_vrf_netns_cmd,
-	"no netns [NAME]",
-	NO_STR
-	"Detach VRF from a Namespace\n"
-	"The file name in " NS_RUN_DIR ", or a full pathname\n")
-{
-	struct ns *ns = NULL;
-
-	VTY_DECLVAR_CONTEXT(vrf, vrf);
-
-	if (!vrf_is_backend_netns()) {
-		vty_out(vty, "VRF backend is not Netns. Aborting\n");
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-	if (!vrf->ns_ctxt) {
-		vty_out(vty, "VRF %s(%u) is not configured with NetNS\n",
-			vrf->name, vrf->vrf_id);
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-
-	ns = (struct ns *)vrf->ns_ctxt;
-
-	ns->vrf_ctxt = NULL;
-	vrf_disable(vrf);
-	/* vrf ID from VRF is necessary for Zebra
-	 * so that propagate to other clients is done
-	 */
-	ns_delete(ns);
-	vrf->ns_ctxt = NULL;
-	return CMD_SUCCESS;
-}
-
 /*
  * Debug CLI for vrf's
  */
@@ -896,8 +760,7 @@ void vrf_install_commands(void)
 	install_element(ENABLE_NODE, &no_vrf_debug_cmd);
 }
 
-void vrf_cmd_init(int (*writefunc)(struct vty *vty),
-		  struct zebra_privs_t *daemon_privs)
+void vrf_cmd_init(int (*writefunc)(struct vty *vty))
 {
 	install_element(CONFIG_NODE, &vrf_cmd);
 	install_element(CONFIG_NODE, &no_vrf_cmd);
@@ -905,41 +768,11 @@ void vrf_cmd_init(int (*writefunc)(struct vty *vty),
 	install_node(&vrf_node);
 	install_default(VRF_NODE);
 	install_element(VRF_NODE, &vrf_exit_cmd);
-	if (vrf_is_backend_netns() && ns_have_netns()) {
-		/* Install NS commands. */
-		vrf_daemon_privs = daemon_privs;
-		install_element(VRF_NODE, &vrf_netns_cmd);
-		install_element(VRF_NODE, &no_vrf_netns_cmd);
-	}
 }
 
-void vrf_set_default_name(const char *default_name, bool force)
+void vrf_set_default_name(const char *default_name)
 {
-	struct vrf *def_vrf;
-	static bool def_vrf_forced;
-
-	def_vrf = vrf_lookup_by_id(VRF_DEFAULT);
-	assert(default_name);
-	if (def_vrf && !force && def_vrf_forced) {
-		zlog_debug("VRF: %s, avoid changing name to %s, previously forced (%u)",
-			   def_vrf->name, default_name,
-			   def_vrf->vrf_id);
-		return;
-	}
-	if (strmatch(vrf_default_name, default_name))
-		return;
 	snprintf(vrf_default_name, VRF_NAMSIZ, "%s", default_name);
-	if (def_vrf) {
-		if (force)
-			def_vrf_forced = true;
-		RB_REMOVE(vrf_name_head, &vrfs_by_name, def_vrf);
-		strlcpy(def_vrf->data.l.netns_name,
-			vrf_default_name, NS_NAMSIZ);
-		strlcpy(def_vrf->name, vrf_default_name, sizeof(def_vrf->name));
-		RB_INSERT(vrf_name_head, &vrfs_by_name, def_vrf);
-		if (vrf_master.vrf_update_name_hook)
-			(*vrf_master.vrf_update_name_hook)(def_vrf);
-	}
 }
 
 const char *vrf_get_default_name(void)
@@ -947,25 +780,50 @@ const char *vrf_get_default_name(void)
 	return vrf_default_name;
 }
 
-int vrf_bind(vrf_id_t vrf_id, int fd, const char *name)
+int vrf_bind(vrf_id_t vrf_id, int fd, const char *ifname)
 {
 	int ret = 0;
 	struct interface *ifp;
+	struct vrf *vrf;
+
+	if (fd < 0)
+		return -1;
+
+	if (vrf_id == VRF_UNKNOWN)
+		return -1;
+
+	/* can't bind to a VRF that doesn't exist */
+	vrf = vrf_lookup_by_id(vrf_id);
+	if (!vrf_is_enabled(vrf))
+		return -1;
+
+	if (ifname && strcmp(ifname, vrf->name)) {
+		/* binding to a regular interface */
+
+		/* can't bind to an interface that doesn't exist */
+		ifp = if_lookup_by_name(ifname, vrf_id);
+		if (!ifp)
+			return -1;
+	} else {
+		/* binding to a VRF device */
+
+		/* nothing to do for netns */
+		if (vrf_is_backend_netns())
+			return 0;
+
+		/* nothing to do for default vrf */
+		if (vrf_id == VRF_DEFAULT)
+			return 0;
+
+		ifname = vrf->name;
+	}
 
-	if (fd < 0 || name == NULL)
-		return fd;
-	/* the device should exist
-	 * otherwise we should return
-	 * case ifname = vrf in netns mode => return
-	 */
-	ifp = if_lookup_by_name(name, vrf_id);
-	if (!ifp)
-		return fd;
 #ifdef SO_BINDTODEVICE
-	ret = setsockopt(fd, SOL_SOCKET, SO_BINDTODEVICE, name, strlen(name)+1);
+	ret = setsockopt(fd, SOL_SOCKET, SO_BINDTODEVICE, ifname,
+			 strlen(ifname) + 1);
 	if (ret < 0)
-		zlog_debug("bind to interface %s failed, errno=%d", name,
-			   errno);
+		zlog_err("bind to interface %s failed, errno=%d", ifname,
+			 errno);
 #endif /* SO_BINDTODEVICE */
 	return ret;
 }
@@ -1039,13 +897,6 @@ int vrf_sockunion_socket(const union sockunion *su, vrf_id_t vrf_id,
 	return ret;
 }
 
-vrf_id_t vrf_generate_id(void)
-{
-	static int vrf_id_local;
-
-	return ++vrf_id_local;
-}
-
 /* ------- Northbound callbacks ------- */
 
 /*
@@ -1063,7 +914,7 @@ static int lib_vrf_create(struct nb_cb_create_args *args)
 
 	vrfp = vrf_get(VRF_UNKNOWN, vrfname);
 
-	vrf_set_user_cfged(vrfp);
+	SET_FLAG(vrfp->status, VRF_CONFIGURED);
 	nb_running_set_entry(args->dnode, vrfp);
 
 	return NB_OK;
@@ -1089,7 +940,7 @@ static int lib_vrf_destroy(struct nb_cb_destroy_args *args)
 		vrfp = nb_running_unset_entry(args->dnode);
 
 		/* Clear configured flag and invoke delete. */
-		vrf_reset_user_cfged(vrfp);
+		UNSET_FLAG(vrfp->status, VRF_CONFIGURED);
 		vrf_delete(vrfp);
 		break;
 	}
@@ -1167,7 +1018,8 @@ const struct frr_yang_module_info frr_vrf_info = {
 				.get_next = lib_vrf_get_next,
 				.get_keys = lib_vrf_get_keys,
 				.lookup_entry = lib_vrf_lookup_entry,
-			}
+			},
+			.priority = NB_DFLT_PRIORITY - 2,
 		},
 		{
 			.xpath = "/frr-vrf:lib/vrf/state/id",
