@@ -7,18 +7,23 @@
  * (at your option) any later version.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <netinet/if_ether.h>
 
 #include "zebra.h"
 #include "memory.h"
 #include "thread.h"
 #include "hash.h"
+#include "network.h"
 
 #include "nhrpd.h"
 #include "nhrp_protocol.h"
 #include "os.h"
 
-DEFINE_MTYPE_STATIC(NHRPD, NHRP_PEER, "NHRP peer entry")
+DEFINE_MTYPE_STATIC(NHRPD, NHRP_PEER, "NHRP peer entry");
 
 struct ipv6hdr {
 	uint8_t priority_version;
@@ -39,14 +44,18 @@ static void nhrp_peer_check_delete(struct nhrp_peer *p)
 	if (p->ref || notifier_active(&p->notifier_list))
 		return;
 
+	debugf(NHRP_DEBUG_COMMON, "Deleting peer ref:%d remote:%pSU local:%pSU",
+	       p->ref, &p->vc->remote.nbma, &p->vc->local.nbma);
+
 	THREAD_OFF(p->t_fallback);
+	THREAD_OFF(p->t_timer);
 	hash_release(nifp->peer_hash, p);
 	nhrp_interface_notify_del(p->ifp, &p->ifp_notifier);
 	nhrp_vc_notify_del(p->vc, &p->vc_notifier);
 	XFREE(MTYPE_NHRP_PEER, p);
 }
 
-static int nhrp_peer_notify_up(struct thread *t)
+static void nhrp_peer_notify_up(struct thread *t)
 {
 	struct nhrp_peer *p = THREAD_ARG(t);
 	struct nhrp_vc *vc = p->vc;
@@ -60,8 +69,6 @@ static int nhrp_peer_notify_up(struct thread *t)
 		notifier_call(&p->notifier_list, NOTIFY_PEER_UP);
 		nhrp_peer_unref(p);
 	}
-
-	return 0;
 }
 
 static void __nhrp_peer_check(struct nhrp_peer *p)
@@ -140,6 +147,10 @@ static void nhrp_peer_ifp_notify(struct notifier_block *n, unsigned long cmd)
 	case NOTIFY_INTERFACE_ADDRESS_CHANGED:
 		notifier_call(&p->notifier_list, NOTIFY_PEER_IFCONFIG_CHANGED);
 		break;
+	case NOTIFY_INTERFACE_IPSEC_CHANGED:
+		__nhrp_peer_check(p);
+		notifier_call(&p->notifier_list, NOTIFY_PEER_IFCONFIG_CHANGED);
+		break;
 	case NOTIFY_INTERFACE_MTU_CHANGED:
 		notifier_call(&p->notifier_list, NOTIFY_PEER_MTU_CHANGED);
 		break;
@@ -147,16 +158,17 @@ static void nhrp_peer_ifp_notify(struct notifier_block *n, unsigned long cmd)
 	nhrp_peer_unref(p);
 }
 
-static unsigned int nhrp_peer_key(void *peer_data)
+static unsigned int nhrp_peer_key(const void *peer_data)
 {
-	struct nhrp_peer *p = peer_data;
+	const struct nhrp_peer *p = peer_data;
 	return sockunion_hash(&p->vc->remote.nbma);
 }
 
-static int nhrp_peer_cmp(const void *cache_data, const void *key_data)
+static bool nhrp_peer_cmp(const void *cache_data, const void *key_data)
 {
 	const struct nhrp_peer *a = cache_data;
 	const struct nhrp_peer *b = key_data;
+
 	return a->ifp == b->ifp && a->vc == b->vc;
 }
 
@@ -170,8 +182,7 @@ static void *nhrp_peer_create(void *data)
 		.ref = 0,
 		.ifp = key->ifp,
 		.vc = key->vc,
-		.notifier_list =
-		NOTIFIER_LIST_INITIALIZER(&p->notifier_list),
+		.notifier_list = NOTIFIER_LIST_INITIALIZER(&p->notifier_list),
 	};
 	nhrp_vc_notify_add(p->vc, &p->vc_notifier, nhrp_peer_vc_notify);
 	nhrp_interface_notify_add(p->ifp, &p->ifp_notifier,
@@ -180,6 +191,28 @@ static void *nhrp_peer_create(void *data)
 	return p;
 }
 
+static void do_peer_hash_free(void *hb_data)
+{
+	struct nhrp_peer *p = (struct nhrp_peer *)hb_data;
+
+	nhrp_peer_check_delete(p);
+}
+
+void nhrp_peer_interface_del(struct interface *ifp)
+{
+	struct nhrp_interface *nifp = ifp->info;
+
+	debugf(NHRP_DEBUG_COMMON, "Cleaning up undeleted peer entries (%lu)",
+	       nifp->peer_hash ? nifp->peer_hash->count : 0);
+
+	if (nifp->peer_hash) {
+		hash_clean(nifp->peer_hash, do_peer_hash_free);
+		assert(nifp->peer_hash->count == 0);
+		hash_free(nifp->peer_hash);
+		nifp->peer_hash = NULL;
+	}
+}
+
 struct nhrp_peer *nhrp_peer_get(struct interface *ifp,
 				const union sockunion *remote_nbma)
 {
@@ -224,17 +257,16 @@ void nhrp_peer_unref(struct nhrp_peer *p)
 	}
 }
 
-static int nhrp_peer_request_timeout(struct thread *t)
+static void nhrp_peer_request_timeout(struct thread *t)
 {
 	struct nhrp_peer *p = THREAD_ARG(t);
 	struct nhrp_vc *vc = p->vc;
 	struct interface *ifp = p->ifp;
 	struct nhrp_interface *nifp = ifp->info;
 
-	p->t_fallback = NULL;
 
 	if (p->online)
-		return 0;
+		return;
 
 	if (nifp->ipsec_fallback_profile && !p->prio
 	    && !p->fallback_requested) {
@@ -246,8 +278,29 @@ static int nhrp_peer_request_timeout(struct thread *t)
 	} else {
 		p->requested = p->fallback_requested = 0;
 	}
+}
 
-	return 0;
+static void nhrp_peer_defer_vici_request(struct thread *t)
+{
+	struct nhrp_peer *p = THREAD_ARG(t);
+	struct nhrp_vc *vc = p->vc;
+	struct interface *ifp = p->ifp;
+	struct nhrp_interface *nifp = ifp->info;
+
+	THREAD_OFF(p->t_timer);
+
+	if (p->online) {
+		debugf(NHRP_DEBUG_COMMON,
+		       "IPsec connection to %pSU already established",
+		       &vc->remote.nbma);
+	} else {
+		vici_request_vc(nifp->ipsec_profile, &vc->local.nbma,
+				&vc->remote.nbma, p->prio);
+		thread_add_timer(
+			master, nhrp_peer_request_timeout, p,
+			(nifp->ipsec_fallback_profile && !p->prio) ? 15 : 30,
+			&p->t_fallback);
+	}
 }
 
 int nhrp_peer_check(struct nhrp_peer *p, int establish)
@@ -266,14 +319,30 @@ int nhrp_peer_check(struct nhrp_peer *p, int establish)
 		return 0;
 	if (sockunion_family(&vc->local.nbma) == AF_UNSPEC)
 		return 0;
+	if (vc->ipsec)
+		return 1;
 
 	p->prio = establish > 1;
 	p->requested = 1;
-	vici_request_vc(nifp->ipsec_profile, &vc->local.nbma, &vc->remote.nbma,
-			p->prio);
-	thread_add_timer(master, nhrp_peer_request_timeout, p,
-			 (nifp->ipsec_fallback_profile && !p->prio) ? 15 : 30,
-			 &p->t_fallback);
+
+	/* All NHRP registration requests are prioritized */
+	if (p->prio) {
+		vici_request_vc(nifp->ipsec_profile, &vc->local.nbma,
+				&vc->remote.nbma, p->prio);
+		thread_add_timer(
+			master, nhrp_peer_request_timeout, p,
+			(nifp->ipsec_fallback_profile && !p->prio) ? 15 : 30,
+			&p->t_fallback);
+	} else {
+		/* Maximum timeout is 1 second */
+		int r_time_ms = frr_weak_random() % 1000;
+
+		debugf(NHRP_DEBUG_COMMON,
+		       "Initiating IPsec connection request to %pSU after %d ms:",
+		       &vc->remote.nbma, r_time_ms);
+		thread_add_timer_msec(master, nhrp_peer_defer_vici_request,
+				      p, r_time_ms, &p->t_timer);
+	}
 
 	return 0;
 }
@@ -286,46 +355,103 @@ void nhrp_peer_notify_add(struct nhrp_peer *p, struct notifier_block *n,
 
 void nhrp_peer_notify_del(struct nhrp_peer *p, struct notifier_block *n)
 {
-	notifier_del(n);
+	notifier_del(n, &p->notifier_list);
 	nhrp_peer_check_delete(p);
 }
 
 void nhrp_peer_send(struct nhrp_peer *p, struct zbuf *zb)
 {
-	char buf[2][256];
-
 	nhrp_packet_debug(zb, "Send");
 
 	if (!p->online)
 		return;
 
-	debugf(NHRP_DEBUG_KERNEL, "PACKET: Send %s -> %s",
-	       sockunion2str(&p->vc->local.nbma, buf[0], sizeof buf[0]),
-	       sockunion2str(&p->vc->remote.nbma, buf[1], sizeof buf[1]));
+	debugf(NHRP_DEBUG_KERNEL, "PACKET: Send %pSU -> %pSU",
+	       &p->vc->local.nbma, &p->vc->remote.nbma);
 
 	os_sendmsg(zb->head, zbuf_used(zb), p->ifp->ifindex,
 		   sockunion_get_addr(&p->vc->remote.nbma),
-		   sockunion_get_addrlen(&p->vc->remote.nbma));
+		   sockunion_get_addrlen(&p->vc->remote.nbma), ETH_P_NHRP);
 	zbuf_reset(zb);
 }
 
-static void nhrp_handle_resolution_req(struct nhrp_packet_parser *p)
+static void nhrp_process_nat_extension(struct nhrp_packet_parser *pp,
+				       union sockunion *proto,
+				       union sockunion *cie_nbma)
 {
+	union sockunion cie_proto;
+	struct zbuf payload;
+	struct nhrp_extension_header *ext;
+	struct zbuf *extensions;
+
+	if (!cie_nbma)
+		return;
+
+	sockunion_family(cie_nbma) = AF_UNSPEC;
+
+	if (!proto || sockunion_family(proto) == AF_UNSPEC)
+		return;
+
+	/* Handle extensions */
+	extensions = zbuf_alloc(zbuf_used(&pp->extensions));
+	if (extensions) {
+		zbuf_copy_peek(extensions, &pp->extensions,
+				 zbuf_used(&pp->extensions));
+		while ((ext = nhrp_ext_pull(extensions, &payload)) != NULL) {
+			switch (htons(ext->type)
+				& ~NHRP_EXTENSION_FLAG_COMPULSORY) {
+			case NHRP_EXTENSION_NAT_ADDRESS:
+				/* Process the NBMA and proto address in NAT
+				 * extension and update the cache without which
+				 * the neighbor table in the kernel contains the
+				 * source NBMA address which is not reachable
+				 * since it is behind a NAT device
+				 */
+				debugf(NHRP_DEBUG_COMMON,
+				       "shortcut res_resp: Processing NAT Extension for %pSU",
+				       proto);
+				while (nhrp_cie_pull(&payload, pp->hdr,
+						     cie_nbma, &cie_proto)) {
+					if (sockunion_family(&cie_proto)
+					    == AF_UNSPEC)
+						continue;
+
+					if (!sockunion_cmp(proto, &cie_proto)) {
+						debugf(NHRP_DEBUG_COMMON,
+						       "cie_nbma for proto %pSU is %pSU",
+						       proto, cie_nbma);
+						break;
+					}
+				}
+			}
+		}
+		zbuf_free(extensions);
+	}
+}
+
+static void nhrp_handle_resolution_req(struct nhrp_packet_parser *pp)
+{
+	struct interface *ifp = pp->ifp;
 	struct zbuf *zb, payload;
 	struct nhrp_packet_header *hdr;
 	struct nhrp_cie_header *cie;
 	struct nhrp_extension_header *ext;
-	struct nhrp_interface *nifp;
+	struct nhrp_cache *c;
+	union sockunion cie_nbma, cie_nbma_nat, cie_proto, *proto_addr,
+		*nbma_addr, *claimed_nbma_addr;
+	int holdtime, prefix_len, hostprefix_len;
+	struct nhrp_interface *nifp = ifp->info;
 	struct nhrp_peer *peer;
+	size_t paylen;
 
-	if (!(p->if_ad->flags & NHRP_IFF_SHORTCUT)) {
+	if (!(pp->if_ad->flags & NHRP_IFF_SHORTCUT)) {
 		debugf(NHRP_DEBUG_COMMON, "Shortcuts disabled");
 		/* FIXME: Send error indication? */
 		return;
 	}
 
-	if (p->if_ad->network_id && p->route_type == NHRP_ROUTE_OFF_NBMA
-	    && p->route_prefix.prefixlen < 8) {
+	if (pp->if_ad->network_id && pp->route_type == NHRP_ROUTE_OFF_NBMA
+	    && pp->route_prefix.prefixlen < 8) {
 		debugf(NHRP_DEBUG_COMMON,
 		       "Shortcut to more generic than /8 dropped");
 		return;
@@ -333,66 +459,158 @@ static void nhrp_handle_resolution_req(struct nhrp_packet_parser *p)
 
 	debugf(NHRP_DEBUG_COMMON, "Parsing and replying to Resolution Req");
 
-	if (nhrp_route_address(p->ifp, &p->src_proto, NULL, &peer)
+	if (nhrp_route_address(ifp, &pp->src_proto, NULL, &peer)
 	    != NHRP_ROUTE_NBMA_NEXTHOP)
 		return;
 
-#if 0
-	/* FIXME: Update requestors binding if CIE specifies holding time */
-	nhrp_cache_update_binding(
-			NHRP_CACHE_CACHED, &p->src_proto,
-			nhrp_peer_get(p->ifp, &p->src_nbma),
-			htons(cie->holding_time));
-#endif
+	/* Copy payload CIE */
+	hostprefix_len = 8 * sockunion_get_addrlen(&pp->if_ad->addr);
+	paylen = zbuf_used(&pp->payload);
+	debugf(NHRP_DEBUG_COMMON, "shortcut res_rep: paylen %zu", paylen);
 
-	nifp = peer->ifp->info;
+	while ((cie = nhrp_cie_pull(&pp->payload, pp->hdr, &cie_nbma,
+				    &cie_proto))
+	       != NULL) {
+		prefix_len = cie->prefix_length;
+		debugf(NHRP_DEBUG_COMMON,
+		       "shortcut res_rep: parsing CIE with prefixlen=%u",
+		       prefix_len);
+		if (prefix_len == 0 || prefix_len >= hostprefix_len)
+			prefix_len = hostprefix_len;
+
+		if (prefix_len != hostprefix_len
+		    && !(pp->hdr->flags
+			 & htons(NHRP_FLAG_REGISTRATION_UNIQUE))) {
+			cie->code = NHRP_CODE_BINDING_NON_UNIQUE;
+			continue;
+		}
+
+		/* We currently support only unique prefix registrations */
+		if (prefix_len != hostprefix_len) {
+			cie->code = NHRP_CODE_ADMINISTRATIVELY_PROHIBITED;
+			continue;
+		}
+
+		proto_addr = (sockunion_family(&cie_proto) == AF_UNSPEC)
+				     ? &pp->src_proto
+				     : &cie_proto;
+
+		/* Check for this proto_addr in NHRP_NAT_EXTENSION */
+		nhrp_process_nat_extension(pp, proto_addr, &cie_nbma_nat);
+
+		if (sockunion_family(&cie_nbma_nat) == AF_UNSPEC) {
+			/* It may be possible that this resolution reply is
+			 * coming directly from NATTED Spoke and there is not
+			 * NAT Extension present
+			 */
+			debugf(NHRP_DEBUG_COMMON,
+			       "shortcut res_rep: No NAT Extension for %pSU",
+			       proto_addr);
+
+			if (!sockunion_same(&pp->src_nbma,
+					    &pp->peer->vc->remote.nbma)
+			    && !nhrp_nhs_match_ip(&pp->peer->vc->remote.nbma,
+						  nifp)) {
+				cie_nbma_nat = pp->peer->vc->remote.nbma;
+				debugf(NHRP_DEBUG_COMMON,
+				       "shortcut res_rep: NAT detected using %pSU as cie_nbma",
+				       &cie_nbma_nat);
+			}
+		}
+
+		if (sockunion_family(&cie_nbma_nat) != AF_UNSPEC)
+			nbma_addr = &cie_nbma_nat;
+		else if (sockunion_family(&cie_nbma) != AF_UNSPEC)
+			nbma_addr = &cie_nbma;
+		else
+			nbma_addr = &pp->src_nbma;
+
+		if (sockunion_family(&cie_nbma) != AF_UNSPEC)
+			claimed_nbma_addr = &cie_nbma;
+		else
+			claimed_nbma_addr = &pp->src_nbma;
+
+		holdtime = htons(cie->holding_time);
+		debugf(NHRP_DEBUG_COMMON,
+		       "shortcut res_rep: holdtime is %u (if 0, using %u)",
+		       holdtime, pp->if_ad->holdtime);
+		if (!holdtime)
+			holdtime = pp->if_ad->holdtime;
+
+		c = nhrp_cache_get(ifp, proto_addr, 1);
+		if (!c) {
+			debugf(NHRP_DEBUG_COMMON,
+			       "shortcut res_rep: no cache found");
+			cie->code = NHRP_CODE_INSUFFICIENT_RESOURCES;
+			continue;
+		}
+
+		debugf(NHRP_DEBUG_COMMON,
+		       "shortcut res_rep: updating binding for nmba addr %pSU",
+		       nbma_addr);
+		if (!nhrp_cache_update_binding(
+			    c, NHRP_CACHE_DYNAMIC, holdtime,
+			    nhrp_peer_get(pp->ifp, nbma_addr), htons(cie->mtu),
+			    nbma_addr, claimed_nbma_addr)) {
+			cie->code = NHRP_CODE_ADMINISTRATIVELY_PROHIBITED;
+			continue;
+		}
+
+		cie->code = NHRP_CODE_SUCCESS;
+	}
 
 	/* Create reply */
 	zb = zbuf_alloc(1500);
-	hdr = nhrp_packet_push(zb, NHRP_PACKET_RESOLUTION_REPLY, &p->src_nbma,
-			       &p->src_proto, &p->dst_proto);
+	hdr = nhrp_packet_push(zb, NHRP_PACKET_RESOLUTION_REPLY, &pp->src_nbma,
+			       &pp->src_proto, &pp->dst_proto);
 
 	/* Copied information from request */
-	hdr->flags =
-		p->hdr->flags & htons(NHRP_FLAG_RESOLUTION_SOURCE_IS_ROUTER
-				      | NHRP_FLAG_RESOLUTION_SOURCE_STABLE);
+	hdr->flags = pp->hdr->flags
+		     & htons(NHRP_FLAG_RESOLUTION_SOURCE_IS_ROUTER
+			     | NHRP_FLAG_RESOLUTION_SOURCE_STABLE);
 	hdr->flags |= htons(NHRP_FLAG_RESOLUTION_DESTINATION_STABLE
 			    | NHRP_FLAG_RESOLUTION_AUTHORATIVE);
-	hdr->u.request_id = p->hdr->u.request_id;
+	hdr->u.request_id = pp->hdr->u.request_id;
 
-	/* CIE payload */
+	/* CIE payload for the reply packet */
 	cie = nhrp_cie_push(zb, NHRP_CODE_SUCCESS, &nifp->nbma,
-			    &p->if_ad->addr);
-	cie->holding_time = htons(p->if_ad->holdtime);
-	cie->mtu = htons(p->if_ad->mtu);
-	if (p->if_ad->network_id && p->route_type == NHRP_ROUTE_OFF_NBMA)
-		cie->prefix_length = p->route_prefix.prefixlen;
+			    &pp->if_ad->addr);
+	cie->holding_time = htons(pp->if_ad->holdtime);
+	cie->mtu = htons(pp->if_ad->mtu);
+	if (pp->if_ad->network_id && pp->route_type == NHRP_ROUTE_OFF_NBMA)
+		cie->prefix_length = pp->route_prefix.prefixlen;
 	else
-		cie->prefix_length = 8 * sockunion_get_addrlen(&p->if_ad->addr);
+		cie->prefix_length =
+			8 * sockunion_get_addrlen(&pp->if_ad->addr);
 
 	/* Handle extensions */
-	while ((ext = nhrp_ext_pull(&p->extensions, &payload)) != NULL) {
+	while ((ext = nhrp_ext_pull(&pp->extensions, &payload)) != NULL) {
 		switch (htons(ext->type) & ~NHRP_EXTENSION_FLAG_COMPULSORY) {
 		case NHRP_EXTENSION_NAT_ADDRESS:
-			if (sockunion_family(&nifp->nat_nbma) == AF_UNSPEC)
-				break;
 			ext = nhrp_ext_push(zb, hdr,
 					    NHRP_EXTENSION_NAT_ADDRESS);
 			if (!ext)
 				goto err;
-			cie = nhrp_cie_push(zb, NHRP_CODE_SUCCESS,
-					    &nifp->nat_nbma, &p->if_ad->addr);
-			if (!cie)
-				goto err;
-			nhrp_ext_complete(zb, ext);
+			if (sockunion_family(&nifp->nat_nbma) != AF_UNSPEC) {
+				cie = nhrp_cie_push(zb, NHRP_CODE_SUCCESS,
+						    &nifp->nat_nbma,
+						    &pp->if_ad->addr);
+				if (!cie)
+					goto err;
+				cie->prefix_length =
+					8 * sockunion_get_addrlen(
+							&pp->if_ad->addr);
+
+				cie->mtu = htons(pp->if_ad->mtu);
+				nhrp_ext_complete(zb, ext);
+			}
 			break;
 		default:
-			if (nhrp_ext_reply(zb, hdr, p->ifp, ext, &payload) < 0)
+			if (nhrp_ext_reply(zb, hdr, ifp, ext, &payload) < 0)
 				goto err;
 			break;
 		}
 	}
-
 	nhrp_packet_complete(zb, hdr);
 	nhrp_peer_send(peer, zb);
 err:
@@ -465,7 +683,11 @@ static void nhrp_handle_registration_request(struct nhrp_packet_parser *p)
 				    : &cie_nbma;
 		nbma_natoa = NULL;
 		if (natted) {
-			nbma_natoa = nbma_addr;
+			nbma_natoa =
+				(sockunion_family(&p->peer->vc->remote.nbma)
+				 == AF_UNSPEC)
+					? nbma_addr
+					: &p->peer->vc->remote.nbma;
 		}
 
 		holdtime = htons(cie->holding_time);
@@ -480,7 +702,8 @@ static void nhrp_handle_registration_request(struct nhrp_packet_parser *p)
 
 		if (!nhrp_cache_update_binding(c, NHRP_CACHE_DYNAMIC, holdtime,
 					       nhrp_peer_ref(p->peer),
-					       htons(cie->mtu), nbma_natoa)) {
+					       htons(cie->mtu), nbma_natoa,
+					       nbma_addr)) {
 			cie->code = NHRP_CODE_ADMINISTRATIVELY_PROHIBITED;
 			continue;
 		}
@@ -498,9 +721,13 @@ static void nhrp_handle_registration_request(struct nhrp_packet_parser *p)
 				goto err;
 			zbuf_copy(zb, &payload, zbuf_used(&payload));
 			if (natted) {
-				nhrp_cie_push(zb, NHRP_CODE_SUCCESS,
-					      &p->peer->vc->remote.nbma,
-					      &p->src_proto);
+				cie = nhrp_cie_push(zb, NHRP_CODE_SUCCESS,
+						    &p->peer->vc->remote.nbma,
+						    &p->src_proto);
+				cie->prefix_length =
+					8 * sockunion_get_addrlen(
+						    &p->if_ad->addr);
+				cie->mtu = htons(p->if_ad->mtu);
 			}
 			nhrp_ext_complete(zb, ext);
 			break;
@@ -562,7 +789,6 @@ void nhrp_peer_send_indication(struct interface *ifp, uint16_t protocol_type,
 	struct nhrp_afi_data *if_ad;
 	struct nhrp_packet_header *hdr;
 	struct nhrp_peer *p;
-	char buf[2][SU_ADDRSTRLEN];
 
 	if (!nifp->enabled)
 		return;
@@ -577,23 +803,20 @@ void nhrp_peer_send_indication(struct interface *ifp, uint16_t protocol_type,
 	if_ad = &nifp->afi[family2afi(sockunion_family(&dst))];
 	if (!(if_ad->flags & NHRP_IFF_REDIRECT)) {
 		debugf(NHRP_DEBUG_COMMON,
-		       "Send Traffic Indication to %s about packet to %s ignored",
-		       sockunion2str(&p->vc->remote.nbma, buf[0],
-				     sizeof buf[0]),
-		       sockunion2str(&dst, buf[1], sizeof buf[1]));
+		       "Send Traffic Indication to %pSU about packet to %pSU ignored",
+		       &p->vc->remote.nbma, &dst);
 		return;
 	}
 
 	debugf(NHRP_DEBUG_COMMON,
-	       "Send Traffic Indication to %s (online=%d) about packet to %s",
-	       sockunion2str(&p->vc->remote.nbma, buf[0], sizeof buf[0]),
-	       p->online, sockunion2str(&dst, buf[1], sizeof buf[1]));
+	       "Send Traffic Indication to %pSU (online=%d) about packet to %pSU",
+	       &p->vc->remote.nbma, p->online, &dst);
 
 	/* Create reply */
 	zb = zbuf_alloc(1500);
 	hdr = nhrp_packet_push(zb, NHRP_PACKET_TRAFFIC_INDICATION, &nifp->nbma,
 			       &if_ad->addr, &dst);
-	hdr->hop_count = 0;
+	hdr->hop_count = 1;
 
 	/* Payload is the packet causing indication */
 	zbuf_copy(zb, pkt, zbuf_used(pkt));
@@ -609,16 +832,14 @@ static void nhrp_handle_error_ind(struct nhrp_packet_parser *pp)
 	struct nhrp_packet_header *hdr;
 	struct nhrp_reqid *reqid;
 	union sockunion src_nbma, src_proto, dst_proto;
-	char buf[2][SU_ADDRSTRLEN];
 
 	hdr = nhrp_packet_pull(&origmsg, &src_nbma, &src_proto, &dst_proto);
 	if (!hdr)
 		return;
 
 	debugf(NHRP_DEBUG_COMMON,
-	       "Error Indication from %s about packet to %s ignored",
-	       sockunion2str(&pp->src_proto, buf[0], sizeof buf[0]),
-	       sockunion2str(&dst_proto, buf[1], sizeof buf[1]));
+	       "Error Indication from %pSU about packet to %pSU ignored",
+	       &pp->src_proto, &dst_proto);
 
 	reqid = nhrp_reqid_lookup(&nhrp_packet_reqid, htonl(hdr->u.request_id));
 	if (reqid)
@@ -628,16 +849,14 @@ static void nhrp_handle_error_ind(struct nhrp_packet_parser *pp)
 static void nhrp_handle_traffic_ind(struct nhrp_packet_parser *p)
 {
 	union sockunion dst;
-	char buf[2][SU_ADDRSTRLEN];
 
 	if (!parse_ether_packet(&p->payload, htons(p->hdr->protocol_type), NULL,
 				&dst))
 		return;
 
 	debugf(NHRP_DEBUG_COMMON,
-	       "Traffic Indication from %s about packet to %s: %s",
-	       sockunion2str(&p->src_proto, buf[0], sizeof buf[0]),
-	       sockunion2str(&dst, buf[1], sizeof buf[1]),
+	       "Traffic Indication from %pSU about packet to %pSU: %s",
+	       &p->src_proto, &dst,
 	       (p->if_ad->flags & NHRP_IFF_SHORTCUT) ? "trying shortcut"
 						     : "ignored");
 
@@ -708,20 +927,23 @@ static struct {
 static void nhrp_peer_forward(struct nhrp_peer *p,
 			      struct nhrp_packet_parser *pp)
 {
-	struct zbuf *zb, extpl;
+	struct zbuf *zb, *zb_copy, extpl;
 	struct nhrp_packet_header *hdr;
 	struct nhrp_extension_header *ext, *dst;
 	struct nhrp_cie_header *cie;
 	struct nhrp_interface *nifp = pp->ifp->info;
 	struct nhrp_afi_data *if_ad = pp->if_ad;
-	union sockunion cie_nbma, cie_protocol;
+	union sockunion cie_nbma, cie_protocol, cie_protocol_mandatory, *proto;
 	uint16_t type, len;
+	struct nhrp_cache *c;
 
 	if (pp->hdr->hop_count == 0)
 		return;
 
 	/* Create forward packet - copy header */
 	zb = zbuf_alloc(1500);
+	zb_copy = zbuf_alloc(1500);
+
 	hdr = nhrp_packet_push(zb, pp->hdr->type, &pp->src_nbma, &pp->src_proto,
 			       &pp->dst_proto);
 	hdr->flags = pp->hdr->flags;
@@ -729,8 +951,13 @@ static void nhrp_peer_forward(struct nhrp_peer *p,
 	hdr->u.request_id = pp->hdr->u.request_id;
 
 	/* Copy payload */
+	zbuf_copy_peek(zb_copy, &pp->payload, zbuf_used(&pp->payload));
 	zbuf_copy(zb, &pp->payload, zbuf_used(&pp->payload));
 
+	/* Get CIE Extension from Mandatory part */
+	sockunion_family(&cie_protocol_mandatory) = AF_UNSPEC;
+	nhrp_cie_pull(zb_copy, pp->hdr, &cie_nbma, &cie_protocol_mandatory);
+
 	/* Copy extensions */
 	while ((ext = nhrp_ext_pull(&pp->extensions, &extpl)) != NULL) {
 		type = htons(ext->type) & ~NHRP_EXTENSION_FLAG_COMPULSORY;
@@ -750,10 +977,9 @@ static void nhrp_peer_forward(struct nhrp_peer *p,
 			if ((type == NHRP_EXTENSION_REVERSE_TRANSIT_NHS)
 			    == (packet_types[hdr->type].type == PACKET_REPLY)) {
 				/* Check NHS list for forwarding loop */
-				while ((cie = nhrp_cie_pull(&extpl, pp->hdr,
-							    &cie_nbma,
-							    &cie_protocol))
-				       != NULL) {
+				while (nhrp_cie_pull(&extpl, pp->hdr,
+						     &cie_nbma,
+						     &cie_protocol) != NULL) {
 					if (sockunion_same(&p->vc->remote.nbma,
 							   &cie_nbma))
 						goto err;
@@ -763,19 +989,82 @@ static void nhrp_peer_forward(struct nhrp_peer *p,
 						    &nifp->nbma, &if_ad->addr);
 				if (!cie)
 					goto err;
+				cie->mtu = htons(if_ad->mtu);
 				cie->holding_time = htons(if_ad->holdtime);
 			}
 			break;
+		case NHRP_EXTENSION_NAT_ADDRESS:
+			c = NULL;
+			proto = NULL;
+
+			/* If NAT extension is empty then attempt to populate
+			 * it with cached NBMA information
+			 */
+			if (len == 0) {
+				if (packet_types[hdr->type].type
+				    == PACKET_REQUEST) {
+					debugf(NHRP_DEBUG_COMMON,
+					       "Processing NHRP_EXTENSION_NAT_ADDRESS while forwarding the request packet");
+					proto = &pp->src_proto;
+				} else if (packet_types[hdr->type].type
+					   == PACKET_REPLY) {
+					debugf(NHRP_DEBUG_COMMON,
+					       "Processing NHRP_EXTENSION_NAT_ADDRESS while forwarding the reply packet");
+					/* For reply packet use protocol
+					 * specified in CIE of mandatory part
+					 * for cache lookup
+					 */
+					if (sockunion_family(
+						    &cie_protocol_mandatory)
+					    != AF_UNSPEC)
+						proto = &cie_protocol_mandatory;
+				}
+			}
+
+			if (proto) {
+				debugf(NHRP_DEBUG_COMMON, "Proto is %pSU",
+				       proto);
+				c = nhrp_cache_get(nifp->ifp, proto, 0);
+			}
+
+			if (c) {
+				debugf(NHRP_DEBUG_COMMON,
+				       "c->cur.remote_nbma_natoa is %pSU",
+				       &c->cur.remote_nbma_natoa);
+				if (sockunion_family(&c->cur.remote_nbma_natoa)
+				    != AF_UNSPEC) {
+					cie = nhrp_cie_push(
+						zb,
+						NHRP_CODE_SUCCESS,
+						&c->cur.remote_nbma_natoa,
+						proto);
+					if (!cie)
+						goto err;
+				}
+			} else {
+				if (proto)
+					debugf(NHRP_DEBUG_COMMON,
+					       "No cache entry for proto %pSU",
+					       proto);
+				/* Copy existing NAT extension to new packet if
+				 * either it was already not-empty, or we do not
+				 * have valid cache information
+				 */
+				zbuf_put(zb, extpl.head, len);
+			}
+			break;
 		default:
 			if (htons(ext->type) & NHRP_EXTENSION_FLAG_COMPULSORY)
 				/* FIXME: RFC says to just copy, but not
-				 * append our selves to the transit NHS list */
+				 * append our selves to the transit NHS list
+				 */
 				goto err;
 		/* fallthru */
 		case NHRP_EXTENSION_RESPONDER_ADDRESS:
 			/* Supported compulsory extensions, and any
 			 * non-compulsory that is not explicitly handled,
-			 * should be just copied. */
+			 * should be just copied.
+			 */
 			zbuf_copy(zb, &extpl, len);
 			break;
 		}
@@ -785,15 +1074,16 @@ static void nhrp_peer_forward(struct nhrp_peer *p,
 	nhrp_packet_complete(zb, hdr);
 	nhrp_peer_send(p, zb);
 	zbuf_free(zb);
+	zbuf_free(zb_copy);
 	return;
 err:
 	nhrp_packet_debug(pp->pkt, "FWD-FAIL");
 	zbuf_free(zb);
+	zbuf_free(zb_copy);
 }
 
 static void nhrp_packet_debug(struct zbuf *zb, const char *dir)
 {
-	char buf[2][SU_ADDRSTRLEN];
 	union sockunion src_nbma, src_proto, dst_proto;
 	struct nhrp_packet_header *hdr;
 	struct zbuf zhdr;
@@ -805,13 +1095,12 @@ static void nhrp_packet_debug(struct zbuf *zb, const char *dir)
 	zbuf_init(&zhdr, zb->buf, zb->tail - zb->buf, zb->tail - zb->buf);
 	hdr = nhrp_packet_pull(&zhdr, &src_nbma, &src_proto, &dst_proto);
 
-	sockunion2str(&src_proto, buf[0], sizeof buf[0]);
-	sockunion2str(&dst_proto, buf[1], sizeof buf[1]);
-
 	reply = packet_types[hdr->type].type == PACKET_REPLY;
-	debugf(NHRP_DEBUG_COMMON, "%s %s(%d) %s -> %s", dir,
-	       packet_types[hdr->type].name ?: "Unknown", hdr->type,
-	       reply ? buf[1] : buf[0], reply ? buf[0] : buf[1]);
+	debugf(NHRP_DEBUG_COMMON, "%s %s(%d) %pSU -> %pSU", dir,
+	       (packet_types[hdr->type].name ? packet_types[hdr->type].name
+					     : "Unknown"),
+	       hdr->type, reply ? &dst_proto : &src_proto,
+	       reply ? &src_proto : &dst_proto);
 }
 
 static int proto2afi(uint16_t proto)
@@ -833,7 +1122,6 @@ struct nhrp_route_info {
 
 void nhrp_peer_recv(struct nhrp_peer *p, struct zbuf *zb)
 {
-	char buf[2][SU_ADDRSTRLEN];
 	struct nhrp_packet_header *hdr;
 	struct nhrp_vc *vc = p->vc;
 	struct interface *ifp = p->ifp;
@@ -846,9 +1134,8 @@ void nhrp_peer_recv(struct nhrp_peer *p, struct zbuf *zb)
 	unsigned paylen, extoff, extlen, realsize;
 	afi_t nbma_afi, proto_afi;
 
-	debugf(NHRP_DEBUG_KERNEL, "PACKET: Recv %s -> %s",
-	       sockunion2str(&vc->remote.nbma, buf[0], sizeof buf[0]),
-	       sockunion2str(&vc->local.nbma, buf[1], sizeof buf[1]));
+	debugf(NHRP_DEBUG_KERNEL, "PACKET: Recv %pSU -> %pSU", &vc->remote.nbma,
+	       &vc->local.nbma);
 
 	if (!p->online) {
 		info = "peer not online";
@@ -879,10 +1166,9 @@ void nhrp_peer_recv(struct nhrp_peer *p, struct zbuf *zb)
 	    || packet_types[hdr->type].type == PACKET_UNKNOWN
 	    || htons(hdr->packet_size) > realsize) {
 		zlog_info(
-			"From %s: error: packet type %d, version %d, AFI %d, proto %x, size %d (real size %d)",
-			sockunion2str(&vc->remote.nbma, buf[0], sizeof buf[0]),
-			(int)hdr->type, (int)hdr->version, (int)nbma_afi,
-			(int)htons(hdr->protocol_type),
+			"From %pSU: error: packet type %d, version %d, AFI %d, proto %x, size %d (real size %d)",
+			&vc->remote.nbma, (int)hdr->type, (int)hdr->version,
+			(int)nbma_afi, (int)htons(hdr->protocol_type),
 			(int)htons(hdr->packet_size), (int)realsize);
 		goto drop;
 	}
@@ -890,11 +1176,17 @@ void nhrp_peer_recv(struct nhrp_peer *p, struct zbuf *zb)
 
 	extoff = htons(hdr->extension_offset);
 	if (extoff) {
+		assert(zb->head > zb->buf);
+		uint32_t header_offset = zb->head - zb->buf;
 		if (extoff >= realsize) {
 			info = "extoff larger than packet";
 			goto drop;
 		}
-		paylen = extoff - (zb->head - zb->buf);
+		if (extoff < header_offset) {
+			info = "extoff smaller than header offset";
+			goto drop;
+		}
+		paylen = extoff - header_offset;
 	} else {
 		paylen = zbuf_used(zb);
 	}
@@ -953,10 +1245,7 @@ void nhrp_peer_recv(struct nhrp_peer *p, struct zbuf *zb)
 
 drop:
 	if (info) {
-		zlog_info(
-			"From %s: error: %s",
-			sockunion2str(&vc->remote.nbma, buf[0], sizeof buf[0]),
-			info);
+		zlog_info("From %pSU: error: %s", &vc->remote.nbma, info);
 	}
 	if (peer)
 		nhrp_peer_unref(peer);
