@@ -2,15 +2,16 @@
 #include "hash.h"
 #include "nhrpd.h"
 
-static unsigned int nhrp_reqid_key(void *data)
+static unsigned int nhrp_reqid_key(const void *data)
 {
-	struct nhrp_reqid *r = data;
+	const struct nhrp_reqid *r = data;
 	return r->request_id;
 }
 
-static int nhrp_reqid_cmp(const void *data, const void *key)
+static bool nhrp_reqid_cmp(const void *data, const void *key)
 {
 	const struct nhrp_reqid *a = data, *b = key;
+
 	return a->request_id == b->request_id;
 }
 
@@ -28,7 +29,7 @@ uint32_t nhrp_reqid_alloc(struct nhrp_reqid_pool *p, struct nhrp_reqid *r,
 		if (++p->next_request_id == 0)
 			p->next_request_id = 1;
 		r->cb = cb;
-		hash_get(p->reqid_hash, r, hash_alloc_intern);
+		(void)hash_get(p->reqid_hash, r, hash_alloc_intern);
 	}
 	return r->request_id;
 }
