@@ -47,6 +47,7 @@
 #include "ospf6_flood.h"
 #include "ospf6d.h"
 #include "ospf6_spf.h"
+#include "ospf6_gr.h"
 
 unsigned char conf_debug_ospf6_brouter = 0;
 uint32_t conf_debug_ospf6_brouter_specific_router_id;
@@ -84,7 +85,7 @@ static char *ospf6_router_lsa_get_nbr_id(struct ospf6_lsa *lsa, char *buf,
 					  sizeof(buf1));
 				inet_ntop(AF_INET, &lsdesc->neighbor_router_id,
 					  buf2, sizeof(buf2));
-				sprintf(buf, "%s/%s", buf2, buf1);
+				snprintf(buf, buflen, "%s/%s", buf2, buf1);
 
 				return buf;
 			}
@@ -94,12 +95,15 @@ static char *ospf6_router_lsa_get_nbr_id(struct ospf6_lsa *lsa, char *buf,
 	return NULL;
 }
 
-static int ospf6_router_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
+static int ospf6_router_lsa_show(struct vty *vty, struct ospf6_lsa *lsa,
+				 json_object *json_obj, bool use_json)
 {
 	char *start, *end, *current;
 	char buf[32], name[32], bits[16], options[32];
 	struct ospf6_router_lsa *router_lsa;
 	struct ospf6_router_lsdesc *lsdesc;
+	json_object *json_arr;
+	json_object *json_loop;
 
 	router_lsa =
 		(struct ospf6_router_lsa *)((char *)lsa->header
@@ -107,7 +111,12 @@ static int ospf6_router_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 
 	ospf6_capability_printbuf(router_lsa->bits, bits, sizeof(bits));
 	ospf6_options_printbuf(router_lsa->options, options, sizeof(options));
-	vty_out(vty, "    Bits: %s Options: %s\n", bits, options);
+	if (use_json) {
+		json_object_string_add(json_obj, "bits", bits);
+		json_object_string_add(json_obj, "options", options);
+		json_arr = json_object_new_array();
+	} else
+		vty_out(vty, "    Bits: %s Options: %s\n", bits, options);
 
 	start = (char *)router_lsa + sizeof(struct ospf6_router_lsa);
 	end = (char *)lsa->header + ntohs(lsa->header->length);
@@ -128,18 +137,39 @@ static int ospf6_router_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 			snprintf(name, sizeof(name), "Unknown (%#x)",
 				 lsdesc->type);
 
-		vty_out(vty, "    Type: %s Metric: %d\n", name,
-			ntohs(lsdesc->metric));
-		vty_out(vty, "    Interface ID: %s\n",
-			inet_ntop(AF_INET, &lsdesc->interface_id, buf,
-				  sizeof(buf)));
-		vty_out(vty, "    Neighbor Interface ID: %s\n",
-			inet_ntop(AF_INET, &lsdesc->neighbor_interface_id, buf,
-				  sizeof(buf)));
-		vty_out(vty, "    Neighbor Router ID: %s\n",
-			inet_ntop(AF_INET, &lsdesc->neighbor_router_id, buf,
-				  sizeof(buf)));
+		if (use_json) {
+			json_loop = json_object_new_object();
+			json_object_string_add(json_loop, "type", name);
+			json_object_int_add(json_loop, "metric",
+					    ntohs(lsdesc->metric));
+			json_object_string_addf(
+				json_loop, "interfaceId", "%pI4",
+				(in_addr_t *)&lsdesc->interface_id);
+			json_object_string_addf(
+				json_loop, "neighborInterfaceId", "%pI4",
+				(in_addr_t *)&lsdesc->neighbor_interface_id);
+			json_object_string_addf(json_loop, "neighborRouterId",
+						"%pI4",
+						&lsdesc->neighbor_router_id);
+			json_object_array_add(json_arr, json_loop);
+		} else {
+			vty_out(vty, "    Type: %s Metric: %d\n", name,
+				ntohs(lsdesc->metric));
+			vty_out(vty, "    Interface ID: %s\n",
+				inet_ntop(AF_INET, &lsdesc->interface_id, buf,
+					  sizeof(buf)));
+			vty_out(vty, "    Neighbor Interface ID: %s\n",
+				inet_ntop(AF_INET,
+					  &lsdesc->neighbor_interface_id, buf,
+					  sizeof(buf)));
+			vty_out(vty, "    Neighbor Router ID: %s\n",
+				inet_ntop(AF_INET, &lsdesc->neighbor_router_id,
+					  buf, sizeof(buf)));
+		}
 	}
+	if (use_json)
+		json_object_object_add(json_obj, "lsaDescription", json_arr);
+
 	return 0;
 }
 
@@ -149,7 +179,7 @@ static void ospf6_router_lsa_options_set(struct ospf6_area *oa,
 	OSPF6_OPT_CLEAR_ALL(router_lsa->options);
 	memcpy(router_lsa->options, oa->options, 3);
 
-	if (ospf6_is_router_abr(ospf6))
+	if (ospf6_check_and_set_router_abr(oa->ospf6))
 		SET_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_B);
 	else
 		UNSET_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_B);
@@ -160,6 +190,17 @@ static void ospf6_router_lsa_options_set(struct ospf6_area *oa,
 		UNSET_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_E);
 	}
 
+	/* If the router is ASBR and the area-type is NSSA set the
+	 * translate bit in router LSA.
+	 */
+	if (IS_AREA_NSSA(oa)
+	    && (ospf6_asbr_is_asbr(oa->ospf6) || IS_OSPF6_ABR(oa->ospf6))) {
+		if (oa->NSSATranslatorRole == OSPF6_NSSA_ROLE_ALWAYS)
+			SET_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_NT);
+	} else {
+		UNSET_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_NT);
+	}
+
 	UNSET_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_V);
 	UNSET_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_W);
 }
@@ -183,7 +224,7 @@ int ospf6_router_is_stub_router(struct ospf6_lsa *lsa)
 	return OSPF6_NOT_STUB_ROUTER;
 }
 
-int ospf6_router_lsa_originate(struct thread *thread)
+void ospf6_router_lsa_originate(struct thread *thread)
 {
 	struct ospf6_area *oa;
 
@@ -203,7 +244,13 @@ int ospf6_router_lsa_originate(struct thread *thread)
 	int count;
 
 	oa = (struct ospf6_area *)THREAD_ARG(thread);
-	oa->thread_router_lsa = NULL;
+
+	if (oa->ospf6->gr_info.restart_in_progress) {
+		if (IS_DEBUG_OSPF6_GR)
+			zlog_debug(
+				"Graceful Restart in progress, don't originate LSA");
+		return;
+	}
 
 	if (IS_OSPF6_DEBUG_ORIGINATE(ROUTER))
 		zlog_debug("Originate Router-LSA for Area %s", oa->name);
@@ -245,10 +292,9 @@ int ospf6_router_lsa_originate(struct thread *thread)
 			if ((caddr_t)lsdesc
 			    == (caddr_t)router_lsa
 				       + sizeof(struct ospf6_router_lsa)) {
-				if (IS_OSPF6_DEBUG_ORIGINATE(ROUTER))
-					zlog_debug(
-						"Size limit setting for Router-LSA too short");
-				return 0;
+				zlog_warn(
+					"Size limit setting for Router-LSA too short");
+				return;
 			}
 
 			/* Fill LSA Header */
@@ -387,8 +433,6 @@ int ospf6_router_lsa_originate(struct thread *thread)
 	if (count && !link_state_id)
 		ospf6_spf_schedule(oa->ospf6,
 				   OSPF6_SPF_FLAGS_ROUTER_LSA_ORIGINATED);
-
-	return 0;
 }
 
 /*******************************/
@@ -424,33 +468,48 @@ static char *ospf6_network_lsa_get_ar_id(struct ospf6_lsa *lsa, char *buf,
 	return NULL;
 }
 
-static int ospf6_network_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
+static int ospf6_network_lsa_show(struct vty *vty, struct ospf6_lsa *lsa,
+				  json_object *json_obj, bool use_json)
 {
 	char *start, *end, *current;
 	struct ospf6_network_lsa *network_lsa;
 	struct ospf6_network_lsdesc *lsdesc;
 	char buf[128], options[32];
+	json_object *json_arr;
 
 	network_lsa =
 		(struct ospf6_network_lsa *)((caddr_t)lsa->header
 					     + sizeof(struct ospf6_lsa_header));
 
 	ospf6_options_printbuf(network_lsa->options, options, sizeof(options));
-	vty_out(vty, "     Options: %s\n", options);
+	if (use_json)
+		json_object_string_add(json_obj, "options", options);
+	else
+		vty_out(vty, "     Options: %s\n", options);
 
 	start = (char *)network_lsa + sizeof(struct ospf6_network_lsa);
 	end = (char *)lsa->header + ntohs(lsa->header->length);
+	if (use_json)
+		json_arr = json_object_new_array();
+
 	for (current = start;
 	     current + sizeof(struct ospf6_network_lsdesc) <= end;
 	     current += sizeof(struct ospf6_network_lsdesc)) {
 		lsdesc = (struct ospf6_network_lsdesc *)current;
 		inet_ntop(AF_INET, &lsdesc->router_id, buf, sizeof(buf));
-		vty_out(vty, "     Attached Router: %s\n", buf);
+		if (use_json)
+			json_object_array_add(json_arr,
+					      json_object_new_string(buf));
+		else
+			vty_out(vty, "     Attached Router: %s\n", buf);
 	}
+	if (use_json)
+		json_object_object_add(json_obj, "attachedRouter", json_arr);
+
 	return 0;
 }
 
-int ospf6_network_lsa_originate(struct thread *thread)
+void ospf6_network_lsa_originate(struct thread *thread)
 {
 	struct ospf6_interface *oi;
 
@@ -467,13 +526,19 @@ int ospf6_network_lsa_originate(struct thread *thread)
 	uint16_t type;
 
 	oi = (struct ospf6_interface *)THREAD_ARG(thread);
-	oi->thread_network_lsa = NULL;
 
 	/* The interface must be enabled until here. A Network-LSA of a
 	   disabled interface (but was once enabled) should be flushed
 	   by ospf6_lsa_refresh (), and does not come here. */
 	assert(oi->area);
 
+	if (oi->area->ospf6->gr_info.restart_in_progress) {
+		if (IS_DEBUG_OSPF6_GR)
+			zlog_debug(
+				"Graceful Restart in progress, don't originate LSA");
+		return;
+	}
+
 	old = ospf6_lsdb_lookup(htons(OSPF6_LSTYPE_NETWORK),
 				htonl(oi->interface->ifindex),
 				oi->area->ospf6->router_id, oi->area->lsdb);
@@ -491,7 +556,7 @@ int ospf6_network_lsa_originate(struct thread *thread)
 				oi->area->ospf6,
 				OSPF6_SPF_FLAGS_NETWORK_LSA_ORIGINATED);
 		}
-		return 0;
+		return;
 	}
 
 	if (IS_OSPF6_DEBUG_ORIGINATE(NETWORK))
@@ -510,7 +575,7 @@ int ospf6_network_lsa_originate(struct thread *thread)
 			zlog_debug("Interface stub, ignore");
 		if (old)
 			ospf6_lsa_purge(old);
-		return 0;
+		return;
 	}
 
 	/* prepare buffer */
@@ -568,8 +633,6 @@ int ospf6_network_lsa_originate(struct thread *thread)
 
 	/* Originate */
 	ospf6_lsa_originate_area(lsa, oi->area);
-
-	return 0;
 }
 
 
@@ -628,15 +691,18 @@ static char *ospf6_link_lsa_get_prefix_str(struct ospf6_lsa *lsa, char *buf,
 	return NULL;
 }
 
-static int ospf6_link_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
+static int ospf6_link_lsa_show(struct vty *vty, struct ospf6_lsa *lsa,
+			       json_object *json_obj, bool use_json)
 {
 	char *start, *end, *current;
 	struct ospf6_link_lsa *link_lsa;
 	int prefixnum;
 	char buf[128], options[32];
 	struct ospf6_prefix *prefix;
-	const char *p, *mc, *la, *nu;
 	struct in6_addr in6;
+	json_object *json_loop;
+	json_object *json_arr = NULL;
+	char prefix_string[133];
 
 	link_lsa = (struct ospf6_link_lsa *)((caddr_t)lsa->header
 					     + sizeof(struct ospf6_lsa_header));
@@ -645,10 +711,18 @@ static int ospf6_link_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 	inet_ntop(AF_INET6, &link_lsa->linklocal_addr, buf, sizeof(buf));
 	prefixnum = ntohl(link_lsa->prefix_num);
 
-	vty_out(vty, "     Priority: %d Options: %s\n", link_lsa->priority,
-		options);
-	vty_out(vty, "     LinkLocal Address: %s\n", buf);
-	vty_out(vty, "     Number of Prefix: %d\n", prefixnum);
+	if (use_json) {
+		json_arr = json_object_new_array();
+		json_object_int_add(json_obj, "priority", link_lsa->priority);
+		json_object_string_add(json_obj, "options", options);
+		json_object_string_add(json_obj, "linkLocalAddress", buf);
+		json_object_int_add(json_obj, "numberOfPrefix", prefixnum);
+	} else {
+		vty_out(vty, "     Priority: %d Options: %s\n",
+			link_lsa->priority, options);
+		vty_out(vty, "     LinkLocal Address: %s\n", buf);
+		vty_out(vty, "     Number of Prefix: %d\n", prefixnum);
+	}
 
 	start = (char *)link_lsa + sizeof(struct ospf6_link_lsa);
 	end = (char *)lsa->header + ntohs(lsa->header->length);
@@ -659,33 +733,35 @@ static int ospf6_link_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 		    || current + OSPF6_PREFIX_SIZE(prefix) > end)
 			break;
 
-		p = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_P)
-			     ? "P"
-			     : "--");
-		mc = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_MC)
-			      ? "MC"
-			      : "--");
-		la = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_LA)
-			      ? "LA"
-			      : "--");
-		nu = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_NU)
-			      ? "NU"
-			      : "--");
-		vty_out(vty, "     Prefix Options: %s|%s|%s|%s\n", p, mc, la,
-			nu);
+		ospf6_prefix_options_printbuf(prefix->prefix_options, buf,
+					      sizeof(buf));
+		if (use_json) {
+			json_loop = json_object_new_object();
+			json_object_string_add(json_loop, "prefixOption", buf);
+		} else
+			vty_out(vty, "     Prefix Options: %s\n", buf);
 
 		memset(&in6, 0, sizeof(in6));
 		memcpy(&in6, OSPF6_PREFIX_BODY(prefix),
 		       OSPF6_PREFIX_SPACE(prefix->prefix_length));
 		inet_ntop(AF_INET6, &in6, buf, sizeof(buf));
-		vty_out(vty, "     Prefix: %s/%d\n", buf,
-			prefix->prefix_length);
+		if (use_json) {
+			snprintf(prefix_string, sizeof(prefix_string), "%s/%d",
+				 buf, prefix->prefix_length);
+			json_object_string_add(json_loop, "prefix",
+					       prefix_string);
+			json_object_array_add(json_arr, json_loop);
+		} else
+			vty_out(vty, "     Prefix: %s/%d\n", buf,
+				prefix->prefix_length);
 	}
+	if (use_json)
+		json_object_object_add(json_obj, "prefix", json_arr);
 
 	return 0;
 }
 
-int ospf6_link_lsa_originate(struct thread *thread)
+void ospf6_link_lsa_originate(struct thread *thread)
 {
 	struct ospf6_interface *oi;
 
@@ -698,10 +774,17 @@ int ospf6_link_lsa_originate(struct thread *thread)
 	struct ospf6_prefix *op;
 
 	oi = (struct ospf6_interface *)THREAD_ARG(thread);
-	oi->thread_link_lsa = NULL;
 
 	assert(oi->area);
 
+	if (oi->area->ospf6->gr_info.restart_in_progress) {
+		if (IS_DEBUG_OSPF6_GR)
+			zlog_debug(
+				"Graceful Restart in progress, don't originate LSA");
+		return;
+	}
+
+
 	/* find previous LSA */
 	old = ospf6_lsdb_lookup(htons(OSPF6_LSTYPE_LINK),
 				htonl(oi->interface->ifindex),
@@ -710,7 +793,7 @@ int ospf6_link_lsa_originate(struct thread *thread)
 	if (CHECK_FLAG(oi->flag, OSPF6_INTERFACE_DISABLE)) {
 		if (old)
 			ospf6_lsa_purge(old);
-		return 0;
+		return;
 	}
 
 	if (IS_OSPF6_DEBUG_ORIGINATE(LINK))
@@ -725,7 +808,7 @@ int ospf6_link_lsa_originate(struct thread *thread)
 				oi->interface->name);
 		if (old)
 			ospf6_lsa_purge(old);
-		return 0;
+		return;
 	}
 
 	/* prepare buffer */
@@ -748,7 +831,7 @@ int ospf6_link_lsa_originate(struct thread *thread)
 	for (route = ospf6_route_head(oi->route_connected); route;
 	     route = ospf6_route_next(route)) {
 		op->prefix_length = route->prefix.prefixlen;
-		op->prefix_options = route->path.prefix_options;
+		op->prefix_options = route->prefix_options;
 		op->prefix_metric = htons(0);
 		memcpy(OSPF6_PREFIX_BODY(op), &route->prefix.u.prefix6,
 		       OSPF6_PREFIX_SPACE(op->prefix_length));
@@ -773,8 +856,6 @@ int ospf6_link_lsa_originate(struct thread *thread)
 
 	/* Originate */
 	ospf6_lsa_originate_interface(lsa, oi);
-
-	return 0;
 }
 
 
@@ -790,6 +871,7 @@ static char *ospf6_intra_prefix_lsa_get_prefix_str(struct ospf6_lsa *lsa,
 	struct in6_addr in6;
 	int prefixnum, cnt = 0;
 	struct ospf6_prefix *prefix;
+	char tbuf[16];
 
 	if (lsa) {
 		intra_prefix_lsa =
@@ -822,8 +904,9 @@ static char *ospf6_intra_prefix_lsa_get_prefix_str(struct ospf6_lsa *lsa,
 				       OSPF6_PREFIX_SPACE(
 					       prefix->prefix_length));
 				inet_ntop(AF_INET6, &in6, buf, buflen);
-				sprintf(&buf[strlen(buf)], "/%d",
-					prefix->prefix_length);
+				snprintf(tbuf, sizeof(tbuf), "/%d",
+					 prefix->prefix_length);
+				strlcat(buf, tbuf, buflen);
 				return (buf);
 			}
 		}
@@ -831,7 +914,8 @@ static char *ospf6_intra_prefix_lsa_get_prefix_str(struct ospf6_lsa *lsa,
 	return NULL;
 }
 
-static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
+static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa,
+				       json_object *json_obj, bool use_json)
 {
 	char *start, *end, *current;
 	struct ospf6_intra_prefix_lsa *intra_prefix_lsa;
@@ -839,8 +923,10 @@ static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 	char buf[128];
 	struct ospf6_prefix *prefix;
 	char id[16], adv_router[16];
-	const char *p, *mc, *la, *nu;
 	struct in6_addr in6;
+	json_object *json_loop;
+	json_object *json_arr = NULL;
+	char prefix_string[133];
 
 	intra_prefix_lsa = (struct ospf6_intra_prefix_lsa
 				    *)((caddr_t)lsa->header
@@ -848,13 +934,25 @@ static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 
 	prefixnum = ntohs(intra_prefix_lsa->prefix_num);
 
-	vty_out(vty, "     Number of Prefix: %d\n", prefixnum);
+	if (use_json) {
+		json_arr = json_object_new_array();
+		json_object_int_add(json_obj, "numberOfPrefix", prefixnum);
+	} else
+		vty_out(vty, "     Number of Prefix: %d\n", prefixnum);
 
 	inet_ntop(AF_INET, &intra_prefix_lsa->ref_id, id, sizeof(id));
 	inet_ntop(AF_INET, &intra_prefix_lsa->ref_adv_router, adv_router,
 		  sizeof(adv_router));
-	vty_out(vty, "     Reference: %s Id: %s Adv: %s\n",
-		ospf6_lstype_name(intra_prefix_lsa->ref_type), id, adv_router);
+	if (use_json) {
+		json_object_string_add(
+			json_obj, "reference",
+			ospf6_lstype_name(intra_prefix_lsa->ref_type));
+		json_object_string_add(json_obj, "referenceId", id);
+		json_object_string_add(json_obj, "referenceAdv", adv_router);
+	} else
+		vty_out(vty, "     Reference: %s Id: %s Adv: %s\n",
+			ospf6_lstype_name(intra_prefix_lsa->ref_type), id,
+			adv_router);
 
 	start = (char *)intra_prefix_lsa
 		+ sizeof(struct ospf6_intra_prefix_lsa);
@@ -866,33 +964,40 @@ static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 		    || current + OSPF6_PREFIX_SIZE(prefix) > end)
 			break;
 
-		p = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_P)
-			     ? "P"
-			     : "--");
-		mc = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_MC)
-			      ? "MC"
-			      : "--");
-		la = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_LA)
-			      ? "LA"
-			      : "--");
-		nu = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_NU)
-			      ? "NU"
-			      : "--");
-		vty_out(vty, "     Prefix Options: %s|%s|%s|%s\n", p, mc, la,
-			nu);
+		ospf6_prefix_options_printbuf(prefix->prefix_options, buf,
+					      sizeof(buf));
+		if (use_json) {
+			json_loop = json_object_new_object();
+			json_object_string_add(json_loop, "prefixOption", buf);
+		} else
+			vty_out(vty, "     Prefix Options: %s\n", buf);
 
 		memset(&in6, 0, sizeof(in6));
 		memcpy(&in6, OSPF6_PREFIX_BODY(prefix),
 		       OSPF6_PREFIX_SPACE(prefix->prefix_length));
 		inet_ntop(AF_INET6, &in6, buf, sizeof(buf));
-		vty_out(vty, "     Prefix: %s/%d\n", buf,
-			prefix->prefix_length);
+		if (use_json) {
+			snprintf(prefix_string, sizeof(prefix_string), "%s/%d",
+				 buf, prefix->prefix_length);
+			json_object_string_add(json_loop, "prefix",
+					       prefix_string);
+			json_object_int_add(json_loop, "metric",
+					    ntohs(prefix->prefix_metric));
+			json_object_array_add(json_arr, json_loop);
+		} else {
+			vty_out(vty, "     Prefix: %s/%d\n", buf,
+				prefix->prefix_length);
+			vty_out(vty, "     Metric: %d\n",
+				ntohs(prefix->prefix_metric));
+		}
 	}
+	if (use_json)
+		json_object_object_add(json_obj, "prefix", json_arr);
 
 	return 0;
 }
 
-int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
+void ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 {
 	struct ospf6_area *oa;
 
@@ -908,12 +1013,17 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 	struct listnode *i, *j;
 	int full_count = 0;
 	unsigned short prefix_num = 0;
-	char buf[PREFIX2STR_BUFFER];
 	struct ospf6_route_table *route_advertise;
 	int ls_id = 0;
 
 	oa = (struct ospf6_area *)THREAD_ARG(thread);
-	oa->thread_intra_prefix_lsa = NULL;
+
+	if (oa->ospf6->gr_info.restart_in_progress) {
+		if (IS_DEBUG_OSPF6_GR)
+			zlog_debug(
+				"Graceful Restart in progress, don't originate LSA");
+		return;
+	}
 
 	/* find previous LSA */
 	old = ospf6_lsdb_lookup(htons(OSPF6_LSTYPE_INTRA_PREFIX), htonl(0),
@@ -935,7 +1045,7 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 					oa->lsdb);
 			}
 		}
-		return 0;
+		return;
 	}
 
 	if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
@@ -986,10 +1096,8 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 		/* connected prefix to advertise */
 		for (route = ospf6_route_head(oi->route_connected); route;
 		     route = ospf6_route_best_next(route)) {
-			if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX)) {
-				prefix2str(&route->prefix, buf, sizeof(buf));
-				zlog_debug("    include %s", buf);
-			}
+			if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
+				zlog_debug("    include %pFX", &route->prefix);
 			ospf6_route_add(ospf6_route_copy(route),
 					route_advertise);
 		}
@@ -1013,7 +1121,7 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 			}
 		}
 		ospf6_route_table_delete(route_advertise);
-		return 0;
+		return;
 	}
 
 	/* Neighbor change to FULL, if INTRA-AREA-PREFIX LSA
@@ -1083,7 +1191,7 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 		}
 
 		op->prefix_length = route->prefix.prefixlen;
-		op->prefix_options = route->path.prefix_options;
+		op->prefix_options = route->prefix_options;
 		op->prefix_metric = htons(route->path.cost);
 		memcpy(OSPF6_PREFIX_BODY(op), &route->prefix.u.prefix6,
 		       OSPF6_PREFIX_SPACE(op->prefix_length));
@@ -1098,7 +1206,7 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 		if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
 			zlog_debug(
 				"Quit to Advertise Intra-Prefix: no route to advertise");
-		return 0;
+		return;
 	}
 
 	intra_prefix_lsa->prefix_num = htons(prefix_num);
@@ -1121,12 +1229,10 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 
 	/* Originate */
 	ospf6_lsa_originate_area(lsa, oa);
-
-	return 0;
 }
 
 
-int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
+void ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 {
 	struct ospf6_interface *oi;
 
@@ -1145,13 +1251,18 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 	struct ospf6_link_lsa *link_lsa;
 	char *start, *end, *current;
 	uint16_t type;
-	char buf[PREFIX2STR_BUFFER];
 
 	oi = (struct ospf6_interface *)THREAD_ARG(thread);
-	oi->thread_intra_prefix_lsa = NULL;
 
 	assert(oi->area);
 
+	if (oi->area->ospf6->gr_info.restart_in_progress) {
+		if (IS_DEBUG_OSPF6_GR)
+			zlog_debug(
+				"Graceful Restart in progress, don't originate LSA");
+		return;
+	}
+
 	/* find previous LSA */
 	old = ospf6_lsdb_lookup(htons(OSPF6_LSTYPE_INTRA_PREFIX),
 				htonl(oi->interface->ifindex),
@@ -1160,7 +1271,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 	if (CHECK_FLAG(oi->flag, OSPF6_INTERFACE_DISABLE)) {
 		if (old)
 			ospf6_lsa_purge(old);
-		return 0;
+		return;
 	}
 
 	if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
@@ -1185,7 +1296,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 			zlog_debug("  Interface is not DR");
 		if (old)
 			ospf6_lsa_purge(old);
-		return 0;
+		return;
 	}
 
 	full_count = 0;
@@ -1198,7 +1309,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 			zlog_debug("  Interface is stub");
 		if (old)
 			ospf6_lsa_purge(old);
-		return 0;
+		return;
 	}
 
 	/* connected prefix to advertise */
@@ -1236,7 +1347,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 			    || current + OSPF6_PREFIX_SIZE(op) > end)
 				break;
 
-			route = ospf6_route_create();
+			route = ospf6_route_create(oi->area->ospf6);
 
 			route->type = OSPF6_DEST_TYPE_NETWORK;
 			route->prefix.family = AF_INET6;
@@ -1245,6 +1356,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 			       sizeof(struct in6_addr));
 			memcpy(&route->prefix.u.prefix6, OSPF6_PREFIX_BODY(op),
 			       OSPF6_PREFIX_SPACE(op->prefix_length));
+			route->prefix_options = op->prefix_options;
 
 			route->path.origin.type = lsa->header->type;
 			route->path.origin.id = lsa->header->id;
@@ -1252,14 +1364,11 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 			route->path.options[0] = link_lsa->options[0];
 			route->path.options[1] = link_lsa->options[1];
 			route->path.options[2] = link_lsa->options[2];
-			route->path.prefix_options = op->prefix_options;
 			route->path.area_id = oi->area->area_id;
 			route->path.type = OSPF6_PATH_TYPE_INTRA;
 
-			if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX)) {
-				prefix2str(&route->prefix, buf, sizeof(buf));
-				zlog_debug("    include %s", buf);
-			}
+			if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
+				zlog_debug("    include %pFX", &route->prefix);
 
 			ospf6_route_add(route, route_advertise);
 			prefix_num--;
@@ -1275,7 +1384,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 	for (route = ospf6_route_head(route_advertise); route;
 	     route = ospf6_route_best_next(route)) {
 		op->prefix_length = route->prefix.prefixlen;
-		op->prefix_options = route->path.prefix_options;
+		op->prefix_options = route->prefix_options;
 		op->prefix_metric = htons(0);
 		memcpy(OSPF6_PREFIX_BODY(op), &route->prefix.u.prefix6,
 		       OSPF6_PREFIX_SPACE(op->prefix_length));
@@ -1289,7 +1398,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 		if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
 			zlog_debug(
 				"Quit to Advertise Intra-Prefix: no route to advertise");
-		return 0;
+		return;
 	}
 
 	intra_prefix_lsa->prefix_num = htons(prefix_num);
@@ -1312,18 +1421,16 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 
 	/* Originate */
 	ospf6_lsa_originate_area(lsa, oi->area);
-
-	return 0;
 }
 
-static void ospf6_intra_prefix_update_route_origin(struct ospf6_route *oa_route)
+static void ospf6_intra_prefix_update_route_origin(struct ospf6_route *oa_route,
+						   struct ospf6 *ospf6)
 {
 	struct ospf6_path *h_path;
 	struct ospf6_route *g_route, *nroute;
 
 	/* Update Global ospf6 route path */
-	g_route = ospf6_route_lookup(&oa_route->prefix,
-				     ospf6->route_table);
+	g_route = ospf6_route_lookup(&oa_route->prefix, ospf6->route_table);
 
 	assert(g_route);
 
@@ -1369,7 +1476,6 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 	struct listnode *anode, *anext;
 	struct listnode *nnode, *rnode, *rnext;
 	struct ospf6_nexthop *nh, *rnh;
-	char buf[PREFIX2STR_BUFFER];
 	bool route_found = false;
 	struct interface *ifp = NULL;
 	struct ospf6_lsa *lsa;
@@ -1381,8 +1487,14 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 	for (old_route = old; old_route; old_route = old_route->next) {
 		bool route_updated = false;
 
-		if (!ospf6_route_is_same(old_route, route) ||
-			(old_route->path.type != route->path.type))
+		/* The route linked-list is grouped in batches of prefix.
+		 * If the new prefix is not the same as the one of interest
+		 * then we have walked over the end of the batch and so we
+		 * should break rather than continuing unnecessarily.
+		 */
+		if (!ospf6_route_is_same(old_route, route))
+			break;
+		if (old_route->path.type != route->path.type)
 			continue;
 
 		/* Current and New route has same origin,
@@ -1393,9 +1505,8 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 			/* Check old route path and route has same
 			 * origin.
 			 */
-			if (o_path->area_id != route->path.area_id ||
-			    (memcmp(&(o_path)->origin, &(route)->path.origin,
-				   sizeof(struct ospf6_ls_origin)) != 0))
+			if (o_path->area_id != route->path.area_id
+			    || !ospf6_ls_origin_same(o_path, &route->path))
 				continue;
 
 			/* Cost is not same then delete current path */
@@ -1445,15 +1556,15 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 					 * nh_list
 					 */
 					if (oa->route_table->hook_add)
-						(*oa->route_table->hook_add)
-							(old_route);
+						(*oa->route_table->hook_add)(
+							old_route);
 
 					if (old_route->path.origin.id ==
 					route->path.origin.id &&
 					old_route->path.origin.adv_router ==
 						route->path.origin.adv_router) {
 						ospf6_intra_prefix_update_route_origin(
-								old_route);
+							old_route, oa->ospf6);
 					}
 					break;
 				}
@@ -1467,7 +1578,7 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 				}
 				if (oa->route_table->hook_remove)
 					ospf6_route_remove(old_route,
-						   oa->route_table);
+							   oa->route_table);
 				else
 					SET_FLAG(old_route->flag,
 						 OSPF6_ROUTE_REMOVE);
@@ -1480,8 +1591,14 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 
 	for (old_route = old; old_route; old_route = old_route->next) {
 
-		if (!ospf6_route_is_same(old_route, route) ||
-			(old_route->path.type != route->path.type))
+		/* The route linked-list is grouped in batches of prefix.
+		 * If the new prefix is not the same as the one of interest
+		 * then we have walked over the end of the batch and so we
+		 * should break rather than continuing unnecessarily.
+		 */
+		if (!ospf6_route_is_same(old_route, route))
+			break;
+		if (old_route->path.type != route->path.type)
 			continue;
 
 		/* Old Route and New Route have Equal Cost, Merge NHs */
@@ -1493,10 +1610,8 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 			 */
 			for (ALL_LIST_ELEMENTS_RO(old_route->paths, anode,
 						  o_path)) {
-				if (o_path->area_id == route->path.area_id &&
-				    (memcmp(&(o_path)->origin,
-					&(route)->path.origin,
-					sizeof(struct ospf6_ls_origin)) == 0))
+				if (o_path->area_id == route->path.area_id
+				    && ospf6_ls_origin_same(o_path, &route->path))
 					break;
 			}
 			/* If path is not found in old_route paths's list,
@@ -1541,8 +1656,9 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 				if (ls_entry == NULL) {
 					if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
 						zlog_debug(
-							"%s: ls_prfix %s ls_entry not found.",
-							__func__, buf);
+							"%s: ls_prfix %pFX ls_entry not found.",
+							__func__,
+							&o_path->ls_prefix);
 					continue;
 				}
 				lsa = ospf6_lsdb_lookup(o_path->origin.type,
@@ -1557,11 +1673,9 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 						ospf6_linkstate_prefix(
 						o_path->origin.adv_router,
 						o_path->origin.id, &adv_prefix);
-						prefix2str(&adv_prefix, buf,
-							   sizeof(buf));
 						zlog_debug(
-							"%s: adv_router %s lsa not found",
-							__func__, buf);
+							"%s: adv_router %pFX lsa not found",
+							__func__, &adv_prefix);
 					}
 					continue;
 				}
@@ -1593,16 +1707,15 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 				}
 			}
 
-			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
-				prefix2str(&route->prefix, buf, sizeof(buf));
+			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
 				zlog_debug(
-					"%s: route %s %p with final effective paths %u nh%u",
-					__func__, buf, (void *)old_route,
+					"%s: route %pFX %p with final effective paths %u nh %u",
+					__func__, &route->prefix,
+					(void *)old_route,
 					old_route->paths
 						? listcount(old_route->paths)
 						: 0,
 					listcount(old_route->nh_list));
-			}
 
 			/* used in intra_route_calculation() to add to
 			 * global ospf6 route table.
@@ -1652,10 +1765,8 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 	intra_prefix_lsa =
 		(struct ospf6_intra_prefix_lsa *)OSPF6_LSA_HEADER_END(
 			lsa->header);
-	if (intra_prefix_lsa->ref_type == htons(OSPF6_LSTYPE_ROUTER))
-		ospf6_linkstate_prefix(intra_prefix_lsa->ref_adv_router,
-				       intra_prefix_lsa->ref_id, &ls_prefix);
-	else if (intra_prefix_lsa->ref_type == htons(OSPF6_LSTYPE_NETWORK))
+	if (intra_prefix_lsa->ref_type == htons(OSPF6_LSTYPE_ROUTER) ||
+	    intra_prefix_lsa->ref_type == htons(OSPF6_LSTYPE_NETWORK))
 		ospf6_linkstate_prefix(intra_prefix_lsa->ref_adv_router,
 				       intra_prefix_lsa->ref_id, &ls_prefix);
 	else {
@@ -1704,19 +1815,19 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 			continue;
 		}
 
-		route = ospf6_route_create();
+		route = ospf6_route_create(oa->ospf6);
 
 		memset(&route->prefix, 0, sizeof(struct prefix));
 		route->prefix.family = AF_INET6;
 		route->prefix.prefixlen = op->prefix_length;
 		ospf6_prefix_in6_addr(&route->prefix.u.prefix6,
 				      intra_prefix_lsa, op);
+		route->prefix_options = op->prefix_options;
 
 		route->type = OSPF6_DEST_TYPE_NETWORK;
 		route->path.origin.type = lsa->header->type;
 		route->path.origin.id = lsa->header->id;
 		route->path.origin.adv_router = lsa->header->adv_router;
-		route->path.prefix_options = op->prefix_options;
 		route->path.area_id = oa->area_id;
 		route->path.type = OSPF6_PATH_TYPE_INTRA;
 		route->path.metric_type = 1;
@@ -1858,7 +1969,8 @@ static void ospf6_intra_prefix_lsa_remove_update_route(struct ospf6_lsa *lsa,
 		if ((route->path.origin.id == lsa->header->id) &&
 		    (route->path.origin.adv_router ==
 				lsa->header->adv_router)) {
-			ospf6_intra_prefix_update_route_origin(route);
+			ospf6_intra_prefix_update_route_origin(route,
+							       oa->ospf6);
 		}
 	}
 
@@ -1896,7 +2008,7 @@ void ospf6_intra_prefix_lsa_remove(struct ospf6_lsa *lsa)
 			break;
 		prefix_num--;
 
-		memset(&prefix, 0, sizeof(struct prefix));
+		memset(&prefix, 0, sizeof(prefix));
 		prefix.family = AF_INET6;
 		prefix.prefixlen = op->prefix_length;
 		ospf6_prefix_in6_addr(&prefix.u.prefix6, intra_prefix_lsa, op);
@@ -1958,6 +2070,7 @@ void ospf6_intra_route_calculation(struct ospf6_area *oa)
 	struct ospf6_lsa *lsa;
 	void (*hook_add)(struct ospf6_route *) = NULL;
 	void (*hook_remove)(struct ospf6_route *) = NULL;
+	char buf[PREFIX2STR_BUFFER];
 
 	if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
 		zlog_debug("Re-examin intra-routes for area %s", oa->name);
@@ -1979,6 +2092,12 @@ void ospf6_intra_route_calculation(struct ospf6_area *oa)
 	oa->route_table->hook_remove = hook_remove;
 
 	for (route = ospf6_route_head(oa->route_table); route; route = nroute) {
+		if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+			prefix2str(&route->prefix, buf, sizeof(buf));
+			zlog_debug("%s: route %s, flag 0x%x", __func__, buf,
+				   route->flag);
+		}
+
 		nroute = ospf6_route_next(route);
 		if (CHECK_FLAG(route->flag, OSPF6_ROUTE_REMOVE)
 		    && CHECK_FLAG(route->flag, OSPF6_ROUTE_ADD)) {
@@ -1995,7 +2114,10 @@ void ospf6_intra_route_calculation(struct ospf6_area *oa)
 			route->flag = 0;
 		} else {
 			/* Redo the summaries as things might have changed */
-			ospf6_abr_originate_summary(route);
+			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
+				zlog_debug("%s: Originate summary for route %s",
+					   __func__, buf);
+			ospf6_abr_originate_summary(route, oa->ospf6);
 			route->flag = 0;
 		}
 	}
@@ -2014,7 +2136,7 @@ static void ospf6_brouter_debug_print(struct ospf6_route *brouter)
 	char installed[64], changed[64];
 	struct timeval now, res;
 	char id[16], adv_router[16];
-	char capa[16], options[16];
+	char capa[16], options[32];
 
 	brouter_id = ADV_ROUTER_IN_PREFIX(&brouter->prefix);
 	inet_ntop(AF_INET, &brouter_id, brouter_name, sizeof(brouter_name));
@@ -2070,8 +2192,8 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 
 	if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(oa->area_id) ||
 	    IS_OSPF6_DEBUG_ROUTE(MEMORY))
-		zlog_info("%s: border-router calculation for area %s", __func__,
-			  oa->name);
+		zlog_debug("%s: border-router calculation for area %s",
+			   __func__, oa->name);
 
 	hook_add = oa->ospf6->brouter_table->hook_add;
 	hook_remove = oa->ospf6->brouter_table->hook_remove;
@@ -2092,8 +2214,8 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 
 		if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_ROUTER_ID(brouter_id)
 		    || IS_OSPF6_DEBUG_ROUTE(MEMORY)) {
-			zlog_info("%p: mark as removing: area %s brouter %s",
-				  (void *)brouter, oa->name, brouter_name);
+			zlog_debug("%p: mark as removing: area %s brouter %s",
+				   (void *)brouter, oa->name, brouter_name);
 			ospf6_brouter_debug_print(brouter);
 		}
 	}
@@ -2126,8 +2248,8 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 
 		if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_ROUTER_ID(brouter_id)
 		    || IS_OSPF6_DEBUG_ROUTE(MEMORY)) {
-			zlog_info("%p: transfer: area %s brouter %s",
-				  (void *)brouter, oa->name, brouter_name);
+			zlog_debug("%p: transfer: area %s brouter %s",
+				   (void *)brouter, oa->name, brouter_name);
 			ospf6_brouter_debug_print(brouter);
 		}
 	}
@@ -2200,14 +2322,14 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 				       brouter_id)
 			    || IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(
 				       oa->area_id))
-				zlog_info(
+				zlog_debug(
 					"%s: brouter %s disappears via area %s",
 					__func__, brouter_name, oa->name);
 			/* This is used to protect nbrouter from removed from
 			 * the table. For an example, ospf6_abr_examin_summary,
 			 * removes brouters which are marked for remove.
 			 */
-			oa->intra_brouter_calc = 1;
+			oa->intra_brouter_calc = true;
 			ospf6_route_remove(brouter, oa->ospf6->brouter_table);
 			brouter = NULL;
 		} else if (CHECK_FLAG(brouter->flag, OSPF6_ROUTE_ADD)
@@ -2228,10 +2350,11 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 				    brouter_id)
 			    || IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(
 				       oa->area_id))
-				zlog_info("brouter %s still exists via area %s",
-					  brouter_name, oa->name);
+				zlog_debug(
+					"brouter %s still exists via area %s",
+					brouter_name, oa->name);
 			/* But re-originate summaries */
-			ospf6_abr_originate_summary(brouter);
+			ospf6_abr_originate_summary(brouter, oa->ospf6);
 		}
 
 		if (brouter) {
@@ -2239,13 +2362,13 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 			UNSET_FLAG(brouter->flag, OSPF6_ROUTE_CHANGE);
 		}
 		/* Reset for nbrouter */
-		oa->intra_brouter_calc = 0;
+		oa->intra_brouter_calc = false;
 	}
 
 	if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(oa->area_id) ||
 	    IS_OSPF6_DEBUG_ROUTE(MEMORY))
-		zlog_info("%s: border-router calculation for area %s: done",
-			  __func__, oa->name);
+		zlog_debug("%s: border-router calculation for area %s: done",
+			   __func__, oa->name);
 }
 
 static struct ospf6_lsa_handler router_handler = {
@@ -2332,12 +2455,13 @@ DEFUN (debug_ospf6_brouter_router,
 
 DEFUN (no_debug_ospf6_brouter_router,
        no_debug_ospf6_brouter_router_cmd,
-       "no debug ospf6 border-routers router-id",
+       "no debug ospf6 border-routers router-id [A.B.C.D]",
        NO_STR
        DEBUG_STR
        OSPF6_STR
        "Debug border router\n"
        "Debug specific border router\n"
+       "Specify border-router's router-id\n"
       )
 {
 	OSPF6_DEBUG_BROUTER_SPECIFIC_ROUTER_OFF();
@@ -2363,12 +2487,13 @@ DEFUN (debug_ospf6_brouter_area,
 
 DEFUN (no_debug_ospf6_brouter_area,
        no_debug_ospf6_brouter_area_cmd,
-       "no debug ospf6 border-routers area-id",
+       "no debug ospf6 border-routers area-id [A.B.C.D]",
        NO_STR
        DEBUG_STR
        OSPF6_STR
        "Debug border router\n"
        "Debug border routers in specific Area\n"
+       "Specify Area-ID\n"
       )
 {
 	OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_OFF();
