@@ -20,6 +20,7 @@
 
 #include <zebra.h>
 #include <lib/version.h>
+#include <lib/keychain.h>
 #include <stdlib.h>
 
 #include "getopt.h"
@@ -47,6 +48,7 @@
 #include "ospf6_lsa.h"
 #include "ospf6_interface.h"
 #include "ospf6_zebra.h"
+#include "ospf6_routemap_nb.h"
 
 /* Default configuration file name for ospf6d. */
 #define OSPF6_DEFAULT_CONFIG       "ospf6d.conf"
@@ -55,7 +57,7 @@
 #define OSPF6_VTY_PORT             2606
 
 /* ospf6d privileges */
-zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND};
+zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND, ZCAP_SYS_ADMIN};
 
 struct zebra_privs_t ospf6d_privs = {
 #if defined(FRR_USER)
@@ -68,7 +70,7 @@ struct zebra_privs_t ospf6d_privs = {
 	.vty_group = VTY_GROUP,
 #endif
 	.caps_p = _caps_p,
-	.cap_num_p = 2,
+	.cap_num_p = array_size(_caps_p),
 	.cap_num_i = 0};
 
 /* ospf6d options, we use GNU getopt library. */
@@ -81,21 +83,21 @@ static void __attribute__((noreturn)) ospf6_exit(int status)
 {
 	struct vrf *vrf;
 	struct interface *ifp;
+	struct ospf6 *ospf6;
+	struct listnode *node, *nnode;
 
 	frr_early_fini();
 
-	if (ospf6) {
+	bfd_protocol_integration_set_shutdown(true);
+
+	for (ALL_LIST_ELEMENTS(om6->ospf6, node, nnode, ospf6)) {
 		vrf = vrf_lookup_by_id(ospf6->vrf_id);
 		ospf6_delete(ospf6);
 		ospf6 = NULL;
-	} else
-		vrf = vrf_lookup_by_id(VRF_DEFAULT);
-
-	bfd_gbl_exit();
-
-	FOR_ALL_INTERFACES (vrf, ifp)
-		if (ifp->info != NULL)
-			ospf6_interface_delete(ifp->info);
+		FOR_ALL_INTERFACES (vrf, ifp)
+			if (ifp->info != NULL)
+				ospf6_interface_delete(ifp->info);
+	}
 
 	ospf6_message_terminate();
 	ospf6_asbr_terminate();
@@ -147,7 +149,7 @@ static void sigusr1(void)
 	zlog_rotate();
 }
 
-struct quagga_signal_t ospf6_signals[] = {
+struct frr_signal_t ospf6_signals[] = {
 	{
 		.signal = SIGHUP,
 		.handler = &sighup,
@@ -171,6 +173,8 @@ static const struct frr_yang_module_info *const ospf6d_yang_modules[] = {
 	&frr_interface_info,
 	&frr_route_map_info,
 	&frr_vrf_info,
+	&frr_ospf_route_map_info,
+	&frr_ospf6_route_map_info,
 };
 
 FRR_DAEMON_INFO(ospf6d, OSPF6, .vty_port = OSPF6_VTY_PORT,
@@ -181,7 +185,8 @@ FRR_DAEMON_INFO(ospf6d, OSPF6, .vty_port = OSPF6_VTY_PORT,
 		.n_signals = array_size(ospf6_signals),
 
 		.privs = &ospf6d_privs, .yang_modules = ospf6d_yang_modules,
-		.n_yang_modules = array_size(ospf6d_yang_modules), )
+		.n_yang_modules = array_size(ospf6d_yang_modules),
+);
 
 /* Main routine of ospf6d. Treatment of argument and starting ospf finite
    state machine is handled here. */
@@ -204,7 +209,6 @@ int main(int argc, char *argv[], char *envp[])
 			break;
 		default:
 			frr_help_exit(1);
-			break;
 		}
 	}
 
@@ -215,17 +219,18 @@ int main(int argc, char *argv[], char *envp[])
 	}
 
 	/* OSPF6 master init. */
-	ospf6_master_init();
+	ospf6_master_init(frr_init());
 
 	/* thread master */
-	master = frr_init();
+	master = om6->master;
 
-	vrf_init(NULL, NULL, NULL, NULL, NULL);
+	keychain_init();
+	ospf6_vrf_init();
 	access_list_init();
 	prefix_list_init();
 
 	/* initialize ospf6 */
-	ospf6_init();
+	ospf6_init(master);
 
 	frr_config_fork();
 	frr_run(master);
