@@ -1,6 +1,7 @@
 /*
  * API message handling module for OSPF daemon and client.
  * Copyright (C) 2001, 2002 Ralph Keller
+ * Copyright (c) 2022, LabN Consulting, L.L.C.
  *
  * This file is part of GNU Zebra.
  *
@@ -58,7 +59,7 @@
 
 
 /* For debugging only, will be removed */
-void api_opaque_lsa_print(struct lsa_header *data)
+void api_opaque_lsa_print(struct ospf_lsa *lsa)
 {
 	struct opaque_lsa {
 		struct lsa_header header;
@@ -69,11 +70,11 @@ void api_opaque_lsa_print(struct lsa_header *data)
 	int opaquelen;
 	int i;
 
-	ospf_lsa_header_dump(data);
+	ospf_lsa_header_dump(lsa->data);
 
-	olsa = (struct opaque_lsa *)data;
+	olsa = (struct opaque_lsa *)lsa->data;
 
-	opaquelen = ntohs(data->length) - OSPF_LSA_HEADER_SIZE;
+	opaquelen = lsa->size - OSPF_LSA_HEADER_SIZE;
 	zlog_debug("apiserver_lsa_print: opaquelen=%d", opaquelen);
 
 	for (i = 0; i < opaquelen; i++) {
@@ -111,11 +112,16 @@ struct msg *msg_new(uint8_t msgtype, void *msgbody, uint32_t seqnum,
 struct msg *msg_dup(struct msg *msg)
 {
 	struct msg *new;
+	size_t size;
 
 	assert(msg);
 
+	size = ntohs(msg->hdr.msglen);
+	if (size > OSPF_MAX_LSA_SIZE)
+		return NULL;
+
 	new = msg_new(msg->hdr.msgtype, STREAM_DATA(msg->s),
-		      ntohl(msg->hdr.msgseq), ntohs(msg->hdr.msglen));
+		      ntohl(msg->hdr.msgseq), size);
 	return new;
 }
 
@@ -172,6 +178,10 @@ const char *ospf_api_typename(int msgtype)
 		{
 			MSG_NSM_CHANGE, "NSM change",
 		},
+		{
+			MSG_REACHABLE_CHANGE,
+			"Reachable change",
+		},
 	};
 
 	int i, n = array_size(NameTab);
@@ -400,7 +410,7 @@ struct msg *msg_read(int fd)
 	}
 
 	/* Allocate new message */
-	msg = msg_new(hdr.msgtype, buf, ntohl(hdr.msgseq), ntohs(hdr.msglen));
+	msg = msg_new(hdr.msgtype, buf, ntohl(hdr.msgseq), bodylen);
 
 	return msg;
 }
@@ -408,29 +418,34 @@ struct msg *msg_read(int fd)
 int msg_write(int fd, struct msg *msg)
 {
 	uint8_t buf[OSPF_API_MAX_MSG_SIZE];
-	int l;
+	uint16_t l;
 	int wlen;
 
 	assert(msg);
 	assert(msg->s);
 
-	/* Length of message including header */
-	l = sizeof(struct apimsghdr) + ntohs(msg->hdr.msglen);
+	/* Length of OSPF LSA payload */
+	l = ntohs(msg->hdr.msglen);
+	if (l > OSPF_MAX_LSA_SIZE) {
+		zlog_warn("%s: wrong LSA size %d", __func__, l);
+		return -1;
+	}
 
 	/* Make contiguous memory buffer for message */
 	memcpy(buf, &msg->hdr, sizeof(struct apimsghdr));
-	memcpy(buf + sizeof(struct apimsghdr), STREAM_DATA(msg->s),
-	       ntohs(msg->hdr.msglen));
+	memcpy(buf + sizeof(struct apimsghdr), STREAM_DATA(msg->s), l);
 
+	/* Total length of OSPF API Message */
+	l += sizeof(struct apimsghdr);
 	wlen = writen(fd, buf, l);
 	if (wlen < 0) {
-		zlog_warn("msg_write: writen %s", safe_strerror(errno));
+		zlog_warn("%s: writen %s", __func__, safe_strerror(errno));
 		return -1;
 	} else if (wlen == 0) {
-		zlog_warn("msg_write: Connection closed by peer");
+		zlog_warn("%s: Connection closed by peer", __func__);
 		return -1;
 	} else if (wlen != l) {
-		zlog_warn("msg_write: Cannot write API message");
+		zlog_warn("%s: Cannot write API message", __func__);
 		return -1;
 	}
 	return 0;
@@ -517,16 +532,17 @@ struct msg *new_msg_originate_request(uint32_t seqnum, struct in_addr ifaddr,
 	return msg_new(MSG_ORIGINATE_REQUEST, omsg, seqnum, omsglen);
 }
 
-struct msg *new_msg_delete_request(uint32_t seqnum, struct in_addr area_id,
+struct msg *new_msg_delete_request(uint32_t seqnum, struct in_addr addr,
 				   uint8_t lsa_type, uint8_t opaque_type,
-				   uint32_t opaque_id)
+				   uint32_t opaque_id, uint8_t flags)
 {
 	struct msg_delete_request dmsg;
-	dmsg.area_id = area_id;
+	dmsg.addr = addr;
 	dmsg.lsa_type = lsa_type;
 	dmsg.opaque_type = opaque_type;
 	dmsg.opaque_id = htonl(opaque_id);
 	memset(&dmsg.pad, 0, sizeof(dmsg.pad));
+	dmsg.flags = flags;
 
 	return msg_new(MSG_DELETE_REQUEST, &dmsg, seqnum,
 		       sizeof(struct msg_delete_request));
@@ -641,4 +657,39 @@ struct msg *new_msg_lsa_change_notify(uint8_t msgtype, uint32_t seqnum,
 	return msg_new(msgtype, nmsg, seqnum, len);
 }
 
+struct msg *new_msg_reachable_change(uint32_t seqnum, uint16_t nadd,
+				     struct in_addr *add, uint16_t nremove,
+				     struct in_addr *remove)
+{
+	uint8_t buf[OSPF_API_MAX_MSG_SIZE];
+	struct msg_reachable_change *nmsg = (void *)buf;
+	const uint insz = sizeof(*nmsg->router_ids);
+	const uint nmax = (sizeof(buf) - sizeof(*nmsg)) / insz;
+	uint len;
+
+	if (nadd > nmax)
+		nadd = nmax;
+	if (nremove > (nmax - nadd))
+		nremove = (nmax - nadd);
+
+	if (nadd)
+		memcpy(nmsg->router_ids, add, nadd * insz);
+	if (nremove)
+		memcpy(&nmsg->router_ids[nadd], remove, nremove * insz);
+
+	nmsg->nadd = htons(nadd);
+	nmsg->nremove = htons(nremove);
+	len = sizeof(*nmsg) + insz * (nadd + nremove);
+
+	return msg_new(MSG_REACHABLE_CHANGE, nmsg, seqnum, len);
+}
+
+struct msg *new_msg_router_id_change(uint32_t seqnum, struct in_addr router_id)
+{
+	struct msg_router_id_change rmsg = {.router_id = router_id};
+
+	return msg_new(MSG_ROUTER_ID_CHANGE, &rmsg, seqnum,
+		       sizeof(struct msg_router_id_change));
+}
+
 #endif /* SUPPORT_OSPF_API */
