@@ -136,7 +136,7 @@ static void ospf_ase_complete_direct_routes(struct ospf_route *ro,
 	struct ospf_path *op;
 
 	for (ALL_LIST_ELEMENTS_RO(ro->paths, node, op))
-		if (op->nexthop.s_addr == 0)
+		if (op->nexthop.s_addr == INADDR_ANY)
 			op->nexthop.s_addr = nexthop.s_addr;
 }
 
@@ -156,84 +156,6 @@ static int ospf_ase_forward_address_check(struct ospf *ospf,
 	return 1;
 }
 
-#if 0
-/* Calculate ASBR route. */
-static struct ospf_route *
-ospf_ase_calculate_asbr_route (struct ospf *ospf,
-			       struct route_table *rt_network,
-			       struct route_table *rt_router,
-			       struct as_external_lsa *al)
-{
-  struct prefix_ipv4 asbr;
-  struct ospf_route *asbr_route;
-  struct route_node *rn;
-
-  /* Find ASBR route from Router routing table. */
-  asbr.family = AF_INET;
-  asbr.prefix = al->header.adv_router;
-  asbr.prefixlen = IPV4_MAX_BITLEN;
-  apply_mask_ipv4 (&asbr);
-
-  asbr_route = ospf_find_asbr_route (ospf, rt_router, &asbr);
-
-  if (asbr_route == NULL)
-    {
-      if (IS_DEBUG_OSPF (lsa, LSA))
-	zlog_debug ("ospf_ase_calculate(): Route to ASBR %s not found",
-		    inet_ntoa (asbr.prefix));
-      return NULL;
-    }
-
-  if (!(asbr_route->u.std.flags & ROUTER_LSA_EXTERNAL))
-    {
-      if (IS_DEBUG_OSPF (lsa, LSA))
-	zlog_debug ("ospf_ase_calculate(): Originating router is not an ASBR");
-      return NULL;
-    }
-
-  if (al->e[0].fwd_addr.s_addr != 0)
-    {
-      if (IS_DEBUG_OSPF (lsa, LSA))
-	zlog_debug ("ospf_ase_calculate(): Forwarding address is not 0.0.0.0.");
-
-      if (! ospf_ase_forward_address_check (ospf, al->e[0].fwd_addr))
-	{
-	  if (IS_DEBUG_OSPF (lsa, LSA))
-	    zlog_debug ("ospf_ase_calculate(): Forwarding address is one of our addresses, Ignore.");
-	  return NULL;
-        }
-
-      if (IS_DEBUG_OSPF (lsa, LSA))
-	zlog_debug ("ospf_ase_calculate(): Looking up in the Network Routing Table.");
-
-      /* Looking up the path to the fwd_addr from Network route. */
-      asbr.family = AF_INET;
-      asbr.prefix = al->e[0].fwd_addr;
-      asbr.prefixlen = IPV4_MAX_BITLEN;
-
-      rn = route_node_match (rt_network, (struct prefix *) &asbr);
-
-      if (rn == NULL)
-	{
-	  if (IS_DEBUG_OSPF (lsa, LSA))
-	    zlog_debug ("ospf_ase_calculate(): Couldn't find a route to the forwarding address.");
-	  return NULL;
-	}
-
-      route_unlock_node (rn);
-
-      if ((asbr_route = rn->info) == NULL)
-	{
-	  if (IS_DEBUG_OSPF (lsa, LSA))
-	    zlog_debug ("ospf_ase_calculate(): Somehow OSPF route to ASBR is lost");
-	  return NULL;
-	}
-    }
-
-  return asbr_route;
-}
-#endif
-
 static struct ospf_route *
 ospf_ase_calculate_new_route(struct ospf_lsa *lsa,
 			     struct ospf_route *asbr_route, uint32_t metric)
@@ -283,7 +205,6 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 	struct prefix_ipv4 asbr, p;
 	struct route_node *rn;
 	struct ospf_route *new, * or ;
-	char buf1[INET_ADDRSTRLEN];
 	int ret;
 
 	assert(lsa);
@@ -291,21 +212,20 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 
 	if (lsa->data->type == OSPF_AS_NSSA_LSA)
 		if (IS_DEBUG_OSPF_NSSA)
-			zlog_debug("ospf_ase_calc(): Processing Type-7");
+			zlog_debug("%s: Processing Type-7", __func__);
 
 	/* Stay away from any Local Translated Type-7 LSAs */
 	if (CHECK_FLAG(lsa->flags, OSPF_LSA_LOCAL_XLT)) {
 		if (IS_DEBUG_OSPF_NSSA)
-			zlog_debug("ospf_ase_calc(): Rejecting Local Xlt'd");
+			zlog_debug("%s: Rejecting Local Xlt'd", __func__);
 		return 0;
 	}
 
 	if (IS_DEBUG_OSPF(lsa, LSA)) {
-		snprintf(buf1, sizeof(buf1), "%s",
-			 inet_ntoa(al->header.adv_router));
 		zlog_debug(
-			"Route[External]: Calculate AS-external-LSA to %s/%d adv_router %s",
-			inet_ntoa(al->header.id), ip_masklen(al->mask), buf1);
+			"Route[External]: Calculate AS-external-LSA to %pI4/%d adv_router %pI4",
+			&al->header.id, ip_masklen(al->mask),
+			&al->header.adv_router);
 	}
 
 	/* (1) If the cost specified by the LSA is LSInfinity, or if the
@@ -360,6 +280,19 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 		return 0;
 	}
 
+	/* Type-5 shouldn't be calculated if it is originated from NSSA ASBR.
+	 * As per RFC 3101, expectation is to receive type-7 lsas from
+	 * NSSA ASBR. Ignore calculation, if the current LSA is type-5 and
+	 * originated ASBR's area is NSSA.
+	 */
+	if ((lsa->data->type == OSPF_AS_EXTERNAL_LSA)
+	    && (asbr_route->u.std.external_routing != OSPF_AREA_DEFAULT)) {
+		if (IS_DEBUG_OSPF(lsa, LSA))
+			zlog_debug(
+				"Route[External]: Ignore, If type-5 LSA from NSSA area.");
+		return 0;
+	}
+
 	/*     Else, this LSA describes an AS external path to destination
 	       N.  Examine the forwarding address specified in the AS-
 	       external-LSA.  This indicates the IP address to which
@@ -457,9 +390,8 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 
 	if (!rn || (or = rn->info) == NULL) {
 		if (IS_DEBUG_OSPF(lsa, LSA))
-			zlog_debug("Route[External]: Adding a new route %s/%d with paths %u",
-				   inet_ntoa(p.prefix), p.prefixlen,
-				   listcount(asbr_route->paths));
+			zlog_debug("Route[External]: Adding a new route %pFX with paths %u",
+				   &p, listcount(asbr_route->paths));
 
 		ospf_route_add(ospf->new_external_route, &p, new, asbr_route);
 
@@ -632,7 +564,7 @@ static int ospf_ase_compare_tables(struct ospf *ospf,
 	return 0;
 }
 
-static int ospf_ase_calculate_timer(struct thread *t)
+static void ospf_ase_calculate_timer(struct thread *t)
 {
 	struct ospf *ospf;
 	struct ospf_lsa *lsa;
@@ -657,9 +589,8 @@ static int ospf_ase_calculate_timer(struct thread *t)
 		if (ospf->anyNSSA)
 			for (ALL_LIST_ELEMENTS_RO(ospf->areas, node, area)) {
 				if (IS_DEBUG_OSPF_NSSA)
-					zlog_debug(
-						"ospf_ase_calculate_timer(): looking at area %s",
-						inet_ntoa(area->area_id));
+					zlog_debug("%s: looking at area %pI4",
+						   __func__, &area->area_id);
 
 				if (area->external_routing == OSPF_AREA_NSSA)
 					LSDB_LOOP (NSSA_LSDB(area), rn, lsa)
@@ -690,7 +621,15 @@ static int ospf_ase_calculate_timer(struct thread *t)
 					+ (stop_time.tv_usec
 					   - start_time.tv_usec));
 	}
-	return 0;
+
+	/*
+	 * Uninstall remnant routes that were installed before the restart, but
+	 * that are no longer valid.
+	 */
+	if (ospf->gr_info.finishing_restart) {
+		ospf_zebra_gr_disable(ospf);
+		ospf->gr_info.finishing_restart = false;
+	}
 }
 
 void ospf_ase_calculate_schedule(struct ospf *ospf)
@@ -751,8 +690,13 @@ void ospf_ase_unregister_external_lsa(struct ospf_lsa *lsa, struct ospf *top)
 
 	if (rn) {
 		lst = rn->info;
-		listnode_delete(lst, lsa);
-		ospf_lsa_unlock(&lsa); /* external_lsas list */
+		struct listnode *node = listnode_lookup(lst, lsa);
+		/* Unlock lsa only if node is present in the list */
+		if (node) {
+			listnode_delete(lst, lsa);
+			ospf_lsa_unlock(&lsa); /* external_lsas list */
+		}
+
 		route_unlock_node(rn);
 	}
 }
