@@ -23,27 +23,34 @@
 #ifndef _ZEBRA_OSPF_BFD_H
 #define _ZEBRA_OSPF_BFD_H
 
+#include "ospfd/ospf_interface.h"
 #include "json.h"
 
-extern void ospf_bfd_init(void);
+extern void ospf_bfd_init(struct thread_master *tm);
 
 extern void ospf_bfd_write_config(struct vty *vty,
-				  struct ospf_if_params *params);
+				  const struct ospf_if_params *params);
 
 extern void ospf_bfd_trigger_event(struct ospf_neighbor *nbr, int old_state,
 				   int state);
 
-extern void ospf_bfd_interface_show(struct vty *vty, struct interface *ifp,
-				    json_object *json_interface_sub,
-				    bool use_json);
+/**
+ * Legacy information: it is the peers who actually have this information
+ * and the protocol should not need to know about timers.
+ */
+extern void ospf_interface_bfd_show(struct vty *vty,
+				    const struct interface *ifp,
+				    struct json_object *json);
 
-extern void ospf_bfd_info_nbr_create(struct ospf_interface *oi,
-				     struct ospf_neighbor *nbr);
+/**
+ * Disables interface BFD configuration and remove settings from all peers.
+ */
+extern void ospf_interface_disable_bfd(struct interface *ifp,
+				       struct ospf_if_params *oip);
 
-extern void ospf_bfd_show_info(struct vty *vty, void *bfd_info,
-			       json_object *json_obj, bool use_json,
-			       int param_only);
-
-extern void ospf_bfd_info_free(void **bfd_info);
+/**
+ * Create/update BFD session for this OSPF neighbor.
+ */
+extern void ospf_neighbor_bfd_apply(struct ospf_neighbor *nbr);
 
 #endif /* _ZEBRA_OSPF_BFD_H */
