@@ -21,6 +21,7 @@
 
 #include <zebra.h>
 
+#include "lib/bfd.h"
 #include "monotime.h"
 #include "linklist.h"
 #include "thread.h"
@@ -41,6 +42,9 @@
 #include "ospfd/ospf_dump.h"
 #include "ospfd/ospf_packet.h"
 #include "ospfd/ospf_network.h"
+#ifndef VTYSH_EXTRACT_PL
+#include "ospfd/ospf_dump_clippy.c"
+#endif
 
 /* Configuration debug option variables. */
 unsigned long conf_debug_ospf_packet[5] = {0, 0, 0, 0, 0};
@@ -53,7 +57,12 @@ unsigned long conf_debug_ospf_nssa = 0;
 unsigned long conf_debug_ospf_te = 0;
 unsigned long conf_debug_ospf_ext = 0;
 unsigned long conf_debug_ospf_sr = 0;
+unsigned long conf_debug_ospf_ti_lfa = 0;
 unsigned long conf_debug_ospf_defaultinfo = 0;
+unsigned long conf_debug_ospf_ldp_sync = 0;
+unsigned long conf_debug_ospf_gr = 0;
+unsigned long conf_debug_ospf_bfd;
+unsigned long conf_debug_ospf_client_api;
 
 /* Enable debug option variables -- valid only session. */
 unsigned long term_debug_ospf_packet[5] = {0, 0, 0, 0, 0};
@@ -66,7 +75,12 @@ unsigned long term_debug_ospf_nssa = 0;
 unsigned long term_debug_ospf_te = 0;
 unsigned long term_debug_ospf_ext = 0;
 unsigned long term_debug_ospf_sr = 0;
+unsigned long term_debug_ospf_ti_lfa = 0;
 unsigned long term_debug_ospf_defaultinfo;
+unsigned long term_debug_ospf_ldp_sync;
+unsigned long term_debug_ospf_gr = 0;
+unsigned long term_debug_ospf_bfd;
+unsigned long term_debug_ospf_client_api;
 
 const char *ospf_redist_string(unsigned int route_type)
 {
@@ -115,7 +129,9 @@ const char *ospf_area_desc_string(struct ospf_area *area)
 	return buf;
 }
 
-#define OSPF_IF_STRING_MAXLEN  40
+#define OSPF_IF_STRING_MAXLEN 40
+
+/* Display both nbr and ism state of the ospf neighbor.*/
 const char *ospf_if_name_string(struct ospf_interface *oi)
 {
 	static char buf[OSPF_IF_STRING_MAXLEN] = "";
@@ -134,8 +150,14 @@ const char *ospf_if_name_string(struct ospf_interface *oi)
 	return buf;
 }
 
-
+/* Display only the nbr state.*/
 void ospf_nbr_state_message(struct ospf_neighbor *nbr, char *buf, size_t size)
+{
+	snprintf(buf, size, "%s",
+		 lookup_msg(ospf_nsm_state_msg, nbr->state, NULL));
+}
+
+int ospf_nbr_ism_state(struct ospf_neighbor *nbr)
 {
 	int state;
 	struct ospf_interface *oi = nbr->oi;
@@ -147,6 +169,27 @@ void ospf_nbr_state_message(struct ospf_neighbor *nbr, char *buf, size_t size)
 	else
 		state = ISM_DROther;
 
+	return state;
+}
+
+void ospf_nbr_ism_state_message(struct ospf_neighbor *nbr, char *buf,
+				size_t size)
+{
+	int state;
+	struct ospf_interface *oi = nbr->oi;
+
+	if (!oi)
+		return;
+
+	/* network type is point-to-point */
+	if (oi->type == OSPF_IFTYPE_POINTOPOINT) {
+		snprintf(buf, size, "%s/-",
+			 lookup_msg(ospf_nsm_state_msg, nbr->state, NULL));
+		return;
+	}
+
+	state = ospf_nbr_ism_state(nbr);
+
 	snprintf(buf, size, "%s/%s",
 		 lookup_msg(ospf_nsm_state_msg, nbr->state, NULL),
 		 lookup_msg(ospf_ism_state_msg, state, NULL));
@@ -157,8 +200,7 @@ const char *ospf_timeval_dump(struct timeval *t, char *buf, size_t size)
 /* Making formatted timer strings. */
 #define MINUTE_IN_SECONDS	60
 #define HOUR_IN_SECONDS		(60*MINUTE_IN_SECONDS)
-#define DAY_IN_SECONDS		(24*HOUR_IN_SECONDS)
-#define WEEK_IN_SECONDS		(7*DAY_IN_SECONDS)
+
 	unsigned long w, d, h, m, ms, us;
 
 	if (!t)
@@ -179,14 +221,14 @@ const char *ospf_timeval_dump(struct timeval *t, char *buf, size_t size)
 		ms %= 1000;
 	}
 
-	if (t->tv_sec > WEEK_IN_SECONDS) {
-		w = t->tv_sec / WEEK_IN_SECONDS;
-		t->tv_sec -= w * WEEK_IN_SECONDS;
+	if (t->tv_sec > ONE_WEEK_SECOND) {
+		w = t->tv_sec / ONE_WEEK_SECOND;
+		t->tv_sec -= w * ONE_WEEK_SECOND;
 	}
 
-	if (t->tv_sec > DAY_IN_SECONDS) {
-		d = t->tv_sec / DAY_IN_SECONDS;
-		t->tv_sec -= d * DAY_IN_SECONDS;
+	if (t->tv_sec > ONE_DAY_SECOND) {
+		d = t->tv_sec / ONE_DAY_SECOND;
+		t->tv_sec -= d * ONE_DAY_SECOND;
 	}
 
 	if (t->tv_sec >= HOUR_IN_SECONDS) {
@@ -209,7 +251,7 @@ const char *ospf_timeval_dump(struct timeval *t, char *buf, size_t size)
 		snprintf(buf, size, "%luh%02lum%02lds", h, m, (long)t->tv_sec);
 	else if (m)
 		snprintf(buf, size, "%lum%02lds", m, (long)t->tv_sec);
-	else if (ms)
+	else if (t->tv_sec > 0 || ms > 0)
 		snprintf(buf, size, "%ld.%03lus", (long)t->tv_sec, ms);
 	else
 		snprintf(buf, size, "%ld usecs", (long)t->tv_usec);
@@ -230,25 +272,25 @@ const char *ospf_timer_dump(struct thread *t, char *buf, size_t size)
 static void ospf_packet_hello_dump(struct stream *s, uint16_t length)
 {
 	struct ospf_hello *hello;
-	int i;
+	int i, len;
 
 	hello = (struct ospf_hello *)stream_pnt(s);
 
 	zlog_debug("Hello");
-	zlog_debug("  NetworkMask %s", inet_ntoa(hello->network_mask));
+	zlog_debug("  NetworkMask %pI4", &hello->network_mask);
 	zlog_debug("  HelloInterval %d", ntohs(hello->hello_interval));
 	zlog_debug("  Options %d (%s)", hello->options,
 		   ospf_options_dump(hello->options));
 	zlog_debug("  RtrPriority %d", hello->priority);
 	zlog_debug("  RtrDeadInterval %ld",
 		   (unsigned long)ntohl(hello->dead_interval));
-	zlog_debug("  DRouter %s", inet_ntoa(hello->d_router));
-	zlog_debug("  BDRouter %s", inet_ntoa(hello->bd_router));
+	zlog_debug("  DRouter %pI4", &hello->d_router);
+	zlog_debug("  BDRouter %pI4", &hello->bd_router);
 
-	length -= OSPF_HEADER_SIZE + OSPF_HELLO_MIN_SIZE;
-	zlog_debug("  # Neighbors %d", length / 4);
-	for (i = 0; length > 0; i++, length -= sizeof(struct in_addr))
-		zlog_debug("    Neighbor %s", inet_ntoa(hello->neighbors[i]));
+	len = length - OSPF_HEADER_SIZE - OSPF_HELLO_MIN_SIZE;
+	zlog_debug("  # Neighbors %d", len / 4);
+	for (i = 0; len > 0; i++, len -= sizeof(struct in_addr))
+		zlog_debug("    Neighbor %pI4", &hello->neighbors[i]);
 }
 
 static char *ospf_dd_flags_dump(uint8_t flags, char *buf, size_t size)
@@ -274,7 +316,8 @@ static void ospf_router_lsa_dump(struct stream *s, uint16_t length)
 {
 	char buf[BUFSIZ];
 	struct router_lsa *rl;
-	int i, len;
+	struct router_link *rlnk;
+	int i, len, sum;
 
 	rl = (struct router_lsa *)stream_pnt(s);
 
@@ -283,16 +326,15 @@ static void ospf_router_lsa_dump(struct stream *s, uint16_t length)
 		   ospf_router_lsa_flags_dump(rl->flags, buf, BUFSIZ));
 	zlog_debug("    # links %d", ntohs(rl->links));
 
-	len = ntohs(rl->header.length) - OSPF_LSA_HEADER_SIZE - 4;
-	for (i = 0; len > 0; i++) {
-		zlog_debug("    Link ID %s", inet_ntoa(rl->link[i].link_id));
-		zlog_debug("    Link Data %s",
-			   inet_ntoa(rl->link[i].link_data));
-		zlog_debug("    Type %d", (uint8_t)rl->link[i].type);
-		zlog_debug("    TOS %d", (uint8_t)rl->link[i].tos);
-		zlog_debug("    metric %d", ntohs(rl->link[i].metric));
-
-		len -= 12;
+	len = length - OSPF_LSA_HEADER_SIZE - 4;
+	rlnk = &rl->link[0];
+	sum = 0;
+	for (i = 0; sum < len && rlnk; sum += 12, rlnk = &rl->link[++i]) {
+		zlog_debug("    Link ID %pI4", &rlnk->link_id);
+		zlog_debug("    Link Data %pI4", &rlnk->link_data);
+		zlog_debug("    Type %d", (uint8_t)rlnk->type);
+		zlog_debug("    TOS %d", (uint8_t)rlnk->tos);
+		zlog_debug("    metric %d", ntohs(rlnk->metric));
 	}
 }
 
@@ -301,74 +343,73 @@ static void ospf_network_lsa_dump(struct stream *s, uint16_t length)
 	struct network_lsa *nl;
 	int i, cnt;
 
-	nl = (struct network_lsa *)stream_pnt(s);
-	cnt = (ntohs(nl->header.length) - (OSPF_LSA_HEADER_SIZE + 4)) / 4;
-
 	zlog_debug("  Network-LSA");
+
+	nl = (struct network_lsa *)stream_pnt(s);
+	cnt = (length - (OSPF_LSA_HEADER_SIZE + 4)) / 4;
+
 	/*
 	zlog_debug ("LSA total size %d", ntohs (nl->header.length));
 	zlog_debug ("Network-LSA size %d",
 	ntohs (nl->header.length) - OSPF_LSA_HEADER_SIZE);
 	*/
-	zlog_debug("    Network Mask %s", inet_ntoa(nl->mask));
+	zlog_debug("    Network Mask %pI4", &nl->mask);
 	zlog_debug("    # Attached Routers %d", cnt);
 	for (i = 0; i < cnt; i++)
-		zlog_debug("      Attached Router %s",
-			   inet_ntoa(nl->routers[i]));
+		zlog_debug("      Attached Router %pI4",
+			   &nl->routers[i]);
 }
 
 static void ospf_summary_lsa_dump(struct stream *s, uint16_t length)
 {
 	struct summary_lsa *sl;
-	int size;
-	int i;
 
 	sl = (struct summary_lsa *)stream_pnt(s);
 
 	zlog_debug("  Summary-LSA");
-	zlog_debug("    Network Mask %s", inet_ntoa(sl->mask));
-
-	size = ntohs(sl->header.length) - OSPF_LSA_HEADER_SIZE - 4;
-	for (i = 0; size > 0; size -= 4, i++)
-		zlog_debug("    TOS=%d metric %d", sl->tos,
-			   GET_METRIC(sl->metric));
+	zlog_debug("    Network Mask %pI4", &sl->mask);
+	zlog_debug("    TOS=%d metric %d", sl->tos, GET_METRIC(sl->metric));
 }
 
 static void ospf_as_external_lsa_dump(struct stream *s, uint16_t length)
 {
 	struct as_external_lsa *al;
-	int size;
+	struct as_route *asr;
+	int size, sum;
 	int i;
 
 	al = (struct as_external_lsa *)stream_pnt(s);
 	zlog_debug("  %s", ospf_lsa_type_msg[al->header.type].str);
-	zlog_debug("    Network Mask %s", inet_ntoa(al->mask));
+	zlog_debug("    Network Mask %pI4", &al->mask);
 
-	size = ntohs(al->header.length) - OSPF_LSA_HEADER_SIZE - 4;
-	for (i = 0; size > 0; size -= 12, i++) {
+	size = length - OSPF_LSA_HEADER_SIZE - 4;
+	asr = &al->e[0];
+	sum = 0;
+	for (i = 0; sum < size && asr; sum += 12, asr = &al->e[++i]) {
 		zlog_debug("    bit %s TOS=%d metric %d",
-			   IS_EXTERNAL_METRIC(al->e[i].tos) ? "E" : "-",
-			   al->e[i].tos & 0x7f, GET_METRIC(al->e[i].metric));
-		zlog_debug("    Forwarding address %s",
-			   inet_ntoa(al->e[i].fwd_addr));
+			   IS_EXTERNAL_METRIC(asr->tos) ? "E" : "-",
+			   asr->tos & 0x7f, GET_METRIC(asr->metric));
+		zlog_debug("    Forwarding address %pI4", &asr->fwd_addr);
 		zlog_debug("    External Route Tag %" ROUTE_TAG_PRI,
-			   al->e[i].route_tag);
+			   ntohl(asr->route_tag));
 	}
 }
 
 static void ospf_lsa_header_list_dump(struct stream *s, uint16_t length)
 {
 	struct lsa_header *lsa;
+	int len;
 
 	zlog_debug("  # LSA Headers %d", length / OSPF_LSA_HEADER_SIZE);
 
 	/* LSA Headers. */
-	while (length > 0) {
+	len = length;
+	while (len > 0) {
 		lsa = (struct lsa_header *)stream_pnt(s);
 		ospf_lsa_header_dump(lsa);
 
 		stream_forward_getp(s, OSPF_LSA_HEADER_SIZE);
-		length -= OSPF_LSA_HEADER_SIZE;
+		len -= OSPF_LSA_HEADER_SIZE;
 	}
 }
 
@@ -406,6 +447,7 @@ static void ospf_packet_ls_req_dump(struct stream *s, uint16_t length)
 	uint32_t ls_type;
 	struct in_addr ls_id;
 	struct in_addr adv_router;
+	int sum;
 
 	sp = stream_get_getp(s);
 
@@ -414,14 +456,15 @@ static void ospf_packet_ls_req_dump(struct stream *s, uint16_t length)
 	zlog_debug("Link State Request");
 	zlog_debug("  # Requests %d", length / 12);
 
-	for (; length > 0; length -= 12) {
+	sum = 0;
+	for (; sum < length; sum += 12) {
 		ls_type = stream_getl(s);
 		ls_id.s_addr = stream_get_ipv4(s);
 		adv_router.s_addr = stream_get_ipv4(s);
 
 		zlog_debug("  LS type %d", ls_type);
-		zlog_debug("  Link State ID %s", inet_ntoa(ls_id));
-		zlog_debug("  Advertising Router %s", inet_ntoa(adv_router));
+		zlog_debug("  Link State ID %pI4", &ls_id);
+		zlog_debug("  Advertising Router %pI4", &adv_router);
 	}
 
 	stream_set_getp(s, sp);
@@ -431,23 +474,23 @@ static void ospf_packet_ls_upd_dump(struct stream *s, uint16_t length)
 {
 	uint32_t sp;
 	struct lsa_header *lsa;
-	int lsa_len;
+	int lsa_len, len;
 	uint32_t count;
 
-	length -= OSPF_HEADER_SIZE;
+	len = length - OSPF_HEADER_SIZE;
 
 	sp = stream_get_getp(s);
 
 	count = stream_getl(s);
-	length -= 4;
+	len -= 4;
 
 	zlog_debug("Link State Update");
 	zlog_debug("  # LSAs %d", count);
 
-	while (length > 0 && count > 0) {
-		if (length < OSPF_HEADER_SIZE || length % 4 != 0) {
+	while (len > 0 && count > 0) {
+		if ((uint16_t)len < OSPF_LSA_HEADER_SIZE || len % 4 != 0) {
 			zlog_debug("  Remaining %d bytes; Incorrect length.",
-				   length);
+				   len);
 			break;
 		}
 
@@ -455,34 +498,39 @@ static void ospf_packet_ls_upd_dump(struct stream *s, uint16_t length)
 		lsa_len = ntohs(lsa->length);
 		ospf_lsa_header_dump(lsa);
 
+		/* Check that LSA length is valid */
+		if (lsa_len > len || lsa_len % 4 != 0) {
+			zlog_debug("  LSA length %d is incorrect!", lsa_len);
+			break;
+		}
 		switch (lsa->type) {
 		case OSPF_ROUTER_LSA:
-			ospf_router_lsa_dump(s, length);
+			ospf_router_lsa_dump(s, lsa_len);
 			break;
 		case OSPF_NETWORK_LSA:
-			ospf_network_lsa_dump(s, length);
+			ospf_network_lsa_dump(s, lsa_len);
 			break;
 		case OSPF_SUMMARY_LSA:
 		case OSPF_ASBR_SUMMARY_LSA:
-			ospf_summary_lsa_dump(s, length);
+			ospf_summary_lsa_dump(s, lsa_len);
 			break;
 		case OSPF_AS_EXTERNAL_LSA:
-			ospf_as_external_lsa_dump(s, length);
+			ospf_as_external_lsa_dump(s, lsa_len);
 			break;
 		case OSPF_AS_NSSA_LSA:
-			ospf_as_external_lsa_dump(s, length);
+			ospf_as_external_lsa_dump(s, lsa_len);
 			break;
 		case OSPF_OPAQUE_LINK_LSA:
 		case OSPF_OPAQUE_AREA_LSA:
 		case OSPF_OPAQUE_AS_LSA:
-			ospf_opaque_lsa_dump(s, length);
+			ospf_opaque_lsa_dump(s, lsa_len);
 			break;
 		default:
 			break;
 		}
 
 		stream_forward_getp(s, lsa_len);
-		length -= lsa_len;
+		len -= lsa_len;
 		count--;
 	}
 
@@ -512,8 +560,8 @@ static void ospf_header_dump(struct ospf_header *ospfh)
 	zlog_debug("  Type %d (%s)", ospfh->type,
 		   lookup_msg(ospf_packet_type_str, ospfh->type, NULL));
 	zlog_debug("  Packet Len %d", ntohs(ospfh->length));
-	zlog_debug("  Router ID %s", inet_ntoa(ospfh->router_id));
-	zlog_debug("  Area ID %s", inet_ntoa(ospfh->area_id));
+	zlog_debug("  Router ID %pI4", &ospfh->router_id);
+	zlog_debug("  Area ID %pI4", &ospfh->area_id);
 	zlog_debug("  Checksum 0x%x", ntohs(ospfh->checksum));
 	zlog_debug("  AuType %s",
 		   lookup_msg(ospf_auth_type_str, auth_type, NULL));
@@ -607,7 +655,7 @@ DEFUN (debug_ospf_packet,
 
 	if (inst) // user passed instance ID
 	{
-		if (!ospf_lookup_instance(strtoul(argv[2]->arg, NULL, 10)))
+		if (inst != ospf_instance)
 			return CMD_NOT_MY_INSTANCE;
 	}
 
@@ -683,7 +731,7 @@ DEFUN (no_debug_ospf_packet,
 
 	if (inst) // user passed instance ID
 	{
-		if (!ospf_lookup_instance(strtoul(argv[3]->arg, NULL, 10)))
+		if (inst != ospf_instance)
 			return CMD_NOT_MY_INSTANCE;
 	}
 
@@ -754,7 +802,7 @@ DEFUN (debug_ospf_ism,
 
 	if (inst) // user passed instance ID
 	{
-		if (!ospf_lookup_instance(strtoul(argv[2]->arg, NULL, 10)))
+		if (inst != ospf_instance)
 			return CMD_NOT_MY_INSTANCE;
 	}
 
@@ -805,7 +853,7 @@ DEFUN (no_debug_ospf_ism,
 
 	if (inst) // user passed instance ID
 	{
-		if (!ospf_lookup_instance(strtoul(argv[3]->arg, NULL, 10)))
+		if (inst != ospf_instance)
 			return CMD_NOT_MY_INSTANCE;
 	}
 
@@ -900,8 +948,8 @@ DEFUN (debug_ospf_instance_nsm,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+	if (instance != ospf_instance)
+		return CMD_NOT_MY_INSTANCE;
 
 	return debug_ospf_nsm_common(vty, 4, argc, argv);
 }
@@ -972,7 +1020,7 @@ DEFUN (no_debug_ospf_instance_nsm,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
+	if (instance != ospf_instance)
 		return CMD_NOT_MY_INSTANCE;
 
 	return no_debug_ospf_nsm_common(vty, 5, argc, argv);
@@ -994,6 +1042,8 @@ static int debug_ospf_lsa_common(struct vty *vty, int arg_base, int argc,
 				DEBUG_ON(lsa, LSA_INSTALL);
 			else if (strmatch(argv[arg_base]->text, "refresh"))
 				DEBUG_ON(lsa, LSA_REFRESH);
+			else if (strmatch(argv[arg_base]->text, "aggregate"))
+				DEBUG_ON(lsa, EXTNL_LSA_AGGR);
 		}
 
 		return CMD_SUCCESS;
@@ -1011,6 +1061,8 @@ static int debug_ospf_lsa_common(struct vty *vty, int arg_base, int argc,
 			TERM_DEBUG_ON(lsa, LSA_INSTALL);
 		else if (strmatch(argv[arg_base]->text, "refresh"))
 			TERM_DEBUG_ON(lsa, LSA_REFRESH);
+		else if (strmatch(argv[arg_base]->text, "aggregate"))
+			TERM_DEBUG_ON(lsa, EXTNL_LSA_AGGR);
 	}
 
 	return CMD_SUCCESS;
@@ -1018,21 +1070,23 @@ static int debug_ospf_lsa_common(struct vty *vty, int arg_base, int argc,
 
 DEFUN (debug_ospf_lsa,
        debug_ospf_lsa_cmd,
-       "debug ospf lsa [<generate|flooding|install|refresh>]",
+       "debug ospf lsa [<generate|flooding|install|refresh|aggregate>]",
        DEBUG_STR
        OSPF_STR
        "OSPF Link State Advertisement\n"
        "LSA Generation\n"
        "LSA Flooding\n"
        "LSA Install/Delete\n"
-       "LSA Refresh\n")
+       "LSA Refresh\n"
+       "External LSA Aggregation\n")
 {
 	return debug_ospf_lsa_common(vty, 3, argc, argv);
 }
 
 DEFUN (debug_ospf_instance_lsa,
        debug_ospf_instance_lsa_cmd,
-       "debug ospf (1-65535) lsa [<generate|flooding|install|refresh>]",
+       "debug ospf (1-65535) lsa "
+       "[<generate|flooding|install|refresh|aggregate>]",
        DEBUG_STR
        OSPF_STR
        "Instance ID\n"
@@ -1040,13 +1094,14 @@ DEFUN (debug_ospf_instance_lsa,
        "LSA Generation\n"
        "LSA Flooding\n"
        "LSA Install/Delete\n"
-       "LSA Refresh\n")
+       "LSA Refresh\n"
+       "External LSA Aggregation\n")
 {
 	int idx_number = 2;
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
+	if (instance != ospf_instance)
 		return CMD_NOT_MY_INSTANCE;
 
 	return debug_ospf_lsa_common(vty, 4, argc, argv);
@@ -1068,6 +1123,8 @@ static int no_debug_ospf_lsa_common(struct vty *vty, int arg_base, int argc,
 				DEBUG_OFF(lsa, LSA_INSTALL);
 			else if (strmatch(argv[arg_base]->text, "refresh"))
 				DEBUG_OFF(lsa, LSA_REFRESH);
+			else if (strmatch(argv[arg_base]->text, "aggregate"))
+				DEBUG_OFF(lsa, EXTNL_LSA_AGGR);
 		}
 
 		return CMD_SUCCESS;
@@ -1085,6 +1142,8 @@ static int no_debug_ospf_lsa_common(struct vty *vty, int arg_base, int argc,
 			TERM_DEBUG_OFF(lsa, LSA_INSTALL);
 		else if (strmatch(argv[arg_base]->text, "refresh"))
 			TERM_DEBUG_OFF(lsa, LSA_REFRESH);
+		else if (strmatch(argv[arg_base]->text, "aggregate"))
+			TERM_DEBUG_OFF(lsa, EXTNL_LSA_AGGR);
 	}
 
 	return CMD_SUCCESS;
@@ -1092,7 +1151,7 @@ static int no_debug_ospf_lsa_common(struct vty *vty, int arg_base, int argc,
 
 DEFUN (no_debug_ospf_lsa,
        no_debug_ospf_lsa_cmd,
-       "no debug ospf lsa [<generate|flooding|install|refresh>]",
+       "no debug ospf lsa [<generate|flooding|install|refresh|aggregate>]",
        NO_STR
        DEBUG_STR
        OSPF_STR
@@ -1100,14 +1159,16 @@ DEFUN (no_debug_ospf_lsa,
        "LSA Generation\n"
        "LSA Flooding\n"
        "LSA Install/Delete\n"
-       "LSA Refres\n")
+       "LSA Refres\n"
+       "External LSA Aggregation\n")
 {
 	return no_debug_ospf_lsa_common(vty, 4, argc, argv);
 }
 
 DEFUN (no_debug_ospf_instance_lsa,
        no_debug_ospf_instance_lsa_cmd,
-       "no debug ospf (1-65535) lsa [<generate|flooding|install|refresh>]",
+       "no debug ospf (1-65535) lsa "
+       "[<generate|flooding|install|refresh|aggregate>]",
        NO_STR
        DEBUG_STR
        OSPF_STR
@@ -1116,13 +1177,14 @@ DEFUN (no_debug_ospf_instance_lsa,
        "LSA Generation\n"
        "LSA Flooding\n"
        "LSA Install/Delete\n"
-       "LSA Refres\n")
+       "LSA Refres\n"
+       "External LSA Aggregation\n")
 {
 	int idx_number = 3;
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
+	if (instance != ospf_instance)
 		return CMD_NOT_MY_INSTANCE;
 
 	return no_debug_ospf_lsa_common(vty, 5, argc, argv);
@@ -1184,7 +1246,7 @@ DEFUN (debug_ospf_instance_zebra,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
+	if (instance != ospf_instance)
 		return CMD_NOT_MY_INSTANCE;
 
 	return debug_ospf_zebra_common(vty, 4, argc, argv);
@@ -1248,8 +1310,8 @@ DEFUN (no_debug_ospf_instance_zebra,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+	if (instance != ospf_instance)
+		return CMD_NOT_MY_INSTANCE;
 
 	return no_debug_ospf_zebra_common(vty, 5, argc, argv);
 }
@@ -1294,8 +1356,8 @@ DEFUN (debug_ospf_instance_event,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+	if (instance != ospf_instance)
+		return CMD_NOT_MY_INSTANCE;
 
 	if (vty->node == CONFIG_NODE)
 		CONF_DEBUG_ON(event, EVENT);
@@ -1316,8 +1378,8 @@ DEFUN (no_debug_ospf_instance_event,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+	if (instance != ospf_instance)
+		return CMD_NOT_MY_INSTANCE;
 
 	if (vty->node == CONFIG_NODE)
 		CONF_DEBUG_OFF(event, EVENT);
@@ -1364,8 +1426,8 @@ DEFUN (debug_ospf_instance_nssa,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+	if (instance != ospf_instance)
+		return CMD_NOT_MY_INSTANCE;
 
 	if (vty->node == CONFIG_NODE)
 		CONF_DEBUG_ON(nssa, NSSA);
@@ -1386,8 +1448,8 @@ DEFUN (no_debug_ospf_instance_nssa,
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+	if (instance != ospf_instance)
+		return CMD_NOT_MY_INSTANCE;
 
 	if (vty->node == CONFIG_NODE)
 		CONF_DEBUG_OFF(nssa, NSSA);
@@ -1449,6 +1511,24 @@ DEFUN (no_debug_ospf_sr,
 	return CMD_SUCCESS;
 }
 
+DEFUN(debug_ospf_ti_lfa, debug_ospf_ti_lfa_cmd, "debug ospf ti-lfa",
+      DEBUG_STR OSPF_STR "OSPF-SR TI-LFA information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_ON(ti_lfa, TI_LFA);
+	TERM_DEBUG_ON(ti_lfa, TI_LFA);
+	return CMD_SUCCESS;
+}
+
+DEFUN(no_debug_ospf_ti_lfa, no_debug_ospf_ti_lfa_cmd, "no debug ospf ti-lfa",
+      NO_STR DEBUG_STR OSPF_STR "OSPF-SR TI-LFA information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_OFF(ti_lfa, TI_LFA);
+	TERM_DEBUG_OFF(ti_lfa, TI_LFA);
+	return CMD_SUCCESS;
+}
+
 DEFUN (debug_ospf_default_info,
        debug_ospf_default_info_cmd,
        "debug ospf default-information",
@@ -1476,6 +1556,99 @@ DEFUN (no_debug_ospf_default_info,
 	return CMD_SUCCESS;
 }
 
+DEFUN(debug_ospf_ldp_sync,
+      debug_ospf_ldp_sync_cmd,
+      "debug ospf ldp-sync",
+      DEBUG_STR OSPF_STR
+      "OSPF LDP-Sync information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_ON(ldp_sync, LDP_SYNC);
+	TERM_DEBUG_ON(ldp_sync, LDP_SYNC);
+	return CMD_SUCCESS;
+}
+
+DEFUN(no_debug_ospf_ldp_sync,
+      no_debug_ospf_ldp_sync_cmd,
+      "no debug ospf ldp-sync",
+      NO_STR
+      DEBUG_STR
+      OSPF_STR
+      "OSPF LDP-Sync information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_OFF(ldp_sync, LDP_SYNC);
+	TERM_DEBUG_OFF(ldp_sync, LDP_SYNC);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(debug_ospf_gr, debug_ospf_gr_cmd, "[no$no] debug ospf graceful-restart",
+      NO_STR DEBUG_STR OSPF_STR "OSPF Graceful Restart\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_ON(gr, GR);
+
+	if (!no)
+		TERM_DEBUG_ON(gr, GR);
+	else
+		TERM_DEBUG_OFF(gr, GR);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(debug_ospf_bfd, debug_ospf_bfd_cmd,
+      "[no] debug ospf bfd",
+      NO_STR
+      DEBUG_STR
+      OSPF_STR
+      "Bidirection Forwarding Detection\n")
+{
+	if (vty->node == CONFIG_NODE) {
+		if (no) {
+			bfd_protocol_integration_set_debug(false);
+			CONF_DEBUG_OFF(bfd, BFD_LIB);
+		} else {
+			bfd_protocol_integration_set_debug(true);
+			CONF_DEBUG_ON(bfd, BFD_LIB);
+		}
+	}
+
+	if (no)
+		TERM_DEBUG_OFF(bfd, BFD_LIB);
+	else
+		TERM_DEBUG_ON(bfd, BFD_LIB);
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(debug_ospf_client_api,
+      debug_ospf_client_api_cmd,
+      "debug ospf client-api",
+      DEBUG_STR OSPF_STR
+      "OSPF client API information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_ON(client_api, CLIENT_API);
+	TERM_DEBUG_ON(client_api, CLIENT_API);
+	return CMD_SUCCESS;
+}
+
+DEFUN(no_debug_ospf_client_api,
+      no_debug_ospf_client_api_cmd,
+      "no debug ospf client-api",
+      NO_STR
+      DEBUG_STR
+      OSPF_STR
+      "OSPF client API information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_OFF(client_api, CLIENT_API);
+	TERM_DEBUG_OFF(client_api, CLIENT_API);
+
+	return CMD_SUCCESS;
+}
+
 DEFUN (no_debug_ospf,
        no_debug_ospf_cmd,
        "no debug ospf",
@@ -1501,10 +1674,20 @@ DEFUN (no_debug_ospf,
 		DEBUG_OFF(nsm, NSM_EVENTS);
 		DEBUG_OFF(nsm, NSM_STATUS);
 		DEBUG_OFF(nsm, NSM_TIMERS);
+		DEBUG_OFF(event, EVENT);
 		DEBUG_OFF(zebra, ZEBRA);
 		DEBUG_OFF(zebra, ZEBRA_INTERFACE);
 		DEBUG_OFF(zebra, ZEBRA_REDISTRIBUTE);
 		DEBUG_OFF(defaultinfo, DEFAULTINFO);
+		DEBUG_OFF(ldp_sync, LDP_SYNC);
+		DEBUG_OFF(te, TE);
+		DEBUG_OFF(sr, SR);
+		DEBUG_OFF(ti_lfa, TI_LFA);
+		DEBUG_OFF(client_api, CLIENT_API);
+
+		/* BFD debugging is two parts: OSPF and library. */
+		DEBUG_OFF(bfd, BFD_LIB);
+		bfd_protocol_integration_set_debug(false);
 
 		for (i = 0; i < 5; i++)
 			DEBUG_PACKET_OFF(i, flag);
@@ -1532,16 +1715,22 @@ DEFUN (no_debug_ospf,
 	TERM_DEBUG_OFF(zebra, ZEBRA_INTERFACE);
 	TERM_DEBUG_OFF(zebra, ZEBRA_REDISTRIBUTE);
 	TERM_DEBUG_OFF(defaultinfo, DEFAULTINFO);
+	TERM_DEBUG_OFF(ldp_sync, LDP_SYNC);
+	TERM_DEBUG_OFF(te, TE);
+	TERM_DEBUG_OFF(sr, SR);
+	TERM_DEBUG_OFF(ti_lfa, TI_LFA);
+	TERM_DEBUG_OFF(bfd, BFD_LIB);
+	TERM_DEBUG_OFF(client_api, CLIENT_API);
 
 	return CMD_SUCCESS;
 }
 
-static int show_debugging_ospf_common(struct vty *vty, struct ospf *ospf)
+static int show_debugging_ospf_common(struct vty *vty)
 {
 	int i;
 
-	if (ospf->instance)
-		vty_out(vty, "\nOSPF Instance: %d\n\n", ospf->instance);
+	if (ospf_instance)
+		vty_out(vty, "\nOSPF Instance: %d\n\n", ospf_instance);
 
 	vty_out(vty, "OSPF debugging status:\n");
 
@@ -1633,7 +1822,33 @@ static int show_debugging_ospf_common(struct vty *vty, struct ospf *ospf)
 	if (IS_DEBUG_OSPF(nssa, NSSA) == OSPF_DEBUG_NSSA)
 		vty_out(vty, "  OSPF NSSA debugging is on\n");
 
-	vty_out(vty, "\n");
+	/* Show debug status for LDP-SYNC. */
+	if (IS_DEBUG_OSPF(ldp_sync, LDP_SYNC) == OSPF_DEBUG_LDP_SYNC)
+		vty_out(vty, "  OSPF ldp-sync debugging is on\n");
+
+	/* Show debug status for GR. */
+	if (IS_DEBUG_OSPF(gr, GR) == OSPF_DEBUG_GR)
+		vty_out(vty, "  OSPF Graceful Restart debugging is on\n");
+
+	/* Show debug status for TE */
+	if (IS_DEBUG_OSPF(te, TE) == OSPF_DEBUG_TE)
+		vty_out(vty, "  OSPF TE debugging is on\n");
+
+	/* Show debug status for SR */
+	if (IS_DEBUG_OSPF(sr, SR) == OSPF_DEBUG_SR)
+		vty_out(vty, "  OSPF SR debugging is on\n");
+
+	/* Show debug status for TI-LFA */
+	if (IS_DEBUG_OSPF(ti_lfa, TI_LFA) == OSPF_DEBUG_TI_LFA)
+		vty_out(vty, "  OSPF TI-LFA debugging is on\n");
+
+	if (IS_DEBUG_OSPF(bfd, BFD_LIB) == OSPF_DEBUG_BFD_LIB)
+		vty_out(vty,
+			"  OSPF BFD integration library debugging is on\n");
+
+	/* Show debug status for LDP-SYNC. */
+	if (IS_DEBUG_OSPF(client_api, CLIENT_API) == OSPF_DEBUG_CLIENT_API)
+		vty_out(vty, "  OSPF client-api debugging is on\n");
 
 	return CMD_SUCCESS;
 }
@@ -1645,13 +1860,7 @@ DEFUN_NOSH (show_debugging_ospf,
 	    DEBUG_STR
 	    OSPF_STR)
 {
-	struct ospf *ospf = NULL;
-
-	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-	if (ospf == NULL)
-		return CMD_SUCCESS;
-
-	return show_debugging_ospf_common(vty, ospf);
+	return show_debugging_ospf_common(vty);
 }
 
 DEFUN_NOSH (show_debugging_ospf_instance,
@@ -1663,14 +1872,13 @@ DEFUN_NOSH (show_debugging_ospf_instance,
 	    "Instance ID\n")
 {
 	int idx_number = 3;
-	struct ospf *ospf;
 	unsigned short instance = 0;
 
 	instance = strtoul(argv[idx_number]->arg, NULL, 10);
-	if ((ospf = ospf_lookup_instance(instance)) == NULL)
-		return CMD_SUCCESS;
+	if (instance != ospf_instance)
+		return CMD_NOT_MY_INSTANCE;
 
-	return show_debugging_ospf_common(vty, ospf);
+	return show_debugging_ospf_common(vty);
 }
 
 static int config_write_debug(struct vty *vty);
@@ -1693,16 +1901,11 @@ static int config_write_debug(struct vty *vty)
 		"",	" send",	" recv",	"",
 		" detail", " send detail", " recv detail", " detail"};
 
-	struct ospf *ospf;
 	char str[16];
 	memset(str, 0, 16);
 
-	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-	if (ospf == NULL)
-		return CMD_SUCCESS;
-
-	if (ospf->instance)
-		snprintf(str, sizeof(str), " %u", ospf->instance);
+	if (ospf_instance)
+		snprintf(str, sizeof(str), " %u", ospf_instance);
 
 	/* debug ospf ism (status|events|timers). */
 	if (IS_CONF_DEBUG_OSPF(ism, ISM) == OSPF_DEBUG_ISM)
@@ -1814,6 +2017,36 @@ static int config_write_debug(struct vty *vty)
 		write = 1;
 	}
 
+	/* debug ospf sr ti-lfa */
+	if (IS_CONF_DEBUG_OSPF(ti_lfa, TI_LFA) == OSPF_DEBUG_TI_LFA) {
+		vty_out(vty, "debug ospf%s ti-lfa\n", str);
+		write = 1;
+	}
+
+	/* debug ospf ldp-sync */
+	if (IS_CONF_DEBUG_OSPF(ldp_sync, LDP_SYNC) == OSPF_DEBUG_LDP_SYNC) {
+		vty_out(vty, "debug ospf%s ldp-sync\n", str);
+		write = 1;
+	}
+
+	/* debug ospf gr */
+	if (IS_CONF_DEBUG_OSPF(gr, GR) == OSPF_DEBUG_GR) {
+		vty_out(vty, "debug ospf%s graceful-restart\n", str);
+		write = 1;
+	}
+
+	if (IS_CONF_DEBUG_OSPF(bfd, BFD_LIB) == OSPF_DEBUG_BFD_LIB) {
+		vty_out(vty, "debug ospf%s bfd\n", str);
+		write = 1;
+	}
+
+	/* debug ospf client-api */
+	if (IS_CONF_DEBUG_OSPF(client_api, CLIENT_API) ==
+	    OSPF_DEBUG_CLIENT_API) {
+		vty_out(vty, "debug ospf%s client-api\n", str);
+		write = 1;
+	}
+
 	return write;
 }
 
@@ -1831,7 +2064,10 @@ void ospf_debug_init(void)
 	install_element(ENABLE_NODE, &debug_ospf_nssa_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_te_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_sr_cmd);
+	install_element(ENABLE_NODE, &debug_ospf_ti_lfa_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_default_info_cmd);
+	install_element(ENABLE_NODE, &debug_ospf_ldp_sync_cmd);
+	install_element(ENABLE_NODE, &debug_ospf_client_api_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_ism_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_nsm_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_lsa_cmd);
@@ -1840,7 +2076,12 @@ void ospf_debug_init(void)
 	install_element(ENABLE_NODE, &no_debug_ospf_nssa_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_te_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_sr_cmd);
+	install_element(ENABLE_NODE, &no_debug_ospf_ti_lfa_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_default_info_cmd);
+	install_element(ENABLE_NODE, &no_debug_ospf_ldp_sync_cmd);
+	install_element(ENABLE_NODE, &no_debug_ospf_client_api_cmd);
+	install_element(ENABLE_NODE, &debug_ospf_gr_cmd);
+	install_element(ENABLE_NODE, &debug_ospf_bfd_cmd);
 
 	install_element(ENABLE_NODE, &show_debugging_ospf_instance_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_packet_cmd);
@@ -1870,7 +2111,10 @@ void ospf_debug_init(void)
 	install_element(CONFIG_NODE, &debug_ospf_nssa_cmd);
 	install_element(CONFIG_NODE, &debug_ospf_te_cmd);
 	install_element(CONFIG_NODE, &debug_ospf_sr_cmd);
+	install_element(CONFIG_NODE, &debug_ospf_ti_lfa_cmd);
 	install_element(CONFIG_NODE, &debug_ospf_default_info_cmd);
+	install_element(CONFIG_NODE, &debug_ospf_ldp_sync_cmd);
+	install_element(CONFIG_NODE, &debug_ospf_client_api_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_nsm_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_lsa_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_zebra_cmd);
@@ -1878,7 +2122,12 @@ void ospf_debug_init(void)
 	install_element(CONFIG_NODE, &no_debug_ospf_nssa_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_te_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_sr_cmd);
+	install_element(CONFIG_NODE, &no_debug_ospf_ti_lfa_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_default_info_cmd);
+	install_element(CONFIG_NODE, &no_debug_ospf_ldp_sync_cmd);
+	install_element(CONFIG_NODE, &no_debug_ospf_client_api_cmd);
+	install_element(CONFIG_NODE, &debug_ospf_gr_cmd);
+	install_element(CONFIG_NODE, &debug_ospf_bfd_cmd);
 
 	install_element(CONFIG_NODE, &debug_ospf_instance_nsm_cmd);
 	install_element(CONFIG_NODE, &debug_ospf_instance_lsa_cmd);
