@@ -24,14 +24,31 @@
 #include "lib/ferr.h"
 
 enum ospf_log_refs {
-	OSPF_ERR_PKT_PROCESS = OSPF_FERR_START,
-	OSPF_ERR_ROUTER_LSA_MISMATCH,
-	OSPF_ERR_DOMAIN_CORRUPT,
-	OSPF_ERR_INIT_FAIL,
-	OSPF_ERR_SR_INVALID_DB,
-	OSPF_ERR_SR_NODE_CREATE,
-	OSPF_ERR_SR_INVALID_LSA_ID,
-	OSPF_ERR_SR_INVALID_ALGORITHM,
+	EC_OSPF_PKT_PROCESS = OSPF_FERR_START,
+	EC_OSPF_ROUTER_LSA_MISMATCH,
+	EC_OSPF_DOMAIN_CORRUPT,
+	EC_OSPF_INIT_FAIL,
+	EC_OSPF_SR_INVALID_DB,
+	EC_OSPF_SR_NODE_CREATE,
+	EC_OSPF_SR_INVALID_LSA_ID,
+	EC_OSPF_SR_SID_OVERFLOW,
+	EC_OSPF_INVALID_ALGORITHM,
+	EC_OSPF_FSM_INVALID_STATE,
+	EC_OSPF_SET_METRIC_PLUS,
+	EC_OSPF_MD5,
+	EC_OSPF_PACKET,
+	EC_OSPF_LARGE_LSA,
+	EC_OSPF_LSA_UNEXPECTED,
+	EC_OSPF_LSA,
+	EC_OSPF_OPAQUE_REGISTRATION,
+	EC_OSPF_TE_UNEXPECTED,
+	EC_OSPF_LSA_INSTALL_FAILURE,
+	EC_OSPF_LSA_NULL,
+	EC_OSPF_EXT_LSA_UNEXPECTED,
+	EC_OSPF_LSA_MISSING,
+	EC_OSPF_PTP_NEIGHBOR,
+	EC_OSPF_LSA_SIZE,
+	EC_OSPF_LARGE_HELLO,
 };
 
 extern void ospf_error_init(void);
