@@ -45,7 +45,7 @@
 
 DEFINE_HOOK(ospf_ism_change,
 	    (struct ospf_interface * oi, int state, int oldstate),
-	    (oi, state, oldstate))
+	    (oi, state, oldstate));
 
 /* elect DR and BDR. Refer to RFC2319 section 9.4 */
 static struct ospf_neighbor *ospf_dr_election_sub(struct list *routers)
@@ -201,7 +201,7 @@ static void ospf_dr_change(struct ospf *ospf, struct route_table *nbrs)
 	}
 }
 
-static int ospf_dr_election(struct ospf_interface *oi)
+int ospf_dr_election(struct ospf_interface *oi)
 {
 	struct in_addr old_dr, old_bdr;
 	int old_state, new_state;
@@ -223,8 +223,8 @@ static int ospf_dr_election(struct ospf_interface *oi)
 
 	new_state = ospf_ism_state(oi);
 
-	zlog_debug("DR-Election[1st]: Backup %s", inet_ntoa(BDR(oi)));
-	zlog_debug("DR-Election[1st]: DR     %s", inet_ntoa(DR(oi)));
+	zlog_debug("DR-Election[1st]: Backup %pI4", &BDR(oi));
+	zlog_debug("DR-Election[1st]: DR     %pI4", &DR(oi));
 
 	if (new_state != old_state
 	    && !(new_state == ISM_DROther && old_state < ISM_DROther)) {
@@ -233,8 +233,8 @@ static int ospf_dr_election(struct ospf_interface *oi)
 
 		new_state = ospf_ism_state(oi);
 
-		zlog_debug("DR-Election[2nd]: Backup %s", inet_ntoa(BDR(oi)));
-		zlog_debug("DR-Election[2nd]: DR     %s", inet_ntoa(DR(oi)));
+		zlog_debug("DR-Election[2nd]: Backup %pI4", &BDR(oi));
+		zlog_debug("DR-Election[2nd]: DR     %pI4", &DR(oi));
 	}
 
 	list_delete(&el_list);
@@ -248,7 +248,7 @@ static int ospf_dr_election(struct ospf_interface *oi)
 }
 
 
-int ospf_hello_timer(struct thread *thread)
+void ospf_hello_timer(struct thread *thread)
 {
 	struct ospf_interface *oi;
 
@@ -263,11 +263,9 @@ int ospf_hello_timer(struct thread *thread)
 
 	/* Hello timer set. */
 	OSPF_HELLO_TIMER_ON(oi);
-
-	return 0;
 }
 
-static int ospf_wait_timer(struct thread *thread)
+static void ospf_wait_timer(struct thread *thread)
 {
 	struct ospf_interface *oi;
 
@@ -278,8 +276,6 @@ static int ospf_wait_timer(struct thread *thread)
 		zlog_debug("ISM[%s]: Timer (Wait timer expire)", IF_NAME(oi));
 
 	OSPF_ISM_EVENT_SCHEDULE(oi, ISM_WaitTimer);
-
-	return 0;
 }
 
 /* Hook function called after ospf ISM event is occurred. And vty's
@@ -294,16 +290,16 @@ static void ism_timer_set(struct ospf_interface *oi)
 		   interface parameters must be set to initial values, and
 		   timers are
 		   reset also. */
-		OSPF_ISM_TIMER_OFF(oi->t_hello);
-		OSPF_ISM_TIMER_OFF(oi->t_wait);
-		OSPF_ISM_TIMER_OFF(oi->t_ls_ack);
+		THREAD_OFF(oi->t_hello);
+		THREAD_OFF(oi->t_wait);
+		THREAD_OFF(oi->t_ls_ack);
 		break;
 	case ISM_Loopback:
 		/* In this state, the interface may be looped back and will be
 		   unavailable for regular data traffic. */
-		OSPF_ISM_TIMER_OFF(oi->t_hello);
-		OSPF_ISM_TIMER_OFF(oi->t_wait);
-		OSPF_ISM_TIMER_OFF(oi->t_ls_ack);
+		THREAD_OFF(oi->t_hello);
+		THREAD_OFF(oi->t_wait);
+		THREAD_OFF(oi->t_ls_ack);
 		break;
 	case ISM_Waiting:
 		/* The router is trying to determine the identity of DRouter and
@@ -313,7 +309,7 @@ static void ism_timer_set(struct ospf_interface *oi)
 		OSPF_ISM_TIMER_MSEC_ON(oi->t_hello, ospf_hello_timer, 1);
 		OSPF_ISM_TIMER_ON(oi->t_wait, ospf_wait_timer,
 				  OSPF_IF_PARAM(oi, v_wait));
-		OSPF_ISM_TIMER_OFF(oi->t_ls_ack);
+		THREAD_OFF(oi->t_ls_ack);
 		break;
 	case ISM_PointToPoint:
 		/* The interface connects to a physical Point-to-point network
@@ -322,7 +318,7 @@ static void ism_timer_set(struct ospf_interface *oi)
 		   neighboring router. Hello packets are also sent. */
 		/* send first hello immediately */
 		OSPF_ISM_TIMER_MSEC_ON(oi->t_hello, ospf_hello_timer, 1);
-		OSPF_ISM_TIMER_OFF(oi->t_wait);
+		THREAD_OFF(oi->t_wait);
 		OSPF_ISM_TIMER_ON(oi->t_ls_ack, ospf_ls_ack_timer,
 				  oi->v_ls_ack);
 		break;
@@ -332,7 +328,7 @@ static void ism_timer_set(struct ospf_interface *oi)
 		   and the router itself is neither Designated Router nor
 		   Backup Designated Router. */
 		OSPF_HELLO_TIMER_ON(oi);
-		OSPF_ISM_TIMER_OFF(oi->t_wait);
+		THREAD_OFF(oi->t_wait);
 		OSPF_ISM_TIMER_ON(oi->t_ls_ack, ospf_ls_ack_timer,
 				  oi->v_ls_ack);
 		break;
@@ -341,7 +337,7 @@ static void ism_timer_set(struct ospf_interface *oi)
 		   network,
 		   and the router is Backup Designated Router. */
 		OSPF_HELLO_TIMER_ON(oi);
-		OSPF_ISM_TIMER_OFF(oi->t_wait);
+		THREAD_OFF(oi->t_wait);
 		OSPF_ISM_TIMER_ON(oi->t_ls_ack, ospf_ls_ack_timer,
 				  oi->v_ls_ack);
 		break;
@@ -350,7 +346,7 @@ static void ism_timer_set(struct ospf_interface *oi)
 		   network,
 		   and the router is Designated Router. */
 		OSPF_HELLO_TIMER_ON(oi);
-		OSPF_ISM_TIMER_OFF(oi->t_wait);
+		THREAD_OFF(oi->t_wait);
 		OSPF_ISM_TIMER_ON(oi->t_ls_ack, ospf_ls_ack_timer,
 				  oi->v_ls_ack);
 		break;
@@ -575,7 +571,7 @@ static void ism_change_state(struct ospf_interface *oi, int state)
 }
 
 /* Execute ISM event process. */
-int ospf_ism_event(struct thread *thread)
+void ospf_ism_event(struct thread *thread)
 {
 	int event;
 	int next_state;
@@ -601,6 +597,4 @@ int ospf_ism_event(struct thread *thread)
 
 	/* Make sure timer is set. */
 	ism_timer_set(oi);
-
-	return 0;
 }
