@@ -34,6 +34,7 @@
 #include "thread.h"
 #include "hash.h"
 #include "sockunion.h" /* for inet_aton() */
+#include "printfrr.h"
 
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_interface.h"
@@ -56,12 +57,12 @@
 #include "ospfd/ospf_ext.h"
 #include "ospfd/ospf_errors.h"
 
-DEFINE_MTYPE_STATIC(OSPFD, OSPF_OPAQUE_FUNCTAB, "OSPF opaque function table")
-DEFINE_MTYPE_STATIC(OSPFD, OPAQUE_INFO_PER_TYPE, "OSPF opaque per-type info")
-DEFINE_MTYPE_STATIC(OSPFD, OPAQUE_INFO_PER_ID, "OSPF opaque per-ID info")
+DEFINE_MTYPE_STATIC(OSPFD, OSPF_OPAQUE_FUNCTAB, "OSPF opaque function table");
+DEFINE_MTYPE_STATIC(OSPFD, OPAQUE_INFO_PER_TYPE, "OSPF opaque per-type info");
+DEFINE_MTYPE_STATIC(OSPFD, OPAQUE_INFO_PER_ID, "OSPF opaque per-ID info");
 
 /*------------------------------------------------------------------------*
- * Followings are initialize/terminate functions for Opaque-LSAs handling.
+ * Following are initialize/terminate functions for Opaque-LSAs handling.
  *------------------------------------------------------------------------*/
 
 #ifdef SUPPORT_OSPF_API
@@ -74,9 +75,8 @@ int ospf_apiserver_enable;
 static void ospf_opaque_register_vty(void);
 static void ospf_opaque_funclist_init(void);
 static void ospf_opaque_funclist_term(void);
-static void free_opaque_info_per_type(void *val);
+static void free_opaque_info_per_type_del(void *val);
 static void free_opaque_info_per_id(void *val);
-static void free_opaque_info_owner(void *val);
 static int ospf_opaque_lsa_install_hook(struct ospf_lsa *lsa);
 static int ospf_opaque_lsa_delete_hook(struct ospf_lsa *lsa);
 
@@ -132,6 +132,10 @@ void ospf_opaque_finish(void)
 
 	ospf_ext_finish();
 
+#ifdef SUPPORT_OSPF_API
+	ospf_apiserver_term();
+#endif
+
 	ospf_sr_finish();
 }
 
@@ -141,14 +145,14 @@ int ospf_opaque_type9_lsa_init(struct ospf_interface *oi)
 		list_delete(&oi->opaque_lsa_self);
 
 	oi->opaque_lsa_self = list_new();
-	oi->opaque_lsa_self->del = free_opaque_info_per_type;
+	oi->opaque_lsa_self->del = free_opaque_info_per_type_del;
 	oi->t_opaque_lsa_self = NULL;
 	return 0;
 }
 
 void ospf_opaque_type9_lsa_term(struct ospf_interface *oi)
 {
-	OSPF_TIMER_OFF(oi->t_opaque_lsa_self);
+	THREAD_OFF(oi->t_opaque_lsa_self);
 	if (oi->opaque_lsa_self != NULL)
 		list_delete(&oi->opaque_lsa_self);
 	oi->opaque_lsa_self = NULL;
@@ -161,7 +165,7 @@ int ospf_opaque_type10_lsa_init(struct ospf_area *area)
 		list_delete(&area->opaque_lsa_self);
 
 	area->opaque_lsa_self = list_new();
-	area->opaque_lsa_self->del = free_opaque_info_per_type;
+	area->opaque_lsa_self->del = free_opaque_info_per_type_del;
 	area->t_opaque_lsa_self = NULL;
 
 #ifdef MONITOR_LSDB_CHANGE
@@ -177,7 +181,7 @@ void ospf_opaque_type10_lsa_term(struct ospf_area *area)
 	area->lsdb->new_lsa_hook = area->lsdb->del_lsa_hook = NULL;
 #endif /* MONITOR_LSDB_CHANGE */
 
-	OSPF_TIMER_OFF(area->t_opaque_lsa_self);
+	THREAD_OFF(area->t_opaque_lsa_self);
 	if (area->opaque_lsa_self != NULL)
 		list_delete(&area->opaque_lsa_self);
 	return;
@@ -189,7 +193,7 @@ int ospf_opaque_type11_lsa_init(struct ospf *top)
 		list_delete(&top->opaque_lsa_self);
 
 	top->opaque_lsa_self = list_new();
-	top->opaque_lsa_self->del = free_opaque_info_per_type;
+	top->opaque_lsa_self->del = free_opaque_info_per_type_del;
 	top->t_opaque_lsa_self = NULL;
 
 #ifdef MONITOR_LSDB_CHANGE
@@ -205,7 +209,7 @@ void ospf_opaque_type11_lsa_term(struct ospf *top)
 	top->lsdb->new_lsa_hook = top->lsdb->del_lsa_hook = NULL;
 #endif /* MONITOR_LSDB_CHANGE */
 
-	OSPF_TIMER_OFF(top->t_opaque_lsa_self);
+	THREAD_OFF(top->t_opaque_lsa_self);
 	if (top->opaque_lsa_self != NULL)
 		list_delete(&top->opaque_lsa_self);
 	return;
@@ -258,11 +262,14 @@ static const char *ospf_opaque_type_name(uint8_t opaque_type)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are management functions to store user specified callbacks.
+ * Following are management functions to store user specified callbacks.
  *------------------------------------------------------------------------*/
 
 struct opaque_info_per_type; /* Forward declaration. */
 
+static void free_opaque_info_per_type(struct opaque_info_per_type *oipt,
+				      bool cleanup_owner);
+
 struct ospf_opaque_functab {
 	uint8_t opaque_type;
 	struct opaque_info_per_type *oipt;
@@ -274,7 +281,8 @@ struct ospf_opaque_functab {
 	void (*config_write_router)(struct vty *vty);
 	void (*config_write_if)(struct vty *vty, struct interface *ifp);
 	void (*config_write_debug)(struct vty *vty);
-	void (*show_opaque_info)(struct vty *vty, struct ospf_lsa *lsa);
+	void (*show_opaque_info)(struct vty *vty, struct json_object *json,
+				 struct ospf_lsa *lsa);
 	int (*lsa_originator)(void *arg);
 	struct ospf_lsa *(*lsa_refresher)(struct ospf_lsa *lsa);
 	int (*new_lsa_hook)(struct ospf_lsa *lsa);
@@ -355,9 +363,8 @@ static struct list *ospf_get_opaque_funclist(uint8_t lsa_type)
 		funclist = ospf_opaque_type11_funclist;
 		break;
 	default:
-		flog_warn(EC_OSPF_LSA_UNEXPECTED,
-			  "ospf_get_opaque_funclist: Unexpected LSA-type(%u)",
-			  lsa_type);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, lsa_type);
 		break;
 	}
 	return funclist;
@@ -373,7 +380,8 @@ int ospf_register_opaque_functab(
 	void (*config_write_router)(struct vty *vty),
 	void (*config_write_if)(struct vty *vty, struct interface *ifp),
 	void (*config_write_debug)(struct vty *vty),
-	void (*show_opaque_info)(struct vty *vty, struct ospf_lsa *lsa),
+	void (*show_opaque_info)(struct vty *vty, struct json_object *json,
+				 struct ospf_lsa *lsa),
 	int (*lsa_originator)(void *arg),
 	struct ospf_lsa *(*lsa_refresher)(struct ospf_lsa *lsa),
 	int (*new_lsa_hook)(struct ospf_lsa *lsa),
@@ -392,8 +400,8 @@ int ospf_register_opaque_functab(
 		if (functab->opaque_type == opaque_type) {
 			flog_warn(
 				EC_OSPF_LSA,
-				"ospf_register_opaque_functab: Duplicated entry?: lsa_type(%u), opaque_type(%u)",
-				lsa_type, opaque_type);
+				"%s: Duplicated entry?: lsa_type(%u), opaque_type(%u)",
+				__func__, lsa_type, opaque_type);
 			return -1;
 		}
 
@@ -431,12 +439,9 @@ void ospf_delete_opaque_functab(uint8_t lsa_type, uint8_t opaque_type)
 			if (functab->opaque_type == opaque_type) {
 				/* Cleanup internal control information, if it
 				 * still remains. */
-				if (functab->oipt != NULL) {
-					free_opaque_info_owner(functab->oipt);
-					free_opaque_info_per_type(
-						functab->oipt);
-				}
-
+				if (functab->oipt != NULL)
+					free_opaque_info_per_type(functab->oipt,
+								  true);
 				/* Dequeue listnode entry from the list. */
 				listnode_delete(funclist, functab);
 
@@ -465,7 +470,7 @@ ospf_opaque_functab_lookup(struct ospf_lsa *lsa)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are management functions for self-originated LSA entries.
+ * Following are management functions for self-originated LSA entries.
  *------------------------------------------------------------------------*/
 
 /*
@@ -556,8 +561,7 @@ register_opaque_info_per_type(struct ospf_opaque_functab *functab,
 	case OSPF_OPAQUE_AS_LSA:
 		top = ospf_lookup_by_vrf_id(new->vrf_id);
 		if (new->area != NULL && (top = new->area->ospf) == NULL) {
-			free_opaque_info_owner(oipt);
-			free_opaque_info_per_type(oipt);
+			free_opaque_info_per_type(oipt, true);
 			oipt = NULL;
 			goto out; /* This case may not exist. */
 		}
@@ -565,12 +569,9 @@ register_opaque_info_per_type(struct ospf_opaque_functab *functab,
 		listnode_add(top->opaque_lsa_self, oipt);
 		break;
 	default:
-		flog_warn(
-			EC_OSPF_LSA_UNEXPECTED,
-			"register_opaque_info_per_type: Unexpected LSA-type(%u)",
-			new->data->type);
-		free_opaque_info_owner(oipt);
-		free_opaque_info_per_type(oipt);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, new->data->type);
+		free_opaque_info_per_type(oipt, true);
 		oipt = NULL;
 		goto out; /* This case may not exist. */
 	}
@@ -578,7 +579,6 @@ register_opaque_info_per_type(struct ospf_opaque_functab *functab,
 	oipt->lsa_type = new->data->type;
 	oipt->opaque_type = GET_OPAQUE_TYPE(ntohl(new->data->id.s_addr));
 	oipt->status = PROC_NORMAL;
-	oipt->t_opaque_lsa_self = NULL;
 	oipt->functab = functab;
 	functab->oipt = oipt;
 	oipt->id_list = list_new();
@@ -588,42 +588,13 @@ out:
 	return oipt;
 }
 
-/* Remove "oipt" from its owner's self-originated LSA list. */
-static void free_opaque_info_owner(void *val)
+static void free_opaque_info_per_type(struct opaque_info_per_type *oipt,
+				      bool cleanup_owner)
 {
-	struct opaque_info_per_type *oipt = (struct opaque_info_per_type *)val;
-
-	switch (oipt->lsa_type) {
-	case OSPF_OPAQUE_LINK_LSA: {
-		struct ospf_interface *oi =
-			(struct ospf_interface *)(oipt->owner);
-		listnode_delete(oi->opaque_lsa_self, oipt);
-		break;
-	}
-	case OSPF_OPAQUE_AREA_LSA: {
-		struct ospf_area *area = (struct ospf_area *)(oipt->owner);
-		listnode_delete(area->opaque_lsa_self, oipt);
-		break;
-	}
-	case OSPF_OPAQUE_AS_LSA: {
-		struct ospf *top = (struct ospf *)(oipt->owner);
-		listnode_delete(top->opaque_lsa_self, oipt);
-		break;
-	}
-	default:
-		flog_warn(EC_OSPF_LSA_UNEXPECTED,
-			  "free_opaque_info_owner: Unexpected LSA-type(%u)",
-			  oipt->lsa_type);
-		break; /* This case may not exist. */
-	}
-}
-
-static void free_opaque_info_per_type(void *val)
-{
-	struct opaque_info_per_type *oipt = (struct opaque_info_per_type *)val;
 	struct opaque_info_per_id *oipi;
 	struct ospf_lsa *lsa;
 	struct listnode *node, *nnode;
+	struct list *l;
 
 	/* Control information per opaque-id may still exist. */
 	for (ALL_LIST_ELEMENTS(oipt->id_list, node, nnode, oipi)) {
@@ -634,12 +605,39 @@ static void free_opaque_info_per_type(void *val)
 		ospf_opaque_lsa_flush_schedule(lsa);
 	}
 
-	OSPF_TIMER_OFF(oipt->t_opaque_lsa_self);
+	THREAD_OFF(oipt->t_opaque_lsa_self);
 	list_delete(&oipt->id_list);
+	if (cleanup_owner) {
+		/* Remove from its owner's self-originated LSA list. */
+		switch (oipt->lsa_type) {
+		case OSPF_OPAQUE_LINK_LSA:
+			l = ((struct ospf_interface *)oipt->owner)
+				    ->opaque_lsa_self;
+			break;
+		case OSPF_OPAQUE_AREA_LSA:
+			l = ((struct ospf_area *)oipt->owner)->opaque_lsa_self;
+			break;
+		case OSPF_OPAQUE_AS_LSA:
+			l = ((struct ospf *)oipt->owner)->opaque_lsa_self;
+			break;
+		default:
+			flog_warn(
+				EC_OSPF_LSA_UNEXPECTED,
+				"free_opaque_info_owner: Unexpected LSA-type(%u)",
+				oipt->lsa_type);
+			return;
+		}
+		listnode_delete(l, oipt);
+	}
 	XFREE(MTYPE_OPAQUE_INFO_PER_TYPE, oipt);
 	return;
 }
 
+static void free_opaque_info_per_type_del(void *val)
+{
+	free_opaque_info_per_type((struct opaque_info_per_type *)val, false);
+}
+
 static struct opaque_info_per_type *
 lookup_opaque_info_by_type(struct ospf_lsa *lsa)
 {
@@ -679,9 +677,8 @@ lookup_opaque_info_by_type(struct ospf_lsa *lsa)
 		listtop = top->opaque_lsa_self;
 		break;
 	default:
-		flog_warn(EC_OSPF_LSA_UNEXPECTED,
-			  "lookup_opaque_info_by_type: Unexpected LSA-type(%u)",
-			  lsa->data->type);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, lsa->data->type);
 		break;
 	}
 
@@ -703,7 +700,6 @@ register_opaque_info_per_id(struct opaque_info_per_type *oipt,
 		       sizeof(struct opaque_info_per_id));
 
 	oipi->opaque_id = GET_OPAQUE_ID(ntohl(new->data->id.s_addr));
-	oipi->t_opaque_lsa_self = NULL;
 	oipi->opqctl_type = oipt;
 	oipi->lsa = ospf_lsa_lock(new);
 
@@ -716,7 +712,7 @@ static void free_opaque_info_per_id(void *val)
 {
 	struct opaque_info_per_id *oipi = (struct opaque_info_per_id *)val;
 
-	OSPF_TIMER_OFF(oipi->t_opaque_lsa_self);
+	THREAD_OFF(oipi->t_opaque_lsa_self);
 	if (oipi->lsa != NULL)
 		ospf_lsa_unlock(&oipi->lsa);
 	XFREE(MTYPE_OPAQUE_INFO_PER_ID, oipi);
@@ -758,8 +754,15 @@ out:
 	return oipi;
 }
 
+int ospf_opaque_is_owned(struct ospf_lsa *lsa)
+{
+	struct opaque_info_per_type *oipt = lookup_opaque_info_by_type(lsa);
+
+	return (oipt != NULL && lookup_opaque_info_by_id(oipt, lsa) != NULL);
+}
+
 /*------------------------------------------------------------------------*
- * Followings are (vty) configuration functions for Opaque-LSAs handling.
+ * Following are (vty) configuration functions for Opaque-LSAs handling.
  *------------------------------------------------------------------------*/
 
 DEFUN (capability_opaque,
@@ -830,7 +833,7 @@ static void ospf_opaque_register_vty(void)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are collection of user-registered function callers.
+ * Following are collection of user-registered function callers.
  *------------------------------------------------------------------------*/
 
 static int opaque_lsa_new_if_callback(struct list *funclist,
@@ -978,7 +981,7 @@ out:
 }
 
 /*------------------------------------------------------------------------*
- * Followings are glue functions to call Opaque-LSA specific processing.
+ * Following are glue functions to call Opaque-LSA specific processing.
  *------------------------------------------------------------------------*/
 
 int ospf_opaque_new_if(struct interface *ifp)
@@ -1161,23 +1164,48 @@ void ospf_opaque_config_write_debug(struct vty *vty)
 	return;
 }
 
-void show_opaque_info_detail(struct vty *vty, struct ospf_lsa *lsa)
+void show_opaque_info_detail(struct vty *vty, struct ospf_lsa *lsa,
+			     json_object *json)
 {
+	char buf[128], *bp;
 	struct lsa_header *lsah = lsa->data;
 	uint32_t lsid = ntohl(lsah->id.s_addr);
 	uint8_t opaque_type = GET_OPAQUE_TYPE(lsid);
 	uint32_t opaque_id = GET_OPAQUE_ID(lsid);
 	struct ospf_opaque_functab *functab;
+	int len, lenValid;
 
 	/* Switch output functionality by vty address. */
 	if (vty != NULL) {
-		vty_out(vty, "  Opaque-Type %u (%s)\n", opaque_type,
-			ospf_opaque_type_name(opaque_type));
-		vty_out(vty, "  Opaque-ID   0x%x\n", opaque_id);
+		if (!json) {
+			vty_out(vty, "  Opaque-Type %u (%s)\n", opaque_type,
+				ospf_opaque_type_name(opaque_type));
+			vty_out(vty, "  Opaque-ID   0x%x\n", opaque_id);
 
-		vty_out(vty, "  Opaque-Info: %u octets of data%s\n",
-			ntohs(lsah->length) - OSPF_LSA_HEADER_SIZE,
-			VALID_OPAQUE_INFO_LEN(lsah) ? "" : "(Invalid length?)");
+			vty_out(vty, "  Opaque-Info: %u octets of data%s\n",
+				ntohs(lsah->length) - OSPF_LSA_HEADER_SIZE,
+				VALID_OPAQUE_INFO_LEN(lsah)
+					? ""
+					: "(Invalid length?)");
+		} else {
+			json_object_string_add(
+				json, "opaqueType",
+				ospf_opaque_type_name(opaque_type));
+			json_object_int_add(json, "opaqueId", opaque_id);
+			len = ntohs(lsah->length) - OSPF_LSA_HEADER_SIZE;
+			json_object_int_add(json, "opaqueDataLength", len);
+			lenValid = VALID_OPAQUE_INFO_LEN(lsah);
+			json_object_boolean_add(json, "opaqueDataLengthValid",
+						lenValid);
+			if (lenValid) {
+				bp = asnprintfrr(MTYPE_TMP, buf, sizeof(buf),
+						 "%*pHXn", (int)len,
+						 (lsah + 1));
+				json_object_string_add(json, "opaqueData", buf);
+				if (bp != buf)
+					XFREE(MTYPE_TMP, bp);
+			}
+		}
 	} else {
 		zlog_debug("    Opaque-Type %u (%s)", opaque_type,
 			   ospf_opaque_type_name(opaque_type));
@@ -1192,17 +1220,18 @@ void show_opaque_info_detail(struct vty *vty, struct ospf_lsa *lsa)
 	/* Call individual output functions. */
 	if ((functab = ospf_opaque_functab_lookup(lsa)) != NULL)
 		if (functab->show_opaque_info != NULL)
-			(*functab->show_opaque_info)(vty, lsa);
+			(*functab->show_opaque_info)(vty, json, lsa);
 
 	return;
 }
 
 void ospf_opaque_lsa_dump(struct stream *s, uint16_t length)
 {
-	struct ospf_lsa lsa;
+	struct ospf_lsa lsa = {};
 
 	lsa.data = (struct lsa_header *)stream_pnt(s);
-	show_opaque_info_detail(NULL, &lsa);
+	lsa.size = length;
+	show_opaque_info_detail(NULL, &lsa, NULL);
 	return;
 }
 
@@ -1267,12 +1296,12 @@ out:
 }
 
 /*------------------------------------------------------------------------*
- * Followings are Opaque-LSA origination/refresh management functions.
+ * Following are Opaque-LSA origination/refresh management functions.
  *------------------------------------------------------------------------*/
 
-static int ospf_opaque_type9_lsa_originate(struct thread *t);
-static int ospf_opaque_type10_lsa_originate(struct thread *t);
-static int ospf_opaque_type11_lsa_originate(struct thread *t);
+static void ospf_opaque_type9_lsa_originate(struct thread *t);
+static void ospf_opaque_type10_lsa_originate(struct thread *t);
+static void ospf_opaque_type11_lsa_originate(struct thread *t);
 static void ospf_opaque_lsa_reoriginate_resume(struct list *listtop, void *arg);
 
 void ospf_opaque_lsa_originate_schedule(struct ospf_interface *oi, int *delay0)
@@ -1285,16 +1314,14 @@ void ospf_opaque_lsa_originate_schedule(struct ospf_interface *oi, int *delay0)
 
 	if ((top = oi_to_top(oi)) == NULL || (area = oi->area) == NULL) {
 		if (IS_DEBUG_OSPF_EVENT)
-			zlog_debug(
-				"ospf_opaque_lsa_originate_schedule: Invalid argument?");
+			zlog_debug("%s: Invalid argument?", __func__);
 		return;
 	}
 
 	/* It may not a right time to schedule origination now. */
 	if (!CHECK_FLAG(top->opaque, OPAQUE_OPERATION_READY_BIT)) {
 		if (IS_DEBUG_OSPF_EVENT)
-			zlog_debug(
-				"ospf_opaque_lsa_originate_schedule: Not operational.");
+			zlog_debug("%s: Not operational.", __func__);
 		return; /* This is not an error. */
 	}
 
@@ -1444,10 +1471,9 @@ void ospf_opaque_lsa_originate_schedule(struct ospf_interface *oi, int *delay0)
 		*delay0 = delay;
 }
 
-static int ospf_opaque_type9_lsa_originate(struct thread *t)
+static void ospf_opaque_type9_lsa_originate(struct thread *t)
 {
 	struct ospf_interface *oi;
-	int rc;
 
 	oi = THREAD_ARG(t);
 	oi->t_opaque_lsa_self = NULL;
@@ -1456,33 +1482,27 @@ static int ospf_opaque_type9_lsa_originate(struct thread *t)
 		zlog_debug("Timer[Type9-LSA]: Originate Opaque-LSAs for OI %s",
 			   IF_NAME(oi));
 
-	rc = opaque_lsa_originate_callback(ospf_opaque_type9_funclist, oi);
-
-	return rc;
+	opaque_lsa_originate_callback(ospf_opaque_type9_funclist, oi);
 }
 
-static int ospf_opaque_type10_lsa_originate(struct thread *t)
+static void ospf_opaque_type10_lsa_originate(struct thread *t)
 {
 	struct ospf_area *area;
-	int rc;
 
 	area = THREAD_ARG(t);
 	area->t_opaque_lsa_self = NULL;
 
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug(
-			"Timer[Type10-LSA]: Originate Opaque-LSAs for Area %s",
-			inet_ntoa(area->area_id));
+			"Timer[Type10-LSA]: Originate Opaque-LSAs for Area %pI4",
+			&area->area_id);
 
-	rc = opaque_lsa_originate_callback(ospf_opaque_type10_funclist, area);
-
-	return rc;
+	opaque_lsa_originate_callback(ospf_opaque_type10_funclist, area);
 }
 
-static int ospf_opaque_type11_lsa_originate(struct thread *t)
+static void ospf_opaque_type11_lsa_originate(struct thread *t)
 {
 	struct ospf *top;
-	int rc;
 
 	top = THREAD_ARG(t);
 	top->t_opaque_lsa_self = NULL;
@@ -1491,9 +1511,7 @@ static int ospf_opaque_type11_lsa_originate(struct thread *t)
 		zlog_debug(
 			"Timer[Type11-LSA]: Originate AS-External Opaque-LSAs");
 
-	rc = opaque_lsa_originate_callback(ospf_opaque_type11_funclist, top);
-
-	return rc;
+	opaque_lsa_originate_callback(ospf_opaque_type11_funclist, top);
 }
 
 static void ospf_opaque_lsa_reoriginate_resume(struct list *listtop, void *arg)
@@ -1520,10 +1538,8 @@ static void ospf_opaque_lsa_reoriginate_resume(struct list *listtop, void *arg)
 			continue;
 
 		if ((*functab->lsa_originator)(arg) != 0) {
-			flog_warn(
-				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_resume: Failed (opaque-type=%u)",
-				oipt->opaque_type);
+			flog_warn(EC_OSPF_LSA, "%s: Failed (opaque-type=%u)",
+				  __func__, oipt->opaque_type);
 			continue;
 		}
 	}
@@ -1561,8 +1577,7 @@ struct ospf_lsa *ospf_opaque_lsa_install(struct ospf_lsa *lsa, int rt_recalc)
 	}
 	/* Register the new lsa entry */
 	else if (register_opaque_lsa(lsa) == NULL) {
-		flog_warn(EC_OSPF_LSA,
-			  "ospf_opaque_lsa_install: register_opaque_lsa() ?");
+		flog_warn(EC_OSPF_LSA, "%s: register_opaque_lsa() ?", __func__);
 		goto out;
 	}
 
@@ -1574,16 +1589,16 @@ struct ospf_lsa *ospf_opaque_lsa_install(struct ospf_lsa *lsa, int rt_recalc)
 	case OSPF_OPAQUE_LINK_LSA:
 		if ((top = oi_to_top(lsa->oi)) == NULL) {
 			/* Above conditions must have passed. */
-			flog_warn(EC_OSPF_LSA,
-				  "ospf_opaque_lsa_install: Something wrong?");
+			flog_warn(EC_OSPF_LSA, "%s: Something wrong?",
+				  __func__);
 			goto out;
 		}
 		break;
 	case OSPF_OPAQUE_AREA_LSA:
 		if (lsa->area == NULL || (top = lsa->area->ospf) == NULL) {
 			/* Above conditions must have passed. */
-			flog_warn(EC_OSPF_LSA,
-				  "ospf_opaque_lsa_install: Something wrong?");
+			flog_warn(EC_OSPF_LSA, "%s: Something wrong?",
+				  __func__);
 			goto out;
 		}
 		break;
@@ -1591,15 +1606,14 @@ struct ospf_lsa *ospf_opaque_lsa_install(struct ospf_lsa *lsa, int rt_recalc)
 		top = ospf_lookup_by_vrf_id(lsa->vrf_id);
 		if (lsa->area != NULL && (top = lsa->area->ospf) == NULL) {
 			/* Above conditions must have passed. */
-			flog_warn(EC_OSPF_LSA,
-				  "ospf_opaque_lsa_install: Something wrong?");
+			flog_warn(EC_OSPF_LSA, "%s: Something wrong?",
+				  __func__);
 			goto out;
 		}
 		break;
 	default:
-		flog_warn(EC_OSPF_LSA_UNEXPECTED,
-			  "ospf_opaque_lsa_install: Unexpected LSA-type(%u)",
-			  lsa->data->type);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, lsa->data->type);
 		goto out;
 	}
 
@@ -1628,8 +1642,8 @@ struct ospf_lsa *ospf_opaque_lsa_refresh(struct ospf_lsa *lsa)
 		 * LSA from the routing domain.
 		 */
 		if (IS_DEBUG_OSPF_EVENT)
-			zlog_debug("LSA[Type%d:%s]: Flush stray Opaque-LSA",
-				   lsa->data->type, inet_ntoa(lsa->data->id));
+			zlog_debug("LSA[Type%d:%pI4]: Flush stray Opaque-LSA",
+				   lsa->data->type, &lsa->data->id);
 
 		lsa->data->ls_age = htons(OSPF_LSA_MAXAGE);
 		ospf_lsa_flush(ospf, lsa);
@@ -1640,7 +1654,7 @@ struct ospf_lsa *ospf_opaque_lsa_refresh(struct ospf_lsa *lsa)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are re-origination/refresh/flush operations of Opaque-LSAs,
+ * Following are re-origination/refresh/flush operations of Opaque-LSAs,
  * triggered by external interventions (vty session, signaling, etc).
  *------------------------------------------------------------------------*/
 
@@ -1649,10 +1663,10 @@ struct ospf_lsa *ospf_opaque_lsa_refresh(struct ospf_lsa *lsa)
 static struct ospf_lsa *pseudo_lsa(struct ospf_interface *oi,
 				   struct ospf_area *area, uint8_t lsa_type,
 				   uint8_t opaque_type);
-static int ospf_opaque_type9_lsa_reoriginate_timer(struct thread *t);
-static int ospf_opaque_type10_lsa_reoriginate_timer(struct thread *t);
-static int ospf_opaque_type11_lsa_reoriginate_timer(struct thread *t);
-static int ospf_opaque_lsa_refresh_timer(struct thread *t);
+static void ospf_opaque_type9_lsa_reoriginate_timer(struct thread *t);
+static void ospf_opaque_type10_lsa_reoriginate_timer(struct thread *t);
+static void ospf_opaque_type11_lsa_reoriginate_timer(struct thread *t);
+static void ospf_opaque_lsa_refresh_timer(struct thread *t);
 
 void ospf_opaque_lsa_reoriginate_schedule(void *lsa_type_dependent,
 					  uint8_t lsa_type, uint8_t opaque_type)
@@ -1663,23 +1677,21 @@ void ospf_opaque_lsa_reoriginate_schedule(void *lsa_type_dependent,
 
 	struct ospf_lsa *lsa;
 	struct opaque_info_per_type *oipt;
-	int (*func)(struct thread * t) = NULL;
+	void (*func)(struct thread * t) = NULL;
 	int delay;
 
 	switch (lsa_type) {
 	case OSPF_OPAQUE_LINK_LSA:
 		if ((oi = (struct ospf_interface *)lsa_type_dependent)
 		    == NULL) {
-			flog_warn(
-				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_schedule: Type-9 Opaque-LSA: Invalid parameter?");
+			flog_warn(EC_OSPF_LSA,
+				  "%s: Type-9 Opaque-LSA: Invalid parameter?",
+				  __func__);
 			goto out;
 		}
 		if ((top = oi_to_top(oi)) == NULL) {
-			flog_warn(
-				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_schedule: OI(%s) -> TOP?",
-				IF_NAME(oi));
+			flog_warn(EC_OSPF_LSA, "%s: OI(%s) -> TOP?", __func__,
+				  IF_NAME(oi));
 			goto out;
 		}
 		if (!list_isempty(ospf_opaque_type9_funclist)
@@ -1695,16 +1707,14 @@ void ospf_opaque_lsa_reoriginate_schedule(void *lsa_type_dependent,
 		break;
 	case OSPF_OPAQUE_AREA_LSA:
 		if ((area = (struct ospf_area *)lsa_type_dependent) == NULL) {
-			flog_warn(
-				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_schedule: Type-10 Opaque-LSA: Invalid parameter?");
+			flog_warn(EC_OSPF_LSA,
+				  "%s: Type-10 Opaque-LSA: Invalid parameter?",
+				  __func__);
 			goto out;
 		}
 		if ((top = area->ospf) == NULL) {
-			flog_warn(
-				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_schedule: AREA(%s) -> TOP?",
-				inet_ntoa(area->area_id));
+			flog_warn(EC_OSPF_LSA, "%s: AREA(%pI4) -> TOP?",
+				  __func__, &area->area_id);
 			goto out;
 		}
 		if (!list_isempty(ospf_opaque_type10_funclist)
@@ -1712,17 +1722,17 @@ void ospf_opaque_lsa_reoriginate_schedule(void *lsa_type_dependent,
 		    && area->t_opaque_lsa_self != NULL) {
 			flog_warn(
 				EC_OSPF_LSA,
-				"Type-10 Opaque-LSA (opaque_type=%u): Common origination for AREA(%s) has already started",
-				opaque_type, inet_ntoa(area->area_id));
+				"Type-10 Opaque-LSA (opaque_type=%u): Common origination for AREA(%pI4) has already started",
+				opaque_type, &area->area_id);
 			goto out;
 		}
 		func = ospf_opaque_type10_lsa_reoriginate_timer;
 		break;
 	case OSPF_OPAQUE_AS_LSA:
 		if ((top = (struct ospf *)lsa_type_dependent) == NULL) {
-			flog_warn(
-				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_schedule: Type-11 Opaque-LSA: Invalid parameter?");
+			flog_warn(EC_OSPF_LSA,
+				  "%s: Type-11 Opaque-LSA: Invalid parameter?",
+				  __func__);
 			goto out;
 		}
 		if (!list_isempty(ospf_opaque_type11_funclist)
@@ -1742,18 +1752,15 @@ void ospf_opaque_lsa_reoriginate_schedule(void *lsa_type_dependent,
 		func = ospf_opaque_type11_lsa_reoriginate_timer;
 		break;
 	default:
-		flog_warn(
-			EC_OSPF_LSA_UNEXPECTED,
-			"ospf_opaque_lsa_reoriginate_schedule: Unexpected LSA-type(%u)",
-			lsa_type);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, lsa_type);
 		goto out;
 	}
 
 	/* It may not a right time to schedule reorigination now. */
 	if (!CHECK_FLAG(top->opaque, OPAQUE_OPERATION_READY_BIT)) {
 		if (IS_DEBUG_OSPF_EVENT)
-			zlog_debug(
-				"ospf_opaque_lsa_reoriginate_schedule: Not operational.");
+			zlog_debug("%s: Not operational.", __func__);
 		goto out; /* This is not an error. */
 	}
 
@@ -1766,16 +1773,16 @@ void ospf_opaque_lsa_reoriginate_schedule(void *lsa_type_dependent,
 		if ((functab = ospf_opaque_functab_lookup(lsa)) == NULL) {
 			flog_warn(
 				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_schedule: No associated function?: lsa_type(%u), opaque_type(%u)",
-				lsa_type, opaque_type);
+				"%s: No associated function?: lsa_type(%u), opaque_type(%u)",
+				__func__, lsa_type, opaque_type);
 			goto out;
 		}
 		if ((oipt = register_opaque_info_per_type(functab, lsa))
 		    == NULL) {
 			flog_warn(
 				EC_OSPF_LSA,
-				"ospf_opaque_lsa_reoriginate_schedule: Cannot get a control info?: lsa_type(%u), opaque_type(%u)",
-				lsa_type, opaque_type);
+				"%s: Cannot get a control info?: lsa_type(%u), opaque_type(%u)",
+				__func__, lsa_type, opaque_type);
 			goto out;
 		}
 	}
@@ -1830,31 +1837,25 @@ static struct ospf_lsa *pseudo_lsa(struct ospf_interface *oi,
 	return &lsa;
 }
 
-static int ospf_opaque_type9_lsa_reoriginate_timer(struct thread *t)
+static void ospf_opaque_type9_lsa_reoriginate_timer(struct thread *t)
 {
 	struct opaque_info_per_type *oipt;
 	struct ospf_opaque_functab *functab;
 	struct ospf *top;
 	struct ospf_interface *oi;
-	int rc = -1;
 
 	oipt = THREAD_ARG(t);
-	oipt->t_opaque_lsa_self = NULL;
 
 	if ((functab = oipt->functab) == NULL
 	    || functab->lsa_originator == NULL) {
-		flog_warn(
-			EC_OSPF_LSA,
-			"ospf_opaque_type9_lsa_reoriginate_timer: No associated function?");
-		goto out;
+		flog_warn(EC_OSPF_LSA, "%s: No associated function?", __func__);
+		return;
 	}
 
 	oi = (struct ospf_interface *)oipt->owner;
 	if ((top = oi_to_top(oi)) == NULL) {
-		flog_warn(
-			EC_OSPF_LSA,
-			"ospf_opaque_type9_lsa_reoriginate_timer: Something wrong?");
-		goto out;
+		flog_warn(EC_OSPF_LSA, "%s: Something wrong?", __func__);
+		return;
 	}
 
 	if (!CHECK_FLAG(top->config, OSPF_OPAQUE_CAPABLE)
@@ -1866,8 +1867,7 @@ static int ospf_opaque_type9_lsa_reoriginate_timer(struct thread *t)
 				oipt->opaque_type);
 
 		oipt->status = PROC_SUSPEND;
-		rc = 0;
-		goto out;
+		return;
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
@@ -1875,12 +1875,10 @@ static int ospf_opaque_type9_lsa_reoriginate_timer(struct thread *t)
 			"Timer[Type9-LSA]: Re-originate Opaque-LSAs (opaque-type=%u) for OI (%s)",
 			oipt->opaque_type, IF_NAME(oi));
 
-	rc = (*functab->lsa_originator)(oi);
-out:
-	return rc;
+	(*functab->lsa_originator)(oi);
 }
 
-static int ospf_opaque_type10_lsa_reoriginate_timer(struct thread *t)
+static void ospf_opaque_type10_lsa_reoriginate_timer(struct thread *t)
 {
 	struct opaque_info_per_type *oipt;
 	struct ospf_opaque_functab *functab;
@@ -1888,25 +1886,20 @@ static int ospf_opaque_type10_lsa_reoriginate_timer(struct thread *t)
 	struct ospf *top;
 	struct ospf_area *area;
 	struct ospf_interface *oi;
-	int n, rc = -1;
+	int n;
 
 	oipt = THREAD_ARG(t);
-	oipt->t_opaque_lsa_self = NULL;
 
 	if ((functab = oipt->functab) == NULL
 	    || functab->lsa_originator == NULL) {
-		flog_warn(
-			EC_OSPF_LSA,
-			"ospf_opaque_type10_lsa_reoriginate_timer: No associated function?");
-		goto out;
+		flog_warn(EC_OSPF_LSA, "%s: No associated function?", __func__);
+		return;
 	}
 
 	area = (struct ospf_area *)oipt->owner;
 	if (area == NULL || (top = area->ospf) == NULL) {
-		flog_warn(
-			EC_OSPF_LSA,
-			"ospf_opaque_type10_lsa_reoriginate_timer: Something wrong?");
-		goto out;
+		flog_warn(EC_OSPF_LSA, "%s: Something wrong?", __func__);
+		return;
 	}
 
 	/* There must be at least one "opaque-capable, full-state" neighbor. */
@@ -1923,43 +1916,34 @@ static int ospf_opaque_type10_lsa_reoriginate_timer(struct thread *t)
 				oipt->opaque_type);
 
 		oipt->status = PROC_SUSPEND;
-		rc = 0;
-		goto out;
+		return;
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug(
-			"Timer[Type10-LSA]: Re-originate Opaque-LSAs (opaque-type=%u) for Area %s",
-			oipt->opaque_type, inet_ntoa(area->area_id));
+			"Timer[Type10-LSA]: Re-originate Opaque-LSAs (opaque-type=%u) for Area %pI4",
+			oipt->opaque_type, &area->area_id);
 
-	rc = (*functab->lsa_originator)(area);
-out:
-	return rc;
+	(*functab->lsa_originator)(area);
 }
 
-static int ospf_opaque_type11_lsa_reoriginate_timer(struct thread *t)
+static void ospf_opaque_type11_lsa_reoriginate_timer(struct thread *t)
 {
 	struct opaque_info_per_type *oipt;
 	struct ospf_opaque_functab *functab;
 	struct ospf *top;
-	int rc = -1;
 
 	oipt = THREAD_ARG(t);
-	oipt->t_opaque_lsa_self = NULL;
 
 	if ((functab = oipt->functab) == NULL
 	    || functab->lsa_originator == NULL) {
-		flog_warn(
-			EC_OSPF_LSA,
-			"ospf_opaque_type11_lsa_reoriginate_timer: No associated function?");
-		goto out;
+		flog_warn(EC_OSPF_LSA, "%s: No associated function?", __func__);
+		return;
 	}
 
 	if ((top = (struct ospf *)oipt->owner) == NULL) {
-		flog_warn(
-			EC_OSPF_LSA,
-			"ospf_opaque_type11_lsa_reoriginate_timer: Something wrong?");
-		goto out;
+		flog_warn(EC_OSPF_LSA, "%s: Something wrong?", __func__);
+		return;
 	}
 
 	if (!CHECK_FLAG(top->config, OSPF_OPAQUE_CAPABLE)) {
@@ -1969,8 +1953,7 @@ static int ospf_opaque_type11_lsa_reoriginate_timer(struct thread *t)
 				oipt->opaque_type);
 
 		oipt->status = PROC_SUSPEND;
-		rc = 0;
-		goto out;
+		return;
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
@@ -1978,9 +1961,7 @@ static int ospf_opaque_type11_lsa_reoriginate_timer(struct thread *t)
 			"Timer[Type11-LSA]: Re-originate Opaque-LSAs (opaque-type=%u).",
 			oipt->opaque_type);
 
-	rc = (*functab->lsa_originator)(top);
-out:
-	return rc;
+	(*functab->lsa_originator)(top);
 }
 
 void ospf_opaque_lsa_refresh_schedule(struct ospf_lsa *lsa0)
@@ -1993,16 +1974,13 @@ void ospf_opaque_lsa_refresh_schedule(struct ospf_lsa *lsa0)
 
 	if ((oipt = lookup_opaque_info_by_type(lsa0)) == NULL
 	    || (oipi = lookup_opaque_info_by_id(oipt, lsa0)) == NULL) {
-		flog_warn(
-			EC_OSPF_LSA,
-			"ospf_opaque_lsa_refresh_schedule: Invalid parameter?");
+		flog_warn(EC_OSPF_LSA, "%s: Invalid parameter?", __func__);
 		goto out;
 	}
 
 	/* Given "lsa0" and current "oipi->lsa" may different, but harmless. */
 	if ((lsa = oipi->lsa) == NULL) {
-		flog_warn(EC_OSPF_LSA,
-			  "ospf_opaque_lsa_refresh_schedule: Something wrong?");
+		flog_warn(EC_OSPF_LSA, "%s: Something wrong?", __func__);
 		goto out;
 	}
 
@@ -2029,10 +2007,8 @@ void ospf_opaque_lsa_refresh_schedule(struct ospf_lsa *lsa0)
 		ospf_ls_retransmit_delete_nbr_as(top, lsa);
 		break;
 	default:
-		flog_warn(
-			EC_OSPF_LSA_UNEXPECTED,
-			"ospf_opaque_lsa_refresh_schedule: Unexpected LSA-type(%u)",
-			lsa->data->type);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, lsa->data->type);
 		goto out;
 	}
 
@@ -2051,7 +2027,7 @@ out:
 	return;
 }
 
-static int ospf_opaque_lsa_refresh_timer(struct thread *t)
+static void ospf_opaque_lsa_refresh_timer(struct thread *t)
 {
 	struct opaque_info_per_id *oipi;
 	struct ospf_opaque_functab *functab;
@@ -2061,14 +2037,11 @@ static int ospf_opaque_lsa_refresh_timer(struct thread *t)
 		zlog_debug("Timer[Opaque-LSA]: (Opaque-LSA Refresh expire)");
 
 	oipi = THREAD_ARG(t);
-	oipi->t_opaque_lsa_self = NULL;
 
 	if ((lsa = oipi->lsa) != NULL)
 		if ((functab = oipi->opqctl_type->functab) != NULL)
 			if (functab->lsa_refresher != NULL)
 				(*functab->lsa_refresher)(lsa);
-
-	return 0;
 }
 
 void ospf_opaque_lsa_flush_schedule(struct ospf_lsa *lsa0)
@@ -2082,18 +2055,27 @@ void ospf_opaque_lsa_flush_schedule(struct ospf_lsa *lsa0)
 
 	if ((oipt = lookup_opaque_info_by_type(lsa0)) == NULL
 	    || (oipi = lookup_opaque_info_by_id(oipt, lsa0)) == NULL) {
-		flog_warn(EC_OSPF_LSA,
-			  "ospf_opaque_lsa_flush_schedule: Invalid parameter?");
+		flog_warn(EC_OSPF_LSA, "%s: Invalid parameter?", __func__);
 		goto out;
 	}
 
 	/* Given "lsa0" and current "oipi->lsa" may different, but harmless. */
 	if ((lsa = oipi->lsa) == NULL) {
-		flog_warn(EC_OSPF_LSA,
-			  "ospf_opaque_lsa_flush_schedule: Something wrong?");
+		flog_warn(EC_OSPF_LSA, "%s: Something wrong?", __func__);
 		goto out;
 	}
 
+	if (lsa->opaque_zero_len_delete &&
+	    lsa->data->length != htons(sizeof(struct lsa_header))) {
+		/* minimize the size of the withdrawal: */
+		/*     increment the sequence number and make len just header */
+		/*     and update checksum */
+		lsa->data->ls_seqnum = lsa_seqnum_increment(lsa);
+		lsa->data->length = htons(sizeof(struct lsa_header));
+		lsa->data->checksum = 0;
+		lsa->data->checksum = ospf_lsa_checksum(lsa->data);
+	}
+
 	/* Delete this lsa from neighbor retransmit-list. */
 	switch (lsa->data->type) {
 	case OSPF_OPAQUE_LINK_LSA:
@@ -2106,22 +2088,17 @@ void ospf_opaque_lsa_flush_schedule(struct ospf_lsa *lsa0)
 		ospf_ls_retransmit_delete_nbr_as(top, lsa);
 		break;
 	default:
-		flog_warn(
-			EC_OSPF_LSA_UNEXPECTED,
-			"ospf_opaque_lsa_flush_schedule: Unexpected LSA-type(%u)",
-			lsa->data->type);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, lsa->data->type);
 		goto out;
 	}
 
+	/* This lsa will be flushed and removed eventually. */
+	ospf_lsa_flush(top, lsa);
+
 	/* Dequeue listnode entry from the list. */
 	listnode_delete(oipt->id_list, oipi);
 
-	/* Disassociate internal control information with the given lsa. */
-	free_opaque_info_per_id((void *)oipi);
-
-	/* Force given lsa's age to MaxAge. */
-	lsa->data->ls_age = htons(OSPF_LSA_MAXAGE);
-
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug(
 			"Schedule Type-%u Opaque-LSA to FLUSH: [opaque-type=%u, opaque-id=%x]",
@@ -2129,8 +2106,8 @@ void ospf_opaque_lsa_flush_schedule(struct ospf_lsa *lsa0)
 			GET_OPAQUE_TYPE(ntohl(lsa->data->id.s_addr)),
 			GET_OPAQUE_ID(ntohl(lsa->data->id.s_addr)));
 
-	/* This lsa will be flushed and removed eventually. */
-	ospf_lsa_flush(top, lsa);
+	/* Disassociate internal control information with the given lsa. */
+	free_opaque_info_per_id((void *)oipi);
 
 out:
 	return;
@@ -2144,15 +2121,27 @@ void ospf_opaque_self_originated_lsa_received(struct ospf_neighbor *nbr,
 	if ((top = oi_to_top(nbr->oi)) == NULL)
 		return;
 
+	if (IS_DEBUG_OSPF_EVENT)
+		zlog_debug(
+			"LSA[Type%d:%pI4]: processing self-originated Opaque-LSA",
+			lsa->data->type, &lsa->data->id);
+
 	/*
-	 * Since these LSA entries are not yet installed into corresponding
-	 * LSDB, just flush them without calling ospf_ls_maxage() afterward.
+	 * Install the stale LSA into the Link State Database, add it to the
+	 * MaxAge list, and flush it from the OSPF routing domain. For other
+	 * LSA types, the installation is done in the refresh function. It is
+	 * done inline here since the opaque refresh function is dynamically
+	 * registered when opaque LSAs are originated (which is not the case
+	 * for stale LSAs).
 	 */
 	lsa->data->ls_age = htons(OSPF_LSA_MAXAGE);
+	ospf_lsa_install(
+		top, (lsa->data->type == OSPF_OPAQUE_LINK_LSA) ? nbr->oi : NULL,
+		lsa);
+	ospf_lsa_maxage(top, lsa);
+
 	switch (lsa->data->type) {
 	case OSPF_OPAQUE_LINK_LSA:
-		ospf_flood_through_area(nbr->oi->area, NULL /*inbr*/, lsa);
-		break;
 	case OSPF_OPAQUE_AREA_LSA:
 		ospf_flood_through_area(nbr->oi->area, NULL /*inbr*/, lsa);
 		break;
@@ -2160,17 +2149,14 @@ void ospf_opaque_self_originated_lsa_received(struct ospf_neighbor *nbr,
 		ospf_flood_through_as(top, NULL /*inbr*/, lsa);
 		break;
 	default:
-		flog_warn(
-			EC_OSPF_LSA_UNEXPECTED,
-			"ospf_opaque_self_originated_lsa_received: Unexpected LSA-type(%u)",
-			lsa->data->type);
+		flog_warn(EC_OSPF_LSA_UNEXPECTED, "%s: Unexpected LSA-type(%u)",
+			  __func__, lsa->data->type);
 		return;
 	}
-	ospf_lsa_discard(lsa); /* List "lsas" will be deleted by caller. */
 }
 
 /*------------------------------------------------------------------------*
- * Followings are util functions; probably be used by Opaque-LSAs only...
+ * Following are util functions; probably be used by Opaque-LSAs only...
  *------------------------------------------------------------------------*/
 
 struct ospf *oi_to_top(struct ospf_interface *oi)
