@@ -66,14 +66,16 @@
 static struct ospf_router_info OspfRI;
 
 /*------------------------------------------------------------------------------*
- * Followings are initialize/terminate functions for Router Information
+ * Following are initialize/terminate functions for Router Information
  *handling.
  *------------------------------------------------------------------------------*/
 
 static void ospf_router_info_ism_change(struct ospf_interface *oi,
 					int old_status);
 static void ospf_router_info_config_write_router(struct vty *vty);
-static void ospf_router_info_show_info(struct vty *vty, struct ospf_lsa *lsa);
+static void ospf_router_info_show_info(struct vty *vty,
+				       struct json_object *json,
+				       struct ospf_lsa *lsa);
 static int ospf_router_info_lsa_originate(void *arg);
 static struct ospf_lsa *ospf_router_info_lsa_refresh(struct ospf_lsa *lsa);
 static void ospf_router_info_lsa_schedule(struct ospf_ri_area_info *ai,
@@ -88,7 +90,7 @@ int ospf_router_info_init(void)
 
 	zlog_info("RI (%s): Initialize Router Information", __func__);
 
-	memset(&OspfRI, 0, sizeof(struct ospf_router_info));
+	memset(&OspfRI, 0, sizeof(OspfRI));
 	OspfRI.enabled = false;
 	OspfRI.registered = 0;
 	OspfRI.scope = OSPF_OPAQUE_AS_LSA;
@@ -230,7 +232,7 @@ static struct ospf_ri_area_info *lookup_by_area(struct ospf_area *area)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are control functions for ROUTER INFORMATION parameters
+ * Following are control functions for ROUTER INFORMATION parameters
  *management.
  *------------------------------------------------------------------------*/
 
@@ -294,8 +296,8 @@ static void set_pce_address(struct in_addr ipv4, struct ospf_pce_info *pce)
 	pce->pce_header.header.type = htons(RI_TLV_PCE);
 	/* Set PCE Address */
 	pce->pce_address.header.type = htons(RI_PCE_SUBTLV_ADDRESS);
-	pce->pce_address.header.length = htons(PCE_ADDRESS_LENGTH_IPV4);
-	pce->pce_address.address.type = htons(PCE_ADDRESS_TYPE_IPV4);
+	pce->pce_address.header.length = htons(PCE_ADDRESS_IPV4_SIZE);
+	pce->pce_address.address.type = htons(PCE_ADDRESS_IPV4);
 	pce->pce_address.address.value = ipv4;
 
 	return;
@@ -323,7 +325,7 @@ static void set_pce_domain(uint16_t type, uint32_t domain,
 		      sizeof(struct ri_pce_subtlv_domain));
 
 	new->header.type = htons(RI_PCE_SUBTLV_DOMAIN);
-	new->header.length = htons(PCE_ADDRESS_LENGTH_IPV4);
+	new->header.length = htons(PCE_ADDRESS_IPV4_SIZE);
 	new->type = htons(type);
 	new->value = htonl(domain);
 
@@ -369,7 +371,7 @@ static void set_pce_neighbor(uint16_t type, uint32_t domain,
 		      sizeof(struct ri_pce_subtlv_neighbor));
 
 	new->header.type = htons(RI_PCE_SUBTLV_NEIGHBOR);
-	new->header.length = htons(PCE_ADDRESS_LENGTH_IPV4);
+	new->header.length = htons(PCE_ADDRESS_IPV4_SIZE);
 	new->type = htons(type);
 	new->value = htonl(domain);
 
@@ -447,8 +449,7 @@ static void set_sr_global_label_range(struct sr_block srgb)
 {
 	/* Set Header */
 	TLV_TYPE(OspfRI.sr_info.srgb) = htons(RI_SR_TLV_SRGB_LABEL_RANGE);
-	TLV_LEN(OspfRI.sr_info.srgb) =
-		htons(SUBTLV_SID_LABEL_SIZE + sizeof(uint32_t));
+	TLV_LEN(OspfRI.sr_info.srgb) = htons(RI_SR_TLV_LABEL_RANGE_SIZE);
 	/* Set Range Size */
 	OspfRI.sr_info.srgb.size = htonl(SET_RANGE_SIZE(srgb.range_size));
 	/* Set Lower bound label SubTLV */
@@ -471,8 +472,7 @@ static void set_sr_local_label_range(struct sr_block srlb)
 {
 	/* Set Header */
 	TLV_TYPE(OspfRI.sr_info.srlb) = htons(RI_SR_TLV_SRLB_LABEL_RANGE);
-	TLV_LEN(OspfRI.sr_info.srlb) =
-		htons(SUBTLV_SID_LABEL_SIZE + sizeof(uint32_t));
+	TLV_LEN(OspfRI.sr_info.srlb) = htons(RI_SR_TLV_LABEL_RANGE_SIZE);
 	/* Set Range Size */
 	OspfRI.sr_info.srlb.size = htonl(SET_RANGE_SIZE(srlb.range_size));
 	/* Set Lower bound label SubTLV */
@@ -544,8 +544,8 @@ static void initialize_params(struct ospf_router_info *ori)
 		if (!list_isempty(OspfRI.area_info))
 			list_delete_all_node(OspfRI.area_info);
 		for (ALL_LIST_ELEMENTS(top->areas, node, nnode, area)) {
-			zlog_debug("RI (%s): Add area %s to Router Information",
-				__func__, inet_ntoa(area->area_id));
+			zlog_debug("RI (%s): Add area %pI4 to Router Information",
+				__func__, &area->area_id);
 			new = XCALLOC(MTYPE_OSPF_ROUTER_INFO,
 				sizeof(struct ospf_ri_area_info));
 			new->area = area;
@@ -668,7 +668,7 @@ void ospf_router_info_update_sr(bool enable, struct sr_node *srn)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are callback functions against generic Opaque-LSAs handling.
+ * Following are callback functions against generic Opaque-LSAs handling.
  *------------------------------------------------------------------------*/
 static void ospf_router_info_ism_change(struct ospf_interface *oi,
 					int old_state)
@@ -693,7 +693,7 @@ static void ospf_router_info_ism_change(struct ospf_interface *oi,
 }
 
 /*------------------------------------------------------------------------*
- * Followings are OSPF protocol processing functions for ROUTER INFORMATION
+ * Following are OSPF protocol processing functions for ROUTER INFORMATION
  *------------------------------------------------------------------------*/
 
 static void build_tlv_header(struct stream *s, struct tlv_header *tlvh)
@@ -795,8 +795,8 @@ static struct ospf_lsa *ospf_router_info_lsa_new(struct ospf_area *area)
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE))
 		zlog_debug(
-			"LSA[Type%d:%s]: Create an Opaque-LSA/ROUTER INFORMATION instance",
-			lsa_type, inet_ntoa(lsa_id));
+			"LSA[Type%d:%pI4]: Create an Opaque-LSA/ROUTER INFORMATION instance",
+			lsa_type, &lsa_id);
 
 	top = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
@@ -874,8 +874,8 @@ static int ospf_router_info_lsa_originate_as(void *arg)
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
 		zlog_debug(
-			"LSA[Type%d:%s]: Originate Opaque-LSA/ROUTER INFORMATION",
-			new->data->type, inet_ntoa(new->data->id));
+			"LSA[Type%d:%pI4]: Originate Opaque-LSA/ROUTER INFORMATION",
+			new->data->type, &new->data->id);
 		ospf_lsa_header_dump(new->data);
 	}
 
@@ -943,8 +943,8 @@ static int ospf_router_info_lsa_originate_area(void *arg)
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
 		zlog_debug(
-			"LSA[Type%d:%s]: Originate Opaque-LSA/ROUTER INFORMATION",
-			new->data->type, inet_ntoa(new->data->id));
+			"LSA[Type%d:%pI4]: Originate Opaque-LSA/ROUTER INFORMATION",
+			new->data->type, &new->data->id);
 		ospf_lsa_header_dump(new->data);
 	}
 
@@ -1094,8 +1094,8 @@ static struct ospf_lsa *ospf_router_info_lsa_refresh(struct ospf_lsa *lsa)
 	/* Debug logging. */
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
 		zlog_debug(
-			"LSA[Type%d:%s]: Refresh Opaque-LSA/ROUTER INFORMATION",
-			new->data->type, inet_ntoa(new->data->id));
+			"LSA[Type%d:%pI4]: Refresh Opaque-LSA/ROUTER INFORMATION",
+			new->data->type, &new->data->id);
 		ospf_lsa_header_dump(new->data);
 	}
 
@@ -1223,13 +1223,28 @@ static int ospf_router_info_lsa_update(struct ospf_lsa *lsa)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are vty session control functions.
+ * Following are vty session control functions.
  *------------------------------------------------------------------------*/
 
+#define check_tlv_size(size, msg)                                              \
+	do {                                                                   \
+		if (ntohs(tlvh->length) > size) {                              \
+			if (vty != NULL)                                       \
+				vty_out(vty, "  Wrong %s TLV size: %d(%d)\n",  \
+					msg, ntohs(tlvh->length), size);       \
+			else                                                   \
+				zlog_debug("    Wrong %s TLV size: %d(%d)",    \
+					   msg, ntohs(tlvh->length), size);    \
+			return size + TLV_HDR_SIZE;                            \
+		}                                                              \
+	} while (0)
+
 static uint16_t show_vty_router_cap(struct vty *vty, struct tlv_header *tlvh)
 {
 	struct ri_tlv_router_cap *top = (struct ri_tlv_router_cap *)tlvh;
 
+	check_tlv_size(RI_TLV_CAPABILITIES_SIZE, "Router Capabilities");
+
 	if (vty != NULL)
 		vty_out(vty, "  Router Capabilities: 0x%x\n",
 			ntohl(top->value));
@@ -1245,21 +1260,30 @@ static uint16_t show_vty_pce_subtlv_address(struct vty *vty,
 	struct ri_pce_subtlv_address *top =
 		(struct ri_pce_subtlv_address *)tlvh;
 
-	if (ntohs(top->address.type) == PCE_ADDRESS_TYPE_IPV4) {
+	if (ntohs(top->address.type) == PCE_ADDRESS_IPV4) {
+		check_tlv_size(PCE_ADDRESS_IPV4_SIZE, "PCE Address");
 		if (vty != NULL)
-			vty_out(vty, "  PCE Address: %s\n",
-				inet_ntoa(top->address.value));
+			vty_out(vty, "  PCE Address: %pI4\n",
+				&top->address.value);
 		else
-			zlog_debug("    PCE Address: %s",
-				   inet_ntoa(top->address.value));
-	} else {
+			zlog_debug("    PCE Address: %pI4",
+				   &top->address.value);
+	} else if (ntohs(top->address.type) == PCE_ADDRESS_IPV6) {
 		/* TODO: Add support to IPv6 with inet_ntop() */
+		check_tlv_size(PCE_ADDRESS_IPV6_SIZE, "PCE Address");
 		if (vty != NULL)
 			vty_out(vty, "  PCE Address: 0x%x\n",
 				ntohl(top->address.value.s_addr));
 		else
 			zlog_debug("    PCE Address: 0x%x",
 				   ntohl(top->address.value.s_addr));
+	} else {
+		if (vty != NULL)
+			vty_out(vty, "  Wrong PCE Address type: 0x%x\n",
+				ntohl(top->address.type));
+		else
+			zlog_debug("    Wrong PCE Address type: 0x%x",
+				   ntohl(top->address.type));
 	}
 
 	return TLV_SIZE(tlvh);
@@ -1271,6 +1295,8 @@ static uint16_t show_vty_pce_subtlv_path_scope(struct vty *vty,
 	struct ri_pce_subtlv_path_scope *top =
 		(struct ri_pce_subtlv_path_scope *)tlvh;
 
+	check_tlv_size(RI_PCE_SUBTLV_PATH_SCOPE_SIZE, "PCE Path Scope");
+
 	if (vty != NULL)
 		vty_out(vty, "  PCE Path Scope: 0x%x\n", ntohl(top->value));
 	else
@@ -1285,19 +1311,29 @@ static uint16_t show_vty_pce_subtlv_domain(struct vty *vty,
 	struct ri_pce_subtlv_domain *top = (struct ri_pce_subtlv_domain *)tlvh;
 	struct in_addr tmp;
 
+	check_tlv_size(RI_PCE_SUBTLV_DOMAIN_SIZE, "PCE Domain");
+
 	if (ntohs(top->type) == PCE_DOMAIN_TYPE_AREA) {
 		tmp.s_addr = top->value;
 		if (vty != NULL)
-			vty_out(vty, "  PCE domain Area: %s\n", inet_ntoa(tmp));
+			vty_out(vty, "  PCE Domain Area: %pI4\n", &tmp);
 		else
-			zlog_debug("    PCE domain Area: %s", inet_ntoa(tmp));
-	} else {
+			zlog_debug("    PCE Domain Area: %pI4", &tmp);
+	} else if (ntohs(top->type) == PCE_DOMAIN_TYPE_AS) {
 		if (vty != NULL)
-			vty_out(vty, "  PCE domain AS: %d\n",
+			vty_out(vty, "  PCE Domain AS: %d\n",
 				ntohl(top->value));
 		else
-			zlog_debug("    PCE domain AS: %d", ntohl(top->value));
+			zlog_debug("    PCE Domain AS: %d", ntohl(top->value));
+	} else {
+		if (vty != NULL)
+			vty_out(vty, "  Wrong PCE Domain type: %d\n",
+				ntohl(top->type));
+		else
+			zlog_debug("    Wrong PCE Domain type: %d",
+				   ntohl(top->type));
 	}
+
 	return TLV_SIZE(tlvh);
 }
 
@@ -1309,21 +1345,30 @@ static uint16_t show_vty_pce_subtlv_neighbor(struct vty *vty,
 		(struct ri_pce_subtlv_neighbor *)tlvh;
 	struct in_addr tmp;
 
+	check_tlv_size(RI_PCE_SUBTLV_NEIGHBOR_SIZE, "PCE Neighbor");
+
 	if (ntohs(top->type) == PCE_DOMAIN_TYPE_AREA) {
 		tmp.s_addr = top->value;
 		if (vty != NULL)
-			vty_out(vty, "  PCE neighbor Area: %s\n",
-				inet_ntoa(tmp));
+			vty_out(vty, "  PCE Neighbor Area: %pI4\n", &tmp);
 		else
-			zlog_debug("    PCE neighbor Area: %s", inet_ntoa(tmp));
-	} else {
+			zlog_debug("    PCE Neighbor Area: %pI4", &tmp);
+	} else if (ntohs(top->type) == PCE_DOMAIN_TYPE_AS) {
 		if (vty != NULL)
-			vty_out(vty, "  PCE neighbor AS: %d\n",
+			vty_out(vty, "  PCE Neighbor AS: %d\n",
 				ntohl(top->value));
 		else
-			zlog_debug("    PCE neighbor AS: %d",
+			zlog_debug("    PCE Neighbor AS: %d",
 				   ntohl(top->value));
+	} else {
+		if (vty != NULL)
+			vty_out(vty, "  Wrong PCE Neighbor type: %d\n",
+				ntohl(top->type));
+		else
+			zlog_debug("    Wrong PCE Neighbor type: %d",
+				   ntohl(top->type));
 	}
+
 	return TLV_SIZE(tlvh);
 }
 
@@ -1333,6 +1378,8 @@ static uint16_t show_vty_pce_subtlv_cap_flag(struct vty *vty,
 	struct ri_pce_subtlv_cap_flag *top =
 		(struct ri_pce_subtlv_cap_flag *)tlvh;
 
+	check_tlv_size(RI_PCE_SUBTLV_CAP_FLAG_SIZE, "PCE Capabilities");
+
 	if (vty != NULL)
 		vty_out(vty, "  PCE Capabilities Flag: 0x%x\n",
 			ntohl(top->value));
@@ -1343,8 +1390,21 @@ static uint16_t show_vty_pce_subtlv_cap_flag(struct vty *vty,
 	return TLV_SIZE(tlvh);
 }
 
-static uint16_t show_vty_unknown_tlv(struct vty *vty, struct tlv_header *tlvh)
+static uint16_t show_vty_unknown_tlv(struct vty *vty, struct tlv_header *tlvh,
+				     size_t buf_size)
 {
+	if (TLV_SIZE(tlvh) > buf_size) {
+		if (vty != NULL)
+			vty_out(vty,
+				"    TLV size %d exceeds buffer size. Abort!",
+				TLV_SIZE(tlvh));
+		else
+			zlog_debug(
+				"    TLV size %d exceeds buffer size. Abort!",
+				TLV_SIZE(tlvh));
+		return buf_size;
+	}
+
 	if (vty != NULL)
 		vty_out(vty, "  Unknown TLV: [type(0x%x), length(0x%x)]\n",
 			ntohs(tlvh->type), ntohs(tlvh->length));
@@ -1356,12 +1416,21 @@ static uint16_t show_vty_unknown_tlv(struct vty *vty, struct tlv_header *tlvh)
 }
 
 static uint16_t show_vty_pce_info(struct vty *vty, struct tlv_header *ri,
-				  uint32_t total)
+				  size_t buf_size)
 {
 	struct tlv_header *tlvh;
+	uint16_t length = ntohs(ri->length);
 	uint16_t sum = 0;
 
-	for (tlvh = ri; sum < total; tlvh = TLV_HDR_NEXT(tlvh)) {
+	/* Verify that TLV length is valid against remaining buffer size */
+	if (length > buf_size) {
+		vty_out(vty,
+			"  PCE Info TLV size %d exceeds buffer size. Abort!\n",
+			length);
+		return buf_size;
+	}
+
+	for (tlvh = ri; sum < length; tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case RI_PCE_SUBTLV_ADDRESS:
 			sum += show_vty_pce_subtlv_address(vty, tlvh);
@@ -1379,7 +1448,7 @@ static uint16_t show_vty_pce_info(struct vty *vty, struct tlv_header *ri,
 			sum += show_vty_pce_subtlv_cap_flag(vty, tlvh);
 			break;
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh);
+			sum += show_vty_unknown_tlv(vty, tlvh, length - sum);
 			break;
 		}
 	}
@@ -1393,6 +1462,8 @@ static uint16_t show_vty_sr_algorithm(struct vty *vty, struct tlv_header *tlvh)
 		(struct ri_sr_tlv_sr_algorithm *)tlvh;
 	int i;
 
+	check_tlv_size(ALGORITHM_COUNT, "Segment Routing Algorithm");
+
 	if (vty != NULL) {
 		vty_out(vty, "  Segment Routing Algorithm TLV:\n");
 		for (i = 0; i < ntohs(algo->header.length); i++) {
@@ -1411,9 +1482,7 @@ static uint16_t show_vty_sr_algorithm(struct vty *vty, struct tlv_header *tlvh)
 				break;
 			}
 		}
-	}
-
-	else {
+	} else {
 		zlog_debug("  Segment Routing Algorithm TLV:");
 		for (i = 0; i < ntohs(algo->header.length); i++)
 			switch (algo->value[i]) {
@@ -1439,6 +1508,8 @@ static uint16_t show_vty_sr_range(struct vty *vty, struct tlv_header *tlvh)
 	struct ri_sr_tlv_sid_label_range *range =
 		(struct ri_sr_tlv_sid_label_range *)tlvh;
 
+	check_tlv_size(RI_SR_TLV_LABEL_RANGE_SIZE, "SR Label Range");
+
 	if (vty != NULL) {
 		vty_out(vty,
 			"  Segment Routing %s Range TLV:\n"
@@ -1451,9 +1522,7 @@ static uint16_t show_vty_sr_range(struct vty *vty, struct tlv_header *tlvh)
 			GET_LABEL(ntohl(range->lower.value)));
 	} else {
 		zlog_debug(
-			"  Segment Routing %s Range TLV:\n"
-			"    Range Size = %d\n"
-			"    SID Label = %d\n\n",
+			"  Segment Routing %s Range TLV:  Range Size = %d  SID Label = %d",
 			ntohs(range->header.type) == RI_SR_TLV_SRGB_LABEL_RANGE
 				? "Global"
 				: "Local",
@@ -1469,6 +1538,8 @@ static uint16_t show_vty_sr_msd(struct vty *vty, struct tlv_header *tlvh)
 {
 	struct ri_sr_tlv_node_msd *msd = (struct ri_sr_tlv_node_msd *)tlvh;
 
+	check_tlv_size(RI_SR_TLV_NODE_MSD_SIZE, "Node Maximum Stack Depth");
+
 	if (vty != NULL) {
 		vty_out(vty,
 			"  Segment Routing MSD TLV:\n"
@@ -1476,24 +1547,28 @@ static uint16_t show_vty_sr_msd(struct vty *vty, struct tlv_header *tlvh)
 			msd->value);
 	} else {
 		zlog_debug(
-			"  Segment Routing MSD TLV:\n"
-			"    Node Maximum Stack Depth = %d\n",
+			"  Segment Routing MSD TLV:  Node Maximum Stack Depth = %d",
 			msd->value);
 	}
 
 	return TLV_SIZE(tlvh);
 }
 
-static void ospf_router_info_show_info(struct vty *vty, struct ospf_lsa *lsa)
+static void ospf_router_info_show_info(struct vty *vty,
+				       struct json_object *json,
+				       struct ospf_lsa *lsa)
 {
 	struct lsa_header *lsah = lsa->data;
 	struct tlv_header *tlvh;
 	uint16_t length = 0, sum = 0;
 
-	/* Initialize TLV browsing */
-	length = ntohs(lsah->length) - OSPF_LSA_HEADER_SIZE;
+	if (json)
+		return;
 
-	for (tlvh = TLV_HDR_TOP(lsah); sum < length;
+	/* Initialize TLV browsing */
+	length = lsa->size - OSPF_LSA_HEADER_SIZE;
+
+	for (tlvh = TLV_HDR_TOP(lsah); sum < length && tlvh;
 	     tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case RI_TLV_CAPABILITIES:
@@ -1516,7 +1591,7 @@ static void ospf_router_info_show_info(struct vty *vty, struct ospf_lsa *lsa)
 			break;
 
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh);
+			sum += show_vty_unknown_tlv(vty, tlvh, length);
 			break;
 		}
 	}
@@ -1543,8 +1618,8 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 	if (OspfRI.pce_info.enabled) {
 
 		if (pce->pce_address.header.type != 0)
-			vty_out(vty, "  pce address %s\n",
-				inet_ntoa(pce->pce_address.address.value));
+			vty_out(vty, "  pce address %pI4\n",
+				&pce->pce_address.address.value);
 
 		if (pce->pce_cap_flag.header.type != 0)
 			vty_out(vty, "  pce flag 0x%x\n",
@@ -1554,8 +1629,8 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 			if (domain->header.type != 0) {
 				if (domain->type == PCE_DOMAIN_TYPE_AREA) {
 					tmp.s_addr = domain->value;
-					vty_out(vty, "  pce domain area %s\n",
-						inet_ntoa(tmp));
+					vty_out(vty, "  pce domain area %pI4\n",
+						&tmp);
 				} else {
 					vty_out(vty, "  pce domain as %d\n",
 						ntohl(domain->value));
@@ -1567,8 +1642,9 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 			if (neighbor->header.type != 0) {
 				if (neighbor->type == PCE_DOMAIN_TYPE_AREA) {
 					tmp.s_addr = neighbor->value;
-					vty_out(vty, "  pce neighbor area %s\n",
-						inet_ntoa(tmp));
+					vty_out(vty,
+						"  pce neighbor area %pI4\n",
+						&tmp);
 				} else {
 					vty_out(vty, "  pce neighbor as %d\n",
 						ntohl(neighbor->value));
@@ -1584,7 +1660,7 @@ static void ospf_router_info_config_write_router(struct vty *vty)
 }
 
 /*------------------------------------------------------------------------*
- * Followings are vty command functions.
+ * Following are vty command functions.
  *------------------------------------------------------------------------*/
 /* Simple wrapper schedule RI LSA action in function of the scope */
 static void ospf_router_info_schedule(enum lsa_opcode opcode)
