@@ -29,6 +29,7 @@
 #include "prefix.h"
 
 #include "pimd.h"
+#include "pim_instance.h"
 #include "pim_str.h"
 #include "pim_iface.h"
 #include "pim_ifchannel.h"
@@ -62,19 +63,7 @@ int pim_ifchannel_compare(const struct pim_ifchannel *ch1,
 	if (pim_ifp1->mroute_vif_index > pim_ifp2->mroute_vif_index)
 		return 1;
 
-	if (ntohl(ch1->sg.grp.s_addr) < ntohl(ch2->sg.grp.s_addr))
-		return -1;
-
-	if (ntohl(ch1->sg.grp.s_addr) > ntohl(ch2->sg.grp.s_addr))
-		return 1;
-
-	if (ntohl(ch1->sg.src.s_addr) < ntohl(ch2->sg.src.s_addr))
-		return -1;
-
-	if (ntohl(ch1->sg.src.s_addr) > ntohl(ch2->sg.src.s_addr))
-		return 1;
-
-	return 0;
+	return pim_sgaddr_cmp(ch1->sg, ch2->sg);
 }
 
 /*
@@ -107,18 +96,15 @@ static void pim_ifchannel_find_new_children(struct pim_ifchannel *ch)
 	struct pim_ifchannel *child;
 
 	// Basic Sanity that we are not being silly
-	if ((ch->sg.src.s_addr != INADDR_ANY)
-	    && (ch->sg.grp.s_addr != INADDR_ANY))
+	if (!pim_addr_is_any(ch->sg.src) && !pim_addr_is_any(ch->sg.grp))
 		return;
 
-	if ((ch->sg.src.s_addr == INADDR_ANY)
-	    && (ch->sg.grp.s_addr == INADDR_ANY))
+	if (pim_addr_is_any(ch->sg.src) && pim_addr_is_any(ch->sg.grp))
 		return;
 
 	RB_FOREACH (child, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
-		if ((ch->sg.grp.s_addr != INADDR_ANY)
-		    && (child->sg.grp.s_addr == ch->sg.grp.s_addr)
-		    && (child != ch)) {
+		if (!pim_addr_is_any(ch->sg.grp) &&
+		    !pim_addr_cmp(child->sg.grp, ch->sg.grp) && (child != ch)) {
 			child->parent = ch;
 			listnode_add_sort(ch->sources, child);
 		}
@@ -153,7 +139,7 @@ void pim_ifchannel_delete(struct pim_ifchannel *ch)
 	if (ch->upstream->channel_oil) {
 		uint32_t mask = PIM_OIF_FLAG_PROTO_PIM;
 		if (ch->upstream->flags & PIM_UPSTREAM_FLAG_MASK_SRC_IGMP)
-			mask |= PIM_OIF_FLAG_PROTO_IGMP;
+			mask |= PIM_OIF_FLAG_PROTO_GM;
 
 		/*
 		 * A S,G RPT channel can have an empty oil, we also
@@ -162,9 +148,9 @@ void pim_ifchannel_delete(struct pim_ifchannel *ch)
 		 * being inherited.  So let's figure out what
 		 * needs to be done here
 		 */
-		if ((ch->sg.src.s_addr != INADDR_ANY) &&
-				pim_upstream_evaluate_join_desired_interface(
-					ch->upstream, ch, ch->parent))
+		if (!pim_addr_is_any(ch->sg.src) &&
+		    pim_upstream_evaluate_join_desired_interface(
+			    ch->upstream, ch, ch->parent))
 			pim_channel_add_oif(ch->upstream->channel_oil,
 					ch->interface,
 					PIM_OIF_FLAG_PROTO_STAR,
@@ -258,7 +244,7 @@ void pim_ifchannel_delete_all(struct interface *ifp)
 	}
 }
 
-static void delete_on_noinfo(struct pim_ifchannel *ch)
+void delete_on_noinfo(struct pim_ifchannel *ch)
 {
 	if (ch->local_ifmembership == PIM_IFMEMBERSHIP_NOINFO
 	    && ch->ifjoin_state == PIM_IFJOIN_NOINFO
@@ -284,7 +270,7 @@ void pim_ifchannel_ifjoin_switch(const char *caller, struct pim_ifchannel *ch,
 	if (old_state == new_state) {
 		if (PIM_DEBUG_PIM_EVENTS) {
 			zlog_debug(
-				"%s calledby %s: non-transition on state %d (%s)",
+				"%s called by %s: non-transition on state %d (%s)",
 				__func__, caller, new_state,
 				pim_ifchannel_ifjoin_name(new_state, 0));
 		}
@@ -293,7 +279,7 @@ void pim_ifchannel_ifjoin_switch(const char *caller, struct pim_ifchannel *ch,
 
 	ch->ifjoin_state = new_state;
 
-	if (ch->sg.src.s_addr == INADDR_ANY) {
+	if (pim_addr_is_any(ch->sg.src)) {
 		struct pim_upstream *up = ch->upstream;
 		struct pim_upstream *child;
 		struct listnode *up_node;
@@ -321,8 +307,8 @@ void pim_ifchannel_ifjoin_switch(const char *caller, struct pim_ifchannel *ch,
 					 * supplying the implied
 					 * if channel.  So remove it.
 					 */
-					if (c_oil->oil.mfcc_ttls
-						    [pim_ifp->mroute_vif_index])
+					if (oil_if_has(c_oil,
+						       pim_ifp->mroute_vif_index))
 						pim_channel_del_inherited_oif(
 							c_oil, ch->interface,
 							__func__);
@@ -438,16 +424,13 @@ const char *pim_ifchannel_ifassert_name(enum pim_ifassert_state ifassert_state)
 */
 void reset_ifassert_state(struct pim_ifchannel *ch)
 {
-	struct in_addr any = {.s_addr = INADDR_ANY};
-
 	THREAD_OFF(ch->t_ifassert_timer);
 
-	pim_ifassert_winner_set(ch, PIM_IFASSERT_NOINFO, any,
+	pim_ifassert_winner_set(ch, PIM_IFASSERT_NOINFO, PIMADDR_ANY,
 				router->infinite_assert_metric);
 }
 
-struct pim_ifchannel *pim_ifchannel_find(struct interface *ifp,
-					 struct prefix_sg *sg)
+struct pim_ifchannel *pim_ifchannel_find(struct interface *ifp, pim_sgaddr *sg)
 {
 	struct pim_interface *pim_ifp;
 	struct pim_ifchannel *ch;
@@ -456,8 +439,8 @@ struct pim_ifchannel *pim_ifchannel_find(struct interface *ifp,
 	pim_ifp = ifp->info;
 
 	if (!pim_ifp) {
-		zlog_warn("%s: (S,G)=%s: multicast not enabled on interface %s",
-			  __func__, pim_str_sg_dump(sg), ifp->name);
+		zlog_warn("%s: (S,G)=%pSG: multicast not enabled on interface %s",
+			  __func__, sg, ifp->name);
 		return NULL;
 	}
 
@@ -498,7 +481,7 @@ void pim_ifchannel_membership_clear(struct interface *ifp)
 	struct pim_ifchannel *ch;
 
 	pim_ifp = ifp->info;
-	zassert(pim_ifp);
+	assert(pim_ifp);
 
 	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb)
 		ifmembership_set(ch, PIM_IFMEMBERSHIP_NOINFO);
@@ -510,7 +493,7 @@ void pim_ifchannel_delete_on_noinfo(struct interface *ifp)
 	struct pim_ifchannel *ch, *ch_tmp;
 
 	pim_ifp = ifp->info;
-	zassert(pim_ifp);
+	assert(pim_ifp);
 
 	RB_FOREACH_SAFE (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb, ch_tmp)
 		delete_on_noinfo(ch);
@@ -524,13 +507,13 @@ void pim_ifchannel_delete_on_noinfo(struct interface *ifp)
  */
 static struct pim_ifchannel *pim_ifchannel_find_parent(struct pim_ifchannel *ch)
 {
-	struct prefix_sg parent_sg = ch->sg;
+	pim_sgaddr parent_sg = ch->sg;
 	struct pim_ifchannel *parent = NULL;
 
 	// (S,G)
-	if ((parent_sg.src.s_addr != INADDR_ANY)
-	    && (parent_sg.grp.s_addr != INADDR_ANY)) {
-		parent_sg.src.s_addr = INADDR_ANY;
+	if (!pim_addr_is_any(parent_sg.src) &&
+	    !pim_addr_is_any(parent_sg.grp)) {
+		parent_sg.src = PIMADDR_ANY;
 		parent = pim_ifchannel_find(ch->interface, &parent_sg);
 
 		if (parent)
@@ -541,8 +524,7 @@ static struct pim_ifchannel *pim_ifchannel_find_parent(struct pim_ifchannel *ch)
 	return NULL;
 }
 
-struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp,
-					struct prefix_sg *sg,
+struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp, pim_sgaddr *sg,
 					uint8_t source_flags, int up_flags)
 {
 	struct pim_interface *pim_ifp;
@@ -550,8 +532,17 @@ struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp,
 	struct pim_upstream *up;
 
 	ch = pim_ifchannel_find(ifp, sg);
-	if (ch)
+	if (ch) {
+		if (up_flags == PIM_UPSTREAM_FLAG_MASK_SRC_PIM)
+			PIM_IF_FLAG_SET_PROTO_PIM(ch->flags);
+
+		if (up_flags == PIM_UPSTREAM_FLAG_MASK_SRC_IGMP)
+			PIM_IF_FLAG_SET_PROTO_IGMP(ch->flags);
+
+		ch->upstream->flags |= up_flags;
+
 		return ch;
+	}
 
 	pim_ifp = ifp->info;
 
@@ -564,9 +555,9 @@ struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp,
 
 	ch->interface = ifp;
 	ch->sg = *sg;
-	pim_str_sg_set(sg, ch->sg_str);
+	snprintfrr(ch->sg_str, sizeof(ch->sg_str), "%pSG", sg);
 	ch->parent = pim_ifchannel_find_parent(ch);
-	if (ch->sg.src.s_addr == INADDR_ANY) {
+	if (pim_addr_is_any(ch->sg.src)) {
 		ch->sources = list_new();
 		ch->sources->cmp =
 			(int (*)(void *, void *))pim_ifchannel_compare;
@@ -592,7 +583,7 @@ struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp,
 	ch->ifassert_my_metric = pim_macro_ch_my_assert_metric_eval(ch);
 	ch->ifassert_winner_metric = pim_macro_ch_my_assert_metric_eval(ch);
 
-	ch->ifassert_winner.s_addr = INADDR_ANY;
+	ch->ifassert_winner = PIMADDR_ANY;
 
 	/* Assert state */
 	ch->t_ifassert_timer = NULL;
@@ -638,15 +629,19 @@ struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp,
 	return ch;
 }
 
-static void ifjoin_to_noinfo(struct pim_ifchannel *ch, bool ch_del)
+static void ifjoin_to_noinfo(struct pim_ifchannel *ch)
 {
-	pim_forward_stop(ch, !ch_del);
 	pim_ifchannel_ifjoin_switch(__func__, ch, PIM_IFJOIN_NOINFO);
-	if (ch_del)
-		delete_on_noinfo(ch);
+	pim_forward_stop(ch);
+
+	PIM_UPSTREAM_FLAG_UNSET_SRC_PIM(ch->upstream->flags);
+
+	PIM_IF_FLAG_UNSET_PROTO_PIM(ch->flags);
+
+	delete_on_noinfo(ch);
 }
 
-static int on_ifjoin_expiry_timer(struct thread *t)
+static void on_ifjoin_expiry_timer(struct thread *t)
 {
 	struct pim_ifchannel *ch;
 
@@ -656,13 +651,11 @@ static int on_ifjoin_expiry_timer(struct thread *t)
 		zlog_debug("%s: ifchannel %s expiry timer", __func__,
 			   ch->sg_str);
 
-	ifjoin_to_noinfo(ch, true);
+	ifjoin_to_noinfo(ch);
 	/* ch may have been deleted */
-
-	return 0;
 }
 
-static int on_ifjoin_prune_pending_timer(struct thread *t)
+static void on_ifjoin_prune_pending_timer(struct thread *t)
 {
 	struct pim_ifchannel *ch;
 	int send_prune_echo; /* boolean */
@@ -672,10 +665,9 @@ static int on_ifjoin_prune_pending_timer(struct thread *t)
 	ch = THREAD_ARG(t);
 
 	if (PIM_DEBUG_PIM_TRACE)
-		zlog_debug(
-			"%s: IFCHANNEL%s %s Prune Pending Timer Popped",
-			__func__, pim_str_sg_dump(&ch->sg),
-			pim_ifchannel_ifjoin_name(ch->ifjoin_state, ch->flags));
+		zlog_debug("%s: IFCHANNEL%pSG %s Prune Pending Timer Popped",
+			   __func__, &ch->sg,
+			   pim_ifchannel_ifjoin_name(ch->ifjoin_state, ch->flags));
 
 	if (ch->ifjoin_state == PIM_IFJOIN_PRUNE_PENDING) {
 		ifp = ch->interface;
@@ -689,56 +681,52 @@ static int on_ifjoin_prune_pending_timer(struct thread *t)
 				struct pim_rpf rpf;
 
 				rpf.source_nexthop.interface = ifp;
-				rpf.rpf_addr.u.prefix4 =
-					pim_ifp->primary_address;
+				rpf.rpf_addr = pim_ifp->primary_address;
 				pim_jp_agg_single_upstream_send(
 					&rpf, ch->upstream, 0);
 			}
 
-			ifjoin_to_noinfo(ch, true);
+			ifjoin_to_noinfo(ch);
 		} else {
 			/* If SGRpt flag is set on ifchannel, Trigger SGRpt
 			 *  message on RP path upon prune timer expiry.
 			 */
 			ch->ifjoin_state = PIM_IFJOIN_PRUNE;
-			if (ch->upstream) {
-				struct pim_upstream *parent =
-					ch->upstream->parent;
+			struct pim_upstream *parent =
+				ch->upstream->parent;
 
-				pim_upstream_update_join_desired(pim_ifp->pim,
-								 ch->upstream);
+			pim_upstream_update_join_desired(pim_ifp->pim,
+							 ch->upstream);
 
-				pim_jp_agg_single_upstream_send(&parent->rpf,
-								parent, true);
-				/*
-				 * SGRpt prune pending expiry has to install
-				 * SG entry with empty olist to drop the SG
-				 * traffic incase no other intf exists.
-				 * On that scenario, SG entry wouldn't have
-				 * got installed until Prune pending timer
-				 * expired. So install now.
-				 */
-				pim_channel_del_oif(
-					ch->upstream->channel_oil, ifp,
-					PIM_OIF_FLAG_PROTO_STAR, __func__);
-				if (!ch->upstream->channel_oil->installed)
-					pim_upstream_mroute_add(
-						ch->upstream->channel_oil,
-						__PRETTY_FUNCTION__);
-			}
+			pim_jp_agg_single_upstream_send(&parent->rpf,
+							parent, true);
+			/*
+			 * SGRpt prune pending expiry has to install
+			 * SG entry with empty olist to drop the SG
+			 * traffic incase no other intf exists.
+			 * On that scenario, SG entry wouldn't have
+			 * got installed until Prune pending timer
+			 * expired. So install now.
+			 */
+			pim_channel_del_oif(
+				ch->upstream->channel_oil, ifp,
+				PIM_OIF_FLAG_PROTO_STAR, __func__);
+			if (!ch->upstream->channel_oil->installed)
+				pim_upstream_mroute_add(
+					ch->upstream->channel_oil,
+					__func__);
 		}
 		/* from here ch may have been deleted */
 	}
-
-	return 0;
 }
 
 static void check_recv_upstream(int is_join, struct interface *recv_ifp,
-				struct in_addr upstream, struct prefix_sg *sg,
+				pim_addr upstream, pim_sgaddr *sg,
 				uint8_t source_flags, int holdtime)
 {
 	struct pim_upstream *up;
 	struct pim_interface *pim_ifp = recv_ifp->info;
+	pim_addr rpf_addr;
 
 	/* Upstream (S,G) in Joined state ? */
 	up = pim_upstream_find(pim_ifp->pim, sg);
@@ -756,16 +744,13 @@ static void check_recv_upstream(int is_join, struct interface *recv_ifp,
 		return;
 	}
 
+	rpf_addr = up->rpf.rpf_addr;
+
 	/* upstream directed to RPF'(S,G) ? */
-	if (upstream.s_addr != up->rpf.rpf_addr.u.prefix4.s_addr) {
-		char up_str[INET_ADDRSTRLEN];
-		char rpf_str[PREFIX_STRLEN];
-		pim_inet4_dump("<up?>", upstream, up_str, sizeof(up_str));
-		pim_addr_dump("<rpf?>", &up->rpf.rpf_addr, rpf_str,
-			      sizeof(rpf_str));
+	if (pim_addr_cmp(upstream, rpf_addr)) {
 		zlog_warn(
-			"%s %s: (S,G)=%s upstream=%s not directed to RPF'(S,G)=%s on interface %s",
-			__FILE__, __func__, up->sg_str, up_str, rpf_str,
+			"%s %s: (S,G)=%s upstream=%pPAs not directed to RPF'(S,G)=%pPAs on interface %s",
+			__FILE__, __func__, up->sg_str, &upstream, &rpf_addr,
 			recv_ifp->name);
 		return;
 	}
@@ -773,8 +758,7 @@ static void check_recv_upstream(int is_join, struct interface *recv_ifp,
 
 	if (is_join) {
 		/* Join(S,G) to RPF'(S,G) */
-		pim_upstream_join_suppress(up, up->rpf.rpf_addr.u.prefix4,
-					   holdtime);
+		pim_upstream_join_suppress(up, up->rpf.rpf_addr, holdtime);
 		return;
 	}
 
@@ -799,27 +783,25 @@ static void check_recv_upstream(int is_join, struct interface *recv_ifp,
 }
 
 static int nonlocal_upstream(int is_join, struct interface *recv_ifp,
-			     struct in_addr upstream, struct prefix_sg *sg,
+			     pim_addr upstream, pim_sgaddr *sg,
 			     uint8_t source_flags, uint16_t holdtime)
 {
 	struct pim_interface *recv_pim_ifp;
 	int is_local; /* boolean */
 
 	recv_pim_ifp = recv_ifp->info;
-	zassert(recv_pim_ifp);
+	assert(recv_pim_ifp);
 
-	is_local = (upstream.s_addr == recv_pim_ifp->primary_address.s_addr);
+	is_local = !pim_addr_cmp(upstream, recv_pim_ifp->primary_address);
 
 	if (is_local)
 		return 0;
 
-	if (PIM_DEBUG_PIM_TRACE_DETAIL) {
-		char up_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<upstream?>", upstream, up_str, sizeof(up_str));
-		zlog_warn("%s: recv %s (S,G)=%s to non-local upstream=%s on %s",
-			  __func__, is_join ? "join" : "prune",
-			  pim_str_sg_dump(sg), up_str, recv_ifp->name);
-	}
+	if (PIM_DEBUG_PIM_TRACE_DETAIL)
+		zlog_warn(
+			"%s: recv %s (S,G)=%pSG to non-local upstream=%pPAs on %s",
+			__func__, is_join ? "join" : "prune", sg, &upstream,
+			recv_ifp->name);
 
 	/*
 	 * Since recv upstream addr was not directed to our primary
@@ -852,8 +834,8 @@ static void pim_ifchannel_ifjoin_handler(struct pim_ifchannel *ch,
 }
 
 
-void pim_ifchannel_join_add(struct interface *ifp, struct in_addr neigh_addr,
-			    struct in_addr upstream, struct prefix_sg *sg,
+void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
+			    pim_addr upstream, pim_sgaddr *sg,
 			    uint8_t source_flags, uint16_t holdtime)
 {
 	struct pim_interface *pim_ifp;
@@ -884,17 +866,14 @@ void pim_ifchannel_join_add(struct interface *ifp, struct in_addr neigh_addr,
 	  address of the join message is our primary address.
 	 */
 	if (ch->ifassert_state == PIM_IFASSERT_I_AM_LOSER) {
-		char neigh_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<neigh?>", neigh_addr, neigh_str,
-			       sizeof(neigh_str));
-		zlog_warn("%s: Assert Loser recv Join%s from %s on %s",
-			  __func__, ch->sg_str, neigh_str, ifp->name);
+		zlog_warn("%s: Assert Loser recv Join%s from %pPA on %s",
+			  __func__, ch->sg_str, &neigh_addr, ifp->name);
 
 		assert_action_a5(ch);
 	}
 
 	pim_ifp = ifp->info;
-	zassert(pim_ifp);
+	assert(pim_ifp);
 
 	switch (ch->ifjoin_state) {
 	case PIM_IFJOIN_NOINFO:
@@ -920,7 +899,7 @@ void pim_ifchannel_join_add(struct interface *ifp, struct in_addr neigh_addr,
 		}
 		break;
 	case PIM_IFJOIN_JOIN:
-		zassert(!ch->t_ifjoin_prune_pending_timer);
+		assert(!ch->t_ifjoin_prune_pending_timer);
 
 		/*
 		  In the JOIN state ch->t_ifjoin_expiry_timer may be NULL due to
@@ -965,14 +944,45 @@ void pim_ifchannel_join_add(struct interface *ifp, struct in_addr neigh_addr,
 			pim_ifchannel_ifjoin_handler(ch, pim_ifp);
 		break;
 	case PIM_IFJOIN_PRUNE_PENDING:
+		/*
+		 * Transitions from Prune-Pending State (Receive Join)
+		 * RFC 7761 Sec 4.5.2:
+		 *    The (S,G) downstream state machine on interface I
+		 * transitions to the Join state.  The Prune-Pending Timer is
+		 * canceled (without triggering an expiry event).  The
+		 * Expiry Timer (ET) is restarted and is then set to the
+		 * maximum of its current value and the HoldTime from the
+		 * triggering Join/Prune message.
+		 */
 		THREAD_OFF(ch->t_ifjoin_prune_pending_timer);
-		if (source_flags & PIM_ENCODE_RPT_BIT) {
+
+		/* Check if SGRpt join Received */
+		if ((source_flags & PIM_ENCODE_RPT_BIT) &&
+		    !pim_addr_is_any(sg->src)) {
+			/*
+			 * Transitions from Prune-Pending State (Rcv SGRpt Join)
+			 * RFC 7761 Sec 4.5.3:
+			 * The (S,G,rpt) downstream state machine on interface
+			 * I transitions to the NoInfo state.The ET and PPT are
+			 * cancelled.
+			 */
 			THREAD_OFF(ch->t_ifjoin_expiry_timer);
 			pim_ifchannel_ifjoin_switch(__func__, ch,
 						    PIM_IFJOIN_NOINFO);
-		} else {
-			pim_ifchannel_ifjoin_handler(ch, pim_ifp);
+			return;
 		}
+
+		pim_ifchannel_ifjoin_handler(ch, pim_ifp);
+
+		if (ch->t_ifjoin_expiry_timer) {
+			unsigned long remain = thread_timer_remain_second(
+				ch->t_ifjoin_expiry_timer);
+
+			if (remain > holdtime)
+				return;
+		}
+		THREAD_OFF(ch->t_ifjoin_expiry_timer);
+
 		break;
 	case PIM_IFJOIN_PRUNE_TMP:
 		break;
@@ -986,8 +996,8 @@ void pim_ifchannel_join_add(struct interface *ifp, struct in_addr neigh_addr,
 	}
 }
 
-void pim_ifchannel_prune(struct interface *ifp, struct in_addr upstream,
-			 struct prefix_sg *sg, uint8_t source_flags,
+void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
+			 pim_sgaddr *sg, uint8_t source_flags,
 			 uint16_t holdtime)
 {
 	struct pim_ifchannel *ch;
@@ -1002,10 +1012,9 @@ void pim_ifchannel_prune(struct interface *ifp, struct in_addr upstream,
 	ch = pim_ifchannel_find(ifp, sg);
 	if (!ch && !(source_flags & PIM_ENCODE_RPT_BIT)) {
 		if (PIM_DEBUG_PIM_TRACE)
-			zlog_debug(
-				"%s: Received prune with no relevant ifchannel %s%s state: %d",
-				__func__, ifp->name, pim_str_sg_dump(sg),
-				source_flags);
+			zlog_debug("%s: Received prune with no relevant ifchannel %s%pSG state: %d",
+				   __func__, ifp->name, sg,
+				   source_flags);
 		return;
 	}
 
@@ -1049,7 +1058,14 @@ void pim_ifchannel_prune(struct interface *ifp, struct in_addr upstream,
 		/* nothing to do */
 		break;
 	case PIM_IFJOIN_JOIN:
-		THREAD_OFF(ch->t_ifjoin_expiry_timer);
+		/*
+		 * The (S,G) downstream state machine on interface I
+		 * transitions to the Prune-Pending state.  The
+		 * Prune-Pending Timer is started.  It is set to the
+		 * J/P_Override_Interval(I) if the router has more than one
+		 * neighbor on that interface; otherwise, it is set to zero,
+		 * causing it to expire immediately.
+		 */
 
 		pim_ifchannel_ifjoin_switch(__func__, ch,
 					    PIM_IFJOIN_PRUNE_PENDING);
@@ -1116,8 +1132,8 @@ void pim_ifchannel_prune(struct interface *ifp, struct in_addr upstream,
 	}
 }
 
-int pim_ifchannel_local_membership_add(struct interface *ifp,
-				       struct prefix_sg *sg, bool is_vxlan)
+int pim_ifchannel_local_membership_add(struct interface *ifp, pim_sgaddr *sg,
+				       bool is_vxlan)
 {
 	struct pim_ifchannel *ch, *starch;
 	struct pim_interface *pim_ifp;
@@ -1128,28 +1144,26 @@ int pim_ifchannel_local_membership_add(struct interface *ifp,
 	pim_ifp = ifp->info;
 	if (!pim_ifp) {
 		if (PIM_DEBUG_EVENTS)
-			zlog_debug("%s:%s Expected pim interface setup for %s",
-				   __func__, pim_str_sg_dump(sg), ifp->name);
+			zlog_debug("%s:%pSG Expected pim interface setup for %s",
+				   __func__, sg, ifp->name);
 		return 0;
 	}
 
-	if (!PIM_IF_TEST_PIM(pim_ifp->options)) {
+	if (!pim_ifp->pim_enable) {
 		if (PIM_DEBUG_EVENTS)
-			zlog_debug(
-				"%s:%s PIM is not configured on this interface %s",
-				__func__, pim_str_sg_dump(sg), ifp->name);
+			zlog_debug("%s:%pSG PIM is not configured on this interface %s",
+				   __func__, sg, ifp->name);
 		return 0;
 	}
 
 	pim = pim_ifp->pim;
 
 	/* skip (*,G) ch creation if G is of type SSM */
-	if (sg->src.s_addr == INADDR_ANY) {
+	if (pim_addr_is_any(sg->src)) {
 		if (pim_is_grp_ssm(pim, sg->grp)) {
 			if (PIM_DEBUG_PIM_EVENTS)
-				zlog_debug(
-					"%s: local membership (S,G)=%s ignored as group is SSM",
-					__func__, pim_str_sg_dump(sg));
+				zlog_debug("%s: local membership (S,G)=%pSG ignored as group is SSM",
+					   __func__, sg);
 			return 1;
 		}
 	}
@@ -1163,7 +1177,7 @@ int pim_ifchannel_local_membership_add(struct interface *ifp,
 
 	ifmembership_set(ch, PIM_IFMEMBERSHIP_INCLUDE);
 
-	if (sg->src.s_addr == INADDR_ANY) {
+	if (pim_addr_is_any(sg->src)) {
 		struct pim_upstream *up = pim_upstream_find(pim, sg);
 		struct pim_upstream *child;
 		struct listnode *up_node;
@@ -1176,6 +1190,16 @@ int pim_ifchannel_local_membership_add(struct interface *ifp,
 					   __FILE__, __func__, child->sg_str,
 					   ifp->name, up->sg_str);
 
+			if (!child->rpf.source_nexthop.interface) {
+				/* when iif unknown, do not inherit */
+				if (PIM_DEBUG_EVENTS)
+					zlog_debug(
+						"Skipped (S,G)=%s(%s) from %s: no iif",
+						child->sg_str, ifp->name,
+						up->sg_str);
+				continue;
+			}
+
 			ch = pim_ifchannel_find(ifp, &child->sg);
 			if (pim_upstream_evaluate_join_desired_interface(
 				    child, ch, starch)) {
@@ -1191,29 +1215,26 @@ int pim_ifchannel_local_membership_add(struct interface *ifp,
 				struct prefix_list *plist = prefix_list_lookup(
 					AFI_IP, pim->spt.plist);
 				struct prefix g;
-				g.family = AF_INET;
-				g.prefixlen = IPV4_MAX_PREFIXLEN;
-				g.u.prefix4 = up->sg.grp;
 
-				if (prefix_list_apply(plist, &g)
-				    == PREFIX_DENY) {
+				pim_addr_to_prefix(&g, up->sg.grp);
+				if (prefix_list_apply_ext(plist, NULL, &g,
+							  true) ==
+				    PREFIX_DENY) {
 					pim_channel_add_oif(
 						up->channel_oil, pim->regiface,
-						PIM_OIF_FLAG_PROTO_IGMP,
+						PIM_OIF_FLAG_PROTO_GM,
 						__func__);
 				}
 			}
 		} else
 			pim_channel_add_oif(up->channel_oil, pim->regiface,
-					PIM_OIF_FLAG_PROTO_IGMP,
-					__func__);
+					    PIM_OIF_FLAG_PROTO_GM, __func__);
 	}
 
 	return 1;
 }
 
-void pim_ifchannel_local_membership_del(struct interface *ifp,
-					struct prefix_sg *sg)
+void pim_ifchannel_local_membership_del(struct interface *ifp, pim_sgaddr *sg)
 {
 	struct pim_ifchannel *starch, *ch, *orig;
 	struct pim_interface *pim_ifp;
@@ -1222,7 +1243,7 @@ void pim_ifchannel_local_membership_del(struct interface *ifp,
 	pim_ifp = ifp->info;
 	if (!pim_ifp)
 		return;
-	if (!PIM_IF_TEST_PIM(pim_ifp->options))
+	if (!pim_ifp->pim_enable)
 		return;
 
 	orig = ch = pim_ifchannel_find(ifp, sg);
@@ -1230,7 +1251,7 @@ void pim_ifchannel_local_membership_del(struct interface *ifp,
 		return;
 	ifmembership_set(ch, PIM_IFMEMBERSHIP_NOINFO);
 
-	if (sg->src.s_addr == INADDR_ANY) {
+	if (pim_addr_is_any(sg->src)) {
 		struct pim_upstream *up = pim_upstream_find(pim_ifp->pim, sg);
 		struct pim_upstream *child;
 		struct listnode *up_node, *up_nnode;
@@ -1259,7 +1280,7 @@ void pim_ifchannel_local_membership_del(struct interface *ifp,
 			if (!pim_upstream_evaluate_join_desired_interface(
 				child, ch, starch) ||
 				(!chchannel &&
-				 c_oil->oil.mfcc_ttls[pim_ifp->mroute_vif_index])) {
+				 oil_if_has(c_oil, pim_ifp->mroute_vif_index))) {
 				pim_channel_del_inherited_oif(c_oil, ifp,
 						__func__);
 			}
@@ -1268,6 +1289,13 @@ void pim_ifchannel_local_membership_del(struct interface *ifp,
 			 * parent' delete_no_info */
 		}
 	}
+
+	/* Resettng the IGMP flags here */
+	if (orig->upstream)
+		PIM_UPSTREAM_FLAG_UNSET_SRC_IGMP(orig->upstream->flags);
+
+	PIM_IF_FLAG_UNSET_PROTO_IGMP(orig->flags);
+
 	delete_on_noinfo(orig);
 }
 
@@ -1281,15 +1309,11 @@ void pim_ifchannel_update_could_assert(struct pim_ifchannel *ch)
 	if (new_couldassert == old_couldassert)
 		return;
 
-	if (PIM_DEBUG_PIM_EVENTS) {
-		char src_str[INET_ADDRSTRLEN];
-		char grp_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<src?>", ch->sg.src, src_str, sizeof(src_str));
-		pim_inet4_dump("<grp?>", ch->sg.grp, grp_str, sizeof(grp_str));
-		zlog_debug("%s: CouldAssert(%s,%s,%s) changed from %d to %d",
-			   __func__, src_str, grp_str, ch->interface->name,
-			   old_couldassert, new_couldassert);
-	}
+	if (PIM_DEBUG_PIM_EVENTS)
+		zlog_debug("%s: CouldAssert(%pPAs,%pPAs,%s) changed from %d to %d",
+			   __func__, &ch->sg.src, &ch->sg.grp,
+			   ch->interface->name, old_couldassert,
+			   new_couldassert);
 
 	if (new_couldassert) {
 		/* CouldAssert(S,G,I) switched from false to true */
@@ -1322,27 +1346,17 @@ void pim_ifchannel_update_my_assert_metric(struct pim_ifchannel *ch)
 	if (pim_assert_metric_match(&my_metric_new, &ch->ifassert_my_metric))
 		return;
 
-	if (PIM_DEBUG_PIM_EVENTS) {
-		char src_str[INET_ADDRSTRLEN];
-		char grp_str[INET_ADDRSTRLEN];
-		char old_addr_str[INET_ADDRSTRLEN];
-		char new_addr_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<src?>", ch->sg.src, src_str, sizeof(src_str));
-		pim_inet4_dump("<grp?>", ch->sg.grp, grp_str, sizeof(grp_str));
-		pim_inet4_dump("<old_addr?>", ch->ifassert_my_metric.ip_address,
-			       old_addr_str, sizeof(old_addr_str));
-		pim_inet4_dump("<new_addr?>", my_metric_new.ip_address,
-			       new_addr_str, sizeof(new_addr_str));
+	if (PIM_DEBUG_PIM_EVENTS)
 		zlog_debug(
-			"%s: my_assert_metric(%s,%s,%s) changed from %u,%u,%u,%s to %u,%u,%u,%s",
-			__func__, src_str, grp_str, ch->interface->name,
+			"%s: my_assert_metric(%pPAs,%pPAs,%s) changed from %u,%u,%u,%pPAs to %u,%u,%u,%pPAs",
+			__func__, &ch->sg.src, &ch->sg.grp, ch->interface->name,
 			ch->ifassert_my_metric.rpt_bit_flag,
 			ch->ifassert_my_metric.metric_preference,
-			ch->ifassert_my_metric.route_metric, old_addr_str,
+			ch->ifassert_my_metric.route_metric,
+			&ch->ifassert_my_metric.ip_address,
 			my_metric_new.rpt_bit_flag,
 			my_metric_new.metric_preference,
-			my_metric_new.route_metric, new_addr_str);
-	}
+			my_metric_new.route_metric, &my_metric_new.ip_address);
 
 	ch->ifassert_my_metric = my_metric_new;
 
@@ -1362,16 +1376,11 @@ void pim_ifchannel_update_assert_tracking_desired(struct pim_ifchannel *ch)
 	if (new_atd == old_atd)
 		return;
 
-	if (PIM_DEBUG_PIM_EVENTS) {
-		char src_str[INET_ADDRSTRLEN];
-		char grp_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<src?>", ch->sg.src, src_str, sizeof(src_str));
-		pim_inet4_dump("<grp?>", ch->sg.grp, grp_str, sizeof(grp_str));
+	if (PIM_DEBUG_PIM_EVENTS)
 		zlog_debug(
-			"%s: AssertTrackingDesired(%s,%s,%s) changed from %d to %d",
-			__func__, src_str, grp_str, ch->interface->name,
+			"%s: AssertTrackingDesired(%pPAs,%pPAs,%s) changed from %d to %d",
+			__func__, &ch->sg.src, &ch->sg.grp, ch->interface->name,
 			old_atd, new_atd);
-	}
 
 	if (new_atd) {
 		/* AssertTrackingDesired(S,G,I) switched from false to true */
@@ -1494,10 +1503,3 @@ void pim_ifchannel_set_star_g_join_state(struct pim_ifchannel *ch, int eom,
 	if (send_upstream_starg)
 		pim_jp_agg_single_upstream_send(&starup->rpf, starup, true);
 }
-
-unsigned int pim_ifchannel_hash_key(const void *arg)
-{
-	const struct pim_ifchannel *ch = arg;
-
-	return jhash_2words(ch->sg.src.s_addr, ch->sg.grp.s_addr, 0);
-}
