@@ -25,6 +25,7 @@
 #include "if.h"
 
 #include "pimd.h"
+#include "pim_instance.h"
 #include "pim_msg.h"
 #include "pim_jp_agg.h"
 #include "pim_join.h"
@@ -60,13 +61,7 @@ int pim_jp_agg_group_list_cmp(void *arg1, void *arg2)
 	const struct pim_jp_agg_group *jag2 =
 		(const struct pim_jp_agg_group *)arg2;
 
-	if (jag1->group.s_addr < jag2->group.s_addr)
-		return -1;
-
-	if (jag1->group.s_addr > jag2->group.s_addr)
-		return 1;
-
-	return 0;
+	return pim_addr_cmp(jag1->group, jag2->group);
 }
 
 static int pim_jp_agg_src_cmp(void *arg1, void *arg2)
@@ -80,13 +75,7 @@ static int pim_jp_agg_src_cmp(void *arg1, void *arg2)
 	if (!js1->is_join && js2->is_join)
 		return 1;
 
-	if ((uint32_t)js1->up->sg.src.s_addr < (uint32_t)js2->up->sg.src.s_addr)
-		return -1;
-
-	if ((uint32_t)js1->up->sg.src.s_addr > (uint32_t)js2->up->sg.src.s_addr)
-		return 1;
-
-	return 0;
+	return pim_addr_cmp(js1->up->sg.src, js2->up->sg.src);
 }
 
 /*
@@ -133,14 +122,14 @@ pim_jp_agg_get_interface_upstream_switch_list(struct pim_rpf *rpf)
 
 	for (ALL_LIST_ELEMENTS(pim_ifp->upstream_switch_list, node, nnode,
 			       pius)) {
-		if (pius->address.s_addr == rpf->rpf_addr.u.prefix4.s_addr)
+		if (!pim_addr_cmp(pius->address, rpf->rpf_addr))
 			break;
 	}
 
 	if (!pius) {
 		pius = XCALLOC(MTYPE_PIM_JP_AGG_GROUP,
 			       sizeof(struct pim_iface_upstream_switch));
-		pius->address.s_addr = rpf->rpf_addr.u.prefix4.s_addr;
+		pius->address = rpf->rpf_addr;
 		pius->us = list_new();
 		listnode_add_sort(pim_ifp->upstream_switch_list, pius);
 	}
@@ -156,7 +145,7 @@ void pim_jp_agg_remove_group(struct list *group, struct pim_upstream *up,
 	struct pim_jp_sources *js = NULL;
 
 	for (ALL_LIST_ELEMENTS(group, node, nnode, jag)) {
-		if (jag->group.s_addr == up->sg.grp.s_addr)
+		if (!pim_addr_cmp(jag->group, up->sg.grp))
 			break;
 	}
 
@@ -169,17 +158,10 @@ void pim_jp_agg_remove_group(struct list *group, struct pim_upstream *up,
 	}
 
 	if (nbr) {
-		if (PIM_DEBUG_TRACE) {
-			char src_str[INET_ADDRSTRLEN];
-
-			pim_inet4_dump("<src?>", nbr->source_addr, src_str,
-					sizeof(src_str));
-			zlog_debug(
-				"up %s remove from nbr %s/%s jp-agg-list",
-				up->sg_str,
-				nbr->interface->name,
-				src_str);
-		}
+		if (PIM_DEBUG_TRACE)
+			zlog_debug("up %s remove from nbr %s/%pPAs jp-agg-list",
+				   up->sg_str, nbr->interface->name,
+				   &nbr->source_addr);
 	}
 
 	if (js) {
@@ -202,7 +184,7 @@ int pim_jp_agg_is_in_list(struct list *group, struct pim_upstream *up)
 	struct pim_jp_sources *js = NULL;
 
 	for (ALL_LIST_ELEMENTS(group, node, nnode, jag)) {
-		if (jag->group.s_addr == up->sg.grp.s_addr)
+		if (!pim_addr_cmp(jag->group, up->sg.grp))
 			break;
 	}
 
@@ -276,14 +258,14 @@ void pim_jp_agg_add_group(struct list *group, struct pim_upstream *up,
 	struct pim_jp_sources *js = NULL;
 
 	for (ALL_LIST_ELEMENTS(group, node, nnode, jag)) {
-		if (jag->group.s_addr == up->sg.grp.s_addr)
+		if (!pim_addr_cmp(jag->group, up->sg.grp))
 			break;
 	}
 
 	if (!jag) {
 		jag = XCALLOC(MTYPE_PIM_JP_AGG_GROUP,
 			      sizeof(struct pim_jp_agg_group));
-		jag->group.s_addr = up->sg.grp.s_addr;
+		jag->group = up->sg.grp;
 		jag->sources = list_new();
 		jag->sources->cmp = pim_jp_agg_src_cmp;
 		jag->sources->del = (void (*)(void *))pim_jp_agg_src_free;
@@ -296,17 +278,11 @@ void pim_jp_agg_add_group(struct list *group, struct pim_upstream *up,
 	}
 
 	if (nbr) {
-		if (PIM_DEBUG_TRACE) {
-			char src_str[INET_ADDRSTRLEN];
-
-			pim_inet4_dump("<src?>", nbr->source_addr, src_str,
-					sizeof(src_str));
-			zlog_debug(
-				"up %s add to nbr %s/%s jp-agg-list",
-				up->sg_str,
-				up->rpf.source_nexthop.interface->name,
-				src_str);
-		}
+		if (PIM_DEBUG_TRACE)
+			zlog_debug("up %s add to nbr %s/%pPAs jp-agg-list",
+				   up->sg_str,
+				   up->rpf.source_nexthop.interface->name,
+				   &nbr->source_addr);
 	}
 
 	if (!js) {
@@ -360,32 +336,30 @@ void pim_jp_agg_switch_interface(struct pim_rpf *orpf, struct pim_rpf *nrpf,
 void pim_jp_agg_single_upstream_send(struct pim_rpf *rpf,
 				     struct pim_upstream *up, bool is_join)
 {
-	static struct list *groups = NULL;
-	static struct pim_jp_agg_group jag;
-	static struct pim_jp_sources js;
-
-	static bool first = true;
+	struct list groups, sources;
+	struct pim_jp_agg_group jag;
+	struct pim_jp_sources js;
 
 	/* skip JP upstream messages if source is directly connected */
 	if (!up || !rpf->source_nexthop.interface ||
 		pim_if_connected_to_source(rpf->source_nexthop.interface,
 			up->sg.src) ||
-		if_is_loopback_or_vrf(rpf->source_nexthop.interface))
+		if_is_loopback(rpf->source_nexthop.interface))
 		return;
 
-	if (first) {
-		groups = list_new();
-		jag.sources = list_new();
+	memset(&groups, 0, sizeof(groups));
+	memset(&sources, 0, sizeof(sources));
+	jag.sources = &sources;
 
-		listnode_add(groups, &jag);
-		listnode_add(jag.sources, &js);
+	listnode_add(&groups, &jag);
+	listnode_add(jag.sources, &js);
 
-		first = false;
-	}
-
-	jag.group.s_addr = up->sg.grp.s_addr;
+	jag.group = up->sg.grp;
 	js.up = up;
 	js.is_join = is_join;
 
-	pim_joinprune_send(rpf, groups);
+	pim_joinprune_send(rpf, &groups);
+
+	list_delete_all_node(jag.sources);
+	list_delete_all_node(&groups);
 }
