@@ -26,8 +26,11 @@
 #include "hash.h"
 #include "jhash.h"
 
+#include "lib/printfrr.h"
+
 #include "pimd.h"
 #include "pimd/pim_nht.h"
+#include "pim_instance.h"
 #include "log.h"
 #include "pim_time.h"
 #include "pim_oil.h"
@@ -40,6 +43,7 @@
 #include "pim_zebra.h"
 #include "pim_zlookup.h"
 #include "pim_rp.h"
+#include "pim_addr.h"
 
 /**
  * pim_sendmsg_zebra_rnh -- Format and send a nexthop register/Unregister
@@ -48,23 +52,21 @@
 void pim_sendmsg_zebra_rnh(struct pim_instance *pim, struct zclient *zclient,
 			   struct pim_nexthop_cache *pnc, int command)
 {
-	struct prefix *p;
+	struct prefix p;
 	int ret;
 
-	p = &(pnc->rpf.rpf_addr);
-	ret = zclient_send_rnh(zclient, command, p, false, pim->vrf_id);
-	if (ret < 0)
+	pim_addr_to_prefix(&p, pnc->rpf.rpf_addr);
+	ret = zclient_send_rnh(zclient, command, &p, SAFI_UNICAST, false, false,
+			       pim->vrf->vrf_id);
+	if (ret == ZCLIENT_SEND_FAILURE)
 		zlog_warn("sendmsg_nexthop: zclient_send_message() failed");
 
-	if (PIM_DEBUG_PIM_NHT) {
-		char buf[PREFIX2STR_BUFFER];
-		prefix2str(p, buf, sizeof(buf));
+	if (PIM_DEBUG_PIM_NHT)
 		zlog_debug(
-			"%s: NHT %sregistered addr %s(%s) with Zebra ret:%d ",
+			"%s: NHT %sregistered addr %pFX(%s) with Zebra ret:%d ",
 			__func__,
-			(command == ZEBRA_NEXTHOP_REGISTER) ? " " : "de", buf,
+			(command == ZEBRA_NEXTHOP_REGISTER) ? " " : "de", &p,
 			pim->vrf->name, ret);
-	}
 
 	return;
 }
@@ -75,10 +77,7 @@ struct pim_nexthop_cache *pim_nexthop_cache_find(struct pim_instance *pim,
 	struct pim_nexthop_cache *pnc = NULL;
 	struct pim_nexthop_cache lookup;
 
-	lookup.rpf.rpf_addr.family = rpf->rpf_addr.family;
-	lookup.rpf.rpf_addr.prefixlen = rpf->rpf_addr.prefixlen;
-	lookup.rpf.rpf_addr.u.prefix4.s_addr = rpf->rpf_addr.u.prefix4.s_addr;
-
+	lookup.rpf.rpf_addr = rpf->rpf_addr;
 	pnc = hash_lookup(pim->rpf_hash, &lookup);
 
 	return pnc;
@@ -89,66 +88,63 @@ static struct pim_nexthop_cache *pim_nexthop_cache_add(struct pim_instance *pim,
 {
 	struct pim_nexthop_cache *pnc;
 	char hash_name[64];
-	char buf1[64];
 
 	pnc = XCALLOC(MTYPE_PIM_NEXTHOP_CACHE,
 		      sizeof(struct pim_nexthop_cache));
-	pnc->rpf.rpf_addr.family = rpf_addr->rpf_addr.family;
-	pnc->rpf.rpf_addr.prefixlen = rpf_addr->rpf_addr.prefixlen;
-	pnc->rpf.rpf_addr.u.prefix4.s_addr =
-		rpf_addr->rpf_addr.u.prefix4.s_addr;
+	pnc->rpf.rpf_addr = rpf_addr->rpf_addr;
 
 	pnc = hash_get(pim->rpf_hash, pnc, hash_alloc_intern);
 
 	pnc->rp_list = list_new();
 	pnc->rp_list->cmp = pim_rp_list_cmp;
 
-	snprintf(hash_name, sizeof(hash_name), "PNC %s(%s) Upstream Hash",
-		 prefix2str(&pnc->rpf.rpf_addr, buf1, 64), pim->vrf->name);
+	snprintfrr(hash_name, sizeof(hash_name), "PNC %pPA(%s) Upstream Hash",
+		   &pnc->rpf.rpf_addr, pim->vrf->name);
 	pnc->upstream_hash = hash_create_size(8192, pim_upstream_hash_key,
 					      pim_upstream_equal, hash_name);
 
 	return pnc;
 }
 
-/*
- * pim_find_or_track_nexthop
- *
- * This API is used to Register an address with Zebra
- *
- * 1 -> Success
- * 0 -> Failure
- */
-int pim_find_or_track_nexthop(struct pim_instance *pim, struct prefix *addr,
-			      struct pim_upstream *up, struct rp_info *rp,
-			      bool bsr_track_needed,
-			      struct pim_nexthop_cache *out_pnc)
+static struct pim_nexthop_cache *pim_nht_get(struct pim_instance *pim,
+					     pim_addr addr)
 {
 	struct pim_nexthop_cache *pnc = NULL;
 	struct pim_rpf rpf;
-	struct listnode *ch_node = NULL;
 	struct zclient *zclient = NULL;
 
 	zclient = pim_zebra_zclient_get();
-	memset(&rpf, 0, sizeof(struct pim_rpf));
-	rpf.rpf_addr.family = addr->family;
-	rpf.rpf_addr.prefixlen = addr->prefixlen;
-	rpf.rpf_addr.u.prefix4 = addr->u.prefix4;
+	memset(&rpf, 0, sizeof(rpf));
+	rpf.rpf_addr = addr;
 
 	pnc = pim_nexthop_cache_find(pim, &rpf);
 	if (!pnc) {
 		pnc = pim_nexthop_cache_add(pim, &rpf);
 		pim_sendmsg_zebra_rnh(pim, zclient, pnc,
 				      ZEBRA_NEXTHOP_REGISTER);
-		if (PIM_DEBUG_PIM_NHT) {
-			char buf[PREFIX2STR_BUFFER];
-			prefix2str(addr, buf, sizeof(buf));
+		if (PIM_DEBUG_PIM_NHT_DETAIL)
 			zlog_debug(
-				"%s: NHT cache and zebra notification added for %s(%s)",
-				__func__, buf, pim->vrf->name);
-		}
+				"%s: NHT cache and zebra notification added for %pPA(%s)",
+				__func__, &addr, pim->vrf->name);
 	}
 
+	return pnc;
+}
+
+/* TBD: this does several distinct things and should probably be split up.
+ * (checking state vs. returning pnc vs. adding upstream vs. adding rp)
+ */
+int pim_find_or_track_nexthop(struct pim_instance *pim, pim_addr addr,
+			      struct pim_upstream *up, struct rp_info *rp,
+			      struct pim_nexthop_cache *out_pnc)
+{
+	struct pim_nexthop_cache *pnc;
+	struct listnode *ch_node = NULL;
+
+	pnc = pim_nht_get(pim, addr);
+
+	assertf(up || rp, "addr=%pPA", &addr);
+
 	if (rp != NULL) {
 		ch_node = listnode_lookup(pnc->rp_list, rp);
 		if (ch_node == NULL)
@@ -156,10 +152,7 @@ int pim_find_or_track_nexthop(struct pim_instance *pim, struct prefix *addr,
 	}
 
 	if (up != NULL)
-		hash_get(pnc->upstream_hash, up, hash_alloc_intern);
-
-	if (bsr_track_needed)
-		pnc->bsr_tracking = true;
+		(void)hash_get(pnc->upstream_hash, up, hash_alloc_intern);
 
 	if (CHECK_FLAG(pnc->flags, PIM_NEXTHOP_VALID)) {
 		if (out_pnc)
@@ -170,243 +163,237 @@ int pim_find_or_track_nexthop(struct pim_instance *pim, struct prefix *addr,
 	return 0;
 }
 
-void pim_delete_tracked_nexthop(struct pim_instance *pim, struct prefix *addr,
-				struct pim_upstream *up, struct rp_info *rp,
-				bool del_bsr_tracking)
+void pim_nht_bsr_add(struct pim_instance *pim, pim_addr addr)
+{
+	struct pim_nexthop_cache *pnc;
+
+	pnc = pim_nht_get(pim, addr);
+
+	pnc->bsr_count++;
+}
+
+static void pim_nht_drop_maybe(struct pim_instance *pim,
+			       struct pim_nexthop_cache *pnc)
+{
+	if (PIM_DEBUG_PIM_NHT)
+		zlog_debug(
+			"%s: NHT %pPA(%s) rp_list count:%d upstream count:%ld BSR count:%u",
+			__func__, &pnc->rpf.rpf_addr, pim->vrf->name,
+			pnc->rp_list->count, pnc->upstream_hash->count,
+			pnc->bsr_count);
+
+	if (pnc->rp_list->count == 0 && pnc->upstream_hash->count == 0
+	    && pnc->bsr_count == 0) {
+		struct zclient *zclient = pim_zebra_zclient_get();
+
+		pim_sendmsg_zebra_rnh(pim, zclient, pnc,
+				      ZEBRA_NEXTHOP_UNREGISTER);
+
+		list_delete(&pnc->rp_list);
+		hash_free(pnc->upstream_hash);
+
+		hash_release(pim->rpf_hash, pnc);
+		if (pnc->nexthop)
+			nexthops_free(pnc->nexthop);
+		XFREE(MTYPE_PIM_NEXTHOP_CACHE, pnc);
+	}
+}
+
+void pim_delete_tracked_nexthop(struct pim_instance *pim, pim_addr addr,
+				struct pim_upstream *up, struct rp_info *rp)
 {
 	struct pim_nexthop_cache *pnc = NULL;
 	struct pim_nexthop_cache lookup;
-	struct zclient *zclient = NULL;
 	struct pim_upstream *upstream = NULL;
 
-	zclient = pim_zebra_zclient_get();
-
 	/* Remove from RPF hash if it is the last entry */
-	lookup.rpf.rpf_addr = *addr;
+	lookup.rpf.rpf_addr = addr;
 	pnc = hash_lookup(pim->rpf_hash, &lookup);
-	if (pnc) {
-		if (rp) {
-			/* Release the (*, G)upstream from pnc->upstream_hash,
-			 * whose Group belongs to the RP getting deleted
-			 */
-			frr_each (rb_pim_upstream, &pim->upstream_head,
-				  upstream) {
-				struct prefix grp;
-				struct rp_info *trp_info;
-
-				if (upstream->sg.src.s_addr != INADDR_ANY)
-					continue;
-
-				grp.family = AF_INET;
-				grp.prefixlen = IPV4_MAX_BITLEN;
-				grp.u.prefix4 = upstream->sg.grp;
-
-				trp_info = pim_rp_find_match_group(pim, &grp);
-				if (trp_info == rp)
-					hash_release(pnc->upstream_hash,
-						     upstream);
-			}
-			listnode_delete(pnc->rp_list, rp);
-		}
-
-		if (up)
-			hash_release(pnc->upstream_hash, up);
-
-		if (del_bsr_tracking)
-			pnc->bsr_tracking = false;
-
-		if (PIM_DEBUG_PIM_NHT) {
-			char buf[PREFIX_STRLEN];
-			prefix2str(addr, buf, sizeof(buf));
-			zlog_debug(
-				"%s: NHT %s(%s) rp_list count:%d upstream count:%ld",
-				__func__, buf, pim->vrf->name,
-				pnc->rp_list->count, pnc->upstream_hash->count);
-		}
-
-		if (pnc->rp_list->count == 0
-		    && pnc->upstream_hash->count == 0
-		    && pnc->bsr_tracking == false) {
-			pim_sendmsg_zebra_rnh(pim, zclient, pnc,
-					      ZEBRA_NEXTHOP_UNREGISTER);
-
-			list_delete(&pnc->rp_list);
-			hash_free(pnc->upstream_hash);
-
-			hash_release(pim->rpf_hash, pnc);
-			if (pnc->nexthop)
-				nexthops_free(pnc->nexthop);
-			XFREE(MTYPE_PIM_NEXTHOP_CACHE, pnc);
-		}
-	}
-}
-
-/* Given a source address and a neighbor address, check if the neighbor is one
- * of the next hop to reach the source. search from zebra route database
- */
-bool pim_nexthop_match(struct pim_instance *pim, struct in_addr addr,
-		       struct in_addr ip_src)
-{
-	struct pim_zlookup_nexthop nexthop_tab[MULTIPATH_NUM];
-	int i = 0;
-	ifindex_t first_ifindex = 0;
-	struct interface *ifp = NULL;
-	struct pim_neighbor *nbr = NULL;
-	int num_ifindex;
-
-	if (addr.s_addr == INADDR_NONE)
-		return false;
-
-	memset(nexthop_tab, 0,
-	       sizeof(struct pim_zlookup_nexthop) * MULTIPATH_NUM);
-	num_ifindex = zclient_lookup_nexthop(pim, nexthop_tab, MULTIPATH_NUM,
-					     addr, PIM_NEXTHOP_LOOKUP_MAX);
-	if (num_ifindex < 1) {
-		char addr_str[INET_ADDRSTRLEN];
-
-		pim_inet4_dump("<addr?>", addr, addr_str, sizeof(addr_str));
-		zlog_warn(
-			"%s %s: could not find nexthop ifindex for address %s",
-			__FILE__, __func__, addr_str);
-		return false;
+	if (!pnc) {
+		zlog_warn("attempting to delete nonexistent NHT entry %pPA",
+			  &addr);
+		return;
 	}
 
-	while (i < num_ifindex) {
-		first_ifindex = nexthop_tab[i].ifindex;
+	if (rp) {
+		/* Release the (*, G)upstream from pnc->upstream_hash,
+		 * whose Group belongs to the RP getting deleted
+		 */
+		frr_each (rb_pim_upstream, &pim->upstream_head, upstream) {
+			struct prefix grp;
+			struct rp_info *trp_info;
 
-		ifp = if_lookup_by_index(first_ifindex, pim->vrf_id);
-		if (!ifp) {
-			if (PIM_DEBUG_ZEBRA) {
-				char addr_str[INET_ADDRSTRLEN];
-
-				pim_inet4_dump("<addr?>", addr, addr_str,
-					       sizeof(addr_str));
-				zlog_debug(
-					"%s %s: could not find interface for ifindex %d (address %s)",
-					__FILE__, __func__, first_ifindex,
-					addr_str);
-			}
-			i++;
-			continue;
-		}
-
-		if (!ifp->info) {
-			if (PIM_DEBUG_ZEBRA) {
-				char addr_str[INET_ADDRSTRLEN];
-
-				pim_inet4_dump("<addr?>", addr, addr_str,
-					       sizeof(addr_str));
-				zlog_debug(
-					"%s: multicast not enabled on input interface %s (ifindex=%d, RPF for source %s)",
-					__func__, ifp->name, first_ifindex,
-					addr_str);
-			}
-			i++;
-			continue;
-		}
-
-		if (!pim_if_connected_to_source(ifp, addr)) {
-			nbr = pim_neighbor_find(
-				ifp, nexthop_tab[i].nexthop_addr.u.prefix4);
-			if (PIM_DEBUG_PIM_TRACE_DETAIL)
-				zlog_debug("ifp name: %s, pim nbr: %p",
-					   ifp->name, nbr);
-			if (!nbr && !if_is_loopback(ifp)) {
-				i++;
+			if (!pim_addr_is_any(upstream->sg.src))
 				continue;
-			}
+
+			pim_addr_to_prefix(&grp, upstream->sg.grp);
+			trp_info = pim_rp_find_match_group(pim, &grp);
+			if (trp_info == rp)
+				hash_release(pnc->upstream_hash, upstream);
 		}
-
-		if (nexthop_tab[i].nexthop_addr.u.prefix4.s_addr
-		    == ip_src.s_addr)
-			return true;
-
-		i++;
+		listnode_delete(pnc->rp_list, rp);
 	}
 
-	return false;
+	if (up)
+		hash_release(pnc->upstream_hash, up);
+
+	pim_nht_drop_maybe(pim, pnc);
 }
 
-/* Given a source address and a neighbor address, check if the neighbor is one
- * of the next hop to reach the source. search from pim next hop cache
- */
-bool pim_nexthop_match_nht_cache(struct pim_instance *pim, struct in_addr addr,
-				 struct in_addr ip_src)
+void pim_nht_bsr_del(struct pim_instance *pim, pim_addr addr)
 {
-	struct pim_rpf rpf;
-	ifindex_t first_ifindex;
-	struct interface *ifp = NULL;
-	uint8_t nh_iter = 0;
-	struct pim_neighbor *nbr = NULL;
-	struct nexthop *nh_node = NULL;
 	struct pim_nexthop_cache *pnc = NULL;
+	struct pim_nexthop_cache lookup;
 
-	memset(&rpf, 0, sizeof(struct pim_rpf));
-	rpf.rpf_addr.family = AF_INET;
-	rpf.rpf_addr.prefixlen = IPV4_MAX_BITLEN;
-	rpf.rpf_addr.u.prefix4 = addr;
+	/*
+	 * Nothing to do here if the address to unregister
+	 * is 0.0.0.0 as that the BSR has not been registered
+	 * for tracking yet.
+	 */
+	if (pim_addr_is_any(addr))
+		return;
 
-	pnc = pim_nexthop_cache_find(pim, &rpf);
-	if (!pnc || !pnc->nexthop_num)
-		return false;
+	lookup.rpf.rpf_addr = addr;
 
-	for (nh_node = pnc->nexthop; nh_node; nh_node = nh_node->next) {
-		first_ifindex = nh_node->ifindex;
-		ifp = if_lookup_by_index(first_ifindex, pim->vrf_id);
-		if (!ifp) {
-			if (PIM_DEBUG_PIM_NHT) {
-				char addr_str[INET_ADDRSTRLEN];
+	pnc = hash_lookup(pim->rpf_hash, &lookup);
 
-				pim_inet4_dump("<addr?>", addr, addr_str,
-					       sizeof(addr_str));
-				zlog_debug(
-					"%s %s: could not find interface for ifindex %d (address %s(%s))",
-					__FILE__, __func__, first_ifindex,
-					addr_str, pim->vrf->name);
-			}
-			nh_iter++;
-			continue;
-		}
-		if (!ifp->info) {
-			if (PIM_DEBUG_PIM_NHT) {
-				char addr_str[INET_ADDRSTRLEN];
-
-				pim_inet4_dump("<addr?>", addr, addr_str,
-					       sizeof(addr_str));
-				zlog_debug(
-					"%s: multicast not enabled on input interface %s(%s) (ifindex=%d, RPF for source %s)",
-					__func__, ifp->name, pim->vrf->name,
-					first_ifindex, addr_str);
-			}
-			nh_iter++;
-			continue;
-		}
-
-		if (!pim_if_connected_to_source(ifp, addr)) {
-			nbr = pim_neighbor_find(ifp, nh_node->gate.ipv4);
-			if (!nbr && !if_is_loopback(ifp)) {
-				if (PIM_DEBUG_PIM_NHT)
-					zlog_debug(
-						"%s: pim nbr not found on input interface %s(%s)",
-						__func__, ifp->name,
-						pim->vrf->name);
-				nh_iter++;
-				continue;
-			}
-		}
-
-		if (nh_node->gate.ipv4.s_addr == ip_src.s_addr)
-			return true;
+	if (!pnc) {
+		zlog_warn("attempting to delete nonexistent NHT BSR entry %pPA",
+			  &addr);
+		return;
 	}
 
+	assertf(pnc->bsr_count > 0, "addr=%pPA", &addr);
+	pnc->bsr_count--;
+
+	pim_nht_drop_maybe(pim, pnc);
+}
+
+bool pim_nht_bsr_rpf_check(struct pim_instance *pim, pim_addr bsr_addr,
+			   struct interface *src_ifp, pim_addr src_ip)
+{
+	struct pim_nexthop_cache *pnc = NULL;
+	struct pim_nexthop_cache lookup;
+	struct pim_neighbor *nbr = NULL;
+	struct nexthop *nh;
+	struct interface *ifp;
+
+	lookup.rpf.rpf_addr = bsr_addr;
+
+	pnc = hash_lookup(pim->rpf_hash, &lookup);
+	if (!pnc || !CHECK_FLAG(pnc->flags, PIM_NEXTHOP_ANSWER_RECEIVED)) {
+		/* BSM from a new freshly registered BSR - do a synchronous
+		 * zebra query since otherwise we'd drop the first packet,
+		 * leading to additional delay in picking up BSM data
+		 */
+
+		/* FIXME: this should really be moved into a generic NHT
+		 * function that does "add and get immediate result" or maybe
+		 * "check cache or get immediate result." But until that can
+		 * be worked in, here's a copy of the code below :(
+		 */
+		struct pim_zlookup_nexthop nexthop_tab[router->multipath];
+		ifindex_t i;
+		struct interface *ifp = NULL;
+		int num_ifindex;
+
+		memset(nexthop_tab, 0, sizeof(nexthop_tab));
+		num_ifindex = zclient_lookup_nexthop(
+			pim, nexthop_tab, router->multipath, bsr_addr,
+			PIM_NEXTHOP_LOOKUP_MAX);
+
+		if (num_ifindex <= 0)
+			return false;
+
+		for (i = 0; i < num_ifindex; i++) {
+			struct pim_zlookup_nexthop *znh = &nexthop_tab[i];
+
+			/* pim_zlookup_nexthop has no ->type */
+
+			/* 1:1 match code below with znh instead of nh */
+			ifp = if_lookup_by_index(znh->ifindex,
+						 pim->vrf->vrf_id);
+
+			if (!ifp || !ifp->info)
+				continue;
+
+			if (if_is_loopback(ifp) && if_is_loopback(src_ifp))
+				return true;
+
+			nbr = pim_neighbor_find(ifp, znh->nexthop_addr);
+			if (!nbr)
+				continue;
+
+			return znh->ifindex == src_ifp->ifindex &&
+			       (!pim_addr_cmp(znh->nexthop_addr, src_ip));
+		}
+		return false;
+	}
+
+	if (!CHECK_FLAG(pnc->flags, PIM_NEXTHOP_VALID))
+		return false;
+
+	/* if we accept BSMs from more than one ECMP nexthop, this will cause
+	 * BSM message "multiplication" for each ECMP hop.  i.e. if you have
+	 * 4-way ECMP and 4 hops you end up with 256 copies of each BSM
+	 * message.
+	 *
+	 * so...  only accept the first (IPv4) valid nexthop as source.
+	 */
+
+	for (nh = pnc->nexthop; nh; nh = nh->next) {
+		pim_addr nhaddr;
+
+		switch (nh->type) {
+#if PIM_IPV == 4
+		case NEXTHOP_TYPE_IPV4:
+			if (nh->ifindex == IFINDEX_INTERNAL)
+				continue;
+
+			/* fallthru */
+		case NEXTHOP_TYPE_IPV4_IFINDEX:
+			nhaddr = nh->gate.ipv4;
+			break;
+#else
+		case NEXTHOP_TYPE_IPV6:
+			if (nh->ifindex == IFINDEX_INTERNAL)
+				continue;
+
+			/* fallthru */
+		case NEXTHOP_TYPE_IPV6_IFINDEX:
+			nhaddr = nh->gate.ipv6;
+			break;
+#endif
+		case NEXTHOP_TYPE_IFINDEX:
+			nhaddr = bsr_addr;
+			break;
+
+		default:
+			continue;
+		}
+
+		ifp = if_lookup_by_index(nh->ifindex, pim->vrf->vrf_id);
+		if (!ifp || !ifp->info)
+			continue;
+
+		if (if_is_loopback(ifp) && if_is_loopback(src_ifp))
+			return true;
+
+		/* MRIB (IGP) may be pointing at a router where PIM is down */
+		nbr = pim_neighbor_find(ifp, nhaddr);
+		if (!nbr)
+			continue;
+
+		return nh->ifindex == src_ifp->ifindex &&
+		       (!pim_addr_cmp(nhaddr, src_ip));
+	}
 	return false;
 }
 
 void pim_rp_nexthop_del(struct rp_info *rp_info)
 {
 	rp_info->rp.source_nexthop.interface = NULL;
-	rp_info->rp.source_nexthop.mrib_nexthop_addr.u.prefix4.s_addr =
-		PIM_NET_INADDR_ANY;
+	rp_info->rp.source_nexthop.mrib_nexthop_addr = PIMADDR_ANY;
 	rp_info->rp.source_nexthop.mrib_metric_preference =
 		router->infinite_assert_metric.metric_preference;
 	rp_info->rp.source_nexthop.mrib_route_metric =
@@ -422,12 +409,12 @@ static void pim_update_rp_nh(struct pim_instance *pim,
 
 	/*Traverse RP list and update each RP Nexthop info */
 	for (ALL_LIST_ELEMENTS_RO(pnc->rp_list, node, rp_info)) {
-		if (rp_info->rp.rpf_addr.u.prefix4.s_addr == INADDR_NONE)
+		if (pim_rpf_addr_is_inaddr_any(&rp_info->rp))
 			continue;
 
 		// Compute PIM RPF using cached nexthop
 		if (!pim_ecmp_nexthop_lookup(pim, &rp_info->rp.source_nexthop,
-					     &rp_info->rp.rpf_addr,
+					     rp_info->rp.rpf_addr,
 					     &rp_info->group, 1))
 			pim_rp_nexthop_del(rp_info);
 	}
@@ -449,7 +436,7 @@ static int pim_update_upstream_nh_helper(struct hash_bucket *bucket, void *arg)
 	 * RPF nbr is now unreachable the MFC has already been updated
 	 * by pim_rpf_clear
 	 */
-	if (rpf_result != PIM_RPF_FAILURE)
+	if (rpf_result == PIM_RPF_CHANGED)
 		pim_upstream_mroute_iif_update(up->channel_oil, __func__);
 
 	if (rpf_result == PIM_RPF_CHANGED ||
@@ -485,34 +472,23 @@ static int pim_update_upstream_nh(struct pim_instance *pim,
 uint32_t pim_compute_ecmp_hash(struct prefix *src, struct prefix *grp)
 {
 	uint32_t hash_val;
-	uint32_t s = 0, g = 0;
 
-	if ((!src))
+	if (!src)
 		return 0;
 
-	switch (src->family) {
-	case AF_INET: {
-		s = src->u.prefix4.s_addr;
-		s = s == 0 ? 1 : s;
-		if (grp)
-			g = grp->u.prefix4.s_addr;
-	} break;
-	default:
-		break;
-	}
-
-	hash_val = jhash_2words(g, s, 101);
+	hash_val = prefix_hash_key(src);
+	if (grp)
+		hash_val ^= prefix_hash_key(grp);
 	return hash_val;
 }
 
 static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 				   struct pim_nexthop_cache *pnc,
-				   struct pim_nexthop *nexthop,
-				   struct prefix *src, struct prefix *grp,
-				   int neighbor_needed)
+				   struct pim_nexthop *nexthop, pim_addr src,
+				   struct prefix *grp, int neighbor_needed)
 {
-	struct pim_neighbor *nbrs[MULTIPATH_NUM], *nbr = NULL;
-	struct interface *ifps[MULTIPATH_NUM];
+	struct pim_neighbor *nbrs[router->multipath], *nbr = NULL;
+	struct interface *ifps[router->multipath];
 	struct nexthop *nh_node = NULL;
 	ifindex_t first_ifindex;
 	struct interface *ifp = NULL;
@@ -523,13 +499,16 @@ static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 	if (!pnc || !pnc->nexthop_num || !nexthop)
 		return 0;
 
+	pim_addr nh_addr = nexthop->mrib_nexthop_addr;
+	pim_addr grp_addr = pim_addr_from_prefix(grp);
+
 	memset(&nbrs, 0, sizeof(nbrs));
 	memset(&ifps, 0, sizeof(ifps));
 
+
 	// Current Nexthop is VALID, check to stay on the current path.
-	if (nexthop->interface && nexthop->interface->info
-	    && nexthop->mrib_nexthop_addr.u.prefix4.s_addr
-		       != PIM_NET_INADDR_ANY) {
+	if (nexthop->interface && nexthop->interface->info &&
+	    (!pim_addr_is_any(nh_addr))) {
 		/* User configured knob to explicitly switch
 		   to new path is disabled or current path
 		   metric is less than nexthop update.
@@ -549,12 +528,12 @@ static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 					break;
 			}
 
-			if (curr_route_valid
-			    && !pim_if_connected_to_source(nexthop->interface,
-							   src->u.prefix4)) {
+			if (curr_route_valid &&
+			    !pim_if_connected_to_source(nexthop->interface,
+							src)) {
 				nbr = pim_neighbor_find(
 					nexthop->interface,
-					nexthop->mrib_nexthop_addr.u.prefix4);
+					nexthop->mrib_nexthop_addr);
 				if (!nbr
 				    && !if_is_loopback(nexthop->interface)) {
 					if (PIM_DEBUG_PIM_NHT)
@@ -569,23 +548,13 @@ static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 						pnc->distance;
 					nexthop->mrib_route_metric =
 						pnc->metric;
-					if (PIM_DEBUG_PIM_NHT) {
-						char src_str[INET_ADDRSTRLEN];
-						pim_inet4_dump("<addr?>",
-							       src->u.prefix4,
-							       src_str,
-							       sizeof(src_str));
-						char grp_str[INET_ADDRSTRLEN];
-						pim_inet4_dump("<addr?>",
-							       grp->u.prefix4,
-							       grp_str,
-							       sizeof(grp_str));
+					if (PIM_DEBUG_PIM_NHT)
 						zlog_debug(
-							"%s: (%s,%s)(%s) current nexthop %s is valid, skipping new path selection",
-							__func__, src_str,
-							grp_str, pim->vrf->name,
+							"%s: (%pPA,%pPA)(%s) current nexthop %s is valid, skipping new path selection",
+							__func__, &src,
+							&grp_addr,
+							pim->vrf->name,
 							nexthop->interface->name);
-					}
 					return 1;
 				}
 			}
@@ -598,17 +567,21 @@ static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 	 */
 	for (nh_node = pnc->nexthop, i = 0; nh_node;
 	     nh_node = nh_node->next, i++) {
-		ifps[i] = if_lookup_by_index(nh_node->ifindex, pim->vrf_id);
+		ifps[i] =
+			if_lookup_by_index(nh_node->ifindex, pim->vrf->vrf_id);
 		if (ifps[i]) {
-			nbrs[i] = pim_neighbor_find(ifps[i],
-						    nh_node->gate.ipv4);
-			if (nbrs[i] || pim_if_connected_to_source(ifps[i],
-
-								  src->u.prefix4))
+#if PIM_IPV == 4
+			pim_addr nhaddr = nh_node->gate.ipv4;
+#else
+			pim_addr nhaddr = nh_node->gate.ipv6;
+#endif
+			nbrs[i] = pim_neighbor_find(ifps[i], nhaddr);
+			if (nbrs[i] || pim_if_connected_to_source(ifps[i], src))
 				num_nbrs++;
 		}
 	}
 	if (pim->ecmp_enable) {
+		struct prefix src_pfx;
 		uint32_t consider = pnc->nexthop_num;
 
 		if (neighbor_needed && num_nbrs < consider)
@@ -618,7 +591,8 @@ static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 			return 0;
 
 		// PIM ECMP flag is enable then choose ECMP path.
-		hash_val = pim_compute_ecmp_hash(src, grp);
+		pim_addr_to_prefix(&src_pfx, src);
+		hash_val = pim_compute_ecmp_hash(&src_pfx, grp);
 		mod_val = hash_val % consider;
 	}
 
@@ -627,38 +601,29 @@ static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 		first_ifindex = nh_node->ifindex;
 		ifp = ifps[nh_iter];
 		if (!ifp) {
-			if (PIM_DEBUG_PIM_NHT) {
-				char addr_str[INET_ADDRSTRLEN];
-				pim_inet4_dump("<addr?>", src->u.prefix4,
-					       addr_str, sizeof(addr_str));
+			if (PIM_DEBUG_PIM_NHT)
 				zlog_debug(
-					"%s %s: could not find interface for ifindex %d (address %s(%s))",
-					__FILE__, __func__, first_ifindex,
-					addr_str, pim->vrf->name);
-			}
+					"%s %s: could not find interface for ifindex %d (address %pPA(%s))",
+					__FILE__, __func__, first_ifindex, &src,
+					pim->vrf->name);
 			if (nh_iter == mod_val)
 				mod_val++; // Select nexthpath
 			nh_iter++;
 			continue;
 		}
 		if (!ifp->info) {
-			if (PIM_DEBUG_PIM_NHT) {
-				char addr_str[INET_ADDRSTRLEN];
-				pim_inet4_dump("<addr?>", src->u.prefix4,
-					       addr_str, sizeof(addr_str));
+			if (PIM_DEBUG_PIM_NHT)
 				zlog_debug(
-					"%s: multicast not enabled on input interface %s(%s) (ifindex=%d, RPF for source %s)",
+					"%s: multicast not enabled on input interface %s(%s) (ifindex=%d, RPF for source %pPA)",
 					__func__, ifp->name, pim->vrf->name,
-					first_ifindex, addr_str);
-			}
+					first_ifindex, &src);
 			if (nh_iter == mod_val)
 				mod_val++; // Select nexthpath
 			nh_iter++;
 			continue;
 		}
 
-		if (neighbor_needed
-		    && !pim_if_connected_to_source(ifp, src->u.prefix4)) {
+		if (neighbor_needed && !pim_if_connected_to_source(ifp, src)) {
 			nbr = nbrs[nh_iter];
 			if (!nbr && !if_is_loopback(ifp)) {
 				if (PIM_DEBUG_PIM_NHT)
@@ -675,34 +640,23 @@ static int pim_ecmp_nexthop_search(struct pim_instance *pim,
 
 		if (nh_iter == mod_val) {
 			nexthop->interface = ifp;
-			nexthop->mrib_nexthop_addr.family = AF_INET;
-			nexthop->mrib_nexthop_addr.prefixlen = IPV4_MAX_BITLEN;
-			nexthop->mrib_nexthop_addr.u.prefix4 =
-				nh_node->gate.ipv4;
+#if PIM_IPV == 4
+			nexthop->mrib_nexthop_addr = nh_node->gate.ipv4;
+#else
+			nexthop->mrib_nexthop_addr = nh_node->gate.ipv6;
+#endif
 			nexthop->mrib_metric_preference = pnc->distance;
 			nexthop->mrib_route_metric = pnc->metric;
-			nexthop->last_lookup = src->u.prefix4;
+			nexthop->last_lookup = src;
 			nexthop->last_lookup_time = pim_time_monotonic_usec();
 			nexthop->nbr = nbr;
 			found = 1;
-			if (PIM_DEBUG_PIM_NHT) {
-				char buf[INET_ADDRSTRLEN];
-				char buf2[INET_ADDRSTRLEN];
-				char buf3[INET_ADDRSTRLEN];
-				pim_inet4_dump("<src?>", src->u.prefix4, buf2,
-					       sizeof(buf2));
-				pim_inet4_dump("<grp?>", grp->u.prefix4, buf3,
-					       sizeof(buf3));
-				pim_inet4_dump(
-					"<rpf?>",
-					nexthop->mrib_nexthop_addr.u.prefix4,
-					buf, sizeof(buf));
+			if (PIM_DEBUG_PIM_NHT)
 				zlog_debug(
-					"%s: (%s,%s)(%s) selected nhop interface %s addr %s mod_val %u iter %d ecmp %d",
-					__func__, buf2, buf3, pim->vrf->name,
-					ifp->name, buf, mod_val, nh_iter,
-					pim->ecmp_enable);
-			}
+					"%s: (%pPA,%pPA)(%s) selected nhop interface %s addr %pPAs mod_val %u iter %d ecmp %d",
+					__func__, &src, &grp_addr,
+					pim->vrf->name, ifp->name, &nh_addr,
+					mod_val, nh_iter, pim->ecmp_enable);
 		}
 		nh_iter++;
 	}
@@ -723,34 +677,30 @@ int pim_parse_nexthop_update(ZAPI_CALLBACK_ARGS)
 	int i;
 	struct pim_rpf rpf;
 	struct pim_nexthop_cache *pnc = NULL;
-	struct pim_neighbor *nbr = NULL;
 	struct interface *ifp = NULL;
-	struct interface *ifp1 = NULL;
 	struct vrf *vrf = vrf_lookup_by_id(vrf_id);
 	struct pim_instance *pim;
 	struct zapi_route nhr;
+	struct prefix match;
 
 	if (!vrf)
 		return 0;
 	pim = vrf->info;
 
-	if (!zapi_nexthop_update_decode(zclient->ibuf, &nhr)) {
+	if (!zapi_nexthop_update_decode(zclient->ibuf, &match, &nhr)) {
 		zlog_err("%s: Decode of nexthop update from zebra failed",
 			 __func__);
 		return 0;
 	}
 
 	if (cmd == ZEBRA_NEXTHOP_UPDATE) {
-		prefix_copy(&rpf.rpf_addr, &nhr.prefix);
+		rpf.rpf_addr = pim_addr_from_prefix(&match);
 		pnc = pim_nexthop_cache_find(pim, &rpf);
 		if (!pnc) {
-			if (PIM_DEBUG_PIM_NHT) {
-				char buf[PREFIX2STR_BUFFER];
-				prefix2str(&rpf.rpf_addr, buf, sizeof(buf));
+			if (PIM_DEBUG_PIM_NHT)
 				zlog_debug(
-					"%s: Skipping NHT update, addr %s is not in local cached DB.",
-					__func__, buf);
-			}
+					"%s: Skipping NHT update, addr %pPA is not in local cached DB.",
+					__func__, &rpf.rpf_addr);
 			return 0;
 		}
 	} else {
@@ -768,43 +718,63 @@ int pim_parse_nexthop_update(ZAPI_CALLBACK_ARGS)
 		for (i = 0; i < nhr.nexthop_num; i++) {
 			nexthop = nexthop_from_zapi_nexthop(&nhr.nexthops[i]);
 			switch (nexthop->type) {
-			case NEXTHOP_TYPE_IPV4:
-			case NEXTHOP_TYPE_IPV4_IFINDEX:
-			case NEXTHOP_TYPE_IPV6:
-			case NEXTHOP_TYPE_BLACKHOLE:
-				break;
 			case NEXTHOP_TYPE_IFINDEX:
 				/*
 				 * Connected route (i.e. no nexthop), use
 				 * RPF address from nexthop cache (i.e.
 				 * destination) as PIM nexthop.
 				 */
+#if PIM_IPV == 4
 				nexthop->type = NEXTHOP_TYPE_IPV4_IFINDEX;
-				nexthop->gate.ipv4 =
-					pnc->rpf.rpf_addr.u.prefix4;
+				nexthop->gate.ipv4 = pnc->rpf.rpf_addr;
+#else
+				nexthop->type = NEXTHOP_TYPE_IPV6_IFINDEX;
+				nexthop->gate.ipv6 = pnc->rpf.rpf_addr;
+#endif
 				break;
-			case NEXTHOP_TYPE_IPV6_IFINDEX:
+#if PIM_IPV == 4
+			/* RFC5549 IPv4-over-IPv6 nexthop handling:
+			 * if we get an IPv6 nexthop in IPv4 PIM, hunt down a
+			 * PIM neighbor and use that instead.
+			 */
+			case NEXTHOP_TYPE_IPV6_IFINDEX: {
+				struct interface *ifp1 = NULL;
+				struct pim_neighbor *nbr = NULL;
+
 				ifp1 = if_lookup_by_index(nexthop->ifindex,
-							  pim->vrf_id);
+							  pim->vrf->vrf_id);
 
 				if (!ifp1)
 					nbr = NULL;
 				else
+					/* FIXME: should really use nbr's
+					 * secondary address list here
+					 */
 					nbr = pim_neighbor_find_if(ifp1);
+
 				/* Overwrite with Nbr address as NH addr */
 				if (nbr)
 					nexthop->gate.ipv4 = nbr->source_addr;
-				else {
+				else
 					// Mark nexthop address to 0 until PIM
 					// Nbr is resolved.
-					nexthop->gate.ipv4.s_addr =
-						PIM_NET_INADDR_ANY;
-				}
+					nexthop->gate.ipv4 = PIMADDR_ANY;
 
 				break;
 			}
+#else
+			case NEXTHOP_TYPE_IPV6_IFINDEX:
+#endif
+			case NEXTHOP_TYPE_IPV6:
+			case NEXTHOP_TYPE_IPV4:
+			case NEXTHOP_TYPE_IPV4_IFINDEX:
+			case NEXTHOP_TYPE_BLACKHOLE:
+				/* nothing to do for the other nexthop types */
+				break;
+			}
 
-			ifp = if_lookup_by_index(nexthop->ifindex, pim->vrf_id);
+			ifp = if_lookup_by_index(nexthop->ifindex,
+						 pim->vrf->vrf_id);
 			if (!ifp) {
 				if (PIM_DEBUG_PIM_NHT) {
 					char buf[NEXTHOP_STRLEN];
@@ -820,15 +790,16 @@ int pim_parse_nexthop_update(ZAPI_CALLBACK_ARGS)
 			}
 
 			if (PIM_DEBUG_PIM_NHT) {
-				char p_str[PREFIX2STR_BUFFER];
-
-				prefix2str(&nhr.prefix, p_str, sizeof(p_str));
+#if PIM_IPV == 4
+				pim_addr nhaddr = nexthop->gate.ipv4;
+#else
+				pim_addr nhaddr = nexthop->gate.ipv6;
+#endif
 				zlog_debug(
-					"%s: NHT addr %s(%s) %d-nhop via %s(%s) type %d distance:%u metric:%u ",
-					__func__, p_str, pim->vrf->name, i + 1,
-					inet_ntoa(nexthop->gate.ipv4),
-					ifp->name, nexthop->type, nhr.distance,
-					nhr.metric);
+					"%s: NHT addr %pFX(%s) %d-nhop via %pPA(%s) type %d distance:%u metric:%u ",
+					__func__, &match, pim->vrf->name, i + 1,
+					&nhaddr, ifp->name, nexthop->type,
+					nhr.distance, nhr.metric);
 			}
 
 			if (!ifp->info) {
@@ -879,15 +850,12 @@ int pim_parse_nexthop_update(ZAPI_CALLBACK_ARGS)
 	}
 	SET_FLAG(pnc->flags, PIM_NEXTHOP_ANSWER_RECEIVED);
 
-	if (PIM_DEBUG_PIM_NHT) {
-		char buf[PREFIX2STR_BUFFER];
-		prefix2str(&nhr.prefix, buf, sizeof(buf));
+	if (PIM_DEBUG_PIM_NHT)
 		zlog_debug(
-			"%s: NHT Update for %s(%s) num_nh %d num_pim_nh %d vrf:%u up %ld rp %d",
-			__func__, buf, pim->vrf->name, nhr.nexthop_num,
+			"%s: NHT Update for %pFX(%s) num_nh %d num_pim_nh %d vrf:%u up %ld rp %d",
+			__func__, &match, pim->vrf->name, nhr.nexthop_num,
 			pnc->nexthop_num, vrf_id, pnc->upstream_hash->count,
 			listcount(pnc->rp_list));
-	}
 
 	pim_rpf_set_refresh_time(pim);
 
@@ -900,34 +868,27 @@ int pim_parse_nexthop_update(ZAPI_CALLBACK_ARGS)
 }
 
 int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
-			    struct pim_nexthop *nexthop, struct prefix *src,
+			    struct pim_nexthop *nexthop, pim_addr src,
 			    struct prefix *grp, int neighbor_needed)
 {
 	struct pim_nexthop_cache *pnc;
-	struct pim_zlookup_nexthop nexthop_tab[MULTIPATH_NUM];
-	struct pim_neighbor *nbrs[MULTIPATH_NUM], *nbr = NULL;
+	struct pim_zlookup_nexthop nexthop_tab[router->multipath];
+	struct pim_neighbor *nbrs[router->multipath], *nbr = NULL;
 	struct pim_rpf rpf;
 	int num_ifindex;
-	struct interface *ifps[MULTIPATH_NUM], *ifp;
+	struct interface *ifps[router->multipath], *ifp;
 	int first_ifindex;
 	int found = 0;
 	uint8_t i = 0;
 	uint32_t hash_val = 0, mod_val = 0;
 	uint32_t num_nbrs = 0;
-	char addr_str[PREFIX_STRLEN];
 
-	if (PIM_DEBUG_PIM_NHT) {
-		pim_inet4_dump("<addr?>", src->u.prefix4, addr_str,
-			       sizeof(addr_str));
-		zlog_debug("%s: Looking up: %s(%s), last lookup time: %lld",
-			   __func__, addr_str, pim->vrf->name,
+	if (PIM_DEBUG_PIM_NHT_DETAIL)
+		zlog_debug("%s: Looking up: %pPA(%s), last lookup time: %lld",
+			   __func__, &src, pim->vrf->name,
 			   nexthop->last_lookup_time);
-	}
 
-	memset(&rpf, 0, sizeof(struct pim_rpf));
-	rpf.rpf_addr.family = AF_INET;
-	rpf.rpf_addr.prefixlen = IPV4_MAX_BITLEN;
-	rpf.rpf_addr.u.prefix4 = src->u.prefix4;
+	rpf.rpf_addr = src;
 
 	pnc = pim_nexthop_cache_find(pim, &rpf);
 	if (pnc) {
@@ -937,15 +898,15 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 	}
 
 	memset(nexthop_tab, 0,
-	       sizeof(struct pim_zlookup_nexthop) * MULTIPATH_NUM);
+	       sizeof(struct pim_zlookup_nexthop) * router->multipath);
 	num_ifindex =
-		zclient_lookup_nexthop(pim, nexthop_tab, MULTIPATH_NUM,
-				       src->u.prefix4, PIM_NEXTHOP_LOOKUP_MAX);
+		zclient_lookup_nexthop(pim, nexthop_tab, router->multipath, src,
+				       PIM_NEXTHOP_LOOKUP_MAX);
 	if (num_ifindex < 1) {
 		if (PIM_DEBUG_PIM_NHT)
 			zlog_warn(
-				"%s: could not find nexthop ifindex for address %s(%s)",
-				__func__, addr_str, pim->vrf->name);
+				"%s: could not find nexthop ifindex for address %pPA(%s)",
+				__func__, &src, pim->vrf->name);
 		return 0;
 	}
 
@@ -958,19 +919,18 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 	 */
 	for (i = 0; i < num_ifindex; i++) {
 		ifps[i] = if_lookup_by_index(nexthop_tab[i].ifindex,
-					     pim->vrf_id);
+					     pim->vrf->vrf_id);
 		if (ifps[i]) {
 			nbrs[i] = pim_neighbor_find(
-				ifps[i], nexthop_tab[i].nexthop_addr.u.prefix4);
-			if (nbrs[i]
-			    || pim_if_connected_to_source(ifps[i],
-							  src->u.prefix4))
+				ifps[i], nexthop_tab[i].nexthop_addr);
+			if (nbrs[i] || pim_if_connected_to_source(ifps[i], src))
 				num_nbrs++;
 		}
 	}
 
 	// If PIM ECMP enable then choose ECMP path.
 	if (pim->ecmp_enable) {
+		struct prefix src_pfx;
 		uint32_t consider = num_ifindex;
 
 		if (neighbor_needed && num_nbrs < consider)
@@ -979,7 +939,8 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 		if (consider == 0)
 			return 0;
 
-		hash_val = pim_compute_ecmp_hash(src, grp);
+		pim_addr_to_prefix(&src_pfx, src);
+		hash_val = pim_compute_ecmp_hash(&src_pfx, grp);
 		mod_val = hash_val % consider;
 		if (PIM_DEBUG_PIM_NHT_DETAIL)
 			zlog_debug("%s: hash_val %u mod_val %u", __func__,
@@ -994,9 +955,9 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 		if (!ifp) {
 			if (PIM_DEBUG_PIM_NHT)
 				zlog_debug(
-					"%s %s: could not find interface for ifindex %d (address %s(%s))",
-					__FILE__, __func__, first_ifindex,
-					addr_str, pim->vrf->name);
+					"%s %s: could not find interface for ifindex %d (address %pPA(%s))",
+					__FILE__, __func__, first_ifindex, &src,
+					pim->vrf->name);
 			if (i == mod_val)
 				mod_val++;
 			i++;
@@ -1006,16 +967,15 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 		if (!ifp->info) {
 			if (PIM_DEBUG_PIM_NHT)
 				zlog_debug(
-					"%s: multicast not enabled on input interface %s(%s) (ifindex=%d, RPF for source %s)",
+					"%s: multicast not enabled on input interface %s(%s) (ifindex=%d, RPF for source %pPA)",
 					__func__, ifp->name, pim->vrf->name,
-					first_ifindex, addr_str);
+					first_ifindex, &src);
 			if (i == mod_val)
 				mod_val++;
 			i++;
 			continue;
 		}
-		if (neighbor_needed
-		    && !pim_if_connected_to_source(ifp, src->u.prefix4)) {
+		if (neighbor_needed && !pim_if_connected_to_source(ifp, src)) {
 			nbr = nbrs[i];
 			if (PIM_DEBUG_PIM_NHT_DETAIL)
 				zlog_debug("ifp name: %s(%s), pim nbr: %p",
@@ -1023,30 +983,26 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 			if (!nbr && !if_is_loopback(ifp)) {
 				if (i == mod_val)
 					mod_val++;
-				i++;
 				if (PIM_DEBUG_PIM_NHT)
 					zlog_debug(
-						"%s: NBR not found on input interface %s(%s) (RPF for source %s)",
-						__func__, ifp->name,
-						pim->vrf->name, addr_str);
+						"%s: NBR (%pPA) not found on input interface %s(%s) (RPF for source %pPA)",
+						__func__,
+						&nexthop_tab[i].nexthop_addr,
+						ifp->name, pim->vrf->name,
+						&src);
+				i++;
 				continue;
 			}
 		}
 
 		if (i == mod_val) {
-			if (PIM_DEBUG_PIM_NHT) {
-				char nexthop_str[PREFIX_STRLEN];
-
-				pim_addr_dump("<nexthop?>",
-					      &nexthop_tab[i].nexthop_addr,
-					      nexthop_str, sizeof(nexthop_str));
+			if (PIM_DEBUG_PIM_NHT)
 				zlog_debug(
-					"%s: found nhop %s for addr %s interface %s(%s) metric %d dist %d",
-					__func__, nexthop_str, addr_str,
-					ifp->name, pim->vrf->name,
+					"%s: found nhop %pPA for addr %pPA interface %s(%s) metric %d dist %d",
+					__func__, &nexthop_tab[i].nexthop_addr,
+					&src, ifp->name, pim->vrf->name,
 					nexthop_tab[i].route_metric,
 					nexthop_tab[i].protocol_distance);
-			}
 			/* update nexthop data */
 			nexthop->interface = ifp;
 			nexthop->mrib_nexthop_addr =
@@ -1055,7 +1011,7 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 				nexthop_tab[i].protocol_distance;
 			nexthop->mrib_route_metric =
 				nexthop_tab[i].route_metric;
-			nexthop->last_lookup = src->u.prefix4;
+			nexthop->last_lookup = src;
 			nexthop->last_lookup_time = pim_time_monotonic_usec();
 			nexthop->nbr = nbr;
 			found = 1;
@@ -1069,41 +1025,37 @@ int pim_ecmp_nexthop_lookup(struct pim_instance *pim,
 		return 0;
 }
 
-int pim_ecmp_fib_lookup_if_vif_index(struct pim_instance *pim,
-				     struct prefix *src, struct prefix *grp)
+int pim_ecmp_fib_lookup_if_vif_index(struct pim_instance *pim, pim_addr src,
+				     struct prefix *grp)
 {
 	struct pim_nexthop nhop;
 	int vif_index;
 	ifindex_t ifindex;
-	char addr_str[PREFIX_STRLEN];
-
-	if (PIM_DEBUG_PIM_NHT)
-		pim_inet4_dump("<addr?>", src->u.prefix4, addr_str,
-			       sizeof(addr_str));
 
 	memset(&nhop, 0, sizeof(nhop));
 	if (!pim_ecmp_nexthop_lookup(pim, &nhop, src, grp, 1)) {
 		if (PIM_DEBUG_PIM_NHT)
 			zlog_debug(
-				"%s: could not find nexthop ifindex for address %s(%s)",
-				__func__, addr_str, pim->vrf->name);
+				"%s: could not find nexthop ifindex for address %pPA(%s)",
+				__func__, &src, pim->vrf->name);
 		return -1;
 	}
 
 	ifindex = nhop.interface->ifindex;
 	if (PIM_DEBUG_PIM_NHT)
 		zlog_debug(
-			"%s: found nexthop ifindex=%d (interface %s(%s)) for address %s",
-			__func__, ifindex, ifindex2ifname(ifindex, pim->vrf_id),
-			pim->vrf->name, addr_str);
+			"%s: found nexthop ifindex=%d (interface %s(%s)) for address %pPA",
+			__func__, ifindex,
+			ifindex2ifname(ifindex, pim->vrf->vrf_id),
+			pim->vrf->name, &src);
 
 	vif_index = pim_if_find_vifindex_by_ifindex(pim, ifindex);
 
 	if (vif_index < 0) {
 		if (PIM_DEBUG_PIM_NHT) {
 			zlog_debug(
-				"%s: low vif_index=%d(%s) < 1 nexthop for address %s",
-				__func__, vif_index, pim->vrf->name, addr_str);
+				"%s: low vif_index=%d(%s) < 1 nexthop for address %pPA",
+				__func__, vif_index, pim->vrf->name, &src);
 		}
 		return -2;
 	}
