@@ -184,11 +184,10 @@ void pim_vxlan_update_sg_reg_state(struct pim_instance *pim,
 		pim_vxlan_del_work(vxlan_sg);
 }
 
-static int pim_vxlan_work_timer_cb(struct thread *t)
+static void pim_vxlan_work_timer_cb(struct thread *t)
 {
 	pim_vxlan_do_reg_work();
 	pim_vxlan_work_timer_setup(true /* start */);
-	return 0;
 }
 
 /* global 1second timer used for periodic processing */
@@ -304,7 +303,6 @@ static void pim_vxlan_orig_mr_up_add(struct pim_vxlan_sg *vxlan_sg)
 	struct pim_upstream *up;
 	struct pim_interface *term_ifp;
 	int flags = 0;
-	struct prefix nht_p;
 	struct pim_instance *pim = vxlan_sg->pim;
 
 	if (vxlan_sg->up) {
@@ -354,11 +352,8 @@ static void pim_vxlan_orig_mr_up_add(struct pim_vxlan_sg *vxlan_sg)
 		 * iif
 		 */
 		if (!PIM_UPSTREAM_FLAG_TEST_STATIC_IIF(up->flags)) {
-			nht_p.family = AF_INET;
-			nht_p.prefixlen = IPV4_MAX_BITLEN;
-			nht_p.u.prefix4 = up->upstream_addr;
 			pim_delete_tracked_nexthop(vxlan_sg->pim,
-				&nht_p, up, NULL, false);
+						   up->upstream_addr, up, NULL);
 		}
 		/* We are acting FHR; clear out use_rpt setting if any */
 		pim_upstream_update_use_rpt(up, false /*update_mroute*/);
@@ -390,9 +385,25 @@ static void pim_vxlan_orig_mr_up_add(struct pim_vxlan_sg *vxlan_sg)
 	pim_upstream_keep_alive_timer_start(up, vxlan_sg->pim->keep_alive_time);
 
 	/* register the source with the RP */
-	if (up->reg_state == PIM_REG_NOINFO) {
+	switch (up->reg_state) {
+
+	case PIM_REG_NOINFO:
 		pim_register_join(up);
 		pim_null_register_send(up);
+		break;
+
+	case PIM_REG_JOIN:
+		/* if the pim upstream entry is already in reg-join state
+		 * send null_register right away and add to the register
+		 * worklist
+		 */
+		pim_null_register_send(up);
+		pim_vxlan_update_sg_reg_state(pim, up, true);
+		break;
+
+	case PIM_REG_JOIN_PENDING:
+	case PIM_REG_PRUNE:
+		break;
 	}
 
 	/* update the inherited OIL */
@@ -497,10 +508,10 @@ static void pim_vxlan_orig_mr_del(struct pim_vxlan_sg *vxlan_sg)
 	pim_vxlan_orig_mr_up_del(vxlan_sg);
 }
 
-static void pim_vxlan_orig_mr_iif_update(struct hash_bucket *backet, void *arg)
+static void pim_vxlan_orig_mr_iif_update(struct hash_bucket *bucket, void *arg)
 {
 	struct interface *ifp;
-	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)backet->data;
+	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)bucket->data;
 	struct interface *old_iif = vxlan_sg->iif;
 
 	if (!pim_vxlan_is_orig_mroute(vxlan_sg))
@@ -704,8 +715,7 @@ static unsigned int pim_vxlan_sg_hash_key_make(const void *p)
 {
 	const struct pim_vxlan_sg *vxlan_sg = p;
 
-	return (jhash_2words(vxlan_sg->sg.src.s_addr,
-				vxlan_sg->sg.grp.s_addr, 0));
+	return pim_sgaddr_hash(vxlan_sg->sg, 0);
 }
 
 static bool pim_vxlan_sg_hash_eq(const void *p1, const void *p2)
@@ -713,12 +723,11 @@ static bool pim_vxlan_sg_hash_eq(const void *p1, const void *p2)
 	const struct pim_vxlan_sg *sg1 = p1;
 	const struct pim_vxlan_sg *sg2 = p2;
 
-	return ((sg1->sg.src.s_addr == sg2->sg.src.s_addr)
-			&& (sg1->sg.grp.s_addr == sg2->sg.grp.s_addr));
+	return !pim_sgaddr_cmp(sg1->sg, sg2->sg);
 }
 
 static struct pim_vxlan_sg *pim_vxlan_sg_new(struct pim_instance *pim,
-		struct prefix_sg *sg)
+					     pim_sgaddr *sg)
 {
 	struct pim_vxlan_sg *vxlan_sg;
 
@@ -726,7 +735,7 @@ static struct pim_vxlan_sg *pim_vxlan_sg_new(struct pim_instance *pim,
 
 	vxlan_sg->pim = pim;
 	vxlan_sg->sg = *sg;
-	pim_str_sg_set(sg, vxlan_sg->sg_str);
+	snprintfrr(vxlan_sg->sg_str, sizeof(vxlan_sg->sg_str), "%pSG", sg);
 
 	if (PIM_DEBUG_VXLAN)
 		zlog_debug("vxlan SG %s alloc", vxlan_sg->sg_str);
@@ -744,8 +753,7 @@ static struct pim_vxlan_sg *pim_vxlan_sg_new(struct pim_instance *pim,
 	return vxlan_sg;
 }
 
-struct pim_vxlan_sg *pim_vxlan_sg_find(struct pim_instance *pim,
-		struct prefix_sg *sg)
+struct pim_vxlan_sg *pim_vxlan_sg_find(struct pim_instance *pim, pim_sgaddr *sg)
 {
 	struct pim_vxlan_sg lookup;
 
@@ -753,8 +761,7 @@ struct pim_vxlan_sg *pim_vxlan_sg_find(struct pim_instance *pim,
 	return hash_lookup(pim->vxlan.sg_hash, &lookup);
 }
 
-struct pim_vxlan_sg *pim_vxlan_sg_add(struct pim_instance *pim,
-		struct prefix_sg *sg)
+struct pim_vxlan_sg *pim_vxlan_sg_add(struct pim_instance *pim, pim_sgaddr *sg)
 {
 	struct pim_vxlan_sg *vxlan_sg;
 
@@ -789,7 +796,7 @@ static void pim_vxlan_sg_del_item(struct pim_vxlan_sg *vxlan_sg)
 	XFREE(MTYPE_PIM_VXLAN_SG, vxlan_sg);
 }
 
-void pim_vxlan_sg_del(struct pim_instance *pim, struct prefix_sg *sg)
+void pim_vxlan_sg_del(struct pim_instance *pim, pim_sgaddr *sg)
 {
 	struct pim_vxlan_sg *vxlan_sg;
 
@@ -812,11 +819,11 @@ bool pim_vxlan_do_mlag_reg(void)
  * to the MLAG peer which may mroute it over the underlay if there are any
  * interested receivers.
  */
-static void pim_vxlan_sg_peerlink_oif_update(struct hash_bucket *backet,
+static void pim_vxlan_sg_peerlink_oif_update(struct hash_bucket *bucket,
 					     void *arg)
 {
 	struct interface *new_oif = (struct interface *)arg;
-	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)backet->data;
+	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)bucket->data;
 
 	if (!pim_vxlan_is_orig_mroute(vxlan_sg))
 		return;
@@ -871,6 +878,12 @@ void pim_vxlan_mlag_update(bool enable, bool peer_state, uint32_t role,
 	 */
 	pim = pim_get_pim_instance(VRF_DEFAULT);
 
+	if (!pim) {
+		if (PIM_DEBUG_VXLAN)
+			zlog_debug("%s: Unable to find pim instance", __func__);
+		return;
+	}
+
 	if (enable)
 		vxlan_mlag.flags |= PIM_VXLAN_MLAGF_ENABLED;
 	else
@@ -950,10 +963,10 @@ static void pim_vxlan_up_cost_update(struct pim_instance *pim,
 	}
 }
 
-static void pim_vxlan_term_mr_cost_update(struct hash_bucket *backet, void *arg)
+static void pim_vxlan_term_mr_cost_update(struct hash_bucket *bucket, void *arg)
 {
 	struct interface *old_peerlink_rif = (struct interface *)arg;
-	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)backet->data;
+	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)bucket->data;
 	struct pim_upstream *up;
 	struct listnode *listnode;
 	struct pim_upstream *child;
@@ -975,11 +988,11 @@ static void pim_vxlan_term_mr_cost_update(struct hash_bucket *backet, void *arg)
 				old_peerlink_rif);
 }
 
-static void pim_vxlan_sg_peerlink_rif_update(struct hash_bucket *backet,
+static void pim_vxlan_sg_peerlink_rif_update(struct hash_bucket *bucket,
 					     void *arg)
 {
-	pim_vxlan_orig_mr_iif_update(backet, NULL);
-	pim_vxlan_term_mr_cost_update(backet, arg);
+	pim_vxlan_orig_mr_iif_update(bucket, NULL);
+	pim_vxlan_term_mr_cost_update(bucket, arg);
 }
 
 static void pim_vxlan_set_peerlink_rif(struct pim_instance *pim,
@@ -1032,10 +1045,10 @@ static void pim_vxlan_set_peerlink_rif(struct pim_instance *pim,
 	}
 }
 
-static void pim_vxlan_term_mr_oif_update(struct hash_bucket *backet, void *arg)
+static void pim_vxlan_term_mr_oif_update(struct hash_bucket *bucket, void *arg)
 {
 	struct interface *ifp = (struct interface *)arg;
-	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)backet->data;
+	struct pim_vxlan_sg *vxlan_sg = (struct pim_vxlan_sg *)bucket->data;
 
 	if (pim_vxlan_is_orig_mroute(vxlan_sg))
 		return;
@@ -1076,10 +1089,10 @@ void pim_vxlan_add_vif(struct interface *ifp)
 	struct pim_interface *pim_ifp = ifp->info;
 	struct pim_instance *pim = pim_ifp->pim;
 
-	if (pim->vrf_id != VRF_DEFAULT)
+	if (pim->vrf->vrf_id != VRF_DEFAULT)
 		return;
 
-	if (if_is_loopback_or_vrf(ifp))
+	if (if_is_loopback(ifp))
 		pim_vxlan_set_default_iif(pim, ifp);
 
 	if (vxlan_mlag.flags & PIM_VXLAN_MLAGF_ENABLED &&
@@ -1095,7 +1108,7 @@ void pim_vxlan_del_vif(struct interface *ifp)
 	struct pim_interface *pim_ifp = ifp->info;
 	struct pim_instance *pim = pim_ifp->pim;
 
-	if (pim->vrf_id != VRF_DEFAULT)
+	if (pim->vrf->vrf_id != VRF_DEFAULT)
 		return;
 
 	if (pim->vxlan.default_iif == ifp)
@@ -1128,7 +1141,7 @@ void pim_vxlan_add_term_dev(struct pim_instance *pim,
 	/* enable pim on the term ifp */
 	pim_ifp = (struct pim_interface *)ifp->info;
 	if (pim_ifp) {
-		PIM_IF_DO_PIM(pim_ifp->options);
+		pim_ifp->pim_enable = true;
 		/* ifp is already oper up; activate it as a term dev */
 		if (pim_ifp->mroute_vif_index >= 0)
 			pim_vxlan_term_oif_update(pim, ifp);
@@ -1156,8 +1169,8 @@ void pim_vxlan_del_term_dev(struct pim_instance *pim)
 
 	pim_ifp = (struct pim_interface *)ifp->info;
 	if (pim_ifp) {
-		PIM_IF_DONT_PIM(pim_ifp->options);
-		if (!PIM_IF_TEST_IGMP(pim_ifp->options))
+		pim_ifp->pim_enable = false;
+		if (!pim_ifp->gm_enable)
 			pim_if_delete(ifp);
 	}
 }
