@@ -26,8 +26,11 @@
 
 #define PIM_NEXTHOP_LOOKUP_MAX (3) /* max. recursive route lookup */
 
+struct channel_oil;
+
 struct pim_zlookup_nexthop {
-	struct prefix nexthop_addr;
+	vrf_id_t vrf_id;
+	pim_addr nexthop_addr;
 	ifindex_t ifindex;
 	uint32_t route_metric;
 	uint8_t protocol_distance;
@@ -38,7 +41,7 @@ void zclient_lookup_free(void);
 
 int zclient_lookup_nexthop(struct pim_instance *pim,
 			   struct pim_zlookup_nexthop nexthop_tab[],
-			   const int tab_size, struct in_addr addr,
+			   const int tab_size, pim_addr addr,
 			   int max_lookup);
 
 void pim_zlookup_show_ip_multicast(struct vty *vty);
