@@ -30,6 +30,7 @@
 #include "zclient.h"
 #include "nexthop.h"
 #include "nexthop_group.h"
+#include "link_state.h"
 
 #include "sharp_globals.h"
 #include "sharp_nht.h"
@@ -211,66 +212,231 @@ int sharp_install_lsps_helper(bool install_p, bool update_p,
 		cmd = ZEBRA_MPLS_LABELS_DELETE;
 	}
 
-	ret = zebra_send_mpls_labels(zclient, cmd, &zl);
+	if (zebra_send_mpls_labels(zclient, cmd, &zl) == ZCLIENT_SEND_FAILURE)
+		return -1;
 
-	return ret;
+	return 0;
 }
 
-void sharp_install_routes_helper(struct prefix *p, vrf_id_t vrf_id,
-				 uint8_t instance,
-				 const struct nexthop_group *nhg,
-				 const struct nexthop_group *backup_nhg,
-				 uint32_t routes)
+enum where_to_restart {
+	SHARP_INSTALL_ROUTES_RESTART,
+	SHARP_DELETE_ROUTES_RESTART,
+};
+
+struct buffer_delay {
+	struct prefix p;
+	uint32_t count;
+	uint32_t routes;
+	vrf_id_t vrf_id;
+	uint8_t instance;
+	uint32_t nhgid;
+	uint32_t flags;
+	const struct nexthop_group *nhg;
+	const struct nexthop_group *backup_nhg;
+	enum where_to_restart restart;
+	char *opaque;
+} wb;
+
+/*
+ * route_add - Encodes a route to zebra
+ *
+ * This function returns true when the route was buffered
+ * by the underlying stream system
+ */
+static bool route_add(const struct prefix *p, vrf_id_t vrf_id, uint8_t instance,
+		      uint32_t nhgid, const struct nexthop_group *nhg,
+		      const struct nexthop_group *backup_nhg, uint32_t flags,
+		      char *opaque)
+{
+	struct zapi_route api;
+	struct zapi_nexthop *api_nh;
+	struct nexthop *nh;
+	int i = 0;
+
+	memset(&api, 0, sizeof(api));
+	api.vrf_id = vrf_id;
+	api.type = ZEBRA_ROUTE_SHARP;
+	api.instance = instance;
+	api.safi = SAFI_UNICAST;
+	memcpy(&api.prefix, p, sizeof(*p));
+
+	api.flags = flags;
+	SET_FLAG(api.flags, ZEBRA_FLAG_ALLOW_RECURSION);
+	SET_FLAG(api.message, ZAPI_MESSAGE_NEXTHOP);
+
+	/* Only send via ID if nhgroup has been successfully installed */
+	if (nhgid && sharp_nhgroup_id_is_installed(nhgid)) {
+		SET_FLAG(api.message, ZAPI_MESSAGE_NHG);
+		api.nhgid = nhgid;
+	} else {
+		for (ALL_NEXTHOPS_PTR(nhg, nh)) {
+			api_nh = &api.nexthops[i];
+
+			zapi_nexthop_from_nexthop(api_nh, nh);
+
+			i++;
+		}
+		api.nexthop_num = i;
+	}
+
+	/* Include backup nexthops, if present */
+	if (backup_nhg && backup_nhg->nexthop) {
+		SET_FLAG(api.message, ZAPI_MESSAGE_BACKUP_NEXTHOPS);
+
+		i = 0;
+		for (ALL_NEXTHOPS_PTR(backup_nhg, nh)) {
+			api_nh = &api.backup_nexthops[i];
+
+			zapi_backup_nexthop_from_nexthop(api_nh, nh);
+
+			i++;
+		}
+
+		api.backup_nexthop_num = i;
+	}
+
+	if (strlen(opaque)) {
+		SET_FLAG(api.message, ZAPI_MESSAGE_OPAQUE);
+		api.opaque.length = strlen(opaque) + 1;
+		assert(api.opaque.length <= ZAPI_MESSAGE_OPAQUE_LENGTH);
+		memcpy(api.opaque.data, opaque, api.opaque.length);
+	}
+
+	if (zclient_route_send(ZEBRA_ROUTE_ADD, zclient, &api)
+	    == ZCLIENT_SEND_BUFFERED)
+		return true;
+	else
+		return false;
+}
+
+/*
+ * route_delete - Encodes a route for deletion to zebra
+ *
+ * This function returns true when the route sent was
+ * buffered by the underlying stream system.
+ */
+static bool route_delete(struct prefix *p, vrf_id_t vrf_id, uint8_t instance)
+{
+	struct zapi_route api;
+
+	memset(&api, 0, sizeof(api));
+	api.vrf_id = vrf_id;
+	api.type = ZEBRA_ROUTE_SHARP;
+	api.safi = SAFI_UNICAST;
+	api.instance = instance;
+	memcpy(&api.prefix, p, sizeof(*p));
+
+	if (zclient_route_send(ZEBRA_ROUTE_DELETE, zclient, &api)
+	    == ZCLIENT_SEND_BUFFERED)
+		return true;
+	else
+		return false;
+}
+
+static void sharp_install_routes_restart(struct prefix *p, uint32_t count,
+					 vrf_id_t vrf_id, uint8_t instance,
+					 uint32_t nhgid,
+					 const struct nexthop_group *nhg,
+					 const struct nexthop_group *backup_nhg,
+					 uint32_t routes, uint32_t flags,
+					 char *opaque)
 {
 	uint32_t temp, i;
 	bool v4 = false;
 
-	zlog_debug("Inserting %u routes", routes);
-
 	if (p->family == AF_INET) {
 		v4 = true;
 		temp = ntohl(p->u.prefix4.s_addr);
 	} else
 		temp = ntohl(p->u.val32[3]);
 
+	for (i = count; i < routes; i++) {
+		bool buffered = route_add(p, vrf_id, (uint8_t)instance, nhgid,
+					  nhg, backup_nhg, flags, opaque);
+		if (v4)
+			p->u.prefix4.s_addr = htonl(++temp);
+		else
+			p->u.val32[3] = htonl(++temp);
+
+		if (buffered) {
+			wb.p = *p;
+			wb.count = i+1;
+			wb.routes = routes;
+			wb.vrf_id = vrf_id;
+			wb.instance = instance;
+			wb.nhgid = nhgid;
+			wb.nhg = nhg;
+			wb.flags = flags;
+			wb.backup_nhg = backup_nhg;
+			wb.opaque = opaque;
+			wb.restart = SHARP_INSTALL_ROUTES_RESTART;
+
+			return;
+		}
+	}
+}
+
+void sharp_install_routes_helper(struct prefix *p, vrf_id_t vrf_id,
+				 uint8_t instance, uint32_t nhgid,
+				 const struct nexthop_group *nhg,
+				 const struct nexthop_group *backup_nhg,
+				 uint32_t routes, uint32_t flags, char *opaque)
+{
+	zlog_debug("Inserting %u routes", routes);
+
 	/* Only use backup route/nexthops if present */
 	if (backup_nhg && (backup_nhg->nexthop == NULL))
 		backup_nhg = NULL;
 
 	monotime(&sg.r.t_start);
-	for (i = 0; i < routes; i++) {
-		route_add(p, vrf_id, (uint8_t)instance, nhg, backup_nhg);
-		if (v4)
-			p->u.prefix4.s_addr = htonl(++temp);
-		else
-			p->u.val32[3] = htonl(++temp);
-	}
+	sharp_install_routes_restart(p, 0, vrf_id, instance, nhgid, nhg,
+				     backup_nhg, routes, flags, opaque);
 }
 
-void sharp_remove_routes_helper(struct prefix *p, vrf_id_t vrf_id,
-				uint8_t instance, uint32_t routes)
+static void sharp_remove_routes_restart(struct prefix *p, uint32_t count,
+					vrf_id_t vrf_id, uint8_t instance,
+					uint32_t routes)
 {
 	uint32_t temp, i;
 	bool v4 = false;
 
-	zlog_debug("Removing %u routes", routes);
-
 	if (p->family == AF_INET) {
 		v4 = true;
 		temp = ntohl(p->u.prefix4.s_addr);
 	} else
 		temp = ntohl(p->u.val32[3]);
 
-	monotime(&sg.r.t_start);
-	for (i = 0; i < routes; i++) {
-		route_delete(p, vrf_id, (uint8_t)instance);
+	for (i = count; i < routes; i++) {
+		bool buffered = route_delete(p, vrf_id, (uint8_t)instance);
+
 		if (v4)
 			p->u.prefix4.s_addr = htonl(++temp);
 		else
 			p->u.val32[3] = htonl(++temp);
+
+		if (buffered) {
+			wb.p = *p;
+			wb.count = i + 1;
+			wb.vrf_id = vrf_id;
+			wb.instance = instance;
+			wb.routes = routes;
+			wb.restart = SHARP_DELETE_ROUTES_RESTART;
+
+			return;
+		}
 	}
 }
 
+void sharp_remove_routes_helper(struct prefix *p, vrf_id_t vrf_id,
+				uint8_t instance, uint32_t routes)
+{
+	zlog_debug("Removing %u routes", routes);
+
+	monotime(&sg.r.t_start);
+
+	sharp_remove_routes_restart(p, 0, vrf_id, instance, routes);
+}
+
 static void handle_repeated(bool installed)
 {
 	struct prefix p = sg.r.orig_prefix;
@@ -288,9 +454,26 @@ static void handle_repeated(bool installed)
 	if (!installed) {
 		sg.r.installed_routes = 0;
 		sharp_install_routes_helper(&p, sg.r.vrf_id, sg.r.inst,
-					    &sg.r.nhop_group,
+					    sg.r.nhgid, &sg.r.nhop_group,
 					    &sg.r.backup_nhop_group,
-					    sg.r.total_routes);
+					    sg.r.total_routes, sg.r.flags,
+					    sg.r.opaque);
+	}
+}
+
+static void sharp_zclient_buffer_ready(void)
+{
+	switch (wb.restart) {
+	case SHARP_INSTALL_ROUTES_RESTART:
+		sharp_install_routes_restart(
+			&wb.p, wb.count, wb.vrf_id, wb.instance, wb.nhgid,
+			wb.nhg, wb.backup_nhg, wb.routes, wb.flags,
+			wb.opaque);
+		return;
+	case SHARP_DELETE_ROUTES_RESTART:
+		sharp_remove_routes_restart(&wb.p, wb.count, wb.vrf_id,
+					    wb.instance, wb.routes);
+		return;
 	}
 }
 
@@ -301,7 +484,8 @@ static int route_notify_owner(ZAPI_CALLBACK_ARGS)
 	enum zapi_route_notify_owner note;
 	uint32_t table;
 
-	if (!zapi_route_notify_decode(zclient->ibuf, &p, &table, &note))
+	if (!zapi_route_notify_decode(zclient->ibuf, &p, &table, &note,
+				      NULL, NULL))
 		return -1;
 
 	switch (note) {
@@ -357,66 +541,84 @@ void vrf_label_add(vrf_id_t vrf_id, afi_t afi, mpls_label_t label)
 	zclient_send_vrf_label(zclient, vrf_id, afi, label, ZEBRA_LSP_SHARP);
 }
 
-void route_add(const struct prefix *p, vrf_id_t vrf_id,
-	       uint8_t instance, const struct nexthop_group *nhg,
-	       const struct nexthop_group *backup_nhg)
+void nhg_add(uint32_t id, const struct nexthop_group *nhg,
+	     const struct nexthop_group *backup_nhg)
 {
-	struct zapi_route api;
+	struct zapi_nhg api_nhg = {};
 	struct zapi_nexthop *api_nh;
 	struct nexthop *nh;
-	int i = 0;
-
-	memset(&api, 0, sizeof(api));
-	api.vrf_id = vrf_id;
-	api.type = ZEBRA_ROUTE_SHARP;
-	api.instance = instance;
-	api.safi = SAFI_UNICAST;
-	memcpy(&api.prefix, p, sizeof(*p));
-
-	SET_FLAG(api.flags, ZEBRA_FLAG_ALLOW_RECURSION);
-	SET_FLAG(api.message, ZAPI_MESSAGE_NEXTHOP);
+	bool is_valid = true;
 
+	api_nhg.id = id;
 	for (ALL_NEXTHOPS_PTR(nhg, nh)) {
-		api_nh = &api.nexthops[i];
-
-		zapi_nexthop_from_nexthop(api_nh, nh);
-
-		i++;
-	}
-	api.nexthop_num = i;
-
-	/* Include backup nexthops, if present */
-	if (backup_nhg && backup_nhg->nexthop) {
-		SET_FLAG(api.message, ZAPI_MESSAGE_BACKUP_NEXTHOPS);
-
-		i = 0;
-		for (ALL_NEXTHOPS_PTR(backup_nhg, nh)) {
-			api_nh = &api.backup_nexthops[i];
-
-			zapi_backup_nexthop_from_nexthop(api_nh, nh);
-
-			i++;
+		if (api_nhg.nexthop_num >= MULTIPATH_NUM) {
+			zlog_warn(
+				"%s: number of nexthops greater than max multipath size, truncating",
+				__func__);
+			break;
 		}
 
-		api.backup_nexthop_num = i;
+		/* Unresolved nexthops will lead to failure - only send
+		 * nexthops that zebra will consider valid.
+		 */
+		if (nh->ifindex == 0)
+			continue;
+
+		api_nh = &api_nhg.nexthops[api_nhg.nexthop_num];
+
+		zapi_nexthop_from_nexthop(api_nh, nh);
+		api_nhg.nexthop_num++;
 	}
 
-	zclient_route_send(ZEBRA_ROUTE_ADD, zclient, &api);
+	if (api_nhg.nexthop_num == 0) {
+		zlog_debug("%s: nhg %u not sent: no valid nexthops",
+			   __func__, id);
+		is_valid = false;
+		goto done;
+	}
+
+	if (backup_nhg) {
+		for (ALL_NEXTHOPS_PTR(backup_nhg, nh)) {
+			if (api_nhg.backup_nexthop_num >= MULTIPATH_NUM) {
+				zlog_warn(
+					"%s: number of backup nexthops greater than max multipath size, truncating",
+					__func__);
+				break;
+			}
+
+			/* Unresolved nexthop: will be rejected by zebra.
+			 * That causes a problem, since the primary nexthops
+			 * rely on array indexing into the backup nexthops. If
+			 * that array isn't valid, the backup indexes won't be
+			 * valid.
+			 */
+			if (nh->ifindex == 0) {
+				zlog_debug("%s: nhg %u: invalid backup nexthop",
+					   __func__, id);
+				is_valid = false;
+				break;
+			}
+
+			api_nh = &api_nhg.backup_nexthops
+					  [api_nhg.backup_nexthop_num];
+
+			zapi_backup_nexthop_from_nexthop(api_nh, nh);
+			api_nhg.backup_nexthop_num++;
+		}
+	}
+
+done:
+	if (is_valid)
+		zclient_nhg_send(zclient, ZEBRA_NHG_ADD, &api_nhg);
 }
 
-void route_delete(struct prefix *p, vrf_id_t vrf_id, uint8_t instance)
+void nhg_del(uint32_t id)
 {
-	struct zapi_route api;
+	struct zapi_nhg api_nhg = {};
 
-	memset(&api, 0, sizeof(api));
-	api.vrf_id = vrf_id;
-	api.type = ZEBRA_ROUTE_SHARP;
-	api.safi = SAFI_UNICAST;
-	api.instance = instance;
-	memcpy(&api.prefix, p, sizeof(*p));
-	zclient_route_send(ZEBRA_ROUTE_DELETE, zclient, &api);
+	api_nhg.id = id;
 
-	return;
+	zclient_nhg_send(zclient, ZEBRA_NHG_DEL, &api_nhg);
 }
 
 void sharp_zebra_nexthop_watch(struct prefix *p, vrf_id_t vrf_id, bool import,
@@ -424,26 +626,20 @@ void sharp_zebra_nexthop_watch(struct prefix *p, vrf_id_t vrf_id, bool import,
 {
 	int command;
 
-	if (!import) {
-		command = ZEBRA_NEXTHOP_REGISTER;
+	command = ZEBRA_NEXTHOP_REGISTER;
 
-		if (!watch)
-			command = ZEBRA_NEXTHOP_UNREGISTER;
-	} else {
-		command = ZEBRA_IMPORT_ROUTE_REGISTER;
+	if (!watch)
+		command = ZEBRA_NEXTHOP_UNREGISTER;
 
-		if (!watch)
-			command = ZEBRA_IMPORT_ROUTE_UNREGISTER;
-	}
-
-	if (zclient_send_rnh(zclient, command, p, connected, vrf_id) < 0)
+	if (zclient_send_rnh(zclient, command, p, SAFI_UNICAST, connected,
+			     false, vrf_id)
+	    == ZCLIENT_SEND_FAILURE)
 		zlog_warn("%s: Failure to send nexthop to zebra", __func__);
 }
 
 static int sharp_debug_nexthops(struct zapi_route *api)
 {
 	int i;
-	char buf[PREFIX_STRLEN];
 
 	if (api->nexthop_num == 0) {
 		zlog_debug(
@@ -458,20 +654,16 @@ static int sharp_debug_nexthops(struct zapi_route *api)
 		case NEXTHOP_TYPE_IPV4_IFINDEX:
 		case NEXTHOP_TYPE_IPV4:
 			zlog_debug(
-				"        Nexthop %s, type: %d, ifindex: %d, vrf: %d, label_num: %d",
-				inet_ntop(AF_INET, &znh->gate.ipv4.s_addr, buf,
-					  sizeof(buf)),
-				znh->type, znh->ifindex, znh->vrf_id,
-				znh->label_num);
+				"        Nexthop %pI4, type: %d, ifindex: %d, vrf: %d, label_num: %d",
+				&znh->gate.ipv4.s_addr, znh->type, znh->ifindex,
+				znh->vrf_id, znh->label_num);
 			break;
 		case NEXTHOP_TYPE_IPV6_IFINDEX:
 		case NEXTHOP_TYPE_IPV6:
 			zlog_debug(
-				"        Nexthop %s, type: %d, ifindex: %d, vrf: %d, label_num: %d",
-				inet_ntop(AF_INET6, &znh->gate.ipv6, buf,
-					  sizeof(buf)),
-				znh->type, znh->ifindex, znh->vrf_id,
-				znh->label_num);
+				"        Nexthop %pI6, type: %d, ifindex: %d, vrf: %d, label_num: %d",
+				&znh->gate.ipv6, znh->type, znh->ifindex,
+				znh->vrf_id, znh->label_num);
 			break;
 		case NEXTHOP_TYPE_IFINDEX:
 			zlog_debug("        Nexthop IFINDEX: %d, ifindex: %d",
@@ -489,15 +681,17 @@ static int sharp_nexthop_update(ZAPI_CALLBACK_ARGS)
 {
 	struct sharp_nh_tracker *nht;
 	struct zapi_route nhr;
+	struct prefix matched;
 
-	if (!zapi_nexthop_update_decode(zclient->ibuf, &nhr)) {
+	if (!zapi_nexthop_update_decode(zclient->ibuf, &matched, &nhr)) {
 		zlog_err("%s: Decode of update failed", __func__);
 		return 0;
 	}
 
-	zlog_debug("Received update for %pFX", &nhr.prefix);
+	zlog_debug("Received update for %pFX actual match: %pFX metric: %u",
+		   &matched, &nhr.prefix, nhr.metric);
 
-	nht = sharp_nh_tracker_get(&nhr.prefix);
+	nht = sharp_nh_tracker_get(&matched);
 	nht->nhop_num = nhr.nexthop_num;
 	nht->updates++;
 
@@ -522,6 +716,16 @@ static int sharp_redistribute_route(ZAPI_CALLBACK_ARGS)
 	return 0;
 }
 
+void sharp_redistribute_vrf(struct vrf *vrf, int type)
+{
+	zebra_redistribute_send(ZEBRA_REDISTRIBUTE_ADD, zclient, AFI_IP, type,
+				0, vrf->vrf_id);
+}
+
+static zclient_handler *const sharp_opaque_handlers[] = {
+	[ZEBRA_OPAQUE_MESSAGE] = sharp_opaque_handler,
+};
+
 /* Add a zclient with a specified session id, for testing. */
 int sharp_zclient_create(uint32_t session_id)
 {
@@ -534,15 +738,14 @@ int sharp_zclient_create(uint32_t session_id)
 			return -1;
 	}
 
-	client = zclient_new(master, &zclient_options_default);
+	client = zclient_new(master, &zclient_options_default,
+			     sharp_opaque_handlers,
+			     array_size(sharp_opaque_handlers));
 	client->sock = -1;
 	client->session_id = session_id;
 
 	zclient_init(client, ZEBRA_ROUTE_SHARP, 0, &sharp_privs);
 
-	/* Register handlers for messages we expect this session to see */
-	client->opaque_msg_handler = sharp_opaque_handler;
-
 	/* Enqueue on the list of test clients */
 	add_zclient(client);
 
@@ -578,11 +781,15 @@ int sharp_zclient_delete(uint32_t session_id)
 	return 0;
 }
 
+static const char *const type2txt[] = { "Generic", "Vertex", "Edge", "Subnet" };
+static const char *const status2txt[] = { "Unknown", "New", "Update",
+					  "Delete", "Sync", "Orphan"};
 /* Handler for opaque messages */
 static int sharp_opaque_handler(ZAPI_CALLBACK_ARGS)
 {
 	struct stream *s;
 	struct zapi_opaque_msg info;
+	struct ls_element *lse;
 
 	s = zclient->ibuf;
 
@@ -592,6 +799,20 @@ static int sharp_opaque_handler(ZAPI_CALLBACK_ARGS)
 	zlog_debug("%s: [%u] received opaque type %u", __func__,
 		   zclient->session_id, info.type);
 
+	if (info.type == LINK_STATE_UPDATE) {
+		lse = ls_stream2ted(sg.ted, s, false);
+		if (lse) {
+			zlog_debug(" |- Got %s %s from Link State Database",
+				   status2txt[lse->status],
+				   type2txt[lse->type]);
+			lse->status = SYNC;
+		}
+		else
+			zlog_debug(
+				"%s: Error to convert Stream into Link State",
+				__func__);
+	}
+
 	return 0;
 }
 
@@ -622,7 +843,7 @@ void sharp_opaque_send(uint32_t type, uint32_t proto, uint32_t instance,
 			ret = zclient_send_opaque_unicast(zclient, type, proto,
 							  instance, session_id,
 							  buf, sizeof(buf));
-		if (ret < 0) {
+		if (ret == ZCLIENT_SEND_FAILURE) {
 			zlog_debug("%s: send_opaque() failed => %d",
 				   __func__, ret);
 			break;
@@ -662,11 +883,118 @@ void sharp_opaque_reg_send(bool is_reg, uint32_t proto, uint32_t instance,
 
 }
 
+/* Link State registration */
+void sharp_zebra_register_te(void)
+{
+	/* First register to received Link State Update messages */
+	zclient_register_opaque(zclient, LINK_STATE_UPDATE);
+
+	/* Then, request initial TED with SYNC message */
+	ls_request_sync(zclient);
+}
+
 void sharp_zebra_send_arp(const struct interface *ifp, const struct prefix *p)
 {
 	zclient_send_neigh_discovery_req(zclient, ifp, p);
 }
 
+static int nhg_notify_owner(ZAPI_CALLBACK_ARGS)
+{
+	enum zapi_nhg_notify_owner note;
+	uint32_t id;
+
+	if (!zapi_nhg_notify_decode(zclient->ibuf, &id, &note))
+		return -1;
+
+	switch (note) {
+	case ZAPI_NHG_INSTALLED:
+		sharp_nhgroup_id_set_installed(id, true);
+		zlog_debug("Installed nhg %u", id);
+		break;
+	case ZAPI_NHG_FAIL_INSTALL:
+		zlog_debug("Failed install of nhg %u", id);
+		break;
+	case ZAPI_NHG_REMOVED:
+		zlog_debug("Removed nhg %u", id);
+		break;
+	case ZAPI_NHG_REMOVE_FAIL:
+		zlog_debug("Failed removal of nhg %u", id);
+		break;
+	}
+
+	return 0;
+}
+
+int sharp_zebra_srv6_manager_get_locator_chunk(const char *locator_name)
+{
+	return srv6_manager_get_locator_chunk(zclient, locator_name);
+}
+
+int sharp_zebra_srv6_manager_release_locator_chunk(const char *locator_name)
+{
+	return srv6_manager_release_locator_chunk(zclient, locator_name);
+}
+
+static int sharp_zebra_process_srv6_locator_chunk(ZAPI_CALLBACK_ARGS)
+{
+	struct stream *s = NULL;
+	struct srv6_locator_chunk s6c = {};
+	struct listnode *node, *nnode;
+	struct sharp_srv6_locator *loc;
+
+	s = zclient->ibuf;
+	zapi_srv6_locator_chunk_decode(s, &s6c);
+
+	for (ALL_LIST_ELEMENTS(sg.srv6_locators, node, nnode, loc)) {
+		struct prefix_ipv6 *chunk = NULL;
+		struct listnode *chunk_node;
+		struct prefix_ipv6 *c;
+
+		if (strcmp(loc->name, s6c.locator_name) != 0) {
+			zlog_err("%s: Locator name unmatch %s:%s", __func__,
+				 loc->name, s6c.locator_name);
+			continue;
+		}
+
+		for (ALL_LIST_ELEMENTS_RO(loc->chunks, chunk_node, c))
+			if (!prefix_cmp(c, &s6c.prefix))
+				return 0;
+
+		chunk = prefix_ipv6_new();
+		*chunk = s6c.prefix;
+		listnode_add(loc->chunks, chunk);
+		return 0;
+	}
+
+	zlog_err("%s: can't get locator_chunk!!", __func__);
+	return 0;
+}
+
+int sharp_zebra_send_interface_protodown(struct interface *ifp, bool down)
+{
+	zlog_debug("Sending zebra to set %s protodown %s", ifp->name,
+		   down ? "on" : "off");
+
+	if (zclient_send_interface_protodown(zclient, ifp->vrf->vrf_id, ifp,
+					     down) == ZCLIENT_SEND_FAILURE)
+		return -1;
+
+	return 0;
+}
+
+static zclient_handler *const sharp_handlers[] = {
+	[ZEBRA_INTERFACE_ADDRESS_ADD] = interface_address_add,
+	[ZEBRA_INTERFACE_ADDRESS_DELETE] = interface_address_delete,
+	[ZEBRA_ROUTE_NOTIFY_OWNER] = route_notify_owner,
+	[ZEBRA_NEXTHOP_UPDATE] = sharp_nexthop_update,
+	[ZEBRA_NHG_NOTIFY_OWNER] = nhg_notify_owner,
+	[ZEBRA_REDISTRIBUTE_ROUTE_ADD] = sharp_redistribute_route,
+	[ZEBRA_REDISTRIBUTE_ROUTE_DEL] = sharp_redistribute_route,
+	[ZEBRA_OPAQUE_MESSAGE] = sharp_opaque_handler,
+	[ZEBRA_SRV6_MANAGER_GET_LOCATOR_CHUNK] =
+		sharp_zebra_process_srv6_locator_chunk,
+};
+
 void sharp_zebra_init(void)
 {
 	struct zclient_options opt = {.receive_notify = true};
@@ -674,17 +1002,10 @@ void sharp_zebra_init(void)
 	if_zapi_callbacks(sharp_ifp_create, sharp_ifp_up,
 			  sharp_ifp_down, sharp_ifp_destroy);
 
-	zclient = zclient_new(master, &opt);
+	zclient = zclient_new(master, &opt, sharp_handlers,
+			      array_size(sharp_handlers));
 
 	zclient_init(zclient, ZEBRA_ROUTE_SHARP, 0, &sharp_privs);
 	zclient->zebra_connected = zebra_connected;
-	zclient->interface_address_add = interface_address_add;
-	zclient->interface_address_delete = interface_address_delete;
-	zclient->route_notify_owner = route_notify_owner;
-	zclient->nexthop_update = sharp_nexthop_update;
-	zclient->import_check_update = sharp_nexthop_update;
-
-	zclient->redistribute_route_add = sharp_redistribute_route;
-	zclient->redistribute_route_del = sharp_redistribute_route;
-	zclient->opaque_msg_handler = sharp_opaque_handler;
+	zclient->zebra_buffer_write_ready = sharp_zclient_buffer_ready;
 }
