@@ -44,7 +44,7 @@
 #define OPT_PARAM  2
 
 /* need these to link in libbgp */
-struct zebra_privs_t *bgpd_privs = NULL;
+struct zebra_privs_t bgpd_privs = {};
 struct thread_master *master = NULL;
 
 static int failed = 0;
@@ -649,6 +649,35 @@ static struct test_segment misc_segments[] =
 			2,
 			SHOULD_PARSE,
 		},
+		{
+			"Role",
+			"Role capability",
+			{
+				/* hdr */ 0x9, 0x1,
+				0x1,
+			},
+			3,
+			SHOULD_PARSE,
+		},
+		{
+			"Role-long",
+			"Role capability, but too long",
+			{
+				/* hdr */ 0x9, 0x4,
+				0x0, 0x0, 0x0, 0x1,
+			},
+			6,
+			SHOULD_ERR,
+		},
+		{
+			"Role-empty",
+			"Role capability, but empty.",
+			{
+				/* hdr */ 0x9, 0x0,
+			},
+			2,
+			SHOULD_ERR,
+		},
 		{NULL, NULL, {0}, 0, 0}};
 
 /* DYNAMIC message */
@@ -912,8 +941,8 @@ int main(void)
 
 	qobj_init();
 	master = thread_master_create(NULL);
-	bgp_master_init(master, BGP_SOCKET_SNDBUF_SIZE);
-	vrf_init(NULL, NULL, NULL, NULL, NULL);
+	bgp_master_init(master, BGP_SOCKET_SNDBUF_SIZE, list_new());
+	vrf_init(NULL, NULL, NULL, NULL);
 	bgp_option_set(BGP_OPT_NO_LISTEN);
 
 	frr_pthread_init();
