@@ -1,6 +1,8 @@
 import frrtest
 
+
 class TestIsisLSPDB(frrtest.TestMultiOut):
-    program = './test_isis_lspdb'
+    program = "./test_isis_lspdb"
+
 
 TestIsisLSPDB.exit_cleanly()
