@@ -38,9 +38,12 @@ unsigned long zebra_debug_mpls;
 unsigned long zebra_debug_vxlan;
 unsigned long zebra_debug_pw;
 unsigned long zebra_debug_dplane;
+unsigned long zebra_debug_dplane_dpdk;
 unsigned long zebra_debug_mlag;
 unsigned long zebra_debug_nexthop;
 unsigned long zebra_debug_evpn_mh;
+unsigned long zebra_debug_pbr;
+unsigned long zebra_debug_neigh;
 
 DEFINE_HOOK(zebra_debug_show_debugging, (struct vty *vty), (vty));
 
@@ -93,8 +96,11 @@ DEFUN_NOSH (show_debugging_zebra,
 		vty_out(vty, "  Zebra detailed next-hop tracking debugging is on\n");
 	else if (IS_ZEBRA_DEBUG_NHT)
 		vty_out(vty, "  Zebra next-hop tracking debugging is on\n");
-	if (IS_ZEBRA_DEBUG_MPLS)
+	if (IS_ZEBRA_DEBUG_MPLS_DETAIL)
+		vty_out(vty, "  Zebra detailed MPLS debugging is on\n");
+	else if (IS_ZEBRA_DEBUG_MPLS)
 		vty_out(vty, "  Zebra MPLS debugging is on\n");
+
 	if (IS_ZEBRA_DEBUG_VXLAN)
 		vty_out(vty, "  Zebra VXLAN debugging is on\n");
 	if (IS_ZEBRA_DEBUG_PW)
@@ -103,6 +109,11 @@ DEFUN_NOSH (show_debugging_zebra,
 		vty_out(vty, "  Zebra detailed dataplane debugging is on\n");
 	else if (IS_ZEBRA_DEBUG_DPLANE)
 		vty_out(vty, "  Zebra dataplane debugging is on\n");
+	if (IS_ZEBRA_DEBUG_DPLANE_DPDK_DETAIL)
+		vty_out(vty,
+			"  Zebra detailed dpdk dataplane debugging is on\n");
+	else if (IS_ZEBRA_DEBUG_DPLANE_DPDK)
+		vty_out(vty, "  Zebra dataplane dpdk debugging is on\n");
 	if (IS_ZEBRA_DEBUG_MLAG)
 		vty_out(vty, "  Zebra mlag debugging is on\n");
 	if (IS_ZEBRA_DEBUG_NHG_DETAIL)
@@ -122,6 +133,9 @@ DEFUN_NOSH (show_debugging_zebra,
 	if (IS_ZEBRA_DEBUG_EVPN_MH_NEIGH)
 		vty_out(vty, "  Zebra EVPN-MH Neigh debugging is on\n");
 
+	if (IS_ZEBRA_DEBUG_PBR)
+		vty_out(vty, "  Zebra PBR debugging is on\n");
+
 	hook_call(zebra_debug_show_debugging, vty);
 	return CMD_SUCCESS;
 }
@@ -155,18 +169,23 @@ DEFUN (debug_zebra_nht,
 	return CMD_SUCCESS;
 }
 
-DEFUN (debug_zebra_mpls,
+DEFPY (debug_zebra_mpls,
        debug_zebra_mpls_cmd,
-       "debug zebra mpls",
+       "debug zebra mpls [detailed$detail]",
        DEBUG_STR
        "Zebra configuration\n"
-       "Debug option set for zebra MPLS LSPs\n")
+       "Debug option set for zebra MPLS LSPs\n"
+       "Debug option for detailed info\n")
 {
 	zebra_debug_mpls = ZEBRA_DEBUG_MPLS;
+
+	if (detail)
+		zebra_debug_mpls |= ZEBRA_DEBUG_MPLS_DETAILED;
+
 	return CMD_SUCCESS;
 }
 
-DEFUN (debug_zebra_vxlan,
+DEFPY (debug_zebra_vxlan,
        debug_zebra_vxlan_cmd,
        "debug zebra vxlan",
        DEBUG_STR
@@ -229,15 +248,10 @@ DEFUN (debug_zebra_kernel,
 {
 	SET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL);
 
-	if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV)
-		UNSET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV);
-
-	if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND)
-		UNSET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND);
-
 	return CMD_SUCCESS;
 }
 
+#if defined(HAVE_NETLINK)
 DEFUN (debug_zebra_kernel_msgdump,
        debug_zebra_kernel_msgdump_cmd,
        "debug zebra kernel msgdump [<recv|send>]",
@@ -250,27 +264,18 @@ DEFUN (debug_zebra_kernel_msgdump,
 {
 	int idx = 0;
 
-	if (argv_find(argv, argc, "recv", &idx)) {
+	if (argv_find(argv, argc, "recv", &idx))
 		SET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV);
-
-		if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND)
-			UNSET_FLAG(zebra_debug_kernel,
-				   ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND);
-
-	} else if (argv_find(argv, argc, "send", &idx)) {
+	else if (argv_find(argv, argc, "send", &idx))
 		SET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND);
-
-		if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV)
-			UNSET_FLAG(zebra_debug_kernel,
-				   ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV);
-
-	} else {
+	else {
 		SET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV);
 		SET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND);
 	}
 
 	return CMD_SUCCESS;
 }
+#endif
 
 DEFUN (debug_zebra_rib,
        debug_zebra_rib_cmd,
@@ -318,6 +323,56 @@ DEFUN (debug_zebra_dplane,
 	return CMD_SUCCESS;
 }
 
+DEFPY(debug_zebra_dplane_dpdk, debug_zebra_dplane_dpdk_cmd,
+      "[no$no] debug zebra dplane dpdk [detailed$detail]",
+      NO_STR DEBUG_STR
+      "Zebra configuration\n"
+      "Debug zebra dataplane events\n"
+      "Debug zebra DPDK offload events\n"
+      "Detailed debug information\n")
+{
+	if (no) {
+		UNSET_FLAG(zebra_debug_dplane_dpdk, ZEBRA_DEBUG_DPLANE_DPDK);
+		UNSET_FLAG(zebra_debug_dplane_dpdk,
+			   ZEBRA_DEBUG_DPLANE_DPDK_DETAIL);
+	} else {
+		SET_FLAG(zebra_debug_dplane_dpdk, ZEBRA_DEBUG_DPLANE_DPDK);
+
+		if (detail)
+			SET_FLAG(zebra_debug_dplane,
+				 ZEBRA_DEBUG_DPLANE_DPDK_DETAIL);
+	}
+
+	return CMD_SUCCESS;
+}
+
+DEFUN (debug_zebra_pbr,
+       debug_zebra_pbr_cmd,
+       "debug zebra pbr",
+       DEBUG_STR
+       "Zebra configuration\n"
+       "Debug zebra pbr events\n")
+{
+	SET_FLAG(zebra_debug_pbr, ZEBRA_DEBUG_PBR);
+	return CMD_SUCCESS;
+}
+
+DEFPY (debug_zebra_neigh,
+       debug_zebra_neigh_cmd,
+       "[no$no] debug zebra neigh",
+       NO_STR
+       DEBUG_STR
+       "Zebra configuration\n"
+       "Debug zebra neigh events\n")
+{
+	if (no)
+		UNSET_FLAG(zebra_debug_neigh, ZEBRA_DEBUG_NEIGH);
+	else
+		SET_FLAG(zebra_debug_neigh, ZEBRA_DEBUG_NEIGH);
+
+	return CMD_SUCCESS;
+}
+
 DEFPY (debug_zebra_mlag,
        debug_zebra_mlag_cmd,
        "[no$no] debug zebra mlag",
@@ -407,11 +462,12 @@ DEFUN (no_debug_zebra_nht,
 
 DEFUN (no_debug_zebra_mpls,
        no_debug_zebra_mpls_cmd,
-       "no debug zebra mpls",
+       "no debug zebra mpls [detailed]",
        NO_STR
        DEBUG_STR
        "Zebra configuration\n"
-       "Debug option set for zebra MPLS LSPs\n")
+       "Debug option set for zebra MPLS LSPs\n"
+       "Debug option for zebra detailed info\n")
 {
 	zebra_debug_mpls = 0;
 	return CMD_SUCCESS;
@@ -452,10 +508,12 @@ DEFUN (no_debug_zebra_kernel,
        "Zebra configuration\n"
        "Debug option set for zebra between kernel interface\n")
 {
-	zebra_debug_kernel = 0;
+	UNSET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL);
+
 	return CMD_SUCCESS;
 }
 
+#if defined(HAVE_NETLINK)
 DEFUN (no_debug_zebra_kernel_msgdump,
        no_debug_zebra_kernel_msgdump_cmd,
        "no debug zebra kernel msgdump [<recv|send>]",
@@ -467,9 +525,20 @@ DEFUN (no_debug_zebra_kernel_msgdump,
        "Dump raw netlink messages received\n"
        "Dump raw netlink messages sent\n")
 {
-	zebra_debug_kernel = 0;
+	int idx = 0;
+
+	if (argv_find(argv, argc, "recv", &idx))
+		UNSET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV);
+	else if (argv_find(argv, argc, "send", &idx))
+		UNSET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND);
+	else {
+		UNSET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV);
+		UNSET_FLAG(zebra_debug_kernel, ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND);
+	}
+
 	return CMD_SUCCESS;
 }
+#endif
 
 DEFUN (no_debug_zebra_rib,
        no_debug_zebra_rib_cmd,
@@ -508,6 +577,18 @@ DEFUN (no_debug_zebra_dplane,
 	return CMD_SUCCESS;
 }
 
+DEFUN (no_debug_zebra_pbr,
+       no_debug_zebra_pbr_cmd,
+       "no debug zebra pbr",
+       NO_STR
+       DEBUG_STR
+       "Zebra configuration\n"
+       "Debug zebra pbr events\n")
+{
+	zebra_debug_pbr = 0;
+	return CMD_SUCCESS;
+}
+
 DEFPY (debug_zebra_nexthop,
        debug_zebra_nexthop_cmd,
        "[no$no] debug zebra nexthop [detail$detail]",
@@ -563,21 +644,21 @@ static int config_write_debug(struct vty *vty)
 		}
 	}
 
+	if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND
+	    && IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV) {
+		vty_out(vty, "debug zebra kernel msgdump\n");
+		write++;
+	} else if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV) {
+		vty_out(vty, "debug zebra kernel msgdump recv\n");
+		write++;
+	} else if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND) {
+		vty_out(vty, "debug zebra kernel msgdump send\n");
+		write++;
+	}
+
 	if (IS_ZEBRA_DEBUG_KERNEL) {
-		if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND
-		    && IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV) {
-			vty_out(vty, "debug zebra kernel msgdump\n");
-			write++;
-		} else if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV) {
-			vty_out(vty, "debug zebra kernel msgdump recv\n");
-			write++;
-		} else if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND) {
-			vty_out(vty, "debug zebra kernel msgdump send\n");
-			write++;
-		} else {
-			vty_out(vty, "debug zebra kernel\n");
-			write++;
-		}
+		vty_out(vty, "debug zebra kernel\n");
+		write++;
 	}
 
 	if (CHECK_FLAG(zebra_debug_rib, ZEBRA_DEBUG_RIB_DETAILED)) {
@@ -601,10 +682,14 @@ static int config_write_debug(struct vty *vty)
 		write++;
 	}
 
-	if (IS_ZEBRA_DEBUG_MPLS) {
+	if (IS_ZEBRA_DEBUG_MPLS_DETAIL) {
+		vty_out(vty, "debug zebra mpls detailed\n");
+		write++;
+	} else if (IS_ZEBRA_DEBUG_MPLS) {
 		vty_out(vty, "debug zebra mpls\n");
 		write++;
 	}
+
 	if (IS_ZEBRA_DEBUG_VXLAN) {
 		vty_out(vty, "debug zebra vxlan\n");
 		write++;
@@ -642,6 +727,14 @@ static int config_write_debug(struct vty *vty)
 		write++;
 	}
 
+	if (CHECK_FLAG(zebra_debug_dplane, ZEBRA_DEBUG_DPLANE_DPDK_DETAIL)) {
+		vty_out(vty, "debug zebra dplane dpdk detailed\n");
+		write++;
+	} else if (CHECK_FLAG(zebra_debug_dplane, ZEBRA_DEBUG_DPLANE_DPDK)) {
+		vty_out(vty, "debug zebra dplane dpdk\n");
+		write++;
+	}
+
 	if (CHECK_FLAG(zebra_debug_nexthop, ZEBRA_DEBUG_NHG_DETAILED)) {
 		vty_out(vty, "debug zebra nexthop detail\n");
 		write++;
@@ -650,6 +743,16 @@ static int config_write_debug(struct vty *vty)
 		write++;
 	}
 
+	if (IS_ZEBRA_DEBUG_PBR) {
+		vty_out(vty, "debug zebra pbr\n");
+		write++;
+	}
+
+	if (IS_ZEBRA_DEBUG_NEIGH) {
+		vty_out(vty, "debug zebra neigh\n");
+		write++;
+	}
+
 	return write;
 }
 
@@ -664,10 +767,13 @@ void zebra_debug_init(void)
 	zebra_debug_vxlan = 0;
 	zebra_debug_pw = 0;
 	zebra_debug_dplane = 0;
+	zebra_debug_dplane_dpdk = 0;
 	zebra_debug_mlag = 0;
 	zebra_debug_evpn_mh = 0;
 	zebra_debug_nht = 0;
 	zebra_debug_nexthop = 0;
+	zebra_debug_pbr = 0;
+	zebra_debug_neigh = 0;
 
 	install_node(&debug_node);
 
@@ -680,22 +786,30 @@ void zebra_debug_init(void)
 	install_element(ENABLE_NODE, &debug_zebra_pw_cmd);
 	install_element(ENABLE_NODE, &debug_zebra_packet_cmd);
 	install_element(ENABLE_NODE, &debug_zebra_kernel_cmd);
+#if defined(HAVE_NETLINK)
 	install_element(ENABLE_NODE, &debug_zebra_kernel_msgdump_cmd);
+#endif
 	install_element(ENABLE_NODE, &debug_zebra_rib_cmd);
 	install_element(ENABLE_NODE, &debug_zebra_fpm_cmd);
 	install_element(ENABLE_NODE, &debug_zebra_dplane_cmd);
 	install_element(ENABLE_NODE, &debug_zebra_mlag_cmd);
 	install_element(ENABLE_NODE, &debug_zebra_nexthop_cmd);
+	install_element(ENABLE_NODE, &debug_zebra_pbr_cmd);
+	install_element(ENABLE_NODE, &debug_zebra_neigh_cmd);
+	install_element(ENABLE_NODE, &debug_zebra_dplane_dpdk_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_events_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_nht_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_mpls_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_vxlan_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_packet_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_kernel_cmd);
+#if defined(HAVE_NETLINK)
 	install_element(ENABLE_NODE, &no_debug_zebra_kernel_msgdump_cmd);
+#endif
 	install_element(ENABLE_NODE, &no_debug_zebra_rib_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_fpm_cmd);
 	install_element(ENABLE_NODE, &no_debug_zebra_dplane_cmd);
+	install_element(ENABLE_NODE, &no_debug_zebra_pbr_cmd);
 	install_element(ENABLE_NODE, &debug_zebra_evpn_mh_cmd);
 
 	install_element(CONFIG_NODE, &debug_zebra_events_cmd);
@@ -705,21 +819,30 @@ void zebra_debug_init(void)
 	install_element(CONFIG_NODE, &debug_zebra_pw_cmd);
 	install_element(CONFIG_NODE, &debug_zebra_packet_cmd);
 	install_element(CONFIG_NODE, &debug_zebra_kernel_cmd);
+#if defined(HAVE_NETLINK)
 	install_element(CONFIG_NODE, &debug_zebra_kernel_msgdump_cmd);
+#endif
 	install_element(CONFIG_NODE, &debug_zebra_rib_cmd);
 	install_element(CONFIG_NODE, &debug_zebra_fpm_cmd);
 	install_element(CONFIG_NODE, &debug_zebra_dplane_cmd);
+	install_element(CONFIG_NODE, &debug_zebra_dplane_dpdk_cmd);
 	install_element(CONFIG_NODE, &debug_zebra_nexthop_cmd);
+	install_element(CONFIG_NODE, &debug_zebra_pbr_cmd);
+	install_element(CONFIG_NODE, &debug_zebra_neigh_cmd);
+
 	install_element(CONFIG_NODE, &no_debug_zebra_events_cmd);
 	install_element(CONFIG_NODE, &no_debug_zebra_nht_cmd);
 	install_element(CONFIG_NODE, &no_debug_zebra_mpls_cmd);
 	install_element(CONFIG_NODE, &no_debug_zebra_vxlan_cmd);
 	install_element(CONFIG_NODE, &no_debug_zebra_packet_cmd);
 	install_element(CONFIG_NODE, &no_debug_zebra_kernel_cmd);
+#if defined(HAVE_NETLINK)
 	install_element(CONFIG_NODE, &no_debug_zebra_kernel_msgdump_cmd);
+#endif
 	install_element(CONFIG_NODE, &no_debug_zebra_rib_cmd);
 	install_element(CONFIG_NODE, &no_debug_zebra_fpm_cmd);
 	install_element(CONFIG_NODE, &no_debug_zebra_dplane_cmd);
+	install_element(CONFIG_NODE, &no_debug_zebra_pbr_cmd);
 	install_element(CONFIG_NODE, &debug_zebra_mlag_cmd);
 	install_element(CONFIG_NODE, &debug_zebra_evpn_mh_cmd);
 }
