@@ -43,6 +43,7 @@
 #include "zebra/debug.h"
 #include "zebra/interface.h"
 #include "zebra/zebra_dplane.h"
+#include "zebra/zebra_mpls.h"
 #include "zebra/zebra_router.h"
 #include "zebra/zebra_evpn.h"
 #include "zebra/zebra_evpn_mac.h"
@@ -74,9 +75,6 @@ struct fpm_nl_ctx {
 	int socket;
 	bool disabled;
 	bool connecting;
-	bool nhg_complete;
-	bool rib_complete;
-	bool rmac_complete;
 	bool use_nhg;
 	struct sockaddr_storage addr;
 
@@ -100,9 +98,12 @@ struct fpm_nl_ctx {
 	struct thread *t_read;
 	struct thread *t_write;
 	struct thread *t_event;
+	struct thread *t_nhg;
 	struct thread *t_dequeue;
 
 	/* zebra events. */
+	struct thread *t_lspreset;
+	struct thread *t_lspwalk;
 	struct thread *t_nhgreset;
 	struct thread *t_nhgwalk;
 	struct thread *t_ribreset;
@@ -155,6 +156,8 @@ enum fpm_nl_events {
 	/* Reconnect request by our own code to avoid races. */
 	FNE_INTERNAL_RECONNECT,
 
+	/* LSP walk finished. */
+	FNE_LSP_FINISHED,
 	/* Next hop groups walk finished. */
 	FNE_NHG_FINISHED,
 	/* RIB walk finished. */
@@ -174,39 +177,16 @@ enum fpm_nl_events {
 /*
  * Prototypes.
  */
-static int fpm_process_event(struct thread *t);
+static void fpm_process_event(struct thread *t);
 static int fpm_nl_enqueue(struct fpm_nl_ctx *fnc, struct zebra_dplane_ctx *ctx);
-static int fpm_nhg_send(struct thread *t);
-static int fpm_nhg_reset(struct thread *t);
-static int fpm_rib_send(struct thread *t);
-static int fpm_rib_reset(struct thread *t);
-static int fpm_rmac_send(struct thread *t);
-static int fpm_rmac_reset(struct thread *t);
-
-/*
- * Helper functions.
- */
-
-/**
- * Reorganizes the data on the buffer so it can fit more data.
- *
- * @param s stream pointer.
- */
-static void stream_pulldown(struct stream *s)
-{
-	size_t rlen = STREAM_READABLE(s);
-
-	/* No more data, so just move the pointers. */
-	if (rlen == 0) {
-		stream_reset(s);
-		return;
-	}
-
-	/* Move the available data to the beginning. */
-	memmove(s->data, &s->data[s->getp], rlen);
-	s->getp = 0;
-	s->endp = rlen;
-}
+static void fpm_lsp_send(struct thread *t);
+static void fpm_lsp_reset(struct thread *t);
+static void fpm_nhg_send(struct thread *t);
+static void fpm_nhg_reset(struct thread *t);
+static void fpm_rib_send(struct thread *t);
+static void fpm_rib_reset(struct thread *t);
+static void fpm_rmac_send(struct thread *t);
+static void fpm_rmac_reset(struct thread *t);
 
 /*
  * CLI.
@@ -292,7 +272,7 @@ DEFUN(fpm_use_nhg, fpm_use_nhg_cmd,
 		return CMD_SUCCESS;
 
 	thread_add_event(gfnc->fthread->master, fpm_process_event, gfnc,
-			 FNE_TOGGLE_NHG, &gfnc->t_event);
+			 FNE_TOGGLE_NHG, &gfnc->t_nhg);
 
 	return CMD_SUCCESS;
 }
@@ -308,7 +288,7 @@ DEFUN(no_fpm_use_nhg, no_fpm_use_nhg_cmd,
 		return CMD_SUCCESS;
 
 	thread_add_event(gfnc->fthread->master, fpm_process_event, gfnc,
-			 FNE_TOGGLE_NHG, &gfnc->t_event);
+			 FNE_TOGGLE_NHG, &gfnc->t_nhg);
 
 	return CMD_SUCCESS;
 }
@@ -384,8 +364,7 @@ DEFUN(fpm_show_counters_json, fpm_show_counters_json_cmd,
 	json_object_int_add(jo, "user-configures",
 			    gfnc->counters.user_configures);
 	json_object_int_add(jo, "user-disables", gfnc->counters.user_disables);
-	vty_out(vty, "%s\n", json_object_to_json_string_ext(jo, 0));
-	json_object_free(jo);
+	vty_json(vty, jo);
 
 	return CMD_SUCCESS;
 }
@@ -395,7 +374,6 @@ static int fpm_write_config(struct vty *vty)
 	struct sockaddr_in *sin;
 	struct sockaddr_in6 *sin6;
 	int written = 0;
-	char addrstr[INET6_ADDRSTRLEN];
 
 	if (gfnc->disabled)
 		return written;
@@ -404,8 +382,7 @@ static int fpm_write_config(struct vty *vty)
 	case AF_INET:
 		written = 1;
 		sin = (struct sockaddr_in *)&gfnc->addr;
-		inet_ntop(AF_INET, &sin->sin_addr, addrstr, sizeof(addrstr));
-		vty_out(vty, "fpm address %s", addrstr);
+		vty_out(vty, "fpm address %pI4", &sin->sin_addr);
 		if (sin->sin_port != htons(SOUTHBOUND_DEFAULT_PORT))
 			vty_out(vty, " port %d", ntohs(sin->sin_port));
 
@@ -414,8 +391,7 @@ static int fpm_write_config(struct vty *vty)
 	case AF_INET6:
 		written = 1;
 		sin6 = (struct sockaddr_in6 *)&gfnc->addr;
-		inet_ntop(AF_INET, &sin6->sin6_addr, addrstr, sizeof(addrstr));
-		vty_out(vty, "fpm address %s", addrstr);
+		vty_out(vty, "fpm address %pI6", &sin6->sin6_addr);
 		if (sin6->sin6_port != htons(SOUTHBOUND_DEFAULT_PORT))
 			vty_out(vty, " port %d", ntohs(sin6->sin6_port));
 
@@ -444,11 +420,13 @@ static struct cmd_node fpm_node = {
 /*
  * FPM functions.
  */
-static int fpm_connect(struct thread *t);
+static void fpm_connect(struct thread *t);
 
 static void fpm_reconnect(struct fpm_nl_ctx *fnc)
 {
 	/* Cancel all zebra threads first. */
+	thread_cancel_async(zrouter.master, &fnc->t_lspreset, NULL);
+	thread_cancel_async(zrouter.master, &fnc->t_lspwalk, NULL);
 	thread_cancel_async(zrouter.master, &fnc->t_nhgreset, NULL);
 	thread_cancel_async(zrouter.master, &fnc->t_nhgwalk, NULL);
 	thread_cancel_async(zrouter.master, &fnc->t_ribreset, NULL);
@@ -481,7 +459,7 @@ static void fpm_reconnect(struct fpm_nl_ctx *fnc)
 			 &fnc->t_connect);
 }
 
-static int fpm_read(struct thread *t)
+static void fpm_read(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 	ssize_t rv;
@@ -494,7 +472,7 @@ static int fpm_read(struct thread *t)
 		/* Schedule next read. */
 		thread_add_read(fnc->fthread->master, fpm_read, fnc,
 				fnc->socket, &fnc->t_read);
-		return 0;
+		return;
 	}
 	if (rv == 0) {
 		atomic_fetch_add_explicit(&fnc->counters.connection_closes, 1,
@@ -504,7 +482,7 @@ static int fpm_read(struct thread *t)
 			zlog_debug("%s: connection closed", __func__);
 
 		FPM_RECONNECT(fnc);
-		return 0;
+		return;
 	}
 	if (rv == -1) {
 		atomic_fetch_add_explicit(&fnc->counters.connection_errors, 1,
@@ -512,7 +490,7 @@ static int fpm_read(struct thread *t)
 		zlog_warn("%s: connection failure: %s", __func__,
 			  strerror(errno));
 		FPM_RECONNECT(fnc);
-		return 0;
+		return;
 	}
 	stream_reset(fnc->ibuf);
 
@@ -522,11 +500,9 @@ static int fpm_read(struct thread *t)
 
 	thread_add_read(fnc->fthread->master, fpm_read, fnc, fnc->socket,
 			&fnc->t_read);
-
-	return 0;
 }
 
-static int fpm_write(struct thread *t)
+static void fpm_write(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 	socklen_t statuslen;
@@ -553,11 +529,18 @@ static int fpm_write(struct thread *t)
 				memory_order_relaxed);
 
 			FPM_RECONNECT(fnc);
-			return 0;
+			return;
 		}
 
 		fnc->connecting = false;
 
+		/*
+		 * Starting with LSPs walk all FPM objects, marking them
+		 * as unsent and then replaying them.
+		 */
+		thread_add_timer(zrouter.master, fpm_lsp_reset, fnc, 0,
+				 &fnc->t_lspreset);
+
 		/* Permit receiving messages now. */
 		thread_add_read(fnc->fthread->master, fpm_read, fnc,
 				fnc->socket, &fnc->t_read);
@@ -600,7 +583,7 @@ static int fpm_write(struct thread *t)
 				  strerror(errno));
 
 			FPM_RECONNECT(fnc);
-			return 0;
+			return;
 		}
 
 		/* Account all bytes sent. */
@@ -619,13 +602,11 @@ static int fpm_write(struct thread *t)
 		stream_pulldown(fnc->obuf);
 		thread_add_write(fnc->fthread->master, fpm_write, fnc,
 				 fnc->socket, &fnc->t_write);
-		return 0;
+		return;
 	}
-
-	return 0;
 }
 
-static int fpm_connect(struct thread *t)
+static void fpm_connect(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 	struct sockaddr_in *sin = (struct sockaddr_in *)&fnc->addr;
@@ -640,7 +621,7 @@ static int fpm_connect(struct thread *t)
 			 strerror(errno));
 		thread_add_timer(fnc->fthread->master, fpm_connect, fnc, 3,
 				 &fnc->t_connect);
-		return 0;
+		return;
 	}
 
 	set_nonblocking(sock);
@@ -666,7 +647,7 @@ static int fpm_connect(struct thread *t)
 			  strerror(errno));
 		thread_add_timer(fnc->fthread->master, fpm_connect, fnc, 3,
 				 &fnc->t_connect);
-		return 0;
+		return;
 	}
 
 	fnc->connecting = (errno == EINPROGRESS);
@@ -677,15 +658,15 @@ static int fpm_connect(struct thread *t)
 	thread_add_write(fnc->fthread->master, fpm_write, fnc, sock,
 			 &fnc->t_write);
 
-	/* Mark all routes as unsent. */
-	if (fnc->use_nhg)
-		thread_add_timer(zrouter.master, fpm_nhg_reset, fnc, 0,
-				 &fnc->t_nhgreset);
-	else
-		thread_add_timer(zrouter.master, fpm_rib_reset, fnc, 0,
-				 &fnc->t_ribreset);
-
-	return 0;
+	/*
+	 * Starting with LSPs walk all FPM objects, marking them
+	 * as unsent and then replaying them.
+	 *
+	 * If we are not connected, then delay the objects reset/send.
+	 */
+	if (!fnc->connecting)
+		thread_add_timer(zrouter.master, fpm_lsp_reset, fnc, 0,
+				 &fnc->t_lspreset);
 }
 
 /**
@@ -790,6 +771,17 @@ static int fpm_nl_enqueue(struct fpm_nl_ctx *fnc, struct zebra_dplane_ctx *ctx)
 	case DPLANE_OP_LSP_INSTALL:
 	case DPLANE_OP_LSP_UPDATE:
 	case DPLANE_OP_LSP_DELETE:
+		rv = netlink_lsp_msg_encoder(ctx, nl_buf, sizeof(nl_buf));
+		if (rv <= 0) {
+			zlog_err("%s: netlink_lsp_msg_encoder failed",
+				 __func__);
+			return 0;
+		}
+
+		nl_buf_len += (size_t)rv;
+		break;
+
+	/* Un-handled by FPM at this time. */
 	case DPLANE_OP_PW_INSTALL:
 	case DPLANE_OP_PW_UNINSTALL:
 	case DPLANE_OP_ADDR_INSTALL:
@@ -803,15 +795,33 @@ static int fpm_nl_enqueue(struct fpm_nl_ctx *fnc, struct zebra_dplane_ctx *ctx)
 	case DPLANE_OP_SYS_ROUTE_DELETE:
 	case DPLANE_OP_ROUTE_NOTIFY:
 	case DPLANE_OP_LSP_NOTIFY:
+	case DPLANE_OP_RULE_ADD:
+	case DPLANE_OP_RULE_DELETE:
+	case DPLANE_OP_RULE_UPDATE:
+	case DPLANE_OP_NEIGH_DISCOVER:
+	case DPLANE_OP_BR_PORT_UPDATE:
+	case DPLANE_OP_IPTABLE_ADD:
+	case DPLANE_OP_IPTABLE_DELETE:
+	case DPLANE_OP_IPSET_ADD:
+	case DPLANE_OP_IPSET_DELETE:
+	case DPLANE_OP_IPSET_ENTRY_ADD:
+	case DPLANE_OP_IPSET_ENTRY_DELETE:
+	case DPLANE_OP_NEIGH_IP_INSTALL:
+	case DPLANE_OP_NEIGH_IP_DELETE:
+	case DPLANE_OP_NEIGH_TABLE_UPDATE:
+	case DPLANE_OP_GRE_SET:
+	case DPLANE_OP_INTF_ADDR_ADD:
+	case DPLANE_OP_INTF_ADDR_DEL:
+	case DPLANE_OP_INTF_NETCONFIG:
+	case DPLANE_OP_INTF_INSTALL:
+	case DPLANE_OP_INTF_UPDATE:
+	case DPLANE_OP_INTF_DELETE:
+	case DPLANE_OP_TC_INSTALL:
+	case DPLANE_OP_TC_UPDATE:
+	case DPLANE_OP_TC_DELETE:
 	case DPLANE_OP_NONE:
 		break;
 
-	default:
-		if (IS_ZEBRA_DEBUG_FPM)
-			zlog_debug("%s: unhandled data plane message (%d) %s",
-				   __func__, dplane_ctx_get_op(ctx),
-				   dplane_op2str(dplane_ctx_get_op(ctx)));
-		break;
 	}
 
 	/* Skip empty enqueues. */
@@ -866,6 +876,64 @@ static int fpm_nl_enqueue(struct fpm_nl_ctx *fnc, struct zebra_dplane_ctx *ctx)
 	return 0;
 }
 
+/*
+ * LSP walk/send functions
+ */
+struct fpm_lsp_arg {
+	struct zebra_dplane_ctx *ctx;
+	struct fpm_nl_ctx *fnc;
+	bool complete;
+};
+
+static int fpm_lsp_send_cb(struct hash_bucket *bucket, void *arg)
+{
+	struct zebra_lsp *lsp = bucket->data;
+	struct fpm_lsp_arg *fla = arg;
+
+	/* Skip entries which have already been sent */
+	if (CHECK_FLAG(lsp->flags, LSP_FLAG_FPM))
+		return HASHWALK_CONTINUE;
+
+	dplane_ctx_reset(fla->ctx);
+	dplane_ctx_lsp_init(fla->ctx, DPLANE_OP_LSP_INSTALL, lsp);
+
+	if (fpm_nl_enqueue(fla->fnc, fla->ctx) == -1) {
+		fla->complete = false;
+		return HASHWALK_ABORT;
+	}
+
+	/* Mark entry as sent */
+	SET_FLAG(lsp->flags, LSP_FLAG_FPM);
+	return HASHWALK_CONTINUE;
+}
+
+static void fpm_lsp_send(struct thread *t)
+{
+	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
+	struct zebra_vrf *zvrf = vrf_info_lookup(VRF_DEFAULT);
+	struct fpm_lsp_arg fla;
+
+	fla.fnc = fnc;
+	fla.ctx = dplane_ctx_alloc();
+	fla.complete = true;
+
+	hash_walk(zvrf->lsp_table, fpm_lsp_send_cb, &fla);
+
+	dplane_ctx_fini(&fla.ctx);
+
+	if (fla.complete) {
+		WALK_FINISH(fnc, FNE_LSP_FINISHED);
+
+		/* Now move onto routes */
+		thread_add_timer(zrouter.master, fpm_nhg_reset, fnc, 0,
+				 &fnc->t_nhgreset);
+	} else {
+		/* Didn't finish - reschedule LSP walk */
+		thread_add_timer(zrouter.master, fpm_lsp_send, fnc, 0,
+				 &fnc->t_lspwalk);
+	}
+}
+
 /*
  * Next hop walk/send functions.
  */
@@ -899,7 +967,7 @@ static int fpm_nhg_send_cb(struct hash_bucket *bucket, void *arg)
 	return HASHWALK_CONTINUE;
 }
 
-static int fpm_nhg_send(struct thread *t)
+static void fpm_nhg_send(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 	struct fpm_nhg_arg fna;
@@ -909,7 +977,8 @@ static int fpm_nhg_send(struct thread *t)
 	fna.complete = true;
 
 	/* Send next hops. */
-	hash_walk(zrouter.nhgs_id, fpm_nhg_send_cb, &fna);
+	if (fnc->use_nhg)
+		hash_walk(zrouter.nhgs_id, fpm_nhg_send_cb, &fna);
 
 	/* `free()` allocated memory. */
 	dplane_ctx_fini(&fna.ctx);
@@ -922,14 +991,12 @@ static int fpm_nhg_send(struct thread *t)
 	} else /* Otherwise reschedule next hop group again. */
 		thread_add_timer(zrouter.master, fpm_nhg_send, fnc, 0,
 				 &fnc->t_nhgwalk);
-
-	return 0;
 }
 
 /**
  * Send all RIB installed routes to the connected data plane.
  */
-static int fpm_rib_send(struct thread *t)
+static void fpm_rib_send(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 	rib_dest_t *dest;
@@ -963,7 +1030,7 @@ static int fpm_rib_send(struct thread *t)
 
 				thread_add_timer(zrouter.master, fpm_rib_send,
 						 fnc, 1, &fnc->t_ribwalk);
-				return 0;
+				return;
 			}
 
 			/* Mark as sent. */
@@ -980,8 +1047,6 @@ static int fpm_rib_send(struct thread *t)
 	/* Schedule next event: RMAC reset. */
 	thread_add_event(zrouter.master, fpm_rmac_reset, fnc, 0,
 			 &fnc->t_rmacreset);
-
-	return 0;
 }
 
 /*
@@ -990,14 +1055,14 @@ static int fpm_rib_send(struct thread *t)
 struct fpm_rmac_arg {
 	struct zebra_dplane_ctx *ctx;
 	struct fpm_nl_ctx *fnc;
-	zebra_l3vni_t *zl3vni;
+	struct zebra_l3vni *zl3vni;
 	bool complete;
 };
 
-static void fpm_enqueue_rmac_table(struct hash_bucket *backet, void *arg)
+static void fpm_enqueue_rmac_table(struct hash_bucket *bucket, void *arg)
 {
 	struct fpm_rmac_arg *fra = arg;
-	zebra_mac_t *zrmac = backet->data;
+	struct zebra_mac *zrmac = bucket->data;
 	struct zebra_if *zif = fra->zl3vni->vxlan_if->info;
 	const struct zebra_l2info_vxlan *vxl = &zif->l2info.vxl;
 	struct zebra_if *br_zif;
@@ -1026,16 +1091,16 @@ static void fpm_enqueue_rmac_table(struct hash_bucket *backet, void *arg)
 	}
 }
 
-static void fpm_enqueue_l3vni_table(struct hash_bucket *backet, void *arg)
+static void fpm_enqueue_l3vni_table(struct hash_bucket *bucket, void *arg)
 {
 	struct fpm_rmac_arg *fra = arg;
-	zebra_l3vni_t *zl3vni = backet->data;
+	struct zebra_l3vni *zl3vni = bucket->data;
 
 	fra->zl3vni = zl3vni;
 	hash_iterate(zl3vni->rmac_table, fpm_enqueue_rmac_table, zl3vni);
 }
 
-static int fpm_rmac_send(struct thread *t)
+static void fpm_rmac_send(struct thread *t)
 {
 	struct fpm_rmac_arg fra;
 
@@ -1048,8 +1113,6 @@ static int fpm_rmac_send(struct thread *t)
 	/* RMAC walk completed. */
 	if (fra.complete)
 		WALK_FINISH(fra.fnc, FNE_RMAC_FINISHED);
-
-	return 0;
 }
 
 /*
@@ -1063,23 +1126,41 @@ static void fpm_nhg_reset_cb(struct hash_bucket *bucket, void *arg)
 	UNSET_FLAG(nhe->flags, NEXTHOP_GROUP_FPM);
 }
 
-static int fpm_nhg_reset(struct thread *t)
+static void fpm_nhg_reset(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 
-	fnc->nhg_complete = false;
 	hash_iterate(zrouter.nhgs_id, fpm_nhg_reset_cb, NULL);
 
 	/* Schedule next step: send next hop groups. */
 	thread_add_event(zrouter.master, fpm_nhg_send, fnc, 0, &fnc->t_nhgwalk);
+}
 
-	return 0;
+/*
+ * Resets the LSP FPM flag so we send all LSPs again.
+ */
+static void fpm_lsp_reset_cb(struct hash_bucket *bucket, void *arg)
+{
+	struct zebra_lsp *lsp = bucket->data;
+
+	UNSET_FLAG(lsp->flags, LSP_FLAG_FPM);
+}
+
+static void fpm_lsp_reset(struct thread *t)
+{
+	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
+	struct zebra_vrf *zvrf = vrf_info_lookup(VRF_DEFAULT);
+
+	hash_iterate(zvrf->lsp_table, fpm_lsp_reset_cb, NULL);
+
+	/* Schedule next step: send LSPs */
+	thread_add_event(zrouter.master, fpm_lsp_send, fnc, 0, &fnc->t_lspwalk);
 }
 
 /**
  * Resets the RIB FPM flags so we send all routes again.
  */
-static int fpm_rib_reset(struct thread *t)
+static void fpm_rib_reset(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 	rib_dest_t *dest;
@@ -1087,8 +1168,6 @@ static int fpm_rib_reset(struct thread *t)
 	struct route_table *rt;
 	rib_tables_iter_t rt_iter;
 
-	fnc->rib_complete = false;
-
 	rt_iter.state = RIB_TABLES_ITER_S_INIT;
 	while ((rt = rib_tables_iter_next(&rt_iter))) {
 		for (rn = route_top(rt); rn; rn = srcdest_route_next(rn)) {
@@ -1103,63 +1182,68 @@ static int fpm_rib_reset(struct thread *t)
 
 	/* Schedule next step: send RIB routes. */
 	thread_add_event(zrouter.master, fpm_rib_send, fnc, 0, &fnc->t_ribwalk);
-
-	return 0;
 }
 
 /*
  * The next three function will handle RMAC table reset.
  */
-static void fpm_unset_rmac_table(struct hash_bucket *backet, void *arg)
+static void fpm_unset_rmac_table(struct hash_bucket *bucket, void *arg)
 {
-	zebra_mac_t *zrmac = backet->data;
+	struct zebra_mac *zrmac = bucket->data;
 
 	UNSET_FLAG(zrmac->flags, ZEBRA_MAC_FPM_SENT);
 }
 
-static void fpm_unset_l3vni_table(struct hash_bucket *backet, void *arg)
+static void fpm_unset_l3vni_table(struct hash_bucket *bucket, void *arg)
 {
-	zebra_l3vni_t *zl3vni = backet->data;
+	struct zebra_l3vni *zl3vni = bucket->data;
 
 	hash_iterate(zl3vni->rmac_table, fpm_unset_rmac_table, zl3vni);
 }
 
-static int fpm_rmac_reset(struct thread *t)
+static void fpm_rmac_reset(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 
-	fnc->rmac_complete = false;
 	hash_iterate(zrouter.l3vni_table, fpm_unset_l3vni_table, NULL);
 
 	/* Schedule next event: send RMAC entries. */
 	thread_add_event(zrouter.master, fpm_rmac_send, fnc, 0,
 			 &fnc->t_rmacwalk);
-
-	return 0;
 }
 
-static int fpm_process_queue(struct thread *t)
+static void fpm_process_queue(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
 	struct zebra_dplane_ctx *ctx;
-
-	frr_mutex_lock_autounlock(&fnc->ctxqueue_mutex);
+	bool no_bufs = false;
+	uint64_t processed_contexts = 0;
 
 	while (true) {
 		/* No space available yet. */
-		if (STREAM_WRITEABLE(fnc->obuf) < NL_PKT_BUF_SIZE)
+		if (STREAM_WRITEABLE(fnc->obuf) < NL_PKT_BUF_SIZE) {
+			no_bufs = true;
 			break;
+		}
 
 		/* Dequeue next item or quit processing. */
-		ctx = dplane_ctx_dequeue(&fnc->ctxqueue);
+		frr_with_mutex (&fnc->ctxqueue_mutex) {
+			ctx = dplane_ctx_dequeue(&fnc->ctxqueue);
+		}
 		if (ctx == NULL)
 			break;
 
-		fpm_nl_enqueue(fnc, ctx);
+		/*
+		 * Intentionally ignoring the return value
+		 * as that we are ensuring that we can write to
+		 * the output data in the STREAM_WRITEABLE
+		 * check above, so we can ignore the return
+		 */
+		if (fnc->socket != -1)
+			(void)fpm_nl_enqueue(fnc, ctx);
 
 		/* Account the processed entries. */
-		atomic_fetch_add_explicit(&fnc->counters.dplane_contexts, 1,
-					  memory_order_relaxed);
+		processed_contexts++;
 		atomic_fetch_sub_explicit(&fnc->counters.ctxqueue_len, 1,
 					  memory_order_relaxed);
 
@@ -1167,10 +1251,12 @@ static int fpm_process_queue(struct thread *t)
 		dplane_provider_enqueue_out_ctx(fnc->prov, ctx);
 	}
 
-	/* Check for more items in the queue. */
-	if (atomic_load_explicit(&fnc->counters.ctxqueue_len,
-				 memory_order_relaxed)
-	    > 0)
+	/* Update count of processed contexts */
+	atomic_fetch_add_explicit(&fnc->counters.dplane_contexts,
+				  processed_contexts, memory_order_relaxed);
+
+	/* Re-schedule if we ran out of buffer space */
+	if (no_bufs)
 		thread_add_timer(fnc->fthread->master, fpm_process_queue,
 				 fnc, 0, &fnc->t_dequeue);
 
@@ -1182,17 +1268,15 @@ static int fpm_process_queue(struct thread *t)
 	 */
 	if (dplane_provider_out_ctx_queue_len(fnc->prov) > 0)
 		dplane_provider_work_ready();
-
-	return 0;
 }
 
 /**
  * Handles external (e.g. CLI, data plane or others) events.
  */
-static int fpm_process_event(struct thread *t)
+static void fpm_process_event(struct thread *t)
 {
 	struct fpm_nl_ctx *fnc = THREAD_ARG(t);
-	int event = THREAD_VAL(t);
+	enum fpm_nl_events event = THREAD_VAL(t);
 
 	switch (event) {
 	case FNE_DISABLE:
@@ -1232,29 +1316,20 @@ static int fpm_process_event(struct thread *t)
 		if (IS_ZEBRA_DEBUG_FPM)
 			zlog_debug("%s: next hop groups walk finished",
 				   __func__);
-
-		fnc->nhg_complete = true;
 		break;
 	case FNE_RIB_FINISHED:
 		if (IS_ZEBRA_DEBUG_FPM)
 			zlog_debug("%s: RIB walk finished", __func__);
-
-		fnc->rib_complete = true;
 		break;
 	case FNE_RMAC_FINISHED:
 		if (IS_ZEBRA_DEBUG_FPM)
 			zlog_debug("%s: RMAC walk finished", __func__);
-
-		fnc->rmac_complete = true;
 		break;
-
-	default:
+	case FNE_LSP_FINISHED:
 		if (IS_ZEBRA_DEBUG_FPM)
-			zlog_debug("%s: unhandled event %d", __func__, event);
+			zlog_debug("%s: LSP walk finished", __func__);
 		break;
 	}
-
-	return 0;
 }
 
 /*
@@ -1285,12 +1360,16 @@ static int fpm_nl_start(struct zebra_dplane_provider *prov)
 static int fpm_nl_finish_early(struct fpm_nl_ctx *fnc)
 {
 	/* Disable all events and close socket. */
+	THREAD_OFF(fnc->t_lspreset);
+	THREAD_OFF(fnc->t_lspwalk);
 	THREAD_OFF(fnc->t_nhgreset);
 	THREAD_OFF(fnc->t_nhgwalk);
 	THREAD_OFF(fnc->t_ribreset);
 	THREAD_OFF(fnc->t_ribwalk);
 	THREAD_OFF(fnc->t_rmacreset);
 	THREAD_OFF(fnc->t_rmacwalk);
+	THREAD_OFF(fnc->t_event);
+	THREAD_OFF(fnc->t_nhg);
 	thread_cancel_async(fnc->fthread->master, &fnc->t_read, NULL);
 	thread_cancel_async(fnc->fthread->master, &fnc->t_write, NULL);
 	thread_cancel_async(fnc->fthread->master, &fnc->t_connect, NULL);
@@ -1335,7 +1414,7 @@ static int fpm_nl_process(struct zebra_dplane_provider *prov)
 	struct zebra_dplane_ctx *ctx;
 	struct fpm_nl_ctx *fnc;
 	int counter, limit;
-	uint64_t cur_queue, peak_queue;
+	uint64_t cur_queue, peak_queue = 0, stored_peak_queue;
 
 	fnc = dplane_provider_get_data(prov);
 	limit = dplane_provider_get_work_limit(prov);
@@ -1349,22 +1428,22 @@ static int fpm_nl_process(struct zebra_dplane_provider *prov)
 		 * anyway.
 		 */
 		if (fnc->socket != -1 && fnc->connecting == false) {
-			frr_mutex_lock_autounlock(&fnc->ctxqueue_mutex);
-			dplane_ctx_enqueue_tail(&fnc->ctxqueue, ctx);
-
-			/* Account the number of contexts. */
+			/*
+			 * Update the number of queued contexts *before*
+			 * enqueueing, to ensure counter consistency.
+			 */
 			atomic_fetch_add_explicit(&fnc->counters.ctxqueue_len,
 						  1, memory_order_relaxed);
+
+			frr_with_mutex (&fnc->ctxqueue_mutex) {
+				dplane_ctx_enqueue_tail(&fnc->ctxqueue, ctx);
+			}
+
 			cur_queue = atomic_load_explicit(
 				&fnc->counters.ctxqueue_len,
 				memory_order_relaxed);
-			peak_queue = atomic_load_explicit(
-				&fnc->counters.ctxqueue_len_peak,
-				memory_order_relaxed);
 			if (peak_queue < cur_queue)
-				atomic_store_explicit(
-					&fnc->counters.ctxqueue_len_peak,
-					cur_queue, memory_order_relaxed);
+				peak_queue = cur_queue;
 			continue;
 		}
 
@@ -1372,12 +1451,23 @@ static int fpm_nl_process(struct zebra_dplane_provider *prov)
 		dplane_provider_enqueue_out_ctx(prov, ctx);
 	}
 
+	/* Update peak queue length, if we just observed a new peak */
+	stored_peak_queue = atomic_load_explicit(
+		&fnc->counters.ctxqueue_len_peak, memory_order_relaxed);
+	if (stored_peak_queue < peak_queue)
+		atomic_store_explicit(&fnc->counters.ctxqueue_len_peak,
+				      peak_queue, memory_order_relaxed);
+
 	if (atomic_load_explicit(&fnc->counters.ctxqueue_len,
 				 memory_order_relaxed)
 	    > 0)
 		thread_add_timer(fnc->fthread->master, fpm_process_queue,
 				 fnc, 0, &fnc->t_dequeue);
 
+	/* Ensure dataplane thread is rescheduled if we hit the work limit */
+	if (counter >= limit)
+		dplane_provider_work_ready();
+
 	return 0;
 }
 
@@ -1418,4 +1508,4 @@ FRR_MODULE_SETUP(
 	.version = "0.0.1",
 	.description = "Data plane plugin for FPM using netlink.",
 	.init = fpm_nl_init,
-	)
+);
