@@ -26,6 +26,10 @@
 
 #ifdef HAVE_NETLINK
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * Handle netlink notification informing a rule add or delete.
  */
@@ -36,6 +40,13 @@ extern int netlink_rule_change(struct nlmsghdr *h, ns_id_t ns_id, int startup);
  */
 extern int netlink_rules_read(struct zebra_ns *zns);
 
+extern enum netlink_msg_status
+netlink_put_rule_update_msg(struct nl_batch *bth, struct zebra_dplane_ctx *ctx);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* HAVE_NETLINK */
 
 #endif /* _ZEBRA_RULE_NETLINK_H */
