@@ -310,12 +310,6 @@ static struct log_ref ferr_zebra_err[] = {
 		.suggestion =
 			"Check to see if the nexthop group on the route you tried to install is valid."
 	},
-	{
-		.code = EC_ZEBRA_IF_LOOKUP_FAILED,
-		.title = "Zebra interface lookup failed",
-		.description = "Zebra attempted to look up a interface for a particular vrf_id and interface index, but didn't find anything.",
-		.suggestion = "If you entered a command to trigger this error, make sure you entered the arguments correctly. Check your config file for any potential errors. If these look correct, seek help.",
-	},
 	{
 		.code = EC_ZEBRA_NS_NO_DEFAULT,
 		.title = "Zebra NameSpace failed to find Default",
@@ -791,6 +785,21 @@ static struct log_ref ferr_zebra_err[] = {
 		.description = "Zebra has detected a situation where there are two vrf devices with the exact same tableid.  This is considered a complete misconfiguration of VRF devices and breaks a fundamental assumption in FRR about how VRF's work",
 		.suggestion = "Use different table id's for the VRF's in question"
 	},
+	{
+		.code = EC_ZEBRA_SRV6M_UNRELEASED_LOCATOR_CHUNK,
+		.title = "Zebra did not free any srv6 locator chunks",
+		.description = "Zebra's srv6-locator chunk cleanup procedure ran, but no srv6 locator chunks were released.",
+		.suggestion = "Ignore this error.",
+	},
+	{
+		.code = EC_ZEBRA_INTF_UPDATE_FAILURE,
+		.title =
+			"Zebra failed to update interface in the kernel",
+		.description =
+			"Zebra made an attempt to update an interfce in the kernel, but it was not successful.",
+		.suggestion =
+			"Wait for Zebra to reattempt update.",
+	},
 	{
 		.code = END_FERR,
 	}
