@@ -91,11 +91,11 @@ static int zebra_mpls_transit_lsp(struct vty *vty, int add_cmd,
 	if (gate_str) {
 		/* Gateway is a IPv4 or IPv6 nexthop. */
 		ret = inet_pton(AF_INET6, gate_str, &gate.ipv6);
-		if (ret)
+		if (ret == 1)
 			gtype = NEXTHOP_TYPE_IPV6;
 		else {
 			ret = inet_pton(AF_INET, gate_str, &gate.ipv4);
-			if (ret)
+			if (ret == 1)
 				gtype = NEXTHOP_TYPE_IPV4;
 			else {
 				vty_out(vty, "%% Invalid nexthop\n");
@@ -131,7 +131,7 @@ static int zebra_mpls_transit_lsp(struct vty *vty, int add_cmd,
 		ret = zebra_mpls_static_lsp_del(zvrf, in_label, gtype, &gate,
 						0);
 
-	if (ret) {
+	if (ret != 0) {
 		vty_out(vty, "%% LSP cannot be %s\n",
 			add_cmd ? "added" : "deleted");
 		return CMD_WARNING_CONFIG_FAILED;
@@ -206,7 +206,7 @@ static int zebra_mpls_bind(struct vty *vty, int add_cmd, const char *prefix,
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
-	memset(&p, 0, sizeof(struct prefix));
+	memset(&p, 0, sizeof(p));
 	ret = str2prefix(prefix, &p);
 	if (ret <= 0) {
 		vty_out(vty, "%% Malformed address\n");
