@@ -24,12 +24,22 @@
 
 #include "zebra/zserv.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct mcast_route_data {
-	struct prefix_sg sg;
+	int family;
+	struct ipaddr src;
+	struct ipaddr grp;
 	unsigned int ifindex;
 	unsigned long long lastused;
 };
 
 void zebra_ipmr_route_stats(ZAPI_HANDLER_ARGS);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
