@@ -21,7 +21,6 @@
 #include <zebra.h>
 
 #include "memory.h"
-#include "zebra_memory.h"
 #include "prefix.h"
 #include "rib.h"
 #include "vty.h"
@@ -31,7 +30,7 @@
 #include "plist.h"
 #include "nexthop.h"
 #include "northbound_cli.h"
-#include "route_types.h"
+#include "lib/route_types.h"
 #include "vrf.h"
 #include "frrstr.h"
 
@@ -64,19 +63,17 @@ static void zebra_route_map_set_delay_timer(uint32_t value);
  * Match function return 1 if match is success else return 0
  */
 static enum route_map_cmd_result_t
-route_match_tag(void *rule, const struct prefix *prefix,
-		route_map_object_t type, void *object)
+route_match_tag(void *rule, const struct prefix *prefix, void *object)
 {
 	route_tag_t *tag;
 	struct nh_rmap_obj *nh_data;
 
-	if (type == RMAP_ZEBRA) {
-		tag = rule;
-		nh_data = object;
+	tag = rule;
+	nh_data = object;
+
+	if (nh_data->tag == *tag)
+		return RMAP_MATCH;
 
-		if (nh_data->tag == *tag)
-			return RMAP_MATCH;
-	}
 	return RMAP_NOMATCH;
 }
 
@@ -92,25 +89,23 @@ static const struct route_map_rule_cmd route_match_tag_cmd = {
 /* `match interface IFNAME' */
 /* Match function return 1 if match is success else return zero. */
 static enum route_map_cmd_result_t
-route_match_interface(void *rule, const struct prefix *prefix,
-		      route_map_object_t type, void *object)
+route_match_interface(void *rule, const struct prefix *prefix, void *object)
 {
 	struct nh_rmap_obj *nh_data;
 	char *ifname = rule;
 	ifindex_t ifindex;
 
-	if (type == RMAP_ZEBRA) {
-		if (strcasecmp(ifname, "any") == 0)
-			return RMAP_MATCH;
-		nh_data = object;
-		if (!nh_data || !nh_data->nexthop)
-			return RMAP_NOMATCH;
-		ifindex = ifname2ifindex(ifname, nh_data->vrf_id);
-		if (ifindex == 0)
-			return RMAP_NOMATCH;
-		if (nh_data->nexthop->ifindex == ifindex)
-			return RMAP_MATCH;
-	}
+	if (strcasecmp(ifname, "any") == 0)
+		return RMAP_MATCH;
+	nh_data = object;
+	if (!nh_data || !nh_data->nexthop)
+		return RMAP_NOMATCH;
+	ifindex = ifname2ifindex(ifname, nh_data->vrf_id);
+	if (ifindex == 0)
+		return RMAP_NOMATCH;
+	if (nh_data->nexthop->ifindex == ifindex)
+		return RMAP_MATCH;
+
 	return RMAP_NOMATCH;
 }
 
@@ -271,7 +266,8 @@ static int ip_protocol_rm_add(struct zebra_vrf *zvrf, const char *rmap,
 		/* Process routes of interested address-families. */
 		table = zebra_vrf_table(afi, safi, zvrf->vrf->vrf_id);
 		if (table)
-			rib_update_table(table, RIB_UPDATE_RMAP_CHANGE);
+			rib_update_table(table, RIB_UPDATE_RMAP_CHANGE,
+					 rtype);
 	}
 
 	return CMD_SUCCESS;
@@ -298,7 +294,8 @@ static int ip_protocol_rm_del(struct zebra_vrf *zvrf, const char *rmap,
 			/* Process routes of interested address-families. */
 			table = zebra_vrf_table(afi, safi, zvrf->vrf->vrf_id);
 			if (table)
-				rib_update_table(table, RIB_UPDATE_RMAP_CHANGE);
+				rib_update_table(table, RIB_UPDATE_RMAP_CHANGE,
+						 rtype);
 		}
 		XFREE(MTYPE_ROUTE_MAP_NAME, PROTO_RM_NAME(zvrf, afi, rtype));
 	}
@@ -322,7 +319,7 @@ static int ip_nht_rm_add(struct zebra_vrf *zvrf, const char *rmap, int rtype,
 	route_map_counter_increment(NHT_RM_MAP(zvrf, afi, rtype));
 
 	if (NHT_RM_MAP(zvrf, afi, rtype))
-		zebra_evaluate_rnh(zvrf, AFI_IP, 1, RNH_NEXTHOP_TYPE, NULL);
+		zebra_evaluate_rnh(zvrf, AFI_IP, 1, NULL, SAFI_UNICAST);
 
 	return CMD_SUCCESS;
 }
@@ -343,8 +340,7 @@ static int ip_nht_rm_del(struct zebra_vrf *zvrf, const char *rmap, int rtype,
 					zvrf->vrf->vrf_id, rtype);
 			NHT_RM_MAP(zvrf, afi, rtype) = NULL;
 
-			zebra_evaluate_rnh(zvrf, AFI_IP, 1, RNH_NEXTHOP_TYPE,
-					   NULL);
+			zebra_evaluate_rnh(zvrf, AFI_IP, 1, NULL, SAFI_UNICAST);
 		}
 		XFREE(MTYPE_ROUTE_MAP_NAME, NHT_RM_NAME(zvrf, afi, rtype));
 	}
@@ -360,12 +356,15 @@ DEFPY_YANG(
 	"Match prefix length of IP address\n"
 	"Prefix length\n")
 {
-	const char *xpath = "./match-condition[condition='ipv4-prefix-length']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:ipv4-prefix-length']";
 	char xpath_value[XPATH_MAXLEN];
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
-	snprintf(xpath_value, sizeof(xpath_value),
-		 "%s/frr-zebra:ipv4-prefix-length", xpath);
+	snprintf(
+		xpath_value, sizeof(xpath_value),
+		"%s/rmap-match-condition/frr-zebra-route-map:ipv4-prefix-length",
+		xpath);
 	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, length_str);
 
 	return nb_cli_apply_changes(vty, NULL);
@@ -381,7 +380,8 @@ DEFPY_YANG(
 	"Match prefix length of IP address\n"
 	"Prefix length\n")
 {
-	const char *xpath = "./match-condition[condition='ipv4-prefix-length']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:ipv4-prefix-length']";
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
@@ -397,12 +397,15 @@ DEFPY_YANG(
 	"Match prefix length of IPv6 address\n"
 	"Prefix length\n")
 {
-	const char *xpath = "./match-condition[condition='ipv6-prefix-length']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:ipv6-prefix-length']";
 	char xpath_value[XPATH_MAXLEN];
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
-	snprintf(xpath_value, sizeof(xpath_value),
-		 "%s/frr-zebra:ipv6-prefix-length", xpath);
+	snprintf(
+		xpath_value, sizeof(xpath_value),
+		"%s/rmap-match-condition/frr-zebra-route-map:ipv6-prefix-length",
+		xpath);
 	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, length_str);
 
 	return nb_cli_apply_changes(vty, NULL);
@@ -418,7 +421,8 @@ DEFPY_YANG(
 	"Match prefix length of IPv6 address\n"
 	"Prefix length\n")
 {
-	const char *xpath = "./match-condition[condition='ipv6-prefix-length']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:ipv6-prefix-length']";
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
@@ -435,12 +439,14 @@ DEFPY_YANG(
 	"Prefix length\n")
 {
 	const char *xpath =
-		"./match-condition[condition='ipv4-next-hop-prefix-length']";
+		"./match-condition[condition='frr-zebra-route-map:ipv4-next-hop-prefix-length']";
 	char xpath_value[XPATH_MAXLEN];
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
-	snprintf(xpath_value, sizeof(xpath_value),
-		 "%s/frr-zebra:ipv4-prefix-length", xpath);
+	snprintf(
+		xpath_value, sizeof(xpath_value),
+		"%s/rmap-match-condition/frr-zebra-route-map:ipv4-prefix-length",
+		xpath);
 	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, length_str);
 
 	return nb_cli_apply_changes(vty, NULL);
@@ -457,7 +463,7 @@ DEFPY_YANG(
 	"Prefix length\n")
 {
 	const char *xpath =
-		"./match-condition[condition='ipv4-next-hop-prefix-length']";
+		"./match-condition[condition='frr-zebra-route-map:ipv4-next-hop-prefix-length']";
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
@@ -471,12 +477,14 @@ DEFPY_YANG(
 	"Match protocol via which the route was learnt\n"
 	FRR_REDIST_HELP_STR_ZEBRA)
 {
-	const char *xpath = "./match-condition[condition='source-protocol']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:source-protocol']";
 	char xpath_value[XPATH_MAXLEN];
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
 	snprintf(xpath_value, sizeof(xpath_value),
-		 "%s/frr-zebra:source-protocol", xpath);
+		 "%s/rmap-match-condition/frr-zebra-route-map:source-protocol",
+		 xpath);
 	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, proto);
 
 	return nb_cli_apply_changes(vty, NULL);
@@ -490,7 +498,8 @@ DEFPY_YANG(
 	"Match protocol via which the route was learnt\n"
 	FRR_REDIST_HELP_STR_ZEBRA)
 {
-	const char *xpath = "./match-condition[condition='source-protocol']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:source-protocol']";
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
@@ -504,12 +513,14 @@ DEFPY_YANG(
 	"Match the protocol's instance number\n"
 	"The instance number\n")
 {
-	const char *xpath = "./match-condition[condition='source-instance']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:source-instance']";
 	char xpath_value[XPATH_MAXLEN];
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
 	snprintf(xpath_value, sizeof(xpath_value),
-		 "%s/frr-zebra:source-instance", xpath);
+		 "%s/rmap-match-condition/frr-zebra-route-map:source-instance",
+		 xpath);
 	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, instance_str);
 
 	return nb_cli_apply_changes(vty, NULL);
@@ -522,7 +533,8 @@ DEFPY_YANG(
 	"Match the protocol's instance number\n"
 	"The instance number\n")
 {
-	const char *xpath = "./match-condition[condition='source-instance']";
+	const char *xpath =
+		"./match-condition[condition='frr-zebra-route-map:source-instance']";
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
@@ -539,18 +551,23 @@ DEFPY_YANG(
 	"IPv4 src address\n"
 	"IPv6 src address\n")
 {
-	const char *xpath = "./set-action[action='source']";
+	const char *xpath =
+		"./set-action[action='frr-zebra-route-map:src-address']";
 	char xpath_value[XPATH_MAXLEN];
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
 	if (addrv4_str) {
-		snprintf(xpath_value, sizeof(xpath_value),
-			 "%s/frr-zebra:source-v4", xpath);
+		snprintf(
+			xpath_value, sizeof(xpath_value),
+			"%s/rmap-set-action/frr-zebra-route-map:ipv4-src-address",
+			xpath);
 		nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY,
 				      addrv4_str);
 	} else {
-		snprintf(xpath_value, sizeof(xpath_value),
-			 "%s/frr-zebra:source-v6", xpath);
+		snprintf(
+			xpath_value, sizeof(xpath_value),
+			"%s/rmap-set-action/frr-zebra-route-map:ipv6-src-address",
+			xpath);
 		nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY,
 				      addrv6_str);
 	}
@@ -567,20 +584,21 @@ DEFPY_YANG(
 	"IPv4 address\n"
 	"IPv6 address\n")
 {
-	const char *xpath = "./set-action[action='source']";
+	const char *xpath =
+		"./set-action[action='frr-zebra-route-map:src-address']";
 
 	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
 
 	return nb_cli_apply_changes(vty, NULL);
 }
 
-DEFUN (zebra_route_map_timer,
+DEFUN_YANG (zebra_route_map_timer,
        zebra_route_map_timer_cmd,
        "zebra route-map delay-timer (0-600)",
        ZEBRA_STR
        "Set route-map parameters\n"
        "Time to wait before route-map updates are processed\n"
-       "0 means event-driven updates are disabled\n")
+       "0 means route-map changes are run immediately instead of delaying\n")
 {
 	int idx_number = 3;
 	uint32_t rmap_delay_timer;
@@ -591,14 +609,14 @@ DEFUN (zebra_route_map_timer,
 	return (CMD_SUCCESS);
 }
 
-DEFUN (no_zebra_route_map_timer,
+DEFUN_YANG (no_zebra_route_map_timer,
        no_zebra_route_map_timer_cmd,
        "no zebra route-map delay-timer [(0-600)]",
        NO_STR
        ZEBRA_STR
        "Set route-map parameters\n"
        "Reset delay-timer to default value, 30 secs\n"
-       "0 means event-driven updates are disabled\n")
+       "0 means route-map changes are run immediately instead of delaying\n")
 {
 	zebra_route_map_set_delay_timer(ZEBRA_RMAP_DEFAULT_UPDATE_TIMER);
 
@@ -620,7 +638,7 @@ DEFPY_YANG (ip_protocol,
 	assert(proto);
 	assert(rmap);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -654,7 +672,7 @@ DEFPY_YANG (no_ip_protocol,
 
 	assert(proto);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -701,7 +719,7 @@ DEFPY_YANG (ipv6_protocol,
 	assert(rmap);
 	assert(proto);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -735,7 +753,7 @@ DEFPY_YANG (no_ipv6_protocol,
 
 	assert(proto);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -783,7 +801,7 @@ DEFPY_YANG (ip_protocol_nht_rmap,
 	assert(proto);
 	assert(rmap);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -817,7 +835,7 @@ DEFPY_YANG (no_ip_protocol_nht_rmap,
 
 	assert(proto);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -865,7 +883,7 @@ DEFPY_YANG (ipv6_protocol_nht_rmap,
 	assert(rmap);
 	assert(proto);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -899,7 +917,7 @@ DEFPY_YANG (no_ipv6_protocol_nht_rmap,
 
 	assert(proto);
 
-	ZEBRA_DECLVAR_CONTEXT(vrf, zvrf);
+	ZEBRA_DECLVAR_CONTEXT_VRF(vrf, zvrf);
 
 	if (!zvrf)
 		return CMD_WARNING;
@@ -938,40 +956,35 @@ DEFPY_YANG (show_ipv6_protocol_nht,
 
 /* Match function return 1 if match is success else return zero. */
 static enum route_map_cmd_result_t
-route_match_ip_next_hop(void *rule, const struct prefix *prefix,
-			route_map_object_t type, void *object)
+route_match_ip_next_hop(void *rule, const struct prefix *prefix, void *object)
 {
 	struct access_list *alist;
 	struct nh_rmap_obj *nh_data;
 	struct prefix_ipv4 p;
 
-	if (type == RMAP_ZEBRA) {
-		nh_data = object;
-		if (!nh_data)
-			return RMAP_NOMATCH;
+	nh_data = object;
+	if (!nh_data)
+		return RMAP_NOMATCH;
 
-		switch (nh_data->nexthop->type) {
-		case NEXTHOP_TYPE_IFINDEX:
-			/* Interface routes can't match ip next-hop */
-			return RMAP_NOMATCH;
-		case NEXTHOP_TYPE_IPV4_IFINDEX:
-		case NEXTHOP_TYPE_IPV4:
-			p.family = AF_INET;
-			p.prefix = nh_data->nexthop->gate.ipv4;
-			p.prefixlen = IPV4_MAX_BITLEN;
-			break;
-		default:
-			return RMAP_NOMATCH;
-		}
-		alist = access_list_lookup(AFI_IP, (char *)rule);
-		if (alist == NULL)
-			return RMAP_NOMATCH;
-
-		return (access_list_apply(alist, &p) == FILTER_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
+	switch (nh_data->nexthop->type) {
+	case NEXTHOP_TYPE_IFINDEX:
+		/* Interface routes can't match ip next-hop */
+		return RMAP_NOMATCH;
+	case NEXTHOP_TYPE_IPV4_IFINDEX:
+	case NEXTHOP_TYPE_IPV4:
+		p.family = AF_INET;
+		p.prefix = nh_data->nexthop->gate.ipv4;
+		p.prefixlen = IPV4_MAX_BITLEN;
+		break;
+	default:
+		return RMAP_NOMATCH;
 	}
-	return RMAP_NOMATCH;
+	alist = access_list_lookup(AFI_IP, (char *)rule);
+	if (alist == NULL)
+		return RMAP_NOMATCH;
+
+	return (access_list_apply(alist, &p) == FILTER_DENY ? RMAP_NOMATCH
+							    : RMAP_MATCH);
 }
 
 /* Route map `ip next-hop' match statement.  `arg' should be
@@ -999,39 +1012,35 @@ static const struct route_map_rule_cmd route_match_ip_next_hop_cmd = {
 
 static enum route_map_cmd_result_t
 route_match_ip_next_hop_prefix_list(void *rule, const struct prefix *prefix,
-				    route_map_object_t type, void *object)
+				    void *object)
 {
 	struct prefix_list *plist;
 	struct nh_rmap_obj *nh_data;
 	struct prefix_ipv4 p;
 
-	if (type == RMAP_ZEBRA) {
-		nh_data = (struct nh_rmap_obj *)object;
-		if (!nh_data)
-			return RMAP_NOMATCH;
+	nh_data = (struct nh_rmap_obj *)object;
+	if (!nh_data)
+		return RMAP_NOMATCH;
 
-		switch (nh_data->nexthop->type) {
-		case NEXTHOP_TYPE_IFINDEX:
-			/* Interface routes can't match ip next-hop */
-			return RMAP_NOMATCH;
-		case NEXTHOP_TYPE_IPV4_IFINDEX:
-		case NEXTHOP_TYPE_IPV4:
-			p.family = AF_INET;
-			p.prefix = nh_data->nexthop->gate.ipv4;
-			p.prefixlen = IPV4_MAX_BITLEN;
-			break;
-		default:
-			return RMAP_NOMATCH;
-		}
-		plist = prefix_list_lookup(AFI_IP, (char *)rule);
-		if (plist == NULL)
-			return RMAP_NOMATCH;
-
-		return (prefix_list_apply(plist, &p) == PREFIX_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
+	switch (nh_data->nexthop->type) {
+	case NEXTHOP_TYPE_IFINDEX:
+		/* Interface routes can't match ip next-hop */
+		return RMAP_NOMATCH;
+	case NEXTHOP_TYPE_IPV4_IFINDEX:
+	case NEXTHOP_TYPE_IPV4:
+		p.family = AF_INET;
+		p.prefix = nh_data->nexthop->gate.ipv4;
+		p.prefixlen = IPV4_MAX_BITLEN;
+		break;
+	default:
+		return RMAP_NOMATCH;
 	}
-	return RMAP_NOMATCH;
+	plist = prefix_list_lookup(AFI_IP, (char *)rule);
+	if (plist == NULL)
+		return RMAP_NOMATCH;
+
+	return (prefix_list_apply(plist, &p) == PREFIX_DENY ? RMAP_NOMATCH
+							    : RMAP_MATCH);
 }
 
 static void *route_match_ip_next_hop_prefix_list_compile(const char *arg)
@@ -1058,34 +1067,28 @@ static const struct route_map_rule_cmd
    zero. */
 static enum route_map_cmd_result_t
 route_match_address(afi_t afi, void *rule, const struct prefix *prefix,
-		    route_map_object_t type, void *object)
+		    void *object)
 {
 	struct access_list *alist;
 
-	if (type == RMAP_ZEBRA) {
-		alist = access_list_lookup(afi, (char *)rule);
-		if (alist == NULL)
-			return RMAP_NOMATCH;
+	alist = access_list_lookup(afi, (char *)rule);
+	if (alist == NULL)
+		return RMAP_NOMATCH;
 
-		return (access_list_apply(alist, prefix) == FILTER_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
-	}
-	return RMAP_NOMATCH;
+	return (access_list_apply(alist, prefix) == FILTER_DENY ? RMAP_NOMATCH
+								: RMAP_MATCH);
 }
 
 static enum route_map_cmd_result_t
-route_match_ip_address(void *rule, const struct prefix *prefix,
-		       route_map_object_t type, void *object)
+route_match_ip_address(void *rule, const struct prefix *prefix, void *object)
 {
-	return route_match_address(AFI_IP, rule, prefix, type, object);
+	return route_match_address(AFI_IP, rule, prefix, object);
 }
 
 static enum route_map_cmd_result_t
-route_match_ipv6_address(void *rule, const struct prefix *prefix,
-			 route_map_object_t type, void *object)
+route_match_ipv6_address(void *rule, const struct prefix *prefix, void *object)
 {
-	return route_match_address(AFI_IP6, rule, prefix, type, object);
+	return route_match_address(AFI_IP6, rule, prefix, object);
 }
 
 /* Route map `ip address' match statement.  `arg' should be
@@ -1121,28 +1124,23 @@ static const struct route_map_rule_cmd route_match_ipv6_address_cmd = {
 
 static enum route_map_cmd_result_t
 route_match_address_prefix_list(void *rule, const struct prefix *prefix,
-			route_map_object_t type, void *object, afi_t afi)
+				void *object, afi_t afi)
 {
 	struct prefix_list *plist;
 
-	if (type == RMAP_ZEBRA) {
-		plist = prefix_list_lookup(afi, (char *)rule);
-		if (plist == NULL)
-			return RMAP_NOMATCH;
+	plist = prefix_list_lookup(afi, (char *)rule);
+	if (plist == NULL)
+		return RMAP_NOMATCH;
 
-		return (prefix_list_apply(plist, prefix) == PREFIX_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
-	}
-	return RMAP_NOMATCH;
+	return (prefix_list_apply(plist, prefix) == PREFIX_DENY ? RMAP_NOMATCH
+								: RMAP_MATCH);
 }
 
 static enum route_map_cmd_result_t
 route_match_ip_address_prefix_list(void *rule, const struct prefix *prefix,
-				   route_map_object_t type, void *object)
+				   void *object)
 {
-	return (route_match_address_prefix_list(rule, prefix, type, object,
-						AFI_IP));
+	return (route_match_address_prefix_list(rule, prefix, object, AFI_IP));
 }
 
 static void *route_match_address_prefix_list_compile(const char *arg)
@@ -1165,10 +1163,9 @@ static const struct route_map_rule_cmd
 
 static enum route_map_cmd_result_t
 route_match_ipv6_address_prefix_list(void *rule, const struct prefix *prefix,
-					route_map_object_t type, void *object)
+				     void *object)
 {
-	return (route_match_address_prefix_list(rule, prefix, type, object,
-						AFI_IP6));
+	return (route_match_address_prefix_list(rule, prefix, object, AFI_IP6));
 }
 
 static const struct route_map_rule_cmd
@@ -1183,11 +1180,11 @@ static const struct route_map_rule_cmd
 
 static enum route_map_cmd_result_t
 route_match_ipv6_next_hop_type(void *rule, const struct prefix *prefix,
-			       route_map_object_t type, void *object)
+			       void *object)
 {
 	struct nh_rmap_obj *nh_data;
 
-	if (type == RMAP_ZEBRA && prefix->family == AF_INET6) {
+	if (prefix->family == AF_INET6) {
 		nh_data = (struct nh_rmap_obj *)object;
 		if (!nh_data)
 			return RMAP_NOMATCH;
@@ -1195,6 +1192,7 @@ route_match_ipv6_next_hop_type(void *rule, const struct prefix *prefix,
 		if (nh_data->nexthop->type == NEXTHOP_TYPE_BLACKHOLE)
 			return RMAP_MATCH;
 	}
+
 	return RMAP_NOMATCH;
 }
 
@@ -1220,15 +1218,11 @@ static const struct route_map_rule_cmd
 
 static enum route_map_cmd_result_t
 route_match_address_prefix_len(void *rule, const struct prefix *prefix,
-			       route_map_object_t type, void *object)
+			       void *object)
 {
 	uint32_t *prefixlen = (uint32_t *)rule;
 
-	if (type == RMAP_ZEBRA) {
-		return ((prefix->prefixlen == *prefixlen) ? RMAP_MATCH
-							  : RMAP_NOMATCH);
-	}
-	return RMAP_NOMATCH;
+	return ((prefix->prefixlen == *prefixlen) ? RMAP_MATCH : RMAP_NOMATCH);
 }
 
 static void *route_match_address_prefix_len_compile(const char *arg)
@@ -1277,34 +1271,30 @@ static const struct route_map_rule_cmd
 
 static enum route_map_cmd_result_t
 route_match_ip_nexthop_prefix_len(void *rule, const struct prefix *prefix,
-				  route_map_object_t type, void *object)
+				  void *object)
 {
 	uint32_t *prefixlen = (uint32_t *)rule;
 	struct nh_rmap_obj *nh_data;
 	struct prefix_ipv4 p;
 
-	if (type == RMAP_ZEBRA) {
-		nh_data = (struct nh_rmap_obj *)object;
-		if (!nh_data || !nh_data->nexthop)
-			return RMAP_NOMATCH;
+	nh_data = (struct nh_rmap_obj *)object;
+	if (!nh_data || !nh_data->nexthop)
+		return RMAP_NOMATCH;
 
-		switch (nh_data->nexthop->type) {
-		case NEXTHOP_TYPE_IFINDEX:
-			/* Interface routes can't match ip next-hop */
-			return RMAP_NOMATCH;
-		case NEXTHOP_TYPE_IPV4_IFINDEX:
-		case NEXTHOP_TYPE_IPV4:
-			p.family = AF_INET;
-			p.prefix = nh_data->nexthop->gate.ipv4;
-			p.prefixlen = IPV4_MAX_BITLEN;
-			break;
-		default:
-			return RMAP_NOMATCH;
-		}
-		return ((p.prefixlen == *prefixlen) ? RMAP_MATCH
-						    : RMAP_NOMATCH);
+	switch (nh_data->nexthop->type) {
+	case NEXTHOP_TYPE_IFINDEX:
+		/* Interface routes can't match ip next-hop */
+		return RMAP_NOMATCH;
+	case NEXTHOP_TYPE_IPV4_IFINDEX:
+	case NEXTHOP_TYPE_IPV4:
+		p.family = AF_INET;
+		p.prefix = nh_data->nexthop->gate.ipv4;
+		p.prefixlen = IPV4_MAX_BITLEN;
+		break;
+	default:
+		return RMAP_NOMATCH;
 	}
-	return RMAP_NOMATCH;
+	return ((p.prefixlen == *prefixlen) ? RMAP_MATCH : RMAP_NOMATCH);
 }
 
 static const struct route_map_rule_cmd
@@ -1319,11 +1309,11 @@ static const struct route_map_rule_cmd
 
 static enum route_map_cmd_result_t
 route_match_ip_next_hop_type(void *rule, const struct prefix *prefix,
-			     route_map_object_t type, void *object)
+			     void *object)
 {
 	struct nh_rmap_obj *nh_data;
 
-	if (type == RMAP_ZEBRA && prefix->family == AF_INET) {
+	if (prefix->family == AF_INET) {
 		nh_data = (struct nh_rmap_obj *)object;
 		if (!nh_data)
 			return RMAP_NOMATCH;
@@ -1331,6 +1321,7 @@ route_match_ip_next_hop_type(void *rule, const struct prefix *prefix,
 		if (nh_data->nexthop->type == NEXTHOP_TYPE_BLACKHOLE)
 			return RMAP_MATCH;
 	}
+
 	return RMAP_NOMATCH;
 }
 
@@ -1355,21 +1346,17 @@ static const struct route_map_rule_cmd
 /* `match source-protocol PROTOCOL' */
 
 static enum route_map_cmd_result_t
-route_match_source_protocol(void *rule, const struct prefix *p,
-			    route_map_object_t type, void *object)
+route_match_source_protocol(void *rule, const struct prefix *p, void *object)
 {
 	uint32_t *rib_type = (uint32_t *)rule;
 	struct nh_rmap_obj *nh_data;
 
-	if (type == RMAP_ZEBRA) {
-		nh_data = (struct nh_rmap_obj *)object;
-		if (!nh_data)
-			return RMAP_NOMATCH;
+	nh_data = (struct nh_rmap_obj *)object;
+	if (!nh_data)
+		return RMAP_NOMATCH;
 
-		return ((nh_data->source_protocol == *rib_type) ? RMAP_MATCH
-								: RMAP_NOMATCH);
-	}
-	return RMAP_NOMATCH;
+	return ((nh_data->source_protocol == *rib_type) ? RMAP_MATCH
+							: RMAP_NOMATCH);
 }
 
 static void *route_match_source_protocol_compile(const char *arg)
@@ -1399,15 +1386,11 @@ static const struct route_map_rule_cmd route_match_source_protocol_cmd = {
 
 /* `source-instance` */
 static enum route_map_cmd_result_t
-route_match_source_instance(void *rule, const struct prefix *p,
-			    route_map_object_t type, void *object)
+route_match_source_instance(void *rule, const struct prefix *p, void *object)
 {
 	uint8_t *instance = (uint8_t *)rule;
 	struct nh_rmap_obj *nh_data;
 
-	if (type != RMAP_ZEBRA)
-		return RMAP_NOMATCH;
-
 	nh_data = (struct nh_rmap_obj *)object;
 	if (!nh_data)
 		return RMAP_NOMATCH;
@@ -1444,15 +1427,13 @@ static const struct route_map_rule_cmd route_match_source_instance_cmd = {
 
 /* Set src. */
 static enum route_map_cmd_result_t
-route_set_src(void *rule, const struct prefix *prefix, route_map_object_t type,
-	      void *object)
+route_set_src(void *rule, const struct prefix *prefix, void *object)
 {
 	struct nh_rmap_obj *nh_data;
 
-	if (type == RMAP_ZEBRA) {
-		nh_data = (struct nh_rmap_obj *)object;
-		nh_data->nexthop->rmap_src = *(union g_addr *)rule;
-	}
+	nh_data = (struct nh_rmap_obj *)object;
+	nh_data->nexthop->rmap_src = *(union g_addr *)rule;
+
 	return RMAP_OKAY;
 }
 
@@ -1495,8 +1476,6 @@ static void zebra_rib_table_rm_update(const char *rmap)
 	struct vrf *vrf = NULL;
 	struct zebra_vrf *zvrf = NULL;
 	char *rmap_name;
-	char afi_ip = 0;
-	char afi_ipv6 = 0;
 	struct route_map *old = NULL;
 
 	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
@@ -1527,16 +1506,12 @@ static void zebra_rib_table_rm_update(const char *rmap)
 						PROTO_RM_MAP(zvrf, AFI_IP, i));
 				/* There is single rib table for all protocols
 				 */
-				if (afi_ip == 0) {
-					table = zvrf->table[AFI_IP]
-							   [SAFI_UNICAST];
-					if (table) {
-
-						afi_ip = 1;
-						rib_update_table(
-							table,
-							RIB_UPDATE_RMAP_CHANGE);
-					}
+				table = zvrf->table[AFI_IP][SAFI_UNICAST];
+				if (table) {
+					rib_update_table(
+						table,
+						RIB_UPDATE_RMAP_CHANGE,
+						i);
 				}
 			}
 			rmap_name = PROTO_RM_NAME(zvrf, AFI_IP6, i);
@@ -1556,16 +1531,12 @@ static void zebra_rib_table_rm_update(const char *rmap)
 						PROTO_RM_MAP(zvrf, AFI_IP6, i));
 				/* There is single rib table for all protocols
 				 */
-				if (afi_ipv6 == 0) {
-					table = zvrf->table[AFI_IP6]
-							   [SAFI_UNICAST];
-					if (table) {
-
-						afi_ipv6 = 1;
-						rib_update_table(
-							table,
-							RIB_UPDATE_RMAP_CHANGE);
-					}
+				table = zvrf->table[AFI_IP6][SAFI_UNICAST];
+				if (table) {
+					rib_update_table(
+						table,
+						RIB_UPDATE_RMAP_CHANGE,
+						i);
 				}
 			}
 		}
@@ -1617,8 +1588,8 @@ static void zebra_nht_rm_update(const char *rmap)
 						afi_ip = 1;
 
 						zebra_evaluate_rnh(
-							zvrf, AFI_IP, 1,
-							RNH_NEXTHOP_TYPE, NULL);
+							zvrf, AFI_IP, 1, NULL,
+							SAFI_UNICAST);
 					}
 				}
 			}
@@ -1648,8 +1619,8 @@ static void zebra_nht_rm_update(const char *rmap)
 						afi_ipv6 = 1;
 
 						zebra_evaluate_rnh(
-							zvrf, AFI_IP, 1,
-							RNH_NEXTHOP_TYPE, NULL);
+							zvrf, AFI_IP, 1, NULL,
+							SAFI_UNICAST);
 					}
 				}
 			}
@@ -1667,10 +1638,8 @@ static void zebra_route_map_process_update_cb(char *rmap_name)
 	zebra_nht_rm_update(rmap_name);
 }
 
-static int zebra_route_map_update_timer(struct thread *thread)
+static void zebra_route_map_update_timer(struct thread *thread)
 {
-	zebra_t_rmap_update = NULL;
-
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_debug("Event driven route-map update triggered");
 
@@ -1686,7 +1655,6 @@ static int zebra_route_map_update_timer(struct thread *thread)
 	 * 1) VRF Aware <sigh>
 	 * 2) Route-map aware
 	 */
-	return 0;
 }
 
 static void zebra_route_map_set_delay_timer(uint32_t value)
@@ -1695,8 +1663,8 @@ static void zebra_route_map_set_delay_timer(uint32_t value)
 	if (!value && zebra_t_rmap_update) {
 		/* Event driven route map updates is being disabled */
 		/* But there's a pending timer. Fire it off now */
-		thread_cancel(zebra_t_rmap_update);
-		zebra_route_map_update_timer(zebra_t_rmap_update);
+		THREAD_OFF(zebra_t_rmap_update);
+		zebra_route_map_update_timer(NULL);
 	}
 }
 
@@ -1705,24 +1673,17 @@ void zebra_routemap_finish(void)
 	/* Set zebra_rmap_update_timer to 0 so that it wont schedule again */
 	zebra_rmap_update_timer = 0;
 	/* Thread off if any scheduled already */
-	THREAD_TIMER_OFF(zebra_t_rmap_update);
+	THREAD_OFF(zebra_t_rmap_update);
 	route_map_finish();
 }
 
-void zebra_route_map_write_delay_timer(struct vty *vty)
-{
-	if (vty && (zebra_rmap_update_timer != ZEBRA_RMAP_DEFAULT_UPDATE_TIMER))
-		vty_out(vty, "zebra route-map delay-timer %d\n",
-			zebra_rmap_update_timer);
-	return;
-}
-
 route_map_result_t
-zebra_route_map_check(int family, int rib_type, uint8_t instance,
+zebra_route_map_check(afi_t family, int rib_type, uint8_t instance,
 		      const struct prefix *p, struct nexthop *nexthop,
 		      struct zebra_vrf *zvrf, route_tag_t tag)
 {
 	struct route_map *rmap = NULL;
+	char *rm_name;
 	route_map_result_t ret = RMAP_PERMITMATCH;
 	struct nh_rmap_obj nh_obj;
 
@@ -1733,12 +1694,22 @@ zebra_route_map_check(int family, int rib_type, uint8_t instance,
 	nh_obj.metric = 0;
 	nh_obj.tag = tag;
 
-	if (rib_type >= 0 && rib_type < ZEBRA_ROUTE_MAX)
+	if (rib_type >= 0 && rib_type < ZEBRA_ROUTE_MAX) {
+		rm_name = PROTO_RM_NAME(zvrf, family, rib_type);
 		rmap = PROTO_RM_MAP(zvrf, family, rib_type);
-	if (!rmap && PROTO_RM_NAME(zvrf, family, ZEBRA_ROUTE_MAX))
+
+		if (rm_name && !rmap)
+			return RMAP_DENYMATCH;
+	}
+	if (!rmap) {
+		rm_name = PROTO_RM_NAME(zvrf, family, ZEBRA_ROUTE_MAX);
 		rmap = PROTO_RM_MAP(zvrf, family, ZEBRA_ROUTE_MAX);
+
+		if (rm_name && !rmap)
+			return RMAP_DENYMATCH;
+	}
 	if (rmap) {
-		ret = route_map_apply(rmap, p, RMAP_ZEBRA, &nh_obj);
+		ret = route_map_apply(rmap, p, &nh_obj);
 	}
 
 	return (ret);
@@ -1782,7 +1753,7 @@ zebra_import_table_route_map_check(int family, int re_type, uint8_t instance,
 	if (re_type >= 0 && re_type < ZEBRA_ROUTE_MAX)
 		rmap = route_map_lookup_by_name(rmap_name);
 	if (rmap) {
-		ret = route_map_apply(rmap, p, RMAP_ZEBRA, &nh_obj);
+		ret = route_map_apply(rmap, p, &nh_obj);
 	}
 
 	return (ret);
@@ -1810,7 +1781,7 @@ route_map_result_t zebra_nht_route_map_check(afi_t afi, int client_proto,
 	if (!rmap && NHT_RM_MAP(zvrf, afi, ZEBRA_ROUTE_MAX))
 		rmap = NHT_RM_MAP(zvrf, afi, ZEBRA_ROUTE_MAX);
 	if (rmap)
-		ret = route_map_apply(rmap, p, RMAP_ZEBRA, &nh_obj);
+		ret = route_map_apply(rmap, p, &nh_obj);
 
 	return ret;
 }
@@ -1818,12 +1789,11 @@ route_map_result_t zebra_nht_route_map_check(afi_t afi, int client_proto,
 static void zebra_route_map_mark_update(const char *rmap_name)
 {
 	/* rmap_update_timer of 0 means don't do route updates */
-	if (zebra_rmap_update_timer && !zebra_t_rmap_update) {
-		zebra_t_rmap_update = NULL;
-		thread_add_timer(zrouter.master, zebra_route_map_update_timer,
-				 NULL, zebra_rmap_update_timer,
-				 &zebra_t_rmap_update);
-	}
+	if (zebra_rmap_update_timer)
+		THREAD_OFF(zebra_t_rmap_update);
+
+	thread_add_timer(zrouter.master, zebra_route_map_update_timer,
+			 NULL, zebra_rmap_update_timer, &zebra_t_rmap_update);
 }
 
 static void zebra_route_map_add(const char *rmap_name)
@@ -1850,6 +1820,23 @@ static void zebra_route_map_event(const char *rmap_name)
 	route_map_notify_dependencies(rmap_name, RMAP_EVENT_MATCH_ADDED);
 }
 
+void zebra_routemap_vrf_delete(struct zebra_vrf *zvrf)
+{
+	afi_t afi;
+	uint8_t type;
+
+	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
+		for (type = 0; type <= ZEBRA_ROUTE_MAX; type++) {
+			if (PROTO_RM_NAME(zvrf, afi, type))
+				XFREE(MTYPE_ROUTE_MAP_NAME,
+				      PROTO_RM_NAME(zvrf, afi, type));
+			if (NHT_RM_NAME(zvrf, afi, type))
+				XFREE(MTYPE_ROUTE_MAP_NAME,
+				      NHT_RM_NAME(zvrf, afi, type));
+		}
+	}
+}
+
 /* ip protocol configuration write function */
 void zebra_routemap_config_write_protocol(struct vty *vty,
 					  struct zebra_vrf *zvrf)
